#include <precomp.h>

#include "Logger.h"

#include "AugmentedForm.h"  // class implemented

LOGGER(WARNING);


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

AugmentedForm::AugmentedForm()
    : mA(), mb(), mc(), mActualVariables()
{
}// AugmentedForm

AugmentedForm::AugmentedForm(const Matrix& A,
                             const ColumnVector& b,
                             const ColumnVector& c,
                             const int actualVariables)
{
    mA = A;
    mb = b;
    mc = c;
    mActualVariables = actualVariables;
}// AugmentedForm

AugmentedForm::AugmentedForm(const LinearProblem& problem)
{
    Matrix       A(problem.ConstraintCount(), problem.VariableCount());
    ColumnVector b(problem.ConstraintCount());
    ColumnVector c(problem.VariableCount());
    Constraint::Container::const_iterator iter;
    int i = 1;
    int slack_count = 0;
    int n = problem.VariableCount();

    assert (problem.ConstraintCount() > 0);
    assert (n > 0);

    LOG(INFO, wxT("-----------CreateAugmentedForm-----------"));
    // Wypeniamy macierz A i wektor b
    for (iter  = problem.Constraints().begin();
         iter != problem.Constraints().end();
         iter++, i++)
    {
        const LinearFunction& r_constraint_function =
                dynamic_cast<const LinearFunction&>(iter->Function());
        A.Row(i) = r_constraint_function.Coefficients();
        b(i) = -r_constraint_function.A0();

        // Dodamy tyle zmiennych dopeniajcych, ile jest
        // ogranicze nierwnociowych.
        if (iter->Type() != Constraint::EQUAL)
        {
           slack_count++;
        }
    }

    // Tworzymy macierz dopeniajc i doczamy j do A.
    Matrix A_slacks(A.nrows(), slack_count);
    A_slacks = 0.0;
    A = A | A_slacks;

    // Dodajemy zmienne dopeniajce
    i = 1;
    int slacks_added = 0;
    for (iter  = problem.Constraints().begin();
         iter != problem.Constraints().end();
         iter++, i++)
    {
        if (iter->Type() == Constraint::LESS_EQUAL_THAN)
        {
            A(i, n + slacks_added + 1) = 1.0;
            slacks_added++;
        }
        else if (iter->Type() == Constraint::GREATER_EQUAL_THAN)
        {
            A(i, n + slacks_added + 1) = -1.0;
            slacks_added++;
        }

        // Wszystkie wspczynniki wektora b musz by nieujemne,
        // eby punkt startowy w simpleksie spenia warunek
        // x >= 0 .
        if (b(i) < 0)
        {
            b(i) = -b(i);
            A.Row(i) = -A.Row(i);
        }
    }

    // Wypeniamy wektor c
    c = dynamic_cast<const LinearFunction&>(problem.Function()).Coefficients().t();
    // Tworzymy wektor dopeniajcy i doczamy go do c.
    ColumnVector c_slacks(slack_count);
    c_slacks = 0.0;
    c = c & c_slacks;

    // LOG(INFO, wxT("A:") << endl << A);
    // LOG(INFO, wxT("b:") << endl << b);
    // LOG(INFO, wxT("c:") << endl << c);

    mA = A;
    mb = b;
    mc = c;
    mActualVariables = problem.VariableCount();
    LOG(INFO, wxT("Forma standardowa:") << endl << *this);

} // AugmentedForm


AugmentedForm::AugmentedForm(const AugmentedForm& from)
    : mA(from.mA), mb(from.mb), mc(from.mc), mActualVariables(from.mActualVariables)
{
}// AugmentedForm

AugmentedForm::~AugmentedForm()
{
}// ~AugmentedForm


//============================= OPERATORS ====================================


AugmentedForm&
AugmentedForm::operator=(const AugmentedForm& from)
{
    mA = from.mA;
    mb = from.mb;
    mc = from.mc;
    mActualVariables = from.mActualVariables;
    return *this;
}// =


bool
AugmentedForm::operator==(const AugmentedForm& right) const
{
    return (mA == right.mA && mb == right.mb && mc == right.mc &&
            mActualVariables == right.mActualVariables);
}// ==

wxTextOutputStream&
operator<<(wxTextOutputStream& os, const AugmentedForm&   augmentedForm)
{
    os << wxT("A:") << endl << augmentedForm.mA;
    os << wxT("b:") << endl << augmentedForm.mb;
    os << wxT("c:") << endl << augmentedForm.mc;
    os << wxT("actualVariables: ") << augmentedForm.mActualVariables;

    return os;
} // <<


//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
const Matrix&
AugmentedForm::A() const
{
    return mA;
}// A

const ColumnVector&
AugmentedForm::b() const
{
    return mb;
}// b

const ColumnVector&
AugmentedForm::c() const
{
    return mc;
}// c

int
AugmentedForm::ActualVariables() const
{
    return mActualVariables;
}// ActualVariables

int
AugmentedForm::m() const
{
    return mA.nrows();
} // m

int
AugmentedForm::n() const
{
    return mA.ncols();
}



//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
