/**  Ograniczenie.
 *
 * Ograniczenia reprezentowane s w postaci (nie)rwnoci, np.:
 *
 * <code>x^2 + y - 1 <= 0</code>
 *
 * Typ ograniczenia okrela kierunek nierwnoci (bd rwno) a funkcja lew
 * stron wyraenia.
 */

#ifndef _Constraint_h
#define _Constraint_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "FunctionBase.h"

// FORWARD REFERENCES
//


class Constraint
{
public:

    /** Okno dialogowe edycji ograniczenia.
     */
    friend class ConstraintDialog;

    /** Typ ograniczenia.
     */
    enum ConstraintType {
        EQUAL,                  ///< rwnociowe (lewa strona = 0)
        LESS_EQUAL_THAN,        ///< nierwnociowe (lewa strona <= 0)
        GREATER_EQUAL_THAN      ///< nierwnociowe (lewa strona >= 0)
    };

    /** Typ do przechowywania ogranicze.
     */
    //  Ten typ jest wyjtkowo zdefiniowany nie tam, gdzie bdzie uywany, gdy
    //  kontener z ograniczeniami jest wykorzystywany w LinearProblem,
    //  PenaltyFunction, ConstraintsDialog i *ConstraintsLayer.
    typedef vector<Constraint>  Container;

// LIFECYCLE

    /** Konstruktor domylny.
     */
    Constraint(void);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Constraint(const Constraint& from);

    /** Konstruktor.
     *
     * @param apFunction Funkcja definiujca ograniczenie.
     * @param type Typ ograniczenia.
     */
    Constraint(std::auto_ptr<const FunctionBase> apFunction, ConstraintType type);


    /** Destruktor.
     */
    virtual ~Constraint(void);


// OPERATORS

    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    Constraint&             operator=(const Constraint& from);

// OPERATIONS
//
    /** Zwraca warto ograniczenia w punkcie.
     */
    double                  Value(const ColumnVector& point) const;

    /** Sprawdza, czy dany punkt spenia ograniczenie.
     *
     * @param point Punkt, dla ktrego sprawdzany jest warunek.
     * @param tolerance Tolerancja mierzona w dopuszczalnym przekroczeniu
     * zera przez warto ograniczenia w danym punkcie.
     */
    bool                IsValid(const   ColumnVector&   point,
                                        bool            includeBoundary = true,
                                        double          tolerance = 0.001)
                                                                        const;

    /** Wykonuje kopi ograniczenia wraz ze sklonowaniem funkcji.
     */
    Constraint          DeepCopy() const;

// ACCESS

    /** Zwraca funkcj ograniczenia (lew stron).
     */
    const FunctionBase&     Function() const;

    /** Zwraca typ ograniczenia.
     */
    const ConstraintType    Type() const;

    /** Zwraca reprezentacj tekstow ograniczenia.
     */
    wxString                ToString(bool full = true) const;


// INQUIRY

protected:
    /// Funkcja opisujca ograniczenie.
    CountedPtr<const FunctionBase>  mcpFunction;
    ConstraintType                  mType;      ///< Typ ograniczenia.

public:
    class ProblemSetter
    {
    public:
        ProblemSetter(const ProblemBase* pProblem)
            : mpProblem(pProblem)
        {
        }

        void operator()(const Constraint &constraint)
        {
            if (constraint.mcpFunction.Get() != NULL)
            {
                constraint.mcpFunction->SetProblem(mpProblem);
            }
        }

    private:
        friend class Constraint;

        const ProblemBase* mpProblem;
    };

private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Constraint_h_
