#include <precomp.h>

#include "Configuration.h"
#include "Error.h"
#include "FunctionBase.h"  // class implemented
#include "Logger.h"

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FunctionBase::FunctionBase(bool isDifferentiable)
    : mDifferentiable(isDifferentiable), mpProblem(NULL), mCounter(0)
{
}// FunctionBase


FunctionBase::~FunctionBase()
{
}// ~FunctionBase


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

double
FunctionBase::Evaluate(const ColumnVector& point) const
    throw (EvaluationError)
{
    double value = DoEvaluate(point);

    CheckValidity(value, point);

    IncreaseCounter();
    return value;
}// Evaluate

double
FunctionBase::EvaluateNoThrow(const ColumnVector& point) const
    throw ()
{
    IncreaseCounter();
    return DoEvaluateNoThrow(point);
}// Evaluate

/* virtual */ void
FunctionBase::BeginFixed() const
{
}// BeginFixed

/* virtual */ void
FunctionBase::EndFixed() const
{
}// EndFixed

void
FunctionBase::IncreaseCounter() const
{
    ++mCounter;
}// IncreaseCounter

void
FunctionBase::ResetCounter() const
{
    mCounter = 0;
}// ResetCounters

double
FunctionBase::PartialDerivative(
        const ColumnVector& point,
        int                 variableIndex) const throw(EvaluationError)
{
    assert(this->IsDifferentiable());

    double delta = Configuration::GetInstance().DifferentiationDelta();
    double result;
    const int TRIAL_LIMIT = 5;
    int trials = 0;

    do
    {
        if (trials > 0)
        {
            LOG(WARNING, wxT("Differentiation precision not sufficient!")
                         << wxString::Format(wxT("\nTrials: %d, Delta: %g"),
                                             trials, delta));
        }

        ColumnVector point1(point);
        ColumnVector point2(point);

        point1(variableIndex + 1) -= delta;
        point2(variableIndex + 1) += delta;

        result = (this->Evaluate(point2) - this->Evaluate(point1)) /
                                                                (2 * delta);

        ++trials;
        delta /= 10;

        LOG(DEBUG, wxT("Partial derivative in ") << point.as_row()
                   << wxT(" for variable ") << variableIndex
                   << wxT(" is: ") << result);
    }
    while (isinf(result) && trials <= TRIAL_LIMIT);

    if (isinf(result))
        throw EvaluationError(_("Partial derivative is infinite"),point);

    return result;
}// PartialDerivative

double
FunctionBase::SecondPartialDerivative(
        const ColumnVector& point,
        int                 firstVariableIndex,
        int                 secondVariableIndex) const throw(EvaluationError)
{
    assert(this->IsDifferentiable());

    double delta = Configuration::GetInstance().DifferentiationDelta();
    double result;
    const int TRIAL_LIMIT = 5;
    int trials = 0;

    do
    {
        if (trials > 0)
        {
            LOG(WARNING, wxT("Differentiation precision not sufficient!")
                         << wxString::Format(wxT("\nTrials: %d, Delta: %g"),
                                             trials, delta));
        }

        ColumnVector point1(point);
        ColumnVector point2(point);

        point1(secondVariableIndex + 1) -= delta;
        point2(secondVariableIndex + 1) += delta;

        result = (PartialDerivative(point2, firstVariableIndex) -
                  PartialDerivative(point1, firstVariableIndex)) / (2 * delta);

        ++trials;
        delta /= 10;

        LOG(DEBUG, wxT("Second partial derivative in\n") << point.as_row()
            << wxT(" for variables ")
            << firstVariableIndex << wxT(" and ") << secondVariableIndex
            << wxT(" is: ") << result);
    }
    while (isinf(result) && trials <= TRIAL_LIMIT);

    if (isinf(result))
        throw EvaluationError(_("Second partial derivative is infinite"),point);

    return result;
}// SecondPartialDerivative

ColumnVector
FunctionBase::Gradient(const ColumnVector& point) const throw(EvaluationError)
{
    ColumnVector gradient(point.nrows());

    assert(this->IsDifferentiable());

    for (int i = 0; i < point.nrows(); ++i)
    {
        gradient(i + 1) = PartialDerivative(point, i);
    }

    return gradient;
}// Gradient

SymmetricMatrix
FunctionBase::Hessian(const ColumnVector& point) const throw(EvaluationError)
{
    SymmetricMatrix hessian(point.nrows());

    assert(this->IsDifferentiable());

    for (int i = 0; i < point.nrows(); ++i)
    {
        for (int j = 0; j <= i; ++j)
        {
            hessian(i + 1, j + 1) = SecondPartialDerivative(point, i, j);
        }
    }

    return hessian;
}// Hessian


//============================= ACCESS     ===================================

unsigned
FunctionBase::EvaluationCount() const
{
    return mCounter;
}// TotalCount

/* virtual */ void
FunctionBase::SetProblem(const ProblemBase* pProblem) const
{
    // Funkcja ma zmieniany problem podczas klonowania problemu. nie moe wic
    // by tutaj tej asercji
    // assert(mpProblem == NULL);

    mpProblem = pProblem;
}// SetProblem

//============================= INQUIRY    ===================================

/* virtual */ bool
FunctionBase::IsDifferentiable() const
{
    return mDifferentiable;
}// IsDifferentiable

/////////////////////////////// PROTECTED  ///////////////////////////////////

FunctionBase::FunctionBase(const FunctionBase& from)
    : mDifferentiable(from.mDifferentiable),
      mCounter(from.mCounter)
{
}// FunctionBase
double
FunctionBase::DoEvaluate(const ColumnVector& point) const
        throw (EvaluationError)
{
    return DoEvaluateNoThrow(point);
}// DoEvaluate

void
FunctionBase::CheckValidity(double value, const ColumnVector& point) const
{
    if (isnan(value))
    {
        throw EvaluationError(_("The result is Not-a-Number!"), point);
    }
    if (!isfinite(value))
    {
        throw EvaluationError(_("The result is infinity!"), point);
    }
}

//////////////////////////// PRIVATE    //////////////////////////////////////
