#ifndef _FunctionBase_h
#define _FunctionBase_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "EvaluationError.h"
#include "VariableNames.h"

// FORWARD REFERENCES
//
class ProblemBase;

/**  Klasa reprezentujca funkcj matematyczn.
 *
 * Abstrakcyjna klasa reprezentujca funkcj matematyczn wielu zmiennych.
 */
class FunctionBase
{
public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param isDifferentiable Czy funkcja jest rniczkowalna.
     */
    FunctionBase(bool isDifferentiable);


    /** Destruktor.
     */
    virtual ~FunctionBase(void);


// OPERATORS

// OPERATIONS

    virtual std::auto_ptr<FunctionBase>     Clone() const = 0;

    /** Metoda obliczajca warto funkcji w punkcie
     *
     * @exception FunctionError Sygnalizowany, gdy nie mona obliczy wartoci
     * funkcji (np. punkt poza dziedzin).
     */
    double          Evaluate(const ColumnVector& point) const
        throw(EvaluationError);

    /** Metoda obliczajca warto funkcji w punkcie
     *
     * Nie wyrzuca wyjtkw, zwracane wartoci mog by NaN, nieskoczonci
     * itd.
     */
    double          EvaluateNoThrow(const ColumnVector& point) const throw();

    /** Rozpoczyna fragment kodu, w ktrym zmienne ani funkcja nie zmieniaj
     * si.
     *
     * Pozwala to na dokonanie pewnych optymalizacji obliczania funkcji.
     *
     * Do momentu wywoania EndFixed zmienne ani funkcja (poprzez inne
     * wskaniki) nie mog ulec zmianie.
     */
    virtual void    BeginFixed() const;

    /** Koniec kodu, w ktrym zmienne ani funkcja nie zmieniaj si.
     */
    virtual void    EndFixed() const;

    /** Pochodna czstkowa w punkcie.
     *
     * @param point Punkt, w ktrym obliczana jest warto pochodnej.
     * @param variableIndex Indeks zmiennej (zaczynaj si od 0), po ktrej jest
     * obliczana.
     */
    double          PartialDerivative(const ColumnVector& point,
                                      int                 variableIndex) const
                                            throw(EvaluationError);

    /** Druga pochodna czstkowa w punkcie.
     *
     * @param point Punkt, w ktrym obliczana jest warto pochodnej.
     * @param firstVariableIndex Indeks pierwszej zmiennej (zaczynaj si od 0),
     * po ktrej pochodna jest obliczana.
     * @param secondVariableIndex Indeks drugiej zmiennej,
     * po ktrej pochodna jest obliczana.
     */
    double          SecondPartialDerivative(const ColumnVector& point,
                                            int   firstVariableIndex,
                                            int   secondVariableIndex) const
                                                throw(EvaluationError);

    /** Oblicza gradient w punkcie.
     *
     * @param point Punkt, w ktrym obliczany jest gradient.
     */
    ColumnVector    Gradient(const ColumnVector& point) const
                                                throw(EvaluationError);

    /** Oblicza hesjan w punkcie.
     *
     * @param point Punkt, w ktrym obliczany jest hesjan.
     */
    SymmetricMatrix Hessian(const ColumnVector& point) const
                                                throw(EvaluationError);

    /** Zwiksz warto licznika.
     *
     * Funkcja jest <code>const</code>, gdy operuje na liczniku, ktry jest
     * <code>mutable</code>.
     */
    void                        IncreaseCounter() const;

    /** Zeruj licznik.
     */
    void                        ResetCounter() const;


// ACCESS

    /** Zwraca tekstow reprezentacj funkcji.
     *
     * @param full Czy zwrci peny opis, czy sam wzr.
     */
    virtual wxString            ToString(bool full = true) const = 0;

    /** Cakowita liczba wylicze wartoci funkcji.
     */
    virtual unsigned            EvaluationCount() const;

    //TODO: Zdecydowa si czy zrobi t funkcj wirtualn i dodawa problem do
    //"kaskadowo" (bdzie wymagao modyfikacji FunctionDecorator).
    /** Ustaw wskanik na problem/zadanie.
     *
     * Z zadania pobierane s zmienne, musi wic ono by ustawione przed
     * wywoywaniem Evaluate.
     */
    virtual void        SetProblem(const ProblemBase* pProblem) const;

    /** Zwraca list nazw zmiennych, ktre wystpuj w funkcji.
     */
    virtual VariableNames       PresentVariables() const = 0;

// INQUIRY

    /** Sprawdza, czy funkcja jest rniczkowalna.
     */
    virtual bool                IsDifferentiable() const;

    /** Sprawdza, czy funkcja uywa zmiennych spoza listy.
     */
    virtual bool                ContainsUnknown(const VariableNames& variables)
                                    const = 0;

protected:
    FunctionBase(const FunctionBase& from);

    /** Funkcja obliczajca warto.
     */
    virtual double          DoEvaluateNoThrow(const ColumnVector& point) const
        throw() = 0;

    virtual double          DoEvaluate(const ColumnVector& point) const
        throw(EvaluationError);

    bool                mDifferentiable;   ///< Czy funkcja jest rniczkowalna.
    // Dodane mutable. Brzydki hak, ale SetProblem trzeba wywolac przy
    // kopiowaniu problemu, a funkcja w problemie jest const.
    mutable const ProblemBase*  mpProblem; ///< Problem, z ktrego pobierane s zmienne.

private:
    /** Sprawdza, czy podana warto jest poprawn liczb (nie NaN oraz
     * skoczon).
     *
     * @param value Otrzymana warto.
     * @param point Punkt, w ktrym otrzymano t warto.
     *
     * @exception EvaluationError Wyjtek z odpowiednim opisem, gdy liczba nie
     * jest skoczona.
     */
    void                        CheckValidity(double              value,
                                              const ColumnVector& point) const;

    mutable unsigned mCounter;  ///< Licznik wylicze.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FunctionBase_h_
