#include <precomp.h>

#include "FunctionTree.h"  // class implemented

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


FunctionTree::FunctionTree(double value)
    : mNodeType(VALUE),
      mNodeData(value),
      mLeftNode(NULL),
      mRightNode(NULL)
{
}// FunctionTree

FunctionTree::FunctionTree(const wxString& variableName)
    : mNodeType(VARIABLE_NAME),
      mNodeData(new wxString(variableName)),
      mLeftNode(NULL),
      mRightNode(NULL)
{
}// FunctionTree

FunctionTree::FunctionTree(unsigned variableIndex)
    : mNodeType(VARIABLE_INDEX),
      mNodeData(variableIndex),
      mLeftNode(NULL),
      mRightNode(NULL)
{
    assert (variableIndex > 0);
}// FunctionTree

FunctionTree::FunctionTree(const UnaryOperator                  unaryOperator,
                           std::auto_ptr<const FunctionTree>    apOperand)
    : mNodeType(UNARY),
      mNodeData(unaryOperator),
      mLeftNode(apOperand),
      mRightNode(NULL)
{
}// FunctionTree

FunctionTree::FunctionTree(const BinaryOperator                 binaryOperator,
                           std::auto_ptr<const FunctionTree>    apLeftOperand,
                           std::auto_ptr<const FunctionTree>    apRightOperand)
    : mNodeType(BINARY),
      mNodeData(binaryOperator),
      mLeftNode(apLeftOperand),
      mRightNode(apRightOperand)
{
}// FunctionTree

FunctionTree::FunctionTree(const FunctionTree& from)
    : mNodeType(from.mNodeType),
      mNodeData(from.mNodeData),
      mLeftNode(from.mLeftNode.get() == NULL
              ? NULL
              : new FunctionTree(*from.mLeftNode)),
      mRightNode(from.mRightNode.get() == NULL
              ? NULL
              : new FunctionTree(*from.mRightNode))
{
    if ((mNodeType == VARIABLE_NAME) && (from.mNodeData.mpVariableName != NULL))
    {   // Skopiuj nazw
        mNodeData.mpVariableName = new wxString(*from.mNodeData.mpVariableName);
    }
}// FunctionTree

FunctionTree::~FunctionTree()
{
    if (mNodeType == VARIABLE_NAME)  delete mNodeData.mpVariableName;
}// ~FunctionTree


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

double
FunctionTree::Evaluate(const ColumnVector&  point,
                       const VariableNames& variables) const
{
    switch (mNodeType)
    {
        case VALUE:
            return mNodeData.mValue;
        case VARIABLE_INDEX:
            return point(mNodeData.mVariableIndex);
        case UNARY:
            return mNodeData.mUnaryOperator(
                    mLeftNode->Evaluate(point, variables));
        case BINARY:
            return mNodeData.mBinaryOperator(
                    mLeftNode->Evaluate(point, variables),
                    mRightNode->Evaluate(point, variables));
        case VARIABLE_NAME:
            return point(1 + variables.Index(*mNodeData.mpVariableName));
    }
    abort();
}// Evaluate

std::auto_ptr<const FunctionTree>
FunctionTree::Optimize(const VariableNames& variables) const
{
    switch (mNodeType)
    {
        case VALUE:
            return auto_ptr<const FunctionTree>
                (new FunctionTree(mNodeData.mValue));
        case VARIABLE_NAME:
            return auto_ptr<const FunctionTree>
                (new FunctionTree(static_cast<unsigned>
                        (1 + variables.Index(*mNodeData.mpVariableName))));
        case VARIABLE_INDEX:
            return auto_ptr<const FunctionTree>
                (new FunctionTree(mNodeData.mVariableIndex));
        case UNARY:
            return auto_ptr<const FunctionTree>
                (new FunctionTree(mNodeData.mUnaryOperator,
                                  mLeftNode->Optimize(variables)));
        case BINARY:
            return auto_ptr<const FunctionTree>
                (new FunctionTree(mNodeData.mBinaryOperator,
                                  mLeftNode->Optimize(variables),
                                  mRightNode->Optimize(variables)));
        default:
            // IMPOSSIBLE
            assert (false);
    }
}// Optimize

//============================= ACCESS     ===================================

void
FunctionTree::SetArgument(std::auto_ptr<const FunctionTree> apArgument)
{
    mLeftNode = apArgument;
}// SetArgument


//============================= INQUIRY    ===================================

bool
FunctionTree::ContainsUnknown(const VariableNames& variables) const
{
    VariableNames presentVariables = PresentVariables();

    // Wyszukaj kad zmienn w variables.
    for (size_t i = 0; i < presentVariables.GetCount(); ++i)
    {
        if (variables.Index(presentVariables[i]) == wxNOT_FOUND) return true;
    }

    return false;
}// ContainsUnknown

VariableNames
FunctionTree::PresentVariables() const
{
    VariableNames variables; // Nie jest to demon szybkoci...

    switch (mNodeType)
    {
        case VARIABLE_NAME:
            variables.AddIfNotPresent(*mNodeData.mpVariableName);
            break;
        case UNARY:
            variables = mLeftNode->PresentVariables();
            break;
        case BINARY:
            variables = mLeftNode->PresentVariables();
            variables.AddIfNotPresent(mRightNode->PresentVariables());
            break;
        default:
            break;
    }

    return variables;
}// PresentVariables

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
