/** Drzewo reprezentujce funkcj matematyczn.
 *
 * Klasa reprezentuje wze drzewa funkcji, w ktrym moe si znajdowa warto
 * liczbowa, nazwa zmiennej w postaci cigu znakw, funkcja jedno- bd
 * dwuargumentowa.
 *
 * Argumentami dla funkcji (operatorw) s lewy i prawy wze potomny
 * (lub poddrzewo).
 */

#ifndef _FunctionTree_h
#define _FunctionTree_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "VariableNames.h"

// FORWARD REFERENCES
//


class FunctionTree
{
private:
    /// Typ wza.
    enum        Type { VALUE, VARIABLE_NAME, VARIABLE_INDEX, BINARY, UNARY };

public:

    /// Operator (bd funkcja) jednoargumentowy.
    typedef     double (*UnaryOperator)(double);

    /// Operator (bd funkcja) dwuargumentowy.
    typedef     double (*BinaryOperator)(double, double);

// LIFECYCLE

    /** Konstruktor wza z wartocia liczbow.
     */
    FunctionTree(double value);

    /** Konstruktor wza z nazw zmiennej.
     */
    FunctionTree(const wxString& variableName);

    /** Konstruktor wza z indeksem zmiennej.
     */
    FunctionTree(unsigned variableIndex);

    /** Konstruktor wza z funkcj/operatorem unarn.
     */
    FunctionTree(const UnaryOperator                unaryOperator,
                 std::auto_ptr<const FunctionTree>  apOperand);

    /** Konstruktor wza z funkcj/operatorem binarn.
     */
    FunctionTree(const BinaryOperator               binaryOperator,
                 std::auto_ptr<const FunctionTree>  apLeftOperand,
                 std::auto_ptr<const FunctionTree>  apRightOperand);

    /** Konstruktor kopiujcy.
     */
    FunctionTree(const FunctionTree& from);

    /** Destruktor.
     */
    ~FunctionTree(void);


// OPERATORS

    /** Operator przypisania.
     *
     * @param from Przypisywana warto.
     *
     * @return Referencja do biecego obiektu.
     */
    FunctionTree&                     operator=(const FunctionTree& from);

// OPERATIONS

    /** Oblicza warto funkcji w punkcie.
     *
     * Obiekt <code>variables</code> jest odpowiedzialny za mapowanie nazw
     * zmiennych wystpujcych w funkcji na indeksy wektora <code>point</code>.
     *
     * @param point Wektor wartoci kolejnych zmiennych.
     * @param variables Nazwy zmiennych.
     */
    double      Evaluate(const ColumnVector&    point,
                         const VariableNames&   variables) const;

    /** Przetwarza drzewo w celu optymalizacji obliczania wartoci.
     *
     * W obecnej chwili zamienia jedynie nazwy zmiennych na ich bezporednie
     * indeksy.
     *
     * @return Nowe, zoptymalizowane drzewo funkcji.
     */
    std::auto_ptr<const FunctionTree>   Optimize(const VariableNames& variables)
                                            const;

// ACCESS

    /** Ustawia lewe poddrzewo.
     *
     * (lewe poddrzewo wykorzystywane jest jako argument funkcji
     * jednoargumentowych)
     *
     * Do budowy drzewa funkcji naley stosowa w pierwszej kolejnoci
     * konstruktory!
     */
    void        SetArgument(std::auto_ptr<const FunctionTree> apArgument);

// INQUIRY

    /** Sprawdza, czy drzewo funkcji zawiera zmienne nie znajdujce si w
     * <code>variables</code>.
     */
    bool        ContainsUnknown(const VariableNames& variables) const;

    /** Znajduje wszystkie zmienne wystpujce w drzewie funkcji.
     */
    VariableNames       PresentVariables() const;

protected:

private:


    /// Typ dla danych w wle.
    union Data
    {
        double          mValue;         ///< Warto liczbowa.
        wxString*       mpVariableName; ///< Nazwa zmiennej.
        unsigned        mVariableIndex; ///< Indeks zmiennej (zaczynajc od 1).
        UnaryOperator   mUnaryOperator; ///< Operator (funkcja) unarny.
        BinaryOperator  mBinaryOperator;///< Operator (funkcja) binarny.

        // Konstruktory
        Data(double value) : mValue(value) { };
        Data(unsigned variableIndex) : mVariableIndex(variableIndex) { };
        Data(wxString* pVariableName) : mpVariableName(pVariableName) { };
        Data(UnaryOperator unaryOperator) : mUnaryOperator(unaryOperator) { };
        Data(BinaryOperator binaryOperator) : mBinaryOperator(binaryOperator) { };
    };

    const Type                          mNodeType;  ///< Typ wza.
    Data                                mNodeData;  ///< Dane.
    std::auto_ptr<const FunctionTree>   mLeftNode;  ///< Lewe poddrzewo.
    std::auto_ptr<const FunctionTree>   mRightNode; ///< Prawe poddrzewo.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FunctionTree_h
