#ifndef _Logger_h
#define _Logger_h


// SYSTEM INCLUDES

//#include <cassert>

// PROJECT INCLUDES

// LOCAL INCLUDES

#include "wxTextCtrlOutputStream.h"
#include "util.h"

// FORWARD REFERENCES
//

#undef ERROR        // wxWidgets te definiuje tak sta

#define FILE_LOGGING
//#define WINDOW_LOGGING

/** Prosty logger dziedziczcy z wxTextOutputStream.
 *
 * Logger mona utworzy samemu, bd zda si na gotowe makra:
 *
 * LOGGER(level) - Tworzy obiekt logujcy od poziomu [level]
 * przekazujcy dane na stderr.
 *
 * Aby przeprowadzi logowanie do pliku lub do okna, naley w pliku Logger.h
 * odkomentowa odpowiednio linijki:
 * //#define FILE_LOGGING
 * lub
 * //#define WINDOW_LOGGING
 *
 * LOG(level,message) - Loguje wiadomo o treci zawartej w [message]
 * z poziomem [level].
 *
 * Poziomy logowania w kolejnoci od najniszego do najwyszego to:
 * DEBUG, INFO, MESSAGE, WARNING, ERROR.
 *
 */
class Logger
    : public wxTextOutputStream
{
// LIFECYCLE

public:

    /** Poziomy logowania.
     */
    typedef enum
    {
        DEBUG,      ///<
        INFO,       ///<
        WARNING,    ///<
        ERROR,      ///<
        FATAL       ///<
    } LevelType;

    /** Konstruktor.
     *
     * @param unitName nazwa programu tworzcego logger.
     * @param displayLevel Poziom logowania (domylnie DEBUG).
     * @param targetOutputStream Strumie docelowy (domylnie NULL).
     */
    Logger(wxOutputStream&        targetOutputStream,
             const wxString&        unitName            = wxString(__TFILE__),
             const LevelType        displayLevel        = DEBUG);


	/** Destruktor.
     */
    ~Logger();

    /** Zwr strumie wyjciowy wsplny dla wszystkich Loggerw.
     *
     * Metoda zwraca referencj na obiekt klasy wxFFileOutputStream wsplny dla
     * wszystkich Loggerw.
     *
     * @return referencja na wsplny strumie wyjciowy Loggerw.
     */
    static wxFFileOutputStream&    CommonFfos();

    /** Zwr strumie wyjciowy (do stderr) wsplny dla wszystkich Loggerw.
     *
     * Metoda zwraca referencj na obiekt klasy wxFFileOutputStream wsplny dla
     * wszystkich Loggerw, a przekazujcy dane do stderr.
     *
     * @return referencja na wsplny strumie wyjciowy Loggerw.
     */
    static wxFFileOutputStream&    CommonStderrOs();

// OPERATIONS

    /** Podziel tekst na linie i zapisz do strumienia wyjciowego.
     *
     * Metoda dzieli otrzymany tekst na pojedyncze linie i przekazuje je
     * do stumienia docelowego poprzedzajc prefiksem o formacie:
     * "<nazwa moduu> : <poziom wiadomoci> : ".
     *
     * @param string Tekst do przekazania do stumienia docelowego.
     */
    void WriteString(const wxString& string);

// OPERATORS
// ACCESS

    /** Ustaw poziom logowania nastpnej wiadomoci.
     *
     * @param messageLevel Poziom logowania nastpnej wiadomoci.
     */
    void MessageLevel(const LevelType messageLevel);

    /** Ustaw poziom logowania.
     *
     * @param displayLevel Nowy poziom logowania.
     */
    void DisplayLevel(const LevelType displayLevel);

// INQUIRY

protected:
private:

    static const wxString& LevelName(LevelType level);

    /** Nazwa moduu tworzcego logger.
     */
    wxString mUnitName;

    /** Minimalny poziom, ktry musi posiada wiadomo, aby
     * zostaa zalogowana.
     */
    LevelType mDisplayLevel;

    /** Poziom logowania nastpnych wiadomoci.
     */
    LevelType mMessageLevel;

    /** Czy logger napotka znak nowej linii i ma wywietli prefiks
     * przed tekstem wiadomoci.
     */
    bool mNewLine;

    /** Obecny tekst prefiksu wiadomoci, tworzony na podstawie
     * nazwy moduu i poziomu nastpnej wiadomoci.
     */
    wxString mCurrentPrefix;

    // Deklaracja prywatnego konstruktora kopiujcego i operatora =.
    // Uniemoliwia skopiowanie obiektu tej klasy.
    DECLARE_NO_COPY_CLASS(Logger);

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//


#ifdef NODEBUG

#define LOGGER(level)               ;
#define FILE_LOGGER(level,filename) ;
#define WINDOW_LOGGER(level)        ;
#define WINDOW_LOGGER_ACTIVATE      ;
#define LOG(level,message)          ;
#define LOGGER_ON                   ;

#else   // NODEBUG not defined

#ifdef  WINDOW_LOGGING
#define LOGGER(level)                                                         \
        static wxTextCtrlOutputStream gOutputStream;                          \
        static Logger gLogger(gOutputStream,                                  \
                                wxString(__TFILE__),                          \
                                Logger::level);
#define LOGGER_ON                                                             \
        wxTextCtrlOutputStream::SetTextCtrl();

#else // WINDOW_LOGGING not defined

#ifdef  FILE_LOGGING
#define LOGGER(level)                                                         \
        static Logger gLogger(Logger::CommonFfos(),                           \
                                wxString(__TFILE__),                          \
                                Logger::level);
#define LOGGER_ON ;

#else // FILE_LOGGING not defined

#define LOGGER(level)                                                         \
        static Logger gLogger(Logger::CommonStderrOs(),                       \
                                wxString(__TFILE__),                          \
                                Logger::level);
#define LOGGER_ON ;

#endif  // FILE_LOGGING
#endif  // WINDOW_LOGGING

#define LOG(level,message)                                                    \
        gLogger.MessageLevel(Logger::level);                                  \
        gLogger << message << endl;

#endif // NODEBUG

#endif // _wxLogger_h
