#ifndef _Method_h
#define _Method_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Error.h"
#include "Observable.h"
#include "OptimizationError.h"
#include "Result.h"
#include "StopConditionBase.h"

// FORWARD REFERENCES
//
class ProblemBase;

typedef unsigned    MethodIdType;

/**  Metoda optymalizacji.
 *
 * Klasa bazowa dla metod. Posiada nazw i opis metody.
 *
 * Komunikacja z interfacem uytkownika odbywa si za pomoc obserwatora (metoda
 * informuje cyklicznie o tym, e dziaa. Przy okazji obserwator moe
 * przetwarza zdarzenia GUI) oraz metody Abort (pozwala GUI przerwa operacj
 * optymalizacji - sygnalizowany jest wtedy odp. wyjtek).
 */

class Method
    : public Observable
{
public:
// LIFECYCLE


    /** Destruktor.
     */
    virtual ~Method(void);


// OPERATORS

// OPERATIONS

    /** Przygotowuje i uruchamia proces optymalizacji.
     *
     * Przygotowanie obejmuje m.in. wyzerowanie licznika oblicze funkcji. Po
     * zakoczeniu bd przerwaniu optymalizacji warto licznika jest kopiowana
     * do wyniku.
     *
     * @param problem Problem optymalizacji.
     * @return Wskanik <code>auto_ptr</code> na wynik oraz przebieg
     * optymalizacji.
     */
    std::auto_ptr<Result>   StartOptimize(const ProblemBase& problem) const
        throw(OptimizationError);


    /** Tworzy kopi siebie i zwraca na ni wskanik.
     */
    virtual std::auto_ptr<Method>       Clone() const = 0;


    /** Zatrzymuje proces optymalizacji.
     *
     * Metoda powinna by wywoywana z interface'u, np. przez przycisk
     * "Przerwij" w okienku postpu.
     */
    void                                Abort() const;

    /** Przygotowuje metod do pracy.
     *
     * Automatycznie wywoywane przez StartOptimize itp. przed rozpoczciem
     * optymalizacji.
     */
    void                                Prestart() const;

    /** Inicjuje metody.
     *
     * Obecnie tylko sortuje prototypy metod wg nazw.
     *
     * Metoda powinna zosta wywoana raz przy starcie programu, jeszcze przed
     * tworzeniem okien interface'u.
     */
    static void                         Init();

    virtual void                        UpdateStartingConditions
                                            (const Result* result) const = 0;

    virtual void                        ResetStartingConditions() const = 0;

// ACCESS

    /** Zwraca wektor metod.
     */
    static const vector<Method*>&   Methods();

    /** Znajduje metod o podaniym ID.
     *
     * Przykad:
     * <code>
     * Method* p_method = Method::GetById(Simplex::ClassId());
     * </code>
     *
     * @return Wskanik na prototyp metody, nie naley go zwalnia!
     */
    static Method*                  GetById(MethodIdType id);

    /** Zwraca ID klasy metody.
     */
    static MethodIdType             ClassId();

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const = 0;

    /** Zwraca opis tekstowy metody (nazwa + parametry).
     */
    // TODO: przerobi na abstrakcyjn, jak bd gotowe we wszystkich metodach.
    virtual wxString                ToString(bool standalone = true) const;
                                                                // = 0;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const = 0;

// INQUIRY

    /** Sprawdza, czy dany problem moe by rozwizany t metod.
     */
    virtual bool            CanHandle(const ProblemBase *pProblem) const = 0;


    /** Zwraca list warunkw stopu odpowiednich dla danej metody.
     */
    virtual StopConditions  AllowedStopConditions() const;

protected:

    /** Informuje wiat zewntrzny o pracy metody.
     *
     * Metoda informuje obserwatory o tym, e dziaa (mog one np. wywietla
     * okienko postpu oraz przetwarza zdarzenia GUI), oraz sprawdza, czy
     * uytkownik nie przerwa dziaania (patrz Abort).
     *
     * Powinna by wywoywana w ptli metody Optimize. Powiadamianie
     * obserwatorw nie zachodzi czciej, ni raz na 250ms (ale moe rzadziej).
     *
     * @exception Error Sygnalizowany, gdy proces optymalizacji zostanie
     * przerwany.
     *
     * @see Optimize, Abort
     */
    void                    NotifyYield() const throw(Error);

    /** Konstruktor domylny.
     *
     * Rejestruje automatycznie metod.
     */
    Method(void);

    /** Tworzy nowy numer ID.
     *
     * @see ClassId
     */
    static MethodIdType        GenerateId();

    virtual wxString&               rName() = 0;

    /** Staa ID dla klas abstrakcyjnych.
     */
    enum {
        ABSTRACT_METHOD_ID = 1
    };

private:
    /** Optymalizuj.
     *
     * Przeprowadza optymalizacj za pomoc algorytmu, ktry realizowa ma dana
     * klasa.
     *
     * W kadej ptli Optimize musi by wywoywana metoda
     * Method::NotifyYield.
     *
     * @param problem Problem optymalizacji.
     * @return Wskanik na wynik oraz przebieg optymalizacji.
     */
    virtual std::auto_ptr<Result>   Optimize(const ProblemBase& problem) const
        throw(OptimizationError) = 0;


    mutable bool        mAbort;         ///< Czy przerwa proces optymalizacji.
    mutable wxLongLong  mPreviousTime;  ///< Czas poprzedniego przebiegu ptli.

    // STATIC

    /** Sprawdza, czy metoda ma tak nazw, jak podana.
     */
    static bool CompareName(const Method* pMethod, const wxString* pName);

    /** Sprawdza, czy metoda ma takie ID, jak podane.
     */
    static bool CompareId(const Method* pMethod, MethodIdType id);

    /** Sprawdza, czy nazwa pierwszej metody jest mniejsza od nazwy drugiej
     * metody.
     */
    static bool NameLesser(const Method* method1, const Method* method2);

    /** Zwraca referencj na list metod.
     */
    static vector<Method*>& rMethods();

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Method_h_
