#ifndef _MethodWithLineSearch_h
#define _MethodWithLineSearch_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "FunctionBase.h"
#include "LineSearch.h"
#include "StandardStopCondition.h"
#include "UnconstrainedProblem.h"

// FORWARD REFERENCES
//
class ProblemBase;


/** Metoda optymalizacji z optymalizacj w kierunku.
 *
 * Klasa bazowa dla metod optymalizacji wielu zmiennych, w ktrych wystpuje
 * etap wyboru kierunku kroku, a nastpnie optymalny krok w tym kierunku.
 */
class MethodWithLineSearch
    : public Method
{
public:
// LIFECYCLE


    /** Destruktor.
     */
    virtual ~MethodWithLineSearch(void);


// OPERATORS

// OPERATIONS

    virtual void        AttachObserver(Observer* pObserver) const;

    virtual void        DetachObserver(Observer* pObserver) const;

    virtual void        UpdateStartingConditions(const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS
    virtual wxString    ToString(bool standalone = true) const;

// INQUIRY


protected:

    /** Konstruktor domylny.
     *
     * Rejestruje automatycznie metod.
     */
    MethodWithLineSearch(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, z ktrego wykonywana jest kopia.
     */
    MethodWithLineSearch(const MethodWithLineSearch& from);

    /** Wykonaj optmalny krok w wybranym kierunku.
     *
     * Funkcja oblicza wsprzdne nowego punktu \f$x^{k+1}\f$ wykonujc
     * optymalizacj w podanym kierunku \f$d\f$, wychodzc z podanego
     * punktu \f$x^k\f$:
     * \f[ \tau = \arg \min_{\tau} f(x^k + \tau d) , \f]
     * \f[ x^{k+1} = x^k + \tau d . \f]
     *
     * Jest to krok wsplny dla wszystkich metod optymalizacji, w
     * ktrych najpierw wybierany jest kierunek, a nastpnie wykonywana jest w
     * tym kierunku optymalizacja.
     * Istnienie tej funkcji usuwa konieczno pamitania o normalizacji
     * wektora kierunku przy implementacji metod.
     *
     * @param function Funkcja celu (\f$f\f$).
     * @param point Punkt, przez ktry przechodzi wektor kierunku (\f$x^k\f$).
     * @param direction Wektor kierunku (automatycznie normalizowany) (\f$d\f$).
     * @param pStepLength Opcjonalny wskanik na dugo wykonanego kroku
     * (\f$\tau\f$).
     *
     * @return Nowy punkt (\f$x^{k+1}\f$).
     *
     * @throws Error W przypadku gdy wystpi bd obliczeniowy w optymalizacji
     * liniowej.
     */
    ColumnVector    StepInDirection(const FunctionBase&     function,
                                    const ColumnVector&     point,
                                    const ColumnVector&     direction,
                                    double* const           pStepLength = NULL)
                    const throw (Error);

    /** Punkt startowy (\f$x^0\f$).
     */
    mutable ColumnVector                    mStartingPoint;

    ColumnVector                            mInitialStartingPoint;

    /* Metoda poszukiwa w kierunku.
     */
    CountedPtr<const LineSearch>            mcpLineSearch;

    /** Warunek stopu.
     */
    CountedPtr<const StandardStopCondition> mcpStopCondition;

private:

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _MethodWithLineSearch_h_
