/** Klasa bazowa dla klas mutacji.
 *
 * Zadaniem meod mutacji w metodach ewolucyjnych jest zapewnienie zrnicowania
 * osobnikw populacji. W przypadku zastosowania algorytmw genetycznych w
 * optymalizacji funkcji, gwnym zadaniem mutacji jest zapewnienie odpornoci
 * algorytmu na minima lokalne.
 *
 * @see CauchyMutation, UniformMutation, EvolutionaryMethod
 */

#ifndef _MutationMethod_h
#define _MutationMethod_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "methods/EvolutionaryMethodIteration.h"
#include "Population.h"

// FORWARD REFERENCES
//

class MutationMethod
{
public:

// LIFECYCLE

    /** Destruktor.
     */
    virtual ~MutationMethod(void);

// OPERATORS
// OPERATIONS

    /** Utwrz nowy egzemplarz metody z prototypu.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach metody.
     *
     * @return Kopia prototypu metody
     */
    virtual std::auto_ptr<MutationMethod>      Clone() const = 0;

    /** Funkcja dokonujca mutacji wybranych chromosomw populacji bazowej.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach selekcji.
     *
     * @param population Populacja bazowa, ktrej osobniki bd mutowane.
     * @param radioactivity natenie zjawiska mutacji.
     * @param mutationRate Wspczynnik mutacji interpretowany rnie w
     * poszczeglnych metodach.
     * @param iteration Obiekt, do ktrego zapisywany jest przebieg iteracji
     * metody optymalizacji.
     */
    virtual void            Mutate
                            (Population&                  population,
                             double                       radioactivity,
                             double                       mutationRate,
                             EvolutionaryMethodIteration& iteration) const = 0;

// ACCESS

    /** Zwr referencj na kontener metod mutacji.
     *
     * @return referencja na konterer metod mutacji.
     */
    static const std::vector<MutationMethod*>& MutationMethods();

    /** Zwr opis metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach mutacji.
     *
     * @return Opis metody.
     */
    virtual wxString                            ToString() const = 0;

    /** Zwr nazw metody.
     *
     * Funkcja abstrakcyjna zaimplementowana w konkretnych metodach mutacji.
     *
     * @return Nazwa metody.
     */
    virtual wxString                            Name() const = 0;

    /** Zwr nazw klasy metod manipulacji populacj.
     *
     * Zwraca nazw klasy metod manipulacji populacj. W tym przypadku "Metoda
     * mutacji" lub odpowiednik w innym jezyku.
     *
     * @return Nazwa klasy manipulacji populacj.
     */
    static wxString                             BaseClassName();

// INQUIRY

protected:

    /** Konstruktor domylny.
     */
    MutationMethod(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    MutationMethod(const MutationMethod& from);

private:

    /** Zwraca referencj na kontener metod mutacji.
     *
     * @return referencja na kontener metod mutacji.
     */
    static std::vector<MutationMethod*>&       rMutationMethods();

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _MutationMethod_h
