#include <precomp.h>

#include "Logger.h"
#include "NonlinearFunction.h"
#include "NonlinearParser.h"  // class implemented
#include "UnivariateNonlinearFunction.h"

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NonlinearParser::NonlinearParser()
{
}// NonlinearParser

NonlinearParser::~NonlinearParser()
{
}// ~NonlinearParser


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<FunctionBase>
NonlinearParser::CreateFunction(const wxString& input,
                                VariableNames&  rVariables,
                                bool            resetVariables)
    const throw(ParseError)
{
    if (resetVariables)     rVariables = VariableNames();

    Tokenizer tokenizer(input, rVariables, resetVariables, mFunctions);
    auto_ptr<const FunctionTree> ap_function_tree;

    ap_function_tree = Expression(tokenizer);

    if (tokenizer.Peek() != Tokenizer::END)
    {
        LOG(DEBUG, wxT("Parse error"));
        throw ParseError(input, _("End of input expected"),
                tokenizer.Position());
    }
    LOG(DEBUG, wxT("Parsed successfully"));

    auto_ptr<FunctionBase> ap_function;

    if (ap_function_tree->PresentVariables().GetCount() == 1)
    {
        ap_function.reset
            (new UnivariateNonlinearFunction(ap_function_tree, input,
                                             tokenizer.AllDifferentiable()));
    }
    else
    {
        ap_function.reset
            (new NonlinearFunction(ap_function_tree, input,
                                   tokenizer.AllDifferentiable()));
    }

    return ap_function;
}// CreateFunction

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

std::auto_ptr<const FunctionTree>
NonlinearParser::Expression(Tokenizer& rTokenizer) const
{
    LOG(DEBUG, wxT("Entering Expression"));

    auto_ptr<const FunctionTree> ap_left;

    ap_left = Product(rTokenizer, true);

    Tokenizer::TokenType token = rTokenizer.Peek();

    while ((token == Tokenizer::PLUS) || (token == Tokenizer::MINUS))
    {
        auto_ptr<const FunctionTree> ap_right;

        rTokenizer.Consume();
        ap_right = Product(rTokenizer);

        if (token == Tokenizer::PLUS)
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorAdd, ap_left, ap_right));
        }
        else
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorSubtract,
                        ap_left, ap_right));
        }

        token = rTokenizer.Peek();
    }

    LOG(DEBUG, wxT("Leaving Expression"));
    return ap_left;
}// Expression

std::auto_ptr<const FunctionTree>
NonlinearParser::Product(Tokenizer& rTokenizer, bool first) const
{
    LOG(DEBUG, wxT("Entering Product"));

    auto_ptr<const FunctionTree> ap_left;

    ap_left = Power(rTokenizer, first);

    Tokenizer::TokenType token = rTokenizer.Peek();

    while ((token == Tokenizer::MULTIPLY) || (token == Tokenizer::DIVIDE))
    {
        auto_ptr<const FunctionTree> ap_right;

        rTokenizer.Consume();
        ap_right = Power(rTokenizer);

        // TODO: upikszy
        if (token == Tokenizer::MULTIPLY)
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorMultiply,
                        ap_left, ap_right));
        }
        else
        {
            ap_left = auto_ptr<const FunctionTree>(
                    new FunctionTree(Parser::OperatorDivide,
                        ap_left, ap_right));
        }

        token = rTokenizer.Peek();
    }

    LOG(DEBUG, wxT("Leaving Product"));
    return ap_left;
}// Product

std::auto_ptr<const FunctionTree>
NonlinearParser::Power(Tokenizer& rTokenizer, bool first) const
{
    LOG(DEBUG, wxT("Entering Power"));
    bool negation = false;

    auto_ptr<const FunctionTree> ap_base;
    Tokenizer::TokenType token;

    // Obsu negacj przed potg (mniejszy priorytet)
    token = rTokenizer.Peek();
    if ((first == true) && (token == Tokenizer::MINUS))
    {
        rTokenizer.Consume();
        negation = true;
    }

    ap_base = Operand(rTokenizer);

    token = rTokenizer.Peek();

    if (token == Tokenizer::POWER)
    {
        rTokenizer.Consume();

        auto_ptr<const FunctionTree> ap_exponent(Power(rTokenizer));

        ap_base = auto_ptr<const FunctionTree>(new FunctionTree(pow, ap_base,
                    ap_exponent));
    }

    if (negation)   ap_base = auto_ptr<const FunctionTree>(
            new FunctionTree(Parser::OperatorNegate, ap_base));

    LOG(DEBUG, wxT("Leaving Power"));
    return ap_base;
}// Power

std::auto_ptr<const FunctionTree>
NonlinearParser::Operand(Tokenizer& rTokenizer) const
{
    LOG(DEBUG, wxT("Entering Operand"));

    auto_ptr<const FunctionTree> ap_operand;
    Tokenizer::TokenType token;

    token = rTokenizer.Peek();
    switch (token)
    {
        case    Tokenizer::NUMBER:
        case    Tokenizer::VARIABLE:
            ap_operand = rTokenizer.Consume(); break;
        case    Tokenizer::FUNCTION:
            ap_operand = Function(rTokenizer); break;
        case    Tokenizer::OPEN_PAREN:
            rTokenizer.Consume();
            ap_operand = Expression(rTokenizer);
            rTokenizer.Expect(Tokenizer::CLOSE_PAREN);
            break;
        case    Tokenizer::END:
            throw ParseError(rTokenizer.Input(), _("Unexpected end of input"),
                    rTokenizer.Position());
        default:
            throw ParseError(rTokenizer.Input(), _("Unexpected token type"),
                    rTokenizer.Position());
            break;
    }

    LOG(DEBUG, wxT("Leaving Operand"));
    return ap_operand;
}// Operand

std::auto_ptr<const FunctionTree>
NonlinearParser::Function(Tokenizer& rTokenizer) const
{
    LOG(DEBUG, wxT("Entering Function"));

    auto_ptr<FunctionTree> ap_function;
    auto_ptr<const FunctionTree> ap_argument;

    ap_function = rTokenizer.Consume();

    rTokenizer.Expect(Tokenizer::OPEN_PAREN);
    ap_argument = Expression(rTokenizer);
    rTokenizer.Expect(Tokenizer::CLOSE_PAREN);

    ap_function->SetArgument(ap_argument);

    LOG(DEBUG, wxT("Leaving Function"));
    return auto_ptr<const FunctionTree>(ap_function);
}// Function


/* static */ Tokenizer::FunctionMap
NonlinearParser::MakeFunctionMap()
{
    Tokenizer::FunctionMap function_map;

    // Niestety, rzutowanie, do tego na mod C, wydaje si by niezbdne.
    //             nazwa                                   funkcja   rniczk.
    function_map[wxT("log")]    = (Tokenizer::FunctionInfo){ log,    true };
    function_map[wxT("log10")]  = (Tokenizer::FunctionInfo){ log10,  true };
    function_map[wxT("exp")]    = (Tokenizer::FunctionInfo){ exp,    true };
    function_map[wxT("sqrt")]   = (Tokenizer::FunctionInfo){ sqrt,   true };

    // Trygonometryczne
    function_map[wxT("sin")]    = (Tokenizer::FunctionInfo){ sin,    true };
    function_map[wxT("cos")]    = (Tokenizer::FunctionInfo){ cos,    true };
    function_map[wxT("tg")]     = (Tokenizer::FunctionInfo){ tan,    true };
    function_map[wxT("tan")]    = (Tokenizer::FunctionInfo){ tan,    true };
    function_map[wxT("sinh")]   = (Tokenizer::FunctionInfo){ sinh,   true };
    function_map[wxT("cosh")]   = (Tokenizer::FunctionInfo){ cosh,   true };
    function_map[wxT("tgh")]    = (Tokenizer::FunctionInfo){ tanh,   true };
    function_map[wxT("tanh")]   = (Tokenizer::FunctionInfo){ tanh,   true };
    function_map[wxT("arcsin")] = (Tokenizer::FunctionInfo){ asin,   true };
    function_map[wxT("arccos")] = (Tokenizer::FunctionInfo){ acos,   true };
    function_map[wxT("arctg")]  = (Tokenizer::FunctionInfo){ atan,   true };
    function_map[wxT("arctan")] = (Tokenizer::FunctionInfo){ atan,   true };
    function_map[wxT("arcsinh")]= (Tokenizer::FunctionInfo){ asinh,  true };
    function_map[wxT("arccosh")]= (Tokenizer::FunctionInfo){ acosh,  true };
    function_map[wxT("arctanh")]= (Tokenizer::FunctionInfo){ atanh,  true };
    function_map[wxT("arctgh")] = (Tokenizer::FunctionInfo){ atanh,  true };

    function_map[wxT("abs")]    = (Tokenizer::FunctionInfo){ fabs,   false };

    return function_map;
}// MakeFunctionMap

const /* static */Tokenizer::FunctionMap NonlinearParser::mFunctions =
        NonlinearParser::MakeFunctionMap();
