#ifndef _PenaltyFunction_h
#define _PenaltyFunction_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "Constraint.h"
#include "DoubleParameter.h"
#include "FunctionBase.h"
#include "FunctionDecorator.h"

// FORWARD REFERENCES
//
class ProblemWithPenalty;

/** Typ numeru ID funckji kary.
 */
typedef unsigned    FunctionIdType;


/** Funkcja kary.
 *
 * Funkcja kary dolicza do funkcji podstawowej (f. celu) warto kary zalen od
 * ogranicze.
 *
 * @see Constraint
 */

class PenaltyFunction
    : public FunctionBase, public FunctionDecorator
{
    //TODO: Usun
//    friend class ProblemWithPenalty;

public:
// LIFECYCLE
//

    enum OutOfBoundsReactionType
    {
        EXCEPTION = 0,  ///< Wyrzucanie wyjtku <code>EvaluationError</code>
        MAX_VALUE   ///< Zwracanie wartoci <code>DBL_MAX</code>
    };

    /** Konstruktor domylny.
     */
    PenaltyFunction(void);

    /** Konstruktor.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj, do ktrej ma by
     * dodana funkcja kary.
     */
    PenaltyFunction(CountedPtr<const FunctionBase> cpFunction);


    /** Destruktor.
     */
    ~PenaltyFunction(void);


// OPERATORS

// OPERATIONS

    /** Modyfikuje wspczynnik kary.
     *
     * Madyfikuje wspczynnik kary mnoc wspczynnik kary
     * <code>mPenaltyParam</code> przez <code>mPenaltyParamMultiplier</code>.
     */
    void                ModifyPenalty();

    /** Resetuje wspczynnik kary do pocztkowej wartoci.
     *
     * Przywraca wspczynnik kary <code>mPenaltyParam</code> do wartoci
     * <code>mInitialPenaltyParam</code>.
     */
    void                ResetPenalty();

// ACCESS

    /** Dodaj ograniczenie.
     *
     * Funkcja przejmuje odpowiedzialno za usunicie ogranicze.
     *
     * @param constraint Wskanik na ograniczenie.
     */
    void                AddConstraint(const Constraint& constraint);

    /** Zwraca list ogranicze.
     */
    const Constraint::Container&    Constraints() const;

    // TODO: a moe jednak zadeklarowa ProblemWithPenaltyPanel jako friend?
    /** Zwraca referencj na list ogranicze.
     *
     * Wycznie do uytku wewntrznego.
     */
    Constraint::Container&          rConstraints();

    virtual unsigned                EvaluationCount() const;

    virtual void                    SetProblem(const ProblemBase* pProblem) const;

    /** Wypisuje wszystkie ograniczenia do stringa.
     */
    wxString                        ConstraintsToString() const;

    /** Reakcja na przekroczenie ogranicze.
     *
     * Informuje o tym, co ma si sta gry w procesie optymalizacji pojawi si
     * punkt znajdujcy si poza ograniczeniami (ma to znaczenie tylko w
     * przypadku funkcji bariery). Dwie moliwocie to:
     * <ul>
     * <li>wyrzucenie wyjtku <code>OptimizationError</code>;
     * <li>zwrcenie przez funkcj <code>Evaluate()</code> wartoci
     * <code>DBL_MAX</code>
     * </uk>
     * Domylnym i zalecanym zachowaniem jest drugi wariant.
     */
    OutOfBoundsReactionType&        rOutOfBoundsReaction();

    double                          PenaltyParam() const;

    double&                         rPenaltyParam() const;

    OutOfBoundsReactionType         OutOfBoundsReaction() const;

    virtual VariableNames           PresentVariables() const;

    static FunctionIdType           ClassId();

    /** Zwraca ID funkcji kary.
     */
    virtual FunctionIdType          Id() const = 0;

// INQUIRY

    /** Sprawdza, czy punkt spenia ograniczenia.
     *
     * Wywouje IsValid z podanymi parametrami dla kadego ograniczenia.
     *
     * @param point Punkt, ktrego pozycja jest sprawdzana.
     *
     * @see Constraint::IsValid()
     */
    virtual bool    IsValid(const ColumnVector& point) const = 0;

    virtual bool    IsValidStartingPoint(const ColumnVector& point) const = 0;

    virtual bool    ContainsUnknown(const VariableNames& variables) const;

    virtual int     FunctionId() const = 0;

protected:
    PenaltyFunction(const PenaltyFunction& from);

    /** Pocztkowa warto parametru optymalizacji
     */
    DoubleParameter         mInitialPenaltyParam;
    /** Wspczynnik zmiany parametru optymalizacji
     */
    DoubleParameter         mPenaltyParamMultiplier;
    /** Tolerancja niedokadnoci konieczna przy ograniczeniach rwnociowych
     */
    DoubleParameter         mTolerance;
    /** Reakcja na wyjcie poza ograniczenia
     */
    OutOfBoundsReactionType mOutOfBoundsReaction;

    Constraint::Container   mConstraints;   ///< Ograniczenia

    static FunctionIdType   GenerateId();

private:

    mutable double          mPenaltyParam;  ///< Wspczynnik kary

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _PenaltyFunction_h_
