#include <precomp.h>

#include <stdlib.h>

#include "Logger.h"
#include "Population.h"  // class implemented

//TODO: przerzuci si na porzdniejszy generator liczb pseudolosowych.
#include <time.h>

LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Population::Population(const FunctionBase&  rFunction,
                             size_t         numberOfChromosomes,
                       const ColumnVector&  rangeMinBounds,
                       const ColumnVector&  rangeMaxBounds)
    : mFunction(rFunction),
      mChromosomes(),
      mDesiredSize(numberOfChromosomes),
      mBestIndex(NOT_CHOSEN),
      mAverageChromosome(NullPoint),
      mAverageValue(0.0),
      mChromosomeStandardDeviation(0.0),
      mValueStandardDeviation(0.0)
{
    Generate(numberOfChromosomes, rangeMinBounds, rangeMaxBounds);
}

Population::Population(const Population& from)
    : mFunction(from.mFunction),
      mChromosomes(from.mChromosomes),
      mDesiredSize(from.mDesiredSize),
      mBestIndex(from.mBestIndex),
      mAverageChromosome(from.mAverageChromosome),
      mAverageValue(from.mAverageValue),
      mChromosomeStandardDeviation(from.mChromosomeStandardDeviation),
      mValueStandardDeviation(from.mValueStandardDeviation)
{
}// Population

Population::~Population()
{
}// ~Populatio

//============================= OPERATORS ====================================

PointWithValue&
Population::operator[](size_t   index)
{
    assert (index < Size());

    return mChromosomes[index];
}

//============================= OPERATIONS ===================================

void
Population::Generate(      size_t           numberOfChromosomes,
                     const ColumnVector&    rangeMinBounds,
                     const ColumnVector&    rangeMaxBounds)
{
    assert(numberOfChromosomes >= 2);

    srand(time(NULL));

    ColumnVector spans(rangeMinBounds);

    spans = rangeMaxBounds - rangeMinBounds;

    LOG(DEBUG, wxT("min: ") << rangeMinBounds.as_row());
    LOG(DEBUG, wxT("max: ") << rangeMaxBounds.as_row());
    LOG(DEBUG, wxT("spans: ") << spans.as_row());

    mChromosomes.reserve(numberOfChromosomes);

    for (unsigned ch = 0; ch < numberOfChromosomes; ++ch)
    {
        ColumnVector new_chromosome(rangeMinBounds.nrows());

        for (int i = 1; i <= spans.nrows(); ++i)
        {
            new_chromosome(i) = static_cast<double>(rand()) /
                                static_cast<double>(RAND_MAX) *
                                spans(i);
            new_chromosome(i) += rangeMinBounds(i);
        }

        mChromosomes.push_back(PointWithValue(new_chromosome, DBL_MAX));
    }
    LOG(DEBUG, wxT("Chomosomes created: ") << mChromosomes.size());
    assert(mChromosomes.size() == numberOfChromosomes);

    CalculateGenerationFitness();
    GatherInfo();

}

void
Population::CalculateGenerationFitness()
{
    ChromosomeContainer::iterator   iter;

    for (iter = mChromosomes.begin(); iter != mChromosomes.end(); ++iter)
    {
        iter->mValue = mFunction.Evaluate(iter->mPoint);
    }

}

void
Population::ReevaluateChromosome(size_t index)
{
    assert(index < Size());

    mChromosomes[index].mValue = mFunction.Evaluate(mChromosomes[index].mPoint);

    if (mChromosomes[index].mValue < mChromosomes[mBestIndex].mValue)
        mBestIndex = index;
}

void
Population::GatherInfo(bool findBest)
{
    if (findBest)
        FindBest();
    CalculateAverages();
    CalculateDeviations();
}

//============================= ACCESS     ===================================

const PointWithValue&
Population::Best() const
{
    return mChromosomes[mBestIndex];
}

size_t
Population::BestIndex() const
{
    return mBestIndex;
}

size_t
Population::DesiredSize() const
{
    return mDesiredSize;
}

size_t
Population::Size() const
{
    return mChromosomes.size();
}

void
Population::AddChromosome(const ColumnVector&   newChromosome)
{
    mChromosomes.push_back(PointWithValue(newChromosome,
                                          mFunction.Evaluate(newChromosome)));

    if (mBestIndex != NOT_CHOSEN)
    {
        LOG(DEBUG, wxString::Format
           (wxT("Value in new = %g\nValue in best = %g\nIndex of best = %d\n"),
            mChromosomes.back().mValue,
            mChromosomes[mBestIndex].mValue,
            mBestIndex));

        if (mChromosomes.back().mValue < mChromosomes[mBestIndex].mValue)
        {
            mBestIndex = Size() - 1;
            LOG(DEBUG, wxString::Format(wxT("New index of best = %d\n"),
                                        mBestIndex));
        }
    }
    else
    {
        LOG(DEBUG, wxT("Best chromosome not yet chosen, dead or mutated"));
        mBestIndex = Size() - 1;
    }
}

void
Population::Erase(size_t index)
{
    assert (index < Size());

    if (index == static_cast<unsigned>(mBestIndex))
        mBestIndex = NOT_CHOSEN;

    if (Size() > 1)
    {
        mChromosomes[index] = mChromosomes.back();
        if (static_cast<unsigned>(mBestIndex) == (Size() - 1))
            mBestIndex = index;
    }
    else
    {
        mBestIndex = NOT_CHOSEN;
    }
    mChromosomes.erase(mChromosomes.end() - 1);
}

//============================= INQUIRY    ===================================

ColumnVector
Population::AverageChromosome() const
{
    return mAverageChromosome;
}

double
Population::AverageValue() const
{
    return mAverageValue;
}

double
Population::ChromosomeStandardDeviation() const
{
    return mChromosomeStandardDeviation;
}

double
Population::ValueStandardDeviation() const
{
    return mValueStandardDeviation;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

void
Population::FindBest()
{
    double best_value;

    if (mBestIndex == NOT_CHOSEN)
        best_value = DBL_MAX;
    else
        best_value = mChromosomes[mBestIndex].mValue;

    for (size_t i = 0; i < Size(); ++i)
    {
        if (mChromosomes[i].mValue < best_value)
        {
            best_value = mChromosomes[i].mValue;
            mBestIndex = i;
        }
    }
}

void
Population::CalculateAverages()
{
    mAverageValue = 0.0;

    //Zmienna liczy ile punktw miao warto rn od DBL_MAX (warto funkcji
    //zwracana dla punktw lecych poza ograniczeniami w problemie z
    //ogranczemiami z wewntrzn funkcj kary). Odrzucanie takich punktw
    //jest konieczne by moliwe byo zastosowanie w metodzie
    //ewolucyjnej warunku stopu biorcego pod uwag odchylenie standardowe
    //wartoci funkcji w chromosomach.
    unsigned n_of_accepted_points = 0;

    if (Size() == 0)
        mAverageChromosome  = NullPoint;
    else
    {
        mAverageChromosome.resize(mChromosomes.front().mPoint.nrows());
        mAverageChromosome = 0.0;

        for (size_t i = 0; i < Size(); ++i)
        {
            mAverageChromosome  += mChromosomes[i].mPoint;

            double value_in_point = mChromosomes[i].mValue;
            if (value_in_point != DBL_MAX)
            {
                mAverageValue += value_in_point;
                ++n_of_accepted_points;
            }
        }

        mAverageChromosome  /= Size();
        mAverageValue       /= n_of_accepted_points;
    }

    LOG(DEBUG, wxT("Average chromosome: ") << mAverageChromosome.as_row());
    LOG(DEBUG, wxT("Average value:      ") << mAverageValue);
}

void
Population::CalculateDeviations()
{
    mValueStandardDeviation         = 0.0;
    mChromosomeStandardDeviation    = 0.0;

    //Zmienna liczy ile punktw miao warto rn od DBL_MAX (warto funkcji
    //zwracana dla punktw lecych poza ograniczeniami w problemie z
    //ogranczemiami z wewntrzn funkcj kary). Odrzucanie takich punktw
    //jest konieczne by moliwe byo zastosowanie w metodzie
    //ewolucyjnej warunku stopu biorcego pod uwag odchylenie standardowe
    //wartoci funkcji w chromosomach.
    unsigned n_of_accepted_points = 0;

    if (Size() != 0)
    {
        double  sum_of_norm_squares = 0.0;
        double  sum_of_values = 0.0;

        for (size_t i = 0; i < Size(); ++i)
        {
            ColumnVector    temp_vector
                                = mChromosomes[i].mPoint - mAverageChromosome;
            sum_of_norm_squares += temp_vector.sum_square();

            double value_in_point = mChromosomes[i].mValue;
            if (value_in_point != DBL_MAX)
            {
                double difference = value_in_point - mAverageValue;
                sum_of_values     += difference * difference;
                ++n_of_accepted_points;
            }
        }

        sum_of_norm_squares /= Size();
        sum_of_values       /= n_of_accepted_points;

        mChromosomeStandardDeviation    = sqrt(sum_of_norm_squares);
        mValueStandardDeviation         = sqrt(sum_of_values);

        LOG(DEBUG, wxT("Chromosome standard deviation: ")
                                            << mChromosomeStandardDeviation);
        LOG(DEBUG, wxT("Value standard deviation:      ")
                                            << mValueStandardDeviation);
    }
}

