#include <precomp.h>


#include "Logger.h"
#include "ProblemBase.h"  // class implemented

LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


ProblemBase::ProblemBase(std::auto_ptr<const FunctionBase>    apFunction,
                         std::auto_ptr<      VariableNames>   apVariables)
    : mcpFunction(apFunction), mcpVariables(apVariables), mapMethod()
{
}// ProblemBase

/* virtual */
ProblemBase::~ProblemBase()
{
}// ~ProblemBase


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

//ColumnVector
//ProblemBase::CreateVector() const
//{
//    return mcpVariables->CreateVector();
//}// CreateVector

std::auto_ptr<Result>
ProblemBase::StartSolve() const throw(OptimizationError)
{
    assert(mapMethod.get() != NULL);
    assert(mcpFunction.Get() != NULL);
    assert(mcpVariables.Get() != NULL);

    mcpFunction->ResetCounter();
    mapMethod->ResetStartingConditions();

    std::auto_ptr<Result> ap_result;
    try
    {
        mcpFunction->BeginFixed();
        ap_result = Solve();

        ap_result->rEvaluationCount() = mcpFunction->EvaluationCount();
    }
    // TODO: OptimizationError
    catch(OptimizationError& error)
    {
        error.rResult().rEvaluationCount() = mcpFunction->EvaluationCount();
        mcpFunction->EndFixed();

        LOG(DEBUG, wxT("Optimization Error"));
        throw;
    }

    mcpFunction->EndFixed();

    return ap_result;
}// StartSolve


//============================= ACCESS     ===================================

void
ProblemBase::SetMethod(std::auto_ptr<const Method> apMethod)
{
    mapMethod = apMethod;
}// SetMethod

void
ProblemBase::SetMethod(std::auto_ptr<Method> apMethod)
{
    SetMethod(auto_ptr<const Method>(apMethod));
}// SetMethod

/* virtual */ void
ProblemBase::SetFunction(auto_ptr<FunctionBase>  apFunction,
                         auto_ptr<VariableNames> apVariables)
{
    apFunction->SetProblem(this);

    mcpFunction  = CountedPtr<const FunctionBase>(apFunction);
    mcpVariables = CountedPtr<const VariableNames>(apVariables);
}// SetFunction

const Method&
ProblemBase::UsedMethod() const
{
    assert(mapMethod.get() != NULL);

    return *mapMethod;
}// Method

const FunctionBase&
ProblemBase::Function() const
{
    assert(mcpFunction.Get() != NULL);

    return *mcpFunction;
}

/* virtual */ const FunctionBase&
ProblemBase::BaseFunction() const
{
    return Function();
}

const VariableNames&
ProblemBase::Variables() const
{
    return *mcpVariables;
}// Variables

int
ProblemBase::VariableCount() const
{
    return mcpVariables->Count();
}// VariableCount

/* static */ const vector<ProblemBase*>&
ProblemBase::Problems()
{
    return rProblems();
}// Problems

/* static */ ProblemBase*
ProblemBase::GetById(ProblemIdType id)
{
    vector<ProblemBase*>::const_iterator position;
    const vector<ProblemBase*>& problems = ProblemBase::Problems();

    position = find_if(problems.begin(), problems.end(),
            bind2nd(ptr_fun(ProblemBase::CompareId), id));

    assert(position  != problems.end());
    assert(*position != NULL);

    return *position;
}// GetById

/* static */ ProblemIdType
ProblemBase::ClassId()
{
    const ProblemIdType AbstractProblemId = 0;

    return AbstractProblemId;
}// ClassId

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

ProblemBase::ProblemBase(bool doRegister)
    // zmienne s potrzebne do inicjowania okienek. Jakiekolwiek.
    : mcpFunction(), mcpVariables(new VariableNames), mapMethod()
{
    if (doRegister) ProblemBase::rProblems().push_back(this);
}// ProblemBase

ProblemBase::ProblemBase(const ProblemBase& from)
    : mcpFunction(),
      mcpVariables(),
      mapMethod()
{
    if (from.mapMethod.get() != NULL)
    {
        SetMethod(from.mapMethod->Clone());
    }
    if (from.mcpFunction.Get() != NULL)
    {
        assert (from.mcpVariables.Get() != NULL);

        mcpFunction  = CountedPtr<const FunctionBase>(from.mcpFunction->Clone());
        mcpVariables = CountedPtr<const VariableNames>
            (new VariableNames(*from.mcpVariables));

        mcpFunction->SetProblem(this);
    }
}// ProblemBase

/* virtual */ std::auto_ptr<Result>
ProblemBase::Solve() const throw(OptimizationError)
{
    return mapMethod->StartOptimize(*this);
}// Solve

/* static */ ProblemIdType
ProblemBase::GenerateId()
{
    static ProblemIdType id = 1;

    return id++;
}


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ vector<ProblemBase*>&
ProblemBase::rProblems()
{
    static vector<ProblemBase*> problems;

    return problems;
}

/* static */ bool
ProblemBase::CompareId(const ProblemBase* pProblem, ProblemIdType id)
{
    return (pProblem->Id() == id);
}// CompareId
