#ifndef _ProblemBase_h
#define _ProblemBase_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "CountedPtr.h"
#include "FunctionBase.h"
#include "Method.h"
#include "Result.h"
#include "VariableNames.h"

// FORWARD REFERENCES
//

/** Typ numeru ID problemu.
 */
typedef unsigned    ProblemIdType;

/** Bazowa klasa dla zada optymalizacji.
 *
 * Zadanie, oprcz danych opisowych (nazwa, opis), posiada zmienne (w tym ilo
 * wymiarw funkcji) oraz funkcj celu.
 *
 * Rodzaj funkcji i/lub ew. ogranicze jest okrelany przez klasy pochodne.
 *
 * Kada klasa pochodna powinna tworzy i rejestrowa swj prototyp (patrz:
 * konstruktor), aby bya dostpna w reszcie programu.
 */
class ProblemBase
{
    // Ze wzgldu na wywoywanie FunctionBase::SetProblem w walidatorze.
    friend class FunctionValidator;

public:
// LIFECYCLE

    /** Konstruktor.
     *
     * @param apFunction <code>auto_ptr</code> na funkcj celu.
     * @param apVariables <code>auto_ptr</code> na zmienne.
     *
     * @deprecated Uywa SetFunction
     */
    ProblemBase(std::auto_ptr<const FunctionBase>  apFunction,
                std::auto_ptr<      VariableNames> apVariables);

    /** Destruktor.
     */
    virtual ~ProblemBase(void);


// OPERATORS

// OPERATIONS

    /** Znajduje rozwizanie zadania optymalizacji.
     *
     * Przed rozpoczciem zeruje licznik wylicze funkcji, po zakoczeniu
     * umieszcza jego warto w wyniku.
     */
    virtual std::auto_ptr<Result>       StartSolve() const
        throw(OptimizationError);

//    /** Tworzy wektor do przechowywania wartoci zmiennych.
//     *
//     * @return Wektor kolumnowy o dugoci rwnej iloci zmiennych
//     * (niezainicjowane wartoci).
//     */ // TODO: nazwa? W kocu Create powinno zwaraca auto_ptr
//    ColumnVector                        CreateVector() const;

    /** Tworzy nowy problem tej samej klasy.
     */
    virtual std::auto_ptr<ProblemBase>  Clone() const = 0;

    /** Tworzy tekstowy opis obiektu.
     */
    virtual wxString                    ToString() const = 0;

// ACCESS

    /** Ustawia metod.
     *
     * Wczeniej ustawiona metoda jest usuwana.
     *
     * @param apMethod <code>auto_ptr</code> na now metod.
     */
    void                    SetMethod(std::auto_ptr<const Method> apMethod);

    /** Ustawia metod.
     *
     * Wczeniej ustawiona metoda jest usuwana. Metoda nie jest zmieniana
     * wewntrz problemu, ta wersja SetMethod dodaje <code>const</code>.
     *
     * @param apMethod <code>auto_ptr</code> na now metod.
     */
    void                    SetMethod(std::auto_ptr<      Method> apMethod);

    /** Ustawia now biec funkcj celu oraz zmienne.
     *
     * @param cpFunction <code>CountedPtr</code> na funkcj celu.
     * @param cpVariables <code>CountedPtr</code> na zmienne funkcji.
     *
     * @deprecated Nie uywa poza testami. Funkcje ustawiane s przez panele
     * "konfiguracji" problemu.
     */
    virtual void            SetFunction(
                                auto_ptr<FunctionBase>  cpFunction,
                                auto_ptr<VariableNames> cpVariables);

    /** Zwraca uywan metod optymalizacji.
     */
    // Niestety, mogem albo zamieni Method na MethodBase w cay programie albo
    // da tutaj inn nazw ni Method..
    const Method&               UsedMethod() const;

    /** Zwraca funkcj celu.
     *
     * @return Referencja na funkcj celu.
     */
    const FunctionBase&         Function() const;

    /** Zwraca oryginaln funkcj celu.
     *
     * Funkcja zwraca oryginaln funkcj celu bez uwzgldnienia ewentualnej
     * kary.
     *
     * @return Referencja na oryginaln funkcj celu.
     */
    virtual const FunctionBase& BaseFunction() const;

    /** Zwraca zmienne funkcji celu.
     *
     * @return Referencja na zmienne.
     */
    const VariableNames&        Variables() const;

    /** Zwraca liczb zmiennych.
     */
    int                         VariableCount() const;

    /** Zwraca nazw problemu.
     */
    virtual wxString            Name() const = 0;

    /** Zwraca wektor problemw.
     */
    static const vector<ProblemBase*>&  Problems();

    /** Znajduje problem wg ID i zwraca wskanik na niego.
     */
    static ProblemBase*         GetById(ProblemIdType id);

    /** Zwraca ID problemu.
     *
     * Przykad:
     * <code>
     * auto_ptr<ProblemBase> ap_problem(ProblemBase::GetById(LinearProblem::ClassId()));
     * </code>
     */
    static ProblemIdType             ClassId();

    /** Zwraca ID problemu.
     */
    virtual ProblemIdType            Id() const = 0;

// INQUIRY

protected:
    /** Konstruktor domylny.
     *
     * @param doRegister Czy zarejestrowac problem jako prototyp.
     */
    ProblemBase(bool doRegister = false);

    /** Konstruktor kopiujcy.
     */
    ProblemBase(const ProblemBase& from);

    /** Generuje unikalne ID.
     */
    static ProblemIdType    GenerateId();

    // TODO: teraz to ju moe by auto_ptr i zwyka zmienna na stosie.
    CountedPtr<const FunctionBase>  mcpFunction;  ///< Funkcja celu.
    CountedPtr<const VariableNames> mcpVariables; ///< Zmienne funkcji celu.
    std::auto_ptr<const Method>     mapMethod;    ///< Metoda.

private:

    /** Znajduje rozwizanie zadania optymalizacji.
     */
    virtual std::auto_ptr<Result> Solve() const throw(OptimizationError);


    /** Zwraca referencj na list problemw.
    */
    static vector<ProblemBase*>& rProblems();

    /** Sprawdza, czy podany problem ma podane ID.
     *
     * @param pProblem Wskanik na problem.
     * @param id ID, ktre jest porwnywane z ID <code>pProblem</code>.
     */
    static bool CompareId(const ProblemBase* pProblem, ProblemIdType id);
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _ProblemBase_h_
