#include <precomp.h>

#include "Configuration.h"
#include "UnivariateConstrainedIteration.h"
#include "MultivariateConstrainedIteration.h"
#include "Logger.h"
#include "SimpleUnivariateAdapter.h"

#include "ProblemWithPenalty.h"  // class implemented


LOGGER(WARNING)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


ProblemWithPenalty::~ProblemWithPenalty()
{
}// ~ProblemWithPenalty


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


//============================= ACCESS     ===================================

/* virtual */ void
ProblemWithPenalty::SetFunction(auto_ptr<FunctionBase>  apFunction,
                                auto_ptr<VariableNames> apVariables)
{
    if (apVariables->GetCount() == 1)
    {
        mcpFunction = CountedPtr<const FunctionBase>
            (new SimpleUnivariateAdapter(auto_ptr<FunctionBase>(apFunction)));
    }
    else    mcpFunction = auto_ptr<const FunctionBase>(apFunction);

    mcpVariables = CountedPtr<const VariableNames>(apVariables);

    SetPenaltyFunction();
}

PenaltyFunction* &
ProblemWithPenalty::rpPenalty()
{
    return mpPenaltyFunction;
}// rPenalty

const PenaltyFunction&
ProblemWithPenalty::Penalty() const
{
    return *mpPenaltyFunction;
}// Penalty

/* virtual */ const FunctionBase&
ProblemWithPenalty::BaseFunction() const
{
    assert(mpPenaltyFunction != NULL);

    return mpPenaltyFunction->InnerFunction();
}

/* virtual */ wxString
ProblemWithPenalty::Name() const
{
    return wxGetTranslation(mName);
}// Name

/* virtual */ ProblemIdType
ProblemWithPenalty::Id() const
{
    return ProblemWithPenalty::ClassId();
}// Id

/* static */ ProblemIdType
ProblemWithPenalty::ClassId()
{
    const static ProblemIdType ProblemWithPenaltyId = GenerateId();

    return ProblemWithPenaltyId;
}// ClassId

//============================= INQUIRY    ===================================

bool
ProblemWithPenalty::ReuseStartingPoint() const
{
    return  mReuseStartingPoint;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

ProblemWithPenalty::ProblemWithPenalty(bool doRegister)
    : UnconstrainedProblem(doRegister),
      mEpsilon(0.01),
      mReuseStartingPoint(true),
      mpPenaltyFunction(NULL)
{
}// ProblemWithPenalty

ProblemWithPenalty::ProblemWithPenalty(const ProblemWithPenalty& from)
    : UnconstrainedProblem(from),
      mEpsilon(from.mEpsilon),
      mReuseStartingPoint(from.mReuseStartingPoint),
      mpPenaltyFunction(NULL)
{
    if (mcpFunction.Get() != NULL)  SetPenaltyFunction();
}// ProblemWithPenalty

/* virtual */ auto_ptr<ProblemBase>
ProblemWithPenalty::Clone() const
{
    return auto_ptr<ProblemBase>(new ProblemWithPenalty(*this));
}// Clone

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
ProblemWithPenalty::Solve() const throw(OptimizationError)
{
    assert(mcpFunction.Get()  != NULL);
    assert(mcpVariables.Get() != NULL);
    assert(mapMethod.get()    != NULL);
    assert(mpPenaltyFunction  != NULL);

    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    ColumnVector            current_approximation(VariableCount());
    ColumnVector            previous_approximation(VariableCount());
    ColumnVector            approximation_step(VariableCount());
    unsigned                step_count = 0;
    unsigned                STEP_LIMIT =
                                    Configuration::GetInstance().StepLimit();
    unsigned                evaluation_sum = 0;

    current_approximation = 0.0;
/*
    if (!mpPenaltyFunction->IsValidStartingPoint(mapMethod->StartingPoint()))
        throw OptimizationError(wxT("Starting point invalid!"),
                                        auto_ptr<Result>(ap_result));
*/
    mpPenaltyFunction->ResetPenalty();

    try
    {
        do
        {
            ++step_count;

            LOG(DEBUG,wxT("Iteracja ") << step_count);

            auto_ptr<Result>
                    ap_iteration_result(mapMethod->StartOptimize(*this));

            ap_iteration_result->rEvaluationCount()
                    = mpPenaltyFunction->EvaluationCount() - evaluation_sum;

            previous_approximation = current_approximation;
            current_approximation = ap_iteration_result->FinalResult();
            approximation_step = current_approximation - previous_approximation;


            if (current_approximation.nrows() == 1)
            {
                auto_ptr<UnivariateConstrainedIteration> ap_iteration
                   (new UnivariateConstrainedIteration
                      (previous_approximation(1),
                       mpPenaltyFunction->IsValid(previous_approximation),
                       current_approximation(1),
                       mpPenaltyFunction->IsValid(current_approximation),
                       ap_iteration_result->EvaluationCount(),
                       ap_iteration_result->IterationCount(),
                       mpPenaltyFunction->PenaltyParam()));
                ap_result->AddIteration(auto_ptr<const IterationData>
                                                            (ap_iteration));
            }
            else
            {
                auto_ptr<MultivariateConstrainedIteration> ap_iteration
                   (new MultivariateConstrainedIteration
                      (previous_approximation,
                       mpPenaltyFunction->IsValid(previous_approximation),
                       current_approximation,
                       mpPenaltyFunction->IsValid(current_approximation),
                       ap_iteration_result->EvaluationCount(),
                       ap_iteration_result->IterationCount(),
                       mpPenaltyFunction->PenaltyParam()));
                ap_result->AddIteration(auto_ptr<const IterationData>
                                                            (ap_iteration));
            }

            evaluation_sum = mpPenaltyFunction->EvaluationCount();

            mpPenaltyFunction->ModifyPenalty();
            if (!mReuseStartingPoint)
            {
                mapMethod->UpdateStartingConditions(ap_iteration_result.get());
            }

            LOG(DEBUG, wxT("New approximation_step")
                                << current_approximation.as_row());
            LOG(DEBUG, wxT("Evaluation count: ")
                                << ap_iteration_result->EvaluationCount());
            LOG(DEBUG, wxT("Evaluation sum:   ") << evaluation_sum);
            LOG(DEBUG, wxT("Iteration count:  ")
                                << ap_iteration_result->IterationCount());

            LOG(DEBUG, wxT("\n"));


        } while ((step_count <= STEP_LIMIT) &&
                 (approximation_step.norm_Frobenius() > mEpsilon ||
                  step_count == 1));

        ap_result->rFinalResult() = current_approximation;

    }
    catch (const OptimizationError& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }

    return  ap_result;
}// Solve


void
ProblemWithPenalty::SetPenaltyFunction()
{
    const SimpleUnivariateAdapter* univariate_adapter =
        dynamic_cast<const SimpleUnivariateAdapter*>(mcpFunction.Get());
    // Bd si za to smay w piekle!
    mpPenaltyFunction =
        const_cast<PenaltyFunction*>
            (dynamic_cast<const PenaltyFunction*>(mcpFunction.Get()));

    if (mpPenaltyFunction == NULL)
    {
        if (univariate_adapter != NULL)
        {
            // Bd si za to smay w piekle!
            mpPenaltyFunction =
                const_cast<PenaltyFunction*>
                    (dynamic_cast<const PenaltyFunction*>
                        (&(univariate_adapter->InnerFunction())));

            assert(mpPenaltyFunction != NULL);
        }
        else
        {   // Ani adapter, ani funkcja kary, le.
            assert(false);
        }
    }

    mpPenaltyFunction->SetProblem(this);
}// SetPenaltyFunction

/* static */ const wxString
    ProblemWithPenalty::mName(_("Constrained with Penalty Function"));

/* static */ ProblemWithPenalty ProblemWithPenalty::mProblemWithPenalty(true);
