#include <precomp.h>

#include "Configuration.h"
#include "Error.h"
#include "SectionLineSearch.h"  // class implemented
#include "Logger.h"
#include "OptimizationError.h"
#include "UnconstrainedProblem.h"
#include "UnivariateFunction.h"

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SectionLineSearch::~SectionLineSearch()
{
}// ~SectionLineSearch


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
SectionLineSearch::UpdateStartingConditions(const Result* result) const
{
    assert(result->FinalResult().nrows() == 1);

    //double  middle = (mA + mB) / 2;
    double  middle = result->FinalResult()(1);
    double  half_of_length = fabs((mB - mA) / 2);

    mA = middle - mSectionContractionOnUpdate * half_of_length;
    mB = middle + mSectionContractionOnUpdate * half_of_length;
}

/* virtual */ void
SectionLineSearch::ResetStartingConditions() const
{
    mA = mInitialA;
    mB = mInitialB;
}

//============================= ACCESS     ===================================

void
SectionLineSearch::AddCommonParameterDescriptions(wxArrayString& descriptions,
                                                  bool           standalone)
                                                                    const
{
    descriptions.Add(EpsilonDescription(mEpsilon));
    if (standalone) descriptions.Add(SearchSectionDescription(mInitialA,
                                                              mInitialB));
    else            descriptions.Add(InitialSectionLengthDescription(
                     Configuration::GetInstance().InitialSectionLength()));
}// AddCommonParameterDescriptions

/* virtual */ wxString
SectionLineSearch::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    AddCommonParameterDescriptions(parameter_descriptions, standalone);

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
SectionLineSearch::CanAssist(const ProblemBase* pProblem) const
{
    return dynamic_cast<const UnconstrainedProblem *>(pProblem) != NULL;
}// CanAssist

/////////////////////////////// PROTECTED  ///////////////////////////////////

SectionLineSearch::SectionLineSearch()
    : LineSearch(),
      mA(-10.0),
      mB(10.0),
      mInitialA(-10.0),
      mInitialB(10.0),
      mEpsilon(0.0001, 0, true),
      mSectionContractionOnUpdate(0.5, 0, true, 1, true)
{
}// SectionLineSearch

SectionLineSearch::SectionLineSearch(const SectionLineSearch& from)
    : LineSearch(from),
      mA(from.mA),
      mB(from.mB),
      mInitialA(from.mInitialA),
      mInitialB(from.mInitialB),
      mEpsilon(from.mEpsilon),
      mSectionContractionOnUpdate(from.mSectionContractionOnUpdate)
{
}// SectionLineSearch

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
SectionLineSearch::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    // Funkcja celu.
    const UnivariateFunction& function =
        dynamic_cast<const UnivariateFunction&>(problem.Function());

    return Solve(function, mA, mB, true);
}// Optimize

/* virtual */ double
SectionLineSearch::Optimize(const UnivariateFunction& function) const
{
    double a;
    double b;
    double length = Configuration::GetInstance().InitialSectionLength();

    auto_ptr<Result> ap_result;

    try
    {
        FindInitialSection(function, a, b, length);
        LOG(DEBUG, wxT("a: ") << a << wxT(", b: ") << b);

        ap_result = Solve(function, a, b, false);
    }
    catch (const Error& error)
    {
        throw Error(_("line search error:\n") + error.What());
    }

    assert (ap_result->FinalResult().nrows() == 1);

    return ap_result->FinalResult()(1);
}// Optimize


void
SectionLineSearch::FindInitialSection(
        const UnivariateFunction&   function,
        double&                     rA,
        double&                     rB,
        double                      initialSectionLength) const
{
    double start;  // Aktualny pocztek odcinka.
    double end;    // Aktualny kraniec odcinka.
    double middle; // Aktualny rodek odcinka.
    double value_start;
    double value_end;
    double value_middle;

    const unsigned STEP_LIMIT = Configuration::GetInstance().StepLimit();
    const double GROW_FACTOR = 2;
    unsigned step_count = 0;

    // Tworzymy bazowy odcinek.
    assert(initialSectionLength > 0);
    start   = 0;
    end     = initialSectionLength;
    middle  = initialSectionLength / 2;

    try
    {
        // Dugo odcinka (bdzie zwikszana z kroku na krok).
        double section_length = initialSectionLength;

        value_start     = function.Evaluate(start);
        value_middle    = function.Evaluate(middle);
        value_end       = function.Evaluate(end);

        if (value_start <= value_middle && value_start <= value_end)
        {   // Poszerz przedzia i zawr, jeli funkcja ronie w kierunku
            // dodatnich.

            section_length  = -section_length * GROW_FACTOR;

            middle          = start;
            start           = end;
            end             = -end;

            value_middle    = value_start;
            value_start     = value_end;
            value_end       = function.Evaluate(end);
        }

        // Przesuwamy odcinek do momentu, kiedy warto
        // porodku bdzie mniejsza od obu wartoci na kracach.
        while (!(value_middle <= value_start && value_middle <= value_end) &&
               step_count < STEP_LIMIT)
        {

            section_length  *= GROW_FACTOR;

            middle      = end;
            value_middle= value_end;
            end         = section_length;
            value_end   = function.Evaluate(end);

#if 0
            if (value_a < value_b)
            { // Przesuwamy w lewo.
                b               = middle;
                value_b         = value_middle;
                middle          = b - 0.5 * section_length;
                a               = b - section_length;

                value_middle    = function.Evaluate(middle);
                value_a         = function.Evaluate(a);
            }
            else
            { // Przesuwamy w prawo.
                a               = middle;
                value_a         = value_middle;
                middle          = a + 0.5 * section_length;
                b               = a + section_length;

                value_middle    = function.Evaluate(middle);
                value_b         = function.Evaluate(b);
            }
#endif
            LOG(DEBUG, wxT("start: ") << start << wxT(", middle: ") << middle
                    << wxT(", end: ") << end);

            ++step_count;
        } // while
    }// try
    catch (const EvaluationError& error)
    {
        throw Error(_("While searching for initial section:\n")
                      + error.What());
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw Error(_("While searching for initial section:\n"
                      "Step limit reached!."));
    }

    rA = min<double>(start, end);
    rB = max<double>(start, end);

    LOG(INFO, wxT("Iteration count: ") << step_count);
    LOG(INFO, wxT("Found section length: ") << rB - rA);
} // FindInitialSection

#if 0
void
SectionLineSearch::FindInitialSection(
        const UnivariateFunction&   function,
        double&                     rA,
        double&                     rB,
        double                      initialSectionLength) const
{
    assert(initialSectionLength > 0);

    double  fixed_bound;    // Nieruchomy kraniec przedzialu
    double  moving_bound;   // Ruchomy kraniec przedzialu
    double  initial_left;   // Prbny punkt mniejszy od pocztkowego
    double  initial_right;  // Prbny punkt wikszy od pocztkowego

    // Wartoci funkcji celu w powyszych punktach:
    double  value_in_fixed_bound;
    double  value_in_moving_bound;
    double  value_in_previous_moving_bound;
    double  value_in_initial_left;
    double  value_in_initial_right;

    // Szeroko, o ktr poszerzamy przedzia (zwikszana dwukrotnie w
    // kadej iteracji)
    double  delta;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;


    try
    {
        // Nieruchomy punkt znajduje si w pocztku wzgldnego ukdu
        // wsprzdnych.
        fixed_bound     = 0;
        initial_left    = -0.5 * initialSectionLength;
        initial_right   = 0.5 * initialSectionLength;

        value_in_initial_left   = function.Evaluate(initial_left);
        value_in_initial_right  = function.Evaluate(initial_right);

        LOG(DEBUG, wxT("initial_left:"));
        LOG(DEBUG, wxT("f(") << initial_left << wxT(")=")
                                                << value_in_initial_left);
        LOG(DEBUG, wxT("initial_right:"));
        LOG(DEBUG, wxT("f(") << initial_right << wxT(")=")
                                                << value_in_initial_right);

// Oszukiwaniem si jest przewiadczenie, e w ten sposb sprawdzimy, czy w
// przedziale jest maksimum lokalne (dziaa tylko dla szczeglnego przypadku).
#if 0
        while ((value_in_initial_left <= value_in_fixed_bound)
               && (value_in_fixed_bound >= value_in_initial_right)
               && (step_count < (STEP_LIMIT - 1))
               )
        {
            ++step_count;

            initial_left    *= 2;
            initial_right   *= 2;

            value_in_initial_left   = function.Evaluate(initial_left);
            value_in_initial_right  = function.Evaluate(initial_right);

            LOG(DEBUG,wxT("Niespelniony warunek unimodalnosci, "));
            LOG(DEBUG,wxT("poszerzanie poczatkowego przedzialu"));

            LOG(DEBUG, wxT("initial_left:"));
            LOG(DEBUG, wxT("f(") << initial_left << wxT(")=")
                    << value_in_initial_left);
            LOG(DEBUG, wxT("fixed_bound:"));
            LOG(DEBUG, wxT("f(") << fixed_bound << wxT(")=")
                    << value_in_fixed_bound);
            LOG(DEBUG, wxT("initial_right:"));
            LOG(DEBUG, wxT("f(") << initial_right << wxT(")=")
                                                << value_in_initial_right);
        }
#endif

        // Ustawiamy kraniec, ktry bdzie przesuwany oraz kraniec, ktry bdzie
        // stay (aby i w kierunku domniemanego minimum).
        if (value_in_initial_left >= value_in_initial_right)
        {
            delta                   = initialSectionLength;

            moving_bound            = initial_right;
            value_in_moving_bound   = value_in_initial_right;

            fixed_bound             = initial_left;
            value_in_fixed_bound    = value_in_initial_left;
        }
        else
        {
            delta                   = -initialSectionLength;

            moving_bound            = initial_left;
            value_in_moving_bound   = value_in_initial_left;

            fixed_bound             = initial_right;
            value_in_fixed_bound    = value_in_initial_right;
        }

// Dziki powyszemu, ponisze bdzie zawsze faszywe (poza sytuacj, gdy obie
// wartoci s rwne, ale to nas nie interesuje).
#if 0
        // Jeli warto funkcji w punkcie bazowym jest mniejsza ni w
        // punktach przesunitych, to szukany przedzia jest ograniczony
        // punktami przesunitymi.
        if (value_in_fixed_bound <= value_in_moving_bound)
        {
            rA  = initial_left;
            rB  = initial_right;

            //LOG(DEBUG,
            //    wxT("Minimum znajdowalo sie w poczatkowym przedziale"));

            return;
        }
#endif

        // Waciwa ptla algorytmu, w ktrej powikszamy przedzia
        // poszukiwa w wybranym kierunku o coraz wiksze odcinki.
        do
        {
            ++step_count;

            value_in_previous_moving_bound = value_in_moving_bound;

            delta *= 2;

            moving_bound           += delta;
            value_in_moving_bound   = function.Evaluate(moving_bound);
        }
        while (value_in_moving_bound < value_in_previous_moving_bound &&
               step_count < STEP_LIMIT);


        if (fixed_bound < moving_bound)
        {
            rA  = fixed_bound;
            rB  = moving_bound;
        }
        else
        {
            rA  = moving_bound;
            rB  = fixed_bound;
        }

        LOG(INFO, wxT("Iteration count: ") << step_count);
        LOG(INFO, wxT("Found section length: ") << rB - rA);


    }// try
    catch (const EvaluationError& error)
    {
        throw Error(_("While searching for initial section:\n")
                    + error.What());
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw Error(_("While searching for initial section:\n"
                      "Step limit reached!."));
    }

} // FindInitialSection
#endif
