/**  Klasa bazowa dla metod optymalizacji w kierunku polegajcych
 * na podziale i redukcji odcinka.
 *
 * Metody polegaj na skracaniu zadanego odcinka \f$[a, b]\f$.
 * Jeli dana metoda z tej rodziny suy do rozwizania samodzielnego problemu
 * optymalizacji funkcji jednej zmiennej, odcinek ten jest zadany
 * przez uytkownika.
 * W przypadku, gdy wykonuje tylko minimalizacj kierunkow jako cz
 * metody optymalizacji funkcji wielu zmiennych, odcinek \f$[a, b]\f$ jest
 * znajdowany automatycznie (patrz <code>FindInitialSection</code>).
 *
 * Odcinek \f$[a, b]\f$ jest skracany w taki sposb, aby by zbieny
 * do odcinka o niewielkiej dugoci, obejmujcego punkt,
 * w ktrym znajduje si minimum.
 *
 * TODO: Karolu, czy wynikiem optymalizacji samodzielnej nie jest cay przebieg
 * optymalizacji? Pamitam gdy Simon o tym mwi, dlatego wrzucam to TODO
 * W przypadku gdy metoda dziaa samodzielnie, wynikiem jest odcinek
 * \f$[a^k, b^k]\f$; jeli suy tylko do minimalizacji kierunkowej
 * przy rozwizywaniu problemu optymalizacji funkcji wielu zmiennych,
 * wynikiem jest punkt - najczciej rodek tego odcinka:
 * \f$\frac{a^k + b^k}{2}\f$.
 */

#ifndef _SectionLineSearch_h
#define _SectionLineSearch_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//

#include "DoubleParameter.h"
#include "LineSearch.h"
#include "text-util.h"

// FORWARD REFERENCES
//


class SectionLineSearch
    : public LineSearch
{
public:
// LIFECYCLE

    friend class InitialSectionValidator;

    /** Destruktor.
     */
    ~SectionLineSearch(void);


// OPERATORS

// OPERATIONS

    virtual void        UpdateStartingConditions(const Result* result) const;

    virtual void        ResetStartingConditions() const;

// ACCESS

    void                AddCommonParameterDescriptions
                                        (wxArrayString&   descriptions,
                                         bool             standalone) const;

    virtual wxString    ToString(bool standalone = true) const;

// INQUIRY

    virtual bool        CanAssist(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    SectionLineSearch(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SectionLineSearch(const SectionLineSearch& from);


    /// Pocztek przeszukiwanego odcinka (gdy metoda dziaa samodzielnie).
    mutable DoubleParameter mA;
    /// Koniec przeszukiwanego odcinka.
    mutable DoubleParameter mB;

    DoubleParameter         mInitialA;
    DoubleParameter         mInitialB;

    /// Dokadno (\f$\varepsilon\f$).
    DoubleParameter         mEpsilon;

    DoubleParameter         mSectionContractionOnUpdate;

    // Ta metoda jest protected tylko ze wzgldu na testy!!!
    /** Znajduje odcinek, w ktrym znajduje si minimum.
     *
     * Poszukiwanie rozpoczyna si od przedziau [0,
     * <code>initialSectionLength</code>]. W kolejnych krokach nastpuje
     * podwajanie dugoci przedziau poprzez przesuwanie dodatniego koca.
     * Algorytm koczy si, gdy warto funkcji w punkcie rodkowym jest
     * mniejsza ni na kracach.
     *
     * W przypadku, gdy przed pierwsz iteracj wartoci funkcji w punktach
     * rodkowym oraz <code>initialSectionLength</code> s wiksze ni w punkcie
     * 0, obszar jest poszerzany do:
     * [-<code>initialSectionLength</code>, <code>initialSectionLength</code>]
     * i zmieniany jest kierunek poszukiwa na ujemny.
     *
     * Punkty wynikowe speniaj zaleno <code>rA</code> &lt; <code>rB</code>.
     *
     * @param function Funkcja celu.
     * @param rA Referencja na pocztek odcinka wynikowego.
     * @param rB Referencja na koniec odcinka wynikowego.
     * @param initialSectionLength Dugo przesuwanego odcinka.
     */
    void            FindInitialSection(const   UnivariateFunction& function,
                                       double& rA,
                                       double& rB,
                                       double  initialSectionLength) const;
private:

    /** Optymalizuj.
     *
     * Przeprowadza optymalizacj za pomoc algorytmu, ktry realizowa ma dana
     * klasa.
     *
     * @param problem Problem optymalizacji.
     * @return Wynik oraz przebieg optymalizacji.
     */
    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    /** Znajd optymaln dugo kroku.
     *
     * Wywoywana z FindOptimalStep.
     *
     * @param function Funkcja celu
     * @see FindOptimalStep
     */
    virtual double  Optimize(const UnivariateFunction& function)
                               const;

    /** Znajd minimum w kierunku.
     *
     * Funkcja znajduje minimum funkcji jednowymiarowej.
     * Jest wykorzystywana zarwno gdy minimalizujemy funkcj jednowymiarow w
     * samodzielnym problemie minimalizacji, jak i gdy minimalizacja w kierunku
     * jest etapem minimalizacji w wielu wymairach.
     *
     * @param function Funkcja celu  (\f$f\f$).
     * @param a Lewy kraniec pocztkowego przedziau (\f$a^0\f$).
     * @param b Prawy kraniec pocztkowego przedziau (\f$b^0\f$).
     * @param generateResult Czy generowa przebieg optymalizacji.
     * @return <code>auto_ptr</code> na przebieg optymalizacji. Jeli
     * <code>generateResult</code> jest <code>false</code> to nie zawiera
     * adnych iteracji (ale wynik kocowy tak).
     */
    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError) = 0;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SectionLineSearch_h_
