#include <precomp.h>

#include "Configuration.h"
#include "Error.h"
#include "Logger.h"
#include "OptimizationError.h"
#include "ProblemBase.h"
#include "SinglePointLineSearch.h"  // class implemented
#include "UnconstrainedProblem.h"
#include "UnivariateFunction.h"
#include "text-util.h"

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SinglePointLineSearch::~SinglePointLineSearch()
{
}// ~SinglePointLineSearch


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
SinglePointLineSearch::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
SinglePointLineSearch::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================
void
SinglePointLineSearch::AddCommonParameterDescriptions
                        (wxArrayString& descriptions,
                         bool           standalone) const
{
    descriptions.Add(EpsilonDescription(mEpsilon));
    if (standalone)     descriptions.Add(StartingPointDescriptionUnivariate(
                            mInitialStartingPoint));
}// AddCommonParameterDescriptions

/* virtual */ wxString
SinglePointLineSearch::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    AddCommonParameterDescriptions(parameter_descriptions, standalone);

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
SinglePointLineSearch::CanAssist(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem *>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable());
}// CanAssist

/////////////////////////////// PROTECTED  ///////////////////////////////////

SinglePointLineSearch::SinglePointLineSearch()
    : LineSearch(),
      mStartingPoint(1),
      mInitialStartingPoint(1),
      mEpsilon(0.0001, 0, true)
{
    mStartingPoint          = 0.0;
    mInitialStartingPoint   = 0.0;
}// SinglePointLineSearch

SinglePointLineSearch::SinglePointLineSearch(const SinglePointLineSearch& from)
    : LineSearch(from),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mEpsilon(from.mEpsilon)
{
}// SinglePointLineSearch

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
SinglePointLineSearch::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    // Funkcja celu.
    const UnivariateFunction& r_function =
        dynamic_cast<const UnivariateFunction&>(problem.Function());

    return Solve(r_function, mStartingPoint, true);
}// Optimize

/* virtual */ double
SinglePointLineSearch::Optimize(const UnivariateFunction& function) const
{
    ColumnVector starting_point(1);
    auto_ptr<Result> ap_result;

    starting_point(1) = 0.0;

    try
    {
        ap_result = Solve(function, starting_point, false);
    }
    catch (const Error& error)
    {
        throw Error(_("line search error:\n") + error.What());
    }

    assert (ap_result->FinalResult().nrows() == 1);

    return ap_result->FinalResult()(1);
}// Optimize
