/** Podstawowy warunek stopu.
 *
 * Reprezentuje warunek stopu, ktry jest sprawdzany na podstawie dwch
 * ostatnich krokw metody (ich pozycji i/lub wartoci funkcji).
 *
 * Udostpnia metody <code>IsMet</code>, ktre sprawdzaj, czy warunek jest
 * speniony dla zadanej wartoci <em>epsilon</em> oraz podanych punktw
 * i/lub wartoci funkcji w poprzednim oraz bierzcym kroku.
 *
 * W miar moliwoci (tzn. jeli znane s wartoci funkcji w momencie
 * wywoania) naley stosowa metod <code>IsMet</code> z jak najwiksz liczb
 * argumentw (nie liczc przekazywanej funkcji celu). Dziki temu w niektrych
 * przypadkach mona ograniczy liczb oblicze funkcji celu.
 */

#ifndef _StandardStopCondition_h
#define _StandardStopCondition_h


// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "DoubleParameter.h"
#include "FunctionBase.h"
#include "StopConditionBase.h"

// FORWARD REFERENCES
//


class StandardStopCondition
    : public StopConditionBase
{
public:
// LIFECYCLE

    /** Konstruktor domylny.
     */
    StandardStopCondition(void);

    /** Konstruktor.
     *
     * @param epsilon Dokadno.
     */
    StandardStopCondition(const DoubleParameter epsilon);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    StandardStopCondition(const StandardStopCondition& from);


    /** Destruktor.
     */
    virtual ~StandardStopCondition(void);


// OPERATORS

// OPERATIONS

// ACCESS

    /** Zwraca warto minimaln <em>epsilon</em>.
     */
    virtual DoubleParameter         Epsilon() const;

    /** Zwraca referencj na <em>epsilon</em>.
     */
    virtual DoubleParameter&        rEpsilon();


// INQUIRY

    /** Sprawd warunek stopu.
     *
     * @param previousPoint Punkt wyznaczony w poprzednim kroku.
     * @param lastPoint Punkt wyznaczony w ostatnim kroku.
     * @param previousValue Warto funkcji w poprzednim kroku.
     * @param lastValue Warto funkcji w tym kroku.
     * @param customValue Dodatkowy parametr porwnywany bezporednio
     * z epsilonem,
     *
     * @return Czy warunek stopu jest speniony.
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              double                lastValue,
                              double                customValue = 0) const = 0;

    /** Sprawd warunek stopu.
     *
     * Analogiczne, jak czteroargumentowy <code>IsMet</code>, z t rnic, e
     * jeli potrzebna jest warto funkcji w biecym kroku, to jest ona
     * obliczana na podstawie <code>lastPoint</code>.
     * Jeli jest ona znana, to naley korzysta z metody czteroargumentowej,
     * pozwala to oszczdzi jedno wyliczenie funkcji w niektrych przypadkach.
     *
     * @param previousPoint Punkt wyznaczony w poprzednim kroku.
     * @param lastPoint Punkt wyznaczony w ostatnim kroku.
     * @param previousValue Warto funkcji w poprzednim kroku.
     * @param function Funkcja celu.
     * @param customValue Dodatkowy parametr porwnywany bezporednio
     * z epsilonem,
     * @return Czy warunek stopu jest speniony.
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              double                previousValue,
                              const FunctionBase&   function,
                              double                customValue = 0) const = 0;

    /** Sprawd warunek stopu.
     *
     * Analogiczne, jak czteroargumentowy <code>IsMet</code>, z t rnic, e
     * jeli potrzebne s wartoci funkcji w poprzednim biecym kroku, to s
     * one obliczane na podstawie <code>previousPoint</code> i
     * <code>lastPoint</code>.
     * Jeli jedna lub obie s znane, to naley korzysta z metody trzy- lub
     * czteroargumentowej, pozwala to oszczdzi wyliczenia funkcji w niektrych
     * przypadkach.
     *
     * @param previousPoint Punkt wyznaczony w poprzednim kroku.
     * @param lastPoint Punkt wyznaczony w ostatnim kroku.
     * @param function Funkcja celu.
     * @param customValue Dodatkowy parametr porwnywany bezporednio
     * z epsilonem,
     * @return Czy warunek stopu jest speniony.
     */
    virtual bool        IsMet(const ColumnVector&   previousPoint,
                              const ColumnVector&   lastPoint,
                              const FunctionBase&   function,
                              double                customValue = 0) const = 0;


// PUBLIC MEMBERS

protected:
    DoubleParameter mEpsilon;
private:
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _StandardStopCondition_h
