#include <precomp.h>

#include "StopConditionBase.h"  // class implemented

#include "CustomValueCondition.h"
#include "DistanceCondition.h"
#include "EMIterationsCondition.h"
#include "EMPointDeviationCondition.h"
#include "EMValueDeviationCondition.h"
#include "NMBestWorstCondition.h"
#include "NMCircumferenceCondition.h"
#include "NMPreviousWorstCondition.h"
#include "NMWorstReplacementCondition.h"
#include "SlopeCondition.h"
#include "SquaredCustomValueCondition.h"
#include "ValueCondition.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

StopConditionBase::StopConditionBase()
{
}// StopConditionBase

StopConditionBase::StopConditionBase(const StopConditionBase& from)
    : mName(from.mName)
{
}// StopConditionBase

StopConditionBase::~StopConditionBase()
{
}// ~StopConditionBase


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
/* virtual */ const wxString&
StopConditionBase::Name() const
{
    return mName;
}// Name

/* virtual */ wxString&
StopConditionBase::rName()
{
    return mName;
}// Name

/* virtual */ wxString
StopConditionBase::ToString() const
{
    wxString description;

    description << Name() << wxT(" < ") <<
        wxString::Format(wxT("%g"), static_cast<double>(Epsilon()));

    return description;
}// ToString

//============================= INQUIRY    ===================================
/* static */ StopConditions
StopConditionBase::StandardStopConditions()
{
    StopConditions conditions;

    conditions.push_back(
            CountedPtr<StopConditionBase>(new DistanceCondition));
    conditions.push_back(
            CountedPtr<StopConditionBase>(new ValueCondition));
    conditions.push_back(
            CountedPtr<StopConditionBase>(new SlopeCondition));

    return conditions;
}

/* static */ StopConditions
StopConditionBase::GradientStopConditions()
{
    StopConditions conditions;

    conditions.push_back(
            CountedPtr<StopConditionBase>(
               new CustomValueCondition(wxT("|| grad f(x k) ||"))));
    conditions.push_back(
            CountedPtr<StopConditionBase>(
                new SquaredCustomValueCondition(wxT("|| grad f(x k) ||^2"))));

    const vector< CountedPtr<StopConditionBase> >& standard_conditions =
        StandardStopConditions();

    conditions.insert(conditions.end(),
            standard_conditions.begin(), standard_conditions.end());

    return conditions;
}

/* static */ StopConditions
StopConditionBase::NelderMeadStopConditions()
{
    StopConditions conditions;
    CountedPtr<StandardStopCondition> cp_distance_condition(
            new DistanceCondition);
    CountedPtr<StandardStopCondition> cp_value_condition(
            new ValueCondition);
    CountedPtr<StandardStopCondition> cp_slope_condition(
            new SlopeCondition);

    StopConditionBase* p_nm_condition;

    p_nm_condition = new NMBestWorstCondition(cp_distance_condition);
    p_nm_condition->rName() = wxT("|| x_w - x_b ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMBestWorstCondition(cp_value_condition);
    p_nm_condition->rName() = wxT("| f(x_w) - f(x_b) |");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMBestWorstCondition(cp_slope_condition);
    p_nm_condition->rName() = wxT("| f(x_w) - f(x_b) | / || x_w - x_b ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));


    p_nm_condition = new NMWorstReplacementCondition(cp_distance_condition);
    p_nm_condition->rName() = wxT("|| x_w - x_w' ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMWorstReplacementCondition(cp_value_condition);
    p_nm_condition->rName() = wxT("| f(x_w) - f(x_w') |");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMWorstReplacementCondition(cp_slope_condition);
    p_nm_condition->rName() = wxT("| f(x_w) - f(x_w') | / || x_w - x_w' ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));


    p_nm_condition = new NMPreviousWorstCondition(cp_distance_condition);
    p_nm_condition->rName() = wxT("|| x_w k  - x_w k-1 ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMPreviousWorstCondition(cp_value_condition);
    p_nm_condition->rName() = wxT("| f(x_w k) - f(x_w k-1) |");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMPreviousWorstCondition(cp_slope_condition);
    p_nm_condition->rName() =
        wxT("| f(x_w k) - f(x_w k-1) | / || x_w k - x_w k-1 ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    p_nm_condition = new NMCircumferenceCondition();
    conditions.push_back(CountedPtr<StopConditionBase>(p_nm_condition));

    return conditions;
}

/* static */ StopConditions
StopConditionBase::EvolutionaryMethodStopConditions()
{
    StopConditions conditions;

    StopConditionBase* p_em_condition;

    //-----Warunki specyficzne dla ewolucyjnej-----
    p_em_condition = new EMIterationsCondition();
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    p_em_condition = new EMPointDeviationCondition();
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    p_em_condition = new EMValueDeviationCondition();
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    //-----Warunki standardowe-----
    // TODO: poprawi nazwy
    p_em_condition = new EMStopCondition(CountedPtr<StandardStopCondition>(
            new DistanceCondition(DoubleParameter(0.1, 0, true))));
    p_em_condition->rName() = wxT("|| x_b k+1 - x_b k ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    p_em_condition = new EMStopCondition(CountedPtr<StandardStopCondition>(
            new ValueCondition(DoubleParameter(0.1, 0, true))));
    p_em_condition->rName() = wxT("| f(x_b k+1) - f(x_b k) |");
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    p_em_condition = new EMStopCondition(CountedPtr<StandardStopCondition>(
            new SlopeCondition(DoubleParameter(0.1, 0, true))));
    p_em_condition->rName() =
        wxT("| f(x_b k+1) - f(x_b k) | / || x_b k+1 - x_b k ||");
    conditions.push_back(CountedPtr<StopConditionBase>(p_em_condition));

    return conditions;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
