#include <precomp.h>

#include "Logger.h"
#include "Tokenizer.h"  // class implemented
#include "util.h"


LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Tokenizer::Tokenizer(const wxString&    input,
                     VariableNames&     rVariables,
                     bool               addVariables,
                     FunctionMap        functions)
    : mInput(input),
      mWorkingInput(input),
      mPosition(0),
      mTokenLength(0),
      mTokenType(NONE),
      mDifferentiable(true),
      mrVariables(rVariables),
      mAddVariables(addVariables),
      mFunctionMap(functions)
{
    TrimInput();     // Obetnij pocztkowe spacje
}// Tokenizer

Tokenizer::~Tokenizer()
{
}// ~Tokenizer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

Tokenizer::TokenType
Tokenizer::Peek() throw(ParseError)
{
    if (mTokenType != NONE)
    {
        LOG(DEBUG, wxT("Peeking again"));
        return mTokenType;                  // EXIT
    }
    if (mWorkingInput.IsEmpty())   return END;     // EXIT

    wxChar      first_char = mWorkingInput[0];     // Pierwszy znak
    TokenType   token_type;                 // Typ tokenu

    bool (*check_function)(wxChar);      // Funkcja sprawdzajca kolejne znaki

    check_function = NULL;

    // Rozpoznaj typ tokenu po pierwszym znaku
    if (wxIsdigit(first_char))       check_function = IsDoubleChar;   // Liczba
    else if (wxIsalpha(first_char))  check_function = IsIdentifierChar;
    else
    {
        switch (first_char)
        {
            case wxT('-'):  token_type = MINUS; break;
            case wxT('+'):  token_type = PLUS; break;
            case wxT('*'):  token_type = MULTIPLY; break;
            case wxT('/'):  token_type = DIVIDE; break;
            case wxT('^'):  token_type = POWER; break;
            case wxT('('):  token_type = OPEN_PAREN; break;
            case wxT(')'):  token_type = CLOSE_PAREN; break;
            default:
                    throw ParseError(mInput, _("Unknown token"), mPosition);
        }
    }

    // Jednoznakowy token, koczymy
    if (check_function == NULL) mTokenLength = 1;
    else
    {
        int index = 1;

        // Znajd koniec cigu tego typu znakw
        while (check_function(mWorkingInput[index])) ++index;

        mTokenLength = index;

        wxString token = mWorkingInput.SubString(0, mTokenLength - 1);


        // Rozpoznaj token
        double value;
        FunctionMap::const_iterator function_position;
        if (wxIsdigit(first_char) && token.ToDouble(&value))
        {
            token_type   = NUMBER;
            mapTokenNode= auto_ptr<FunctionTree>(new FunctionTree(value));
        }
        else if ((function_position = mFunctionMap.find(token))
                != mFunctionMap.end())
        {
            token_type   = FUNCTION;

            // Brak dzieci, argument funkcji jest dodawany w parserze.
            mapTokenNode= auto_ptr<FunctionTree>(
                    new FunctionTree(function_position->second.function,
                        auto_ptr<const FunctionTree>()));

            // Czy ta funkcja matematyczna jest nierniczkowalna?
            if ((mDifferentiable) &&
                    (!function_position->second.differentiable))
            {
                mDifferentiable = false;
            }
        }
        else if (mrVariables.Index(token) != wxNOT_FOUND)
        {
            token_type   = VARIABLE;
            mapTokenNode= auto_ptr<FunctionTree>(new FunctionTree(token));
        }
        else if (mAddVariables)
        {
            mrVariables.AddIfNotPresent(token);
            token_type   = VARIABLE;
            mapTokenNode= auto_ptr<FunctionTree>(new FunctionTree(token));
        }
        else
        {
            throw ParseError(mInput, _("Unknown identifier"), mPosition);
        }
    }
    LOG(DEBUG, wxT("token: '") << mWorkingInput.SubString(0, mTokenLength - 1)
            << wxT('\''));

    mTokenType = token_type;
    return token_type;
}// Peek

auto_ptr<FunctionTree>
Tokenizer::Consume()
{
    assert(mTokenLength > 0);

    // Usu biecy token.
    mWorkingInput.Remove(0, mTokenLength);
    mPosition += mTokenLength;

    // Zapomnij o bieacym tokenie.
    mTokenLength = 0;
    mTokenType = NONE;

    TrimInput(); // Obetnij pocztkowe spacje.

    // Oczywicie zwrcenie auto_ptr'a powoduje jego wyzerowanie.
    return mapTokenNode;
}// Consume

void
Tokenizer::Expect(Tokenizer::TokenType expectedToken) throw(ParseError)
{
    TokenType token = Peek();
    if (token == expectedToken) Consume();
    else throw ParseError(mInput, _("Unexpected token"), mPosition);
}// Expect

//============================= ACCESS     ===================================

const wxString&
Tokenizer::Input() const
{
    return mInput;
}// Input

unsigned
Tokenizer::Position() const
{
    return mPosition;
}// Position

//============================= INQUIRY    ===================================

bool
Tokenizer::AllDifferentiable() const
{
    return mDifferentiable;
}// AllDifferentiable

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
Tokenizer::TrimInput()
{
    size_t length = mWorkingInput.Length();
    mWorkingInput.Trim(false);

    mPosition += length - mWorkingInput.Length();
}// Trim

/* static */ bool
Tokenizer::IsDoubleChar(wxChar c)
{
    if (mDecimalSeparator == '\0') {
        mDecimalSeparator = DiscoverDecimalSeparator();
    }

    return wxIsdigit(c) || (c == mDecimalSeparator);
}// IsDoubleChar

/* static */ bool
Tokenizer::IsIdentifierChar(wxChar c)
{   // Samego wxIsalnum nie mona podstawi pod wskanik w uniwersalny
    // (niezaleny od platformy) sposb.

    return wxIsalnum(c) || c == wxT('_');
}// IsIdentifierChar

// \0 oznacza, e trzeba odkry, patrz IsDoubleChar
wxChar Tokenizer::mDecimalSeparator('\0');
