#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "AlphaDivision.h"  // class implemented
#include "AlphaDivisionIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


AlphaDivision::~AlphaDivision()
{
}// ~AlphaDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
AlphaDivision::Clone() const
{
    return auto_ptr<Method>(new AlphaDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
AlphaDivision::Name() const
{
    return AlphaDivision::mName;
}// Name

/* virtual */ wxString&
AlphaDivision::rName()
{
    return AlphaDivision::mName;
}// Name

/* virtual */ MethodIdType
AlphaDivision::Id() const
{
    return AlphaDivision::ClassId();
}// Id

/* static */ MethodIdType
AlphaDivision::ClassId()
{
    const static MethodIdType AlphaDivisionId = Method::GenerateId();

    return AlphaDivisionId;
}// ClassId

/* virtual */ wxString
AlphaDivision::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    parameter_descriptions.Add(AlphaDescriptionAD(mAlpha));
    parameter_descriptions.Add(GoldenSectionDescription(mGoldenSection));
    AddCommonParameterDescriptions(parameter_descriptions, standalone);

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);

    return description;
}// ToString

//============================= INQUIRY    ===================================

/* virtual */ bool
AlphaDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

AlphaDivision::AlphaDivision()
    : SectionLineSearch(),
      mAlpha(((sqrt(5) - 1) / 2), 0.5, true, 1, true),
      mGoldenSection(true)
{
}// AlphaDivision

AlphaDivision::AlphaDivision(const AlphaDivision& from)
    : SectionLineSearch(from),
      mAlpha(from.mAlpha),
      mGoldenSection(from.mGoldenSection)
{
}// AlphaDivision

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
AlphaDivision::Solve(const UnivariateFunction&  function,
                     double                     a,
                     double                     b,
                     bool                       generateResult)
        const throw(OptimizationError)
{
    double working_a;   // Aktualny pocztek odcinka.
    double working_b;   // Aktualny koniec odcinka.
    double length;      // Aktualna dugo odcinka.

    if (a > b) swap(a, b);
    working_a = a;
    working_b = b;
    length = b - a;

    double lambda;      // Pierwszy punkt prbny.
    double mu;          // Drugi punkt prbny.

    // Wartoci funkcji w punktach prbnych.
    double value_lambda;
    double value_mu;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    auto_ptr<Result> ap_result(new Result(1));

    try
    {
        // Badamy warto funkcji w pierwszych punktach prbnych.
        lambda = a + (1 - mAlpha) * (working_b - working_a);
        mu     = a +      mAlpha  * (working_b - working_a);

        value_lambda = function.Evaluate(lambda);
        value_mu = function.Evaluate(mu);

        LOG(DEBUG, wxT("alpha: " << mAlpha));
        LOG(DEBUG, wxT("a: ") << working_a << wxT(".b: ") << working_b);
        LOG(DEBUG, wxT("lambda: ") << lambda << wxT(".mu: ") << mu);

        if (generateResult)
        {
            ap_result->AddIteration(auto_ptr<const IterationData>
                    (new AlphaDivisionIteration(working_a, working_b,
                                                lambda, mu,
                                                value_mu < value_lambda)));
        }

        while (length >= mEpsilon && step_count < STEP_LIMIT)
        {
            ++step_count;

            LOG(DEBUG, wxT("-----") << endl <<
                       wxT("lambda: ") << lambda <<
                       wxT(". f(lambda): ") << value_lambda << endl <<
                       wxT("mu: ") << mu <<
                       wxT(". f(mu): ") << value_mu);
            if (value_lambda <= value_mu)
            {
                // Skracamy do [a, mu].
                working_b = mu;
                length = working_b - working_a;

                // Dokonujemy podziau nowego odcinka.

                if (mGoldenSection == true)
                { // Za nowy drugi punkt przyjmujemy
                  // dotychczasowy pierwszy.
                  mu = lambda;
                  value_mu = value_lambda;
                  LOG(DEBUG, wxT("Golden section right cut"));
                }
                else
                { // Wyliczamy nowy drugi punkt.
                  mu = working_a + mAlpha * length;
                  value_mu = function.Evaluate(mu);
                }
                // Wyliczamy nowy pierwszy punkt.
                lambda = working_a + (1 - mAlpha) * length;
                value_lambda = function.Evaluate(lambda);
                LOG(DEBUG, wxT("a: ") << working_a <<
                           wxT(". b: ") << working_b <<
                           wxT(". |ab| = ") << length);
            }
            else
            {
                // Skracamy do [lambda, b].
                working_a = lambda;
                length = working_b - working_a;

                // Dokonujemy podziau nowego odcinka.

                if (mGoldenSection == true)
                { // Za nowy pierwszy punkt przyjmujemy
                  // dotychczasowy drugi.
                  lambda = mu;
                  value_lambda = value_mu;
                  LOG(DEBUG, wxT("Golden section left cut"));
                }
                else
                { // Wyliczamy  nowy pierwszy punkt.
                  lambda = working_a + (1 - mAlpha) * length;
                  value_lambda = function.Evaluate(lambda);
                }
                // Wyliczamy nowy drugi punkt.
                mu = working_a + mAlpha * length;
                value_mu = function.Evaluate(mu);
                LOG(DEBUG, wxT("a: ") << working_a <<
                           wxT(". b: ") << working_b <<
                           wxT(". |ab| = ") << length);
            } // if

            if (generateResult && length >= mEpsilon)
            {
                ap_result->AddIteration(auto_ptr<const IterationData>
                        (new AlphaDivisionIteration(working_a, working_b,
                                                    lambda, mu,
                                                    value_mu < value_lambda)));
            }

            NotifyYield();
        } // while
    } // try
    catch (const BaseException& error)
    {   // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }

    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }

    double final_result = (working_a + working_b) / 2;

    ap_result->rFinalResult().resize(1);
    ap_result->rFinalResult()(1) = final_result;

    LOG(DEBUG, wxT("mGoldenSection: ") << mGoldenSection);

    return ap_result;
} // Solve

wxString AlphaDivision::mName(_("Alpha Division"));

// Utwrz i zarejestruj prototyp.
AlphaDivision AlphaDivision::mAlphaDivision;
