/** Metoda alfa-podziau.
 *
 * Metoda optymalizacji w kierunku polegajca na stopniowym
 * skracaniu zadanego odcinka (patrz SectionLineSearch).
 *
 * Skracanie odcinka nastpuje w kadej iteracji, na podstawie badania
 * wartoci funkcji w dwch punktach: \f$\lambda\f$ i \f$\mu\f$.
 * Punkty te rozmieszczane s w taki sposb, aby dzieliy odcinek symetrycznie:
 * \f[ \lambda^k = a^k + (1 - \alpha)  (b^k - a^k), \f]
 * \f[ \mu^k = a^k + \alpha  (b^k - a^k), \f]
 * gdzie:<br>
 * \f$a^k, b^k\f$ - koce zadanego odcinka,<br>
 * \f$\alpha\f$ - wspczynnik podziau odcinka zadany przez uytkownika,
 * \f$\alpha \in (0,5; 1).\f$<br>
 * Dziki takiemu podziaowi w kadej iteracji zachodzi:
 * \f[ \lambda^k - a^k = b^k - \mu^k = (1 - \alpha) (b^k - a^k) . \f]
 * Redukcja odcinka polega na odrzuceniu jednego z jego skrajnych fragmentw:
 * \f$[a^k, \lambda^k]\f$ lub \f$[\mu^k, b^k]\f$, w zalenoci od wartoci
 * funkcji w punktach prbnych. W kadej iteracji odcinek jest wic skracany
 * o dugo:
 * \f[ (1 - \alpha) (b^k - a^k) .\f]
 * W przypadku gdy metoda dziaa samodzielnie, wynikiem jest odcinek
 * \f$[a^k, b^k]\f$; jeli suy tylko do minimalizacji kierunkowej
 * przy rozwizywaniu problemu optymalizacji funkcji wielu zmiennych,
 * wynikiem jest punkt - rodek tego odcinka: \f$\frac{a^k + b^k}{2}\f$.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f(x)\f$ - minimalizowana funkcja jednej zmiennej,<br>
 * \f$a^0\f$ - lewy kraniec pocztkowego przedziau poszukiwa,<br>
 * \f$b^0\f$ - prawy kraniec pocztkowego przedziau poszukiwa,<br>
 * \f$\alpha\f$ - wspczynnik podziau odcinka, \f$\alpha \in (0,5; 1),\f$<br>
 * \f$\varepsilon\f$ - wymagana dokadno (graniczna dugo odcinka).
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$a^k\f$ - lewy kraniec aktualnego przedziau poszukiwa,<br>
 * \f$b^k\f$ - prawy kraniec aktualnego przedziau poszukiwa,<br>
 * \f$\lambda^k\f$ - punkt prbny znajdujcy si po lewej stronie,<br>
 * \f$\mu^k\f$ - punkt prbny znajdujcy si po prawej stronie,<br>
 * \f$F_{\lambda^k}\f$ - warto funkcji w punkcie \f$\lambda^k\f$,<br>
 * \f$F_{\mu^k}\f$ - warto funkcji w punkcie \f$\mu^k\f$.<br>
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ a^1 = a^0, \f]
 * \f[ b^1 = b^0, \f]
 * \f[ \lambda^1 = a^1 + (1 - \alpha)  (b^1 - a^1), \f]
 * \f[ \mu^1 = a^1 + \alpha  (b^1 - a^1), \f]
 * \f[ F_{\lambda^1} = f(\lambda^1), \f]
 * \f[ F_{\mu^1} = f(\mu^1), \f]
 * \f[ k = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Jeli \f$F_{\lambda^k} \le F_{\mu^k}\f$, to przechodzimy do kroku
 * 2. W przeciwnym razie przechodzimy do kroku 3.
 *
 * <br><b>Krok 2:</b><br>
 * Redukujemy odcinek do \f$[a^k, \mu^k]\f$:
 * \f[ a^{k+1} = a^k, \f]
 * \f[ b^{k+1} = \mu^k \f]
 * i dokonujemy podziau otrzymanego odcinka:
 * \f[ \lambda^{k+1} = a^{k+1} + (1 - \alpha) (b^{k+1} - a^{k+1}), \f]
 * \f[ \mu^{k+1} =  a^{k+1} + \alpha (b^{k+1} - a^{k+1}) , \f]
 * \f[ F_{\lambda^{k+1}} = f(\lambda^{k+1}), \f]
 * \f[ F_{\mu^{k+1}} = f(\mu^{k+1}). \f]
 * Przechodzimy do kroku 4.
 *
 * <br><b>Krok 3:</b><br>
 * Redukujemy odcinek do \f$[\lambda^k, b^k]\f$:
 * \f[ a^{k+1} = \lambda^k, \f]
 * \f[ b^{k+1} = b^k \f]
 * i dokonujemy podziau otrzymanego odcinka:
 * \f[ \lambda^{k+1} = a^{k+1} + (1 - \alpha) (b^{k+1} - a^{k+1}), \f]
 * \f[ \mu^{k+1} =  a^{k+1} + \alpha (b^{k+1} - a^{k+1}) , \f]
 * \f[ F_{\lambda^{k+1}} = f(\lambda^{k+1}), \f]
 * \f[ F_{\mu^{k+1}} = f(\mu^{k+1}). \f]
 * Przechodzimy do kroku 4.
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy warunek stopu: <br>
 * Jeli \f$ (b^{k+1} - a^{k+1}) < \varepsilon \f$,
 * to STOP. W przeciwnym razie podstawiamy \f$k = k + 1\f$ i przechodzimy
 * do kroku 1.
 *
 * <hr>
 *
 * Szczeglnym, najczciej stosowanym przypadkiem metody jest algorytm
 * z parametrem \f$ \alpha = \frac{\sqrt{5} - 1}{2} \approx 0,618 \f$.
 * Dla tej wartoci parametru odcinek \f$[a, b]\f$ jest w kadej
 * iteracji dzielony przez punkty prbne na pododcinki speniajce
 * warunek "zotego podziau":
 * \f[ \frac{\mu^k - a^k}{b^k - a^k} = \frac{b^k - \mu^k}{\mu^k - a^k} =
 *  \alpha, \f]
 * \f[ \frac{b^k - \lambda^k}{b^k - a^k} = \frac{\lambda^k - a^k}
 * {b^k - \lambda^k} = \alpha, \f]
 * Std:
 * \f[ \frac{\lambda^k - a^k}{\mu^k - a^k} = \alpha . \f]
 * Dziki ostatniej wasnoci procedura podziaw odcinka upraszcza si:
 * w kadej iteracji wyznaczamy tylko jeden nowy punkt prbny i, co
 * za tym idzie, tylko raz obliczamy warto funkcji.
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ a^1 = a^0, \f]
 * \f[ b^1 = b^0, \f]
 * \f[ \lambda^1 = a^1 + (1 - \alpha)  (b^1 - a^1), \f]
 * \f[ \mu^1 = a^1 + \alpha  (b^1 - a^1), \f]
 * \f[ F_{\lambda^1} = f(\lambda^1), \f]
 * \f[ F_{\mu^1} = f(\mu^1), \f]
 * \f[ k = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Jeli \f$F_{\lambda^k} < F_{\mu^k}\f$, to przechodzimy do kroku
 * 2. W przeciwnym razie przechodzimy do kroku 3.
 *
 * <br><b>Krok 2:</b><br>
 * Redukujemy odcinek do \f$[a^k, \mu^k]\f$:
 * \f[ a^{k+1} = a^k, \f]
 * \f[ b^{k+1} = \mu^k \f]
 * i dokonujemy podziau otrzymanego odcinka:
 * \f[ \lambda^{k+1} = a^{k+1} + (1 - \alpha) (b^{k+1} - a^{k+1}), \f]
 * \f[ \mu^{k+1} =  \lambda^k, \f]
 * \f[ F_{\lambda^{k+1}} = f(\lambda^{k+1}), \f]
 * \f[ F_{\mu^{k+1}} = F_{\lambda^k}. \f]
 * Przechodzimy do kroku 4.
 *
 * <br><b>Krok 3:</b><br>
 * Redukujemy odcinek do \f$[\lambda^k, b^k]\f$:
 * \f[ a^{k+1} = \lambda^k, \f]
 * \f[ b^{k+1} = b^k \f]
 * i dokonujemy podziau otrzymanego odcinka:
 * \f[ \lambda^{k+1} = \mu^k, \f]
 * \f[ \mu^{k+1} =  a^{k+1} + \alpha (b^{k+1} - a^{k+1}) , \f]
 * \f[ F_{\lambda^{k+1}} = F_{\mu^k}, \f]
 * \f[ F_{\mu^{k+1}} = f(\mu^{k+1}). \f]
 * Przechodzimy do kroku 4.
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy warunek stopu: <br>
 * Jeli \f$ (b^{k+1} - a^{k+1}) < \varepsilon \f$,
 * to STOP. W przeciwnym razie podstawiamy \f$k = k + 1\f$ i przechodzimy
 * do kroku 1.
 *
 * Taki wariant metody nazywany jest popularnie <i>metod zotego podziau</i>.
 *
 * @see SectionLineSearch
 * @see MethodWithLineSearch
 */

#ifndef _AlphaDivision_h
#define _AlphaDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"

// FORWARD REFERENCES
//


class AlphaDivision
    : public SectionLineSearch
{
public:
    friend class AlphaDivisionPanel;

// LIFECYCLE


    /** Destruktor.
     */
    ~AlphaDivision(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    AlphaDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    AlphaDivision(const AlphaDivision& from);


    /** Wspczynnik podziau odcinka (\f$\alpha \in (0,5; 1)\f$).
     */
    DoubleParameter mAlpha;

    /** Informacja, czy metoda realizuje wariant "zotego podziau".
     */
    bool            mGoldenSection;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);


    static wxString      mName;             ///< Nazwa metody.
    static AlphaDivision mAlphaDivision;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _AlphaDivision_h_
