#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "BolzanoDivision.h"  // class implemented
#include "BolzanoDivisionIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


BolzanoDivision::~BolzanoDivision()
{
}// ~BolzanoDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
BolzanoDivision::Clone() const
{
    return auto_ptr<Method>(new BolzanoDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
BolzanoDivision::Name() const
{
    return BolzanoDivision::mName;
}// Name

/* virtual */ wxString&
BolzanoDivision::rName()
{
    return BolzanoDivision::mName;
}// Name

/* virtual */ MethodIdType
BolzanoDivision::Id() const
{
    return BolzanoDivision::ClassId();
}// Id

/* static */ MethodIdType
BolzanoDivision::ClassId()
{
    const static MethodIdType BolzanoDivisionId = Method::GenerateId();

    return BolzanoDivisionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
BolzanoDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL &&
            pProblem->Function().IsDifferentiable());
}

/* virtual */ bool
BolzanoDivision::CanAssist(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem *>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable());
}// CanAssist

/////////////////////////////// PROTECTED  ///////////////////////////////////

BolzanoDivision::BolzanoDivision()
    : SectionLineSearch()
{
}// BolzanoDivision

BolzanoDivision::BolzanoDivision(const BolzanoDivision& from)
    : SectionLineSearch(from)
{
}// BolzanoDivision

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
BolzanoDivision::Solve(const UnivariateFunction&  function,
                       double                     a,
                       double                     b,
                       bool                       generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result>                    ap_result(new Result(1));
    auto_ptr<BolzanoDivisionIteration>  ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        if (a > b) swap(a, b);

        ColumnVector    left(1);                // Lewy kraniec przedziau
        ColumnVector    middle(1);              // Aktualny rodek odcinka
        ColumnVector    right(1);               // Prawy kraniec przedziau
        double          derivative_in_middle;   // warto pochodnej w middle


        left    = a;
        right   = b;
        middle  = (a + b) / 2;

        derivative_in_middle    = function.Gradient(middle)(1);

        ap_iteration.reset(new BolzanoDivisionIteration
                                            (left(1), middle(1), right(1)));

        if ((right(1) - left(1)) <= mEpsilon ||
            fabs(derivative_in_middle) <= mEpsilon)
                ap_iteration->rHalfChosen() = BolzanoDivisionIteration::NONE;


        LOG(DEBUG,wxT("Krok nr ") << step_count);

        LOG(DEBUG,wxString::Format(wxT("Left   : %6g, derivative: %6g"),
                                    left(1),function.Gradient(left)(1)));
        LOG(DEBUG,wxString::Format(wxT("Middle : %6g, derivative: %6g"),
                                    middle(1),derivative_in_middle));
        LOG(DEBUG,wxString::Format(wxT("Right  : %6g, derivative: %6g"),
                                    right(1),function.Gradient(right)(1)));

        LOG(DEBUG,wxT("-----------------------------------"));

/*
        // TODO: To tu sprawdzamy czy mamy w odcinku bbel.
        if ((function.Gradient(left)(1) > 0 &&
                    function.Gradient(right)(1) < 0))
        {
            throw OptimizationError(
                    _("The initial section contains a maximum."), ap_result);
        }
*/

        while ((right(1) - left(1)) >= mEpsilon &&
               fabs(derivative_in_middle) >= mEpsilon &&
               step_count < STEP_LIMIT)
        {
            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count);


            if (derivative_in_middle < 0)
            {
                left    = middle;

                ap_iteration->rHalfChosen() = BolzanoDivisionIteration::RIGHT;

                LOG(DEBUG, wxT("RIGHT"));
            }
            else // derivative_in_middle > 0 (=0 w warunku ptli)
            {
                right   = middle;

                ap_iteration->rHalfChosen() = BolzanoDivisionIteration::LEFT;

                LOG(DEBUG, wxT("LEFT"));
            }

            middle                  = (left + right) / 2;
            derivative_in_middle    = function.Gradient(middle)(1);

            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));

            ap_iteration.reset(new BolzanoDivisionIteration
                                            (left(1), middle(1), right(1)));

            if ((right(1) - left(1)) <= mEpsilon ||
                fabs(derivative_in_middle) <= mEpsilon)
                   ap_iteration->rHalfChosen() = BolzanoDivisionIteration::NONE;


            LOG(DEBUG,wxString::Format(
                            wxT("Left   : %10.5g, derivative: %10.5g"),
                            left(1),function.Gradient(left)(1)));
            LOG(DEBUG,wxString::Format(
                            wxT("Middle : %10.5g, derivative: %10.5g"),
                            middle(1), derivative_in_middle));
            LOG(DEBUG,wxString::Format(
                            wxT("Right  : %10.5g, derivative: %10.5g"),
                            right(1),function.Gradient(right)(1)));

            LOG(DEBUG,wxT("-----------------------------------"));


            NotifyYield();
        } // while

        ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));

        ap_result->rFinalResult() = middle;

    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve


wxString BolzanoDivision::mName(_("Bolzano Division"));

// Utwrz i zarejestruj prototyp.
BolzanoDivision BolzanoDivision::mBolzanoDivision;
