/** Metoda podziau Bolzano.
 *
 * Metoda BolzanoDivision (inaczej: metoda punktu rodkowego) dziaa przy wykorzystaniu
 * twierdzenia BolzanoDivision-Cauchy'ego, ktre brzmi:
 * Niech funkcja \f$ \phi(x) \f$ bdzie okrelona i ciga w przedziale
 * domknitym \f$ <a,b> \f$ i niech na kocach tego przedziau przyjmuje
 * wartoci rnych znakw. Wwczas pomidzy \f$ a \f$ i \f$ b \f$ znajduje si
 * punkt c, w ktrym funkcja rwna si zeru:
 * \f[ \phi(c)=0, a<c<b \f]
 * <br>
 * W naszym przypadku funkcj \f$ \phi(x) \f$ jest pochodna funkcji celu
 * \f$ f'(x) \f$.
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ a \f$ - lewy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ b \f$ - prawy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ x_m \f$ - punkt w poowie przedziau nieoznaczonoci w danej iteracji<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ a^0 \f$ - lewy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ b^0 \f$ - prawy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 * <br>
 *
 * <b>Krok 1</b>:<br>
 * Obliczy \f$ f'(a^0) \f$ i \f$ f'(b^0) \f$.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Jeli \f$ f'(a^0)>0 \f$, a \f$ f'(b^0)<0 \f$, zakocz dziaanie algorytmu
 * stwierdzajc, e punkt stacjonarny nie znajduje si w przedziale
 * pocztkowym.<br>
 * W przeciwnym wypadku przej do kroku 3.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Oblicz: \f$ x_m=(a+b)/2 \f$ oraz pochodn w tym punkcie \f$ f'(x_m) \f$.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Jeli \f$ f'(x_m)<\varepsilon \f$, zakocz dziaanie algorytmu. Wynikiem jest
 * punkt \f$ x_m \f$.<br>
 * <br>
 * <b>Krok 5a</b>:<br>
 * Jeli \f$ f'(x_m)<0 \f$, podstaw \f$ a=x_m \f$ i przejd do kroku 3.<br>
 * W przeciwnym wypadku przejd do kroku 5b.<br>
 * <br>
 * <b>Krok 5b</b>:<br>
 * Podstaw \f$ b=x_m \f$ i przejd do kroku 3.<br>
 * <br>
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 *
 * @see Linesearch
 **/

#ifndef _BolzanoDivision_h
#define _BolzanoDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"
#include "../ui/SectionLineSearchPanel.h"

// FORWARD REFERENCES
//


class BolzanoDivision
    : public SectionLineSearch
{
public:
    friend class SectionLineSearchPanel<BolzanoDivision>;

// LIFECYCLE


    /** Destruktor.
     */
    ~BolzanoDivision(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual bool                    CanAssist(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    BolzanoDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    BolzanoDivision(const BolzanoDivision& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);


    static wxString      mName;                 ///< Nazwa metody.
    static BolzanoDivision mBolzanoDivision;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _BolzanoDivision_h_
