#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "../text-util.h"
#include "../util.h"

#include "DFP.h"  // class implemented
#include "DFPIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


DFP::~DFP()
{
}// ~DFP


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
DFP::Clone() const
{
    return auto_ptr<Method>(new DFP(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
DFP::Name() const
{
    return DFP::mName;
}// Name

/* virtual */ wxString&
DFP::rName()
{
    return DFP::mName;
}// rName

/* virtual */ MethodIdType
DFP::Id() const
{
    return DFP::ClassId();
}// Id

/* static */ MethodIdType
DFP::ClassId()
{
    const static MethodIdType DFPId = Method::GenerateId();

    return DFPId;
}// ClassId

/* virtual */ wxString
DFP::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << _("Reset direction after N iterations: ");
    description << (mModification ? _("Yes") : _("No"));
    description << wxT("\n\n");
    description << LineSearchDescription(mcpLineSearch->ToString(false));
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}

//============================= INQUIRY    ===================================

/* virtual */ bool
DFP::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL &&
            pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}// CanHandle

/* virtual */ StopConditions
DFP::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    conditions.at(0)->rName() = wxT("|| d ||");
    conditions.at(1)->rName() = wxT("|| d ||^2");

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

DFP::DFP()
    : MethodWithLineSearch(),
      mModification(true)
{
}// DFP

DFP::DFP(const DFP& from)
    : MethodWithLineSearch(from),
      mModification(from.mModification)
{
}// DFP

SquareMatrix
DFP::CalculateNewD(const SquareMatrix& D,
                   const ColumnVector& x_diff,
                   const ColumnVector& grad_diff) const
{
    SquareMatrix new_D;
    SquareMatrix numerator;
    double denominator;

    numerator = x_diff * x_diff.t();
    denominator = (x_diff.t() * grad_diff).as_scalar();
    new_D = D + numerator / denominator;

    numerator = D * grad_diff * grad_diff.t() * D;
    denominator = (grad_diff.t() * D * grad_diff).as_scalar();
    new_D -= numerator / denominator;

    return new_D;
}// CalculateNewD

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
DFP::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    // Funkcja celu.
    const FunctionBase& function = problem.Function();
    // Aktualny punkt.
    ColumnVector x(mStartingPoint);
    // Poprzedni punkt (inicjowany wektorem zerowym, aby moliwe byo
    // sprawdzanie przed iteracj wszystkich warunkw stopu).
    ColumnVector x_prev(NullPoint);
    // Odlego midzy aktualnym i poprzednim punktem.
    ColumnVector x_diff;

    // Gradient w aktualnej iteracji.
    ColumnVector grad;
    // Gradient w poprzedniej iteracji.
    ColumnVector grad_prev;
    // Rnica midzy aktualnym a poprzednim gradientem.
    ColumnVector grad_diff;

    // Macierz przybliajca macierz odwrotn
    // do hesjanu.
    SquareMatrix D;

    // Aktualny kierunek poszukiwa.
    ColumnVector d;
    // Optymalna dugo kroku w aktualnym kierunku.
    double tau;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;

    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);
    assert(function.IsDifferentiable());

    try
    {
        // Tworzymy pierwsz macierz przybliajc -
        // zaczynamy od macierzy jednostkowej.
        D = IdentityMatrix(x.nrows());

        // Obliczamy gradient w punkcie pocztkowym.
        grad = function.Gradient(x);
        // Wyznaczamy pierwszy kierunek poszukiwa.
        d = -grad;

        auto_ptr<DFPIteration> ap_iteration(new DFPIteration(
                    x, grad, d, D));

        while (!mcpStopCondition->IsMet(x_prev, x, function, d.norm_Frobenius())
               && step_count < STEP_LIMIT)

        {
            ++step_count;
            LOG(DEBUG, wxT("---step ") << step_count);

            x_prev = x;
            grad_prev = grad;

            // Wykonujemy optymalny krok w wyznaczonym kierunku.
            x = StepInDirection(function, x_prev, d, &tau);

            ap_iteration->rTau()      = tau;
            ap_iteration->rEndPoint() = x;
            LOG(DEBUG, wxT("x:") << endl << x);

            // Obliczamy gradient w aktualnym punkcie.
            grad = function.Gradient(x);

            ap_iteration->rGradientInEndPoint() = grad;
            LOG(DEBUG, wxT("grad:") << endl << grad);

            // Aby zapobiec utracie przez macierz D wasnoci
            // dodatniej okrelonoci na skutek niedokadnoci
            // oblicze, od czasu do czasu "zapominamy"
            // o dotychczasowych postpach i ponownie startujemy
            // od macierzy jednostkowej.
            if (step_count % x.nrows() == 0 && mModification)
            { // Wracamy do macierzy jednostkowej.
                D = IdentityMatrix(x.nrows());
            }
            else
            { // Wyznaczamy kolejne przyblienie hesjanu.
                x_diff = x - x_prev;
                grad_diff = grad - grad_prev;

                ap_iteration->rPointDifference()    = x_diff;
                ap_iteration->rGradientDifference() = grad_diff;

                if (x_diff.norm_Frobenius()    < 0.0000001 ||
                    grad_diff.norm_Frobenius() < 0.0000001)
                { // W przypadku braku przesunicia punktu w aktualnej
                  // iteracji nie da si obliczy nowej macierzy D -
                  // mielibymy dzielenie przez 0.
                    ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_iteration));
                    throw Error(_("Cannot calculate new D matrix - no step was made in the current iteration.\n\
Please change the stop condition or choose a more exact line search method."));
                }
                D = CalculateNewD(D, x_diff, grad_diff);
            }
            ap_iteration->rNewD() = D;
            LOG(DEBUG, wxT("D:") << endl << D);

            // Wyznaczamy kolejny kierunek.
            d = -D * grad;

            ap_iteration->rNewDirection() = d;
            LOG(DEBUG, wxT("d:") << endl << d);

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_iteration));

            ap_iteration.reset(new DFPIteration(x, grad, d, D));

            NotifyYield();
        } // while

        ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_iteration));
    } // try
    catch (BaseException)
    { // Wyjtek newmata.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x;
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString DFP::mName(_("Davidon-Fletcher-Powell's"));

// Utwrz i zarejestruj prototyp.
DFP DFP::mDFP;
