/**  Metoda Davidona-Fletchera-Powella.
 * Metoda zmiennej metryki.<br>
 * Metoda z rodziny algorytmw quasi-Newtonowskich, opartych na tej
 * samej wasnoci i sposobie postpowania, co zmodyfikowana metoda
 * Newtona: dla funkcji celu o postaci
 * \f[ f(x) = q + b^T x + \frac{1}{2} x^T A x \f]
 * wystarczy znajomo macierzy odwrotnej do hesjanu \f$A^{-1}\f$,
 * aby w jednym kroku wyznaczy minimum. Poszukiwanie przebiega wtedy
 * zgodnie z prost procedur:
 * \f[ x^{k+1} = x^k - \tau A^{-1} \nabla f(x^k). \f]
 * Metody quasi-Newtonowskie zastpuj macierz \f$A^{-1}\f$,
 * ktrej otrzymanie jest kosztowne ze wzgldu na konieczno wyliczania
 * drugich pochodnych, macierz estymujc \f$D\f$. Staje si ona w
 * kolejnych iteracjach coraz lepszym przyblieniem macierzy odwrotnej
 * do hesjanu. Pierwszym, ale efektywnym pomysem na generowanie takiej
 * macierzy by wzr Davidona-Fletchera-Powella:
 * \f[ D^{k+1} = D^k + \frac{p^k (p^k)^T}{(p^k)^T q^k} -
 * \frac{D^k q^k (q^k)^T D^k}{(q^k)^T D^k q^k}, \f]
 * gdzie:
 * \f[ p^k = x^{k+1} - x^k, \f]
 * \f[ q^k = \nabla f(x^{k+1}) - \nabla f(x^k). \f]
 * Gdy przyblienie uznaje si za zadowalajce, procedura
 * koczy si.
 *
 * <b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x_0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\varepsilon\f$ - wymagana dokadno oblicze,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|d\| < \varepsilon\f$</li>
 * <li>\f$\|d\|^2 < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami,<br>
 * informacja, czy realizowana bdzie metoda z modyfikacj.<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$d^k\f$ - aktualny kierunek poszukiwa,<br>
 * \f$x^k\f$ - aktualny punkt,<br>
 * \f$g^k\f$ - gradient funkcji w punkcie \f$x^k\f$,<br>
 * \f$\tau\f$ - dugo wykonywanego aktualnie kroku,<br>
 * \f$D^k\f$ - aktualna macierz estymujca m. odwrotn do hesjanu.<br>
 *
 * <br><b>Procedura.</b><br>
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ D^1 = I(n), \mbox{ gdzie } I(n) - \mbox{ macierz jednostkowa }
 * n \times n , \f]
 * \f[ g^0 = \nabla f(x^0), \f]
 * \f[ g^1 = g^0, \f]
 * \f[ d^1 = -g^0, \f]
 * \f[ x^1 = x^0, \f]
 * \f[ k = 1. \f]
 *
 * <br><b>Krok 1:</b><br>
 * Sprawdzamy warunek stopu.<br>
 * Jeli jest on speniony, to STOP. Wynikiem jest \f$x^k\f$.<br>
 * W przeciwnym wypadku przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Wykonujemy krok o optymalnej dugoci \f$\tau\f$ w aktualnym kierunku:
 * \f[ \tau = \arg \min_{\tau} f(x^k + \tau d^k) , \f]
 * \f[ x^{k+1} = x^k + \tau d^k . \f]
 * Obliczamy gradient w otrzymanym punkcie:
 * \f[ g^{k+1} = \nabla f(x^{k+1}) . \f]
 * Przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Jeli realizowana jest wersja z modyfikacj, sprawdzamy,
 * czy \f$k = n, \: 2n, \: 3n, \: \ldots\f$. Jeli tak,
 * podstawiamy:
 * \f[ D^{k+1} = I(n) \f]
 * i przechodzimy do kroku 5.
 *
 * W przeciwnym razie przechodzimy do kroku 4.
 *
 * <br><b>Krok 4:</b><br>
 * Wyliczamy \f$D^{k+1}\f$ jako kolejne przyblienie macierzy
 * odwrotnej do hesjanu, zgodnie ze wzorem Davidona-Fletchera-Powella:
 * \f[ D^{k+1} = D^k + \frac{p^k (p^k)^T}{(p^k)^T q^k} -
 * \frac{D^k q^k (q^k)^T D^k}{(q^k)^T D^k q^k}, \f]
 * gdzie:
 * \f[ p^k = x^{k+1} - x^k, \f]
 * \f[ q^k = g^{k+1} - g^k. \f]
 *
 * <br><b>Krok 5:</b><br>
 * Wyznaczamy kolejny kierunek:
 * \f[ d^{k+1} = -D^{k+1} g^{k+1} . \f]
 * Podstawiamy \f$ k = k + 1 \f$ i przechodzimy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Metody zmiennej metryki nale do najwydajniejszych algorytmw
 * optymalizacji bez ogranicze. Ze wzgldu jednak na konieczno
 * stopniowego estymowania macierzy odwrotnej do hesjanu, w niektrych
 * wypadkach mog by one wolniej zbiene od zmodyfikowanej metody Newtona
 * (w zamian za wiksz liczb krokw potrzebnych do znalezienia minimum
 * otrzymamy jednak znacznie mniejsz liczb wylicze funkcji celu).
 *
 * Regularne ponowne inicjowanie macierzy \f$D\f$ macierz jednostkow
 * (krok 3)
 * wprowadzono w celu zwikszenia stabilnoci metody. Gdyby aktualizowa
 * macierz zgodnie ze wzorem Davidona-Fletchera-Powella w kadej iteracji,
 * mogaby ona straci, na skutek niedokadnych oblicze i niedokadnoci
 * metody minimalizacji w kierunku, dodatni okrelono, a to mogoby
 * doprowadzi do denia algorytmu w kierunku maksimum. Cen za popraw
 * stabilnoci w taki sposb jest oczywicie mniejsza szybko zbienoci
 * metody.
 *
 * Metoda zaimplementowana na podstawie opisu zamieszczonego w:<br>
 * Findeisen W., Szymanowski J., Wierzbicki A.: <i>Teoria i metody
 * obliczeniowe optymalizacji</i>. Warszawa, PWN 1977.
 *
 * @see MethodWithLineSearch
 * @see NewtonModified
 */

#ifndef _DFP_h
#define _DFP_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CustomValueCondition.h"
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../MethodWithLineSearch.h"
#include "../SquaredCustomValueCondition.h"
#include "../ui/VariantedMethodWithLineSearchPanel.h"

// FORWARD REFERENCES
//


class DFP
    : public MethodWithLineSearch
{
public:
// LIFECYCLE

    /** Panel ustawie metody.
     */
    friend class MethodWithLineSearchPanel<DFP>;
    friend class VariantedMethodWithLineSearchPanel<DFP>;

    /** Destruktor.
     */
    ~DFP(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;


    virtual StopConditions          AllowedStopConditions() const;

protected:
    virtual wxString&               rName();

    /** Konstruktor domylny.
     */
    DFP(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    DFP(const DFP& from);

    /** Wyznacza zaktualizowan macierz D.
     *
     * Wyznacza now, blisz macierzy odwrotnej do hesjanu,
     * macierz \f$D\f$, wedug wzoru Davidona-Fletchera-Powella:
     * \f[ D_{k+1} = D_k + \frac{p_k p_k^T}{p_k^T q_k} -
     * \frac{D_k q_k q_k^T D_k}{q_k^T D_k q_k}, \f]
     * gdzie:
     * \f[ p_k = x_{k+1} - x_k, \f]
     * \f[ q_k = \nabla f(x_{k+1}) - \nabla f(x_k). \f]
     *
     * @param D Obecna macierz przybliajca (\f$D_k\f$).
     * @param x_diff Wektor rnicy punktw (\f$p_k\f$).
     * @param grad_diff Wektor rnicy gradientw (\f$q_k\f$).
     *
     * @return Zaktualizowana macierz prybliajca (\f$D_{k+1}\f$).
     */
    SquareMatrix    CalculateNewD(const SquareMatrix& D,
                                  const ColumnVector& x_diff,
                                  const ColumnVector& grad_diff) const;

    /** Czy do algorytmu ma zosta wprowadzone usprawninie (szczegy w opisie
     * metody).
     */
    bool                            mModification;

private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);
    static wxString      mName; ///< Nazwa metody.
    static DFP mDFP;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DFP_h_
