#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "../text-util.h"
#include "Derivative.h"  // class implemented
#include "DerivativeIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


Derivative::~Derivative()
{
}// ~Derivative


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
Derivative::Clone() const
{
    return auto_ptr<Method>(new Derivative(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
Derivative::Name() const
{
    return Derivative::mName;
}// Name

/* virtual */ wxString&
Derivative::rName()
{
    return Derivative::mName;
}// rName

/* virtual */ MethodIdType
Derivative::Id() const
{
    return Derivative::ClassId();
}// Id

/* static */ MethodIdType
Derivative::ClassId()
{
    const static MethodIdType DerivativeId = Method::GenerateId();

    return DerivativeId;
}// ClassId

/* virtual */ wxString
Derivative::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    parameter_descriptions.Add(AlphaDescriptionDerivative(mAlpha));
    parameter_descriptions.Add(GammaDescriptionDerivative(mGamma));
    AddCommonParameterDescriptions(parameter_descriptions, standalone);

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
Derivative::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>
                        (&(pProblem->Function())) != NULL &&
            pProblem->Function().IsDifferentiable());
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

Derivative::Derivative()
    : SinglePointLineSearch(),
      mAlpha(1.0, 0, true),
      mGamma(0.5, 0, true, 1, true)
{
}// Derivative

Derivative::Derivative(const Derivative& from)
    : SinglePointLineSearch(from),
      mAlpha(from.mAlpha),
      mGamma(from.mGamma)
{
}// Derivative

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
Derivative::Solve(const UnivariateFunction& function,
                  const ColumnVector&       startingPoint,
                  bool                      generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result>                ap_result(new Result(1));
    auto_ptr<DerivativeIteration>   ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        ColumnVector    x(1);                   // Punkt w danej iteracji
        ColumnVector    x_candidate(1);         // Punkt z poprzedniej iteracji
        double          value_in_x;             // Warto funkcji w punkcie
        double          value_in_x_candidate;
        double          derivative_in_x;        // Pierwsza pochodna w punkcie x
        double          derivative_in_candidate;
        double          alpha = mAlpha;

        x               = startingPoint;
        value_in_x      = function.Evaluate(x);
        derivative_in_x = function.Gradient(x)(1);


        LOG(DEBUG,wxT("-----------------------------------\n"));

        do
        {
            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count << endl);

            LOG(DEBUG,wxString::Format(wxT("point           : %10.6g"),
                                            x(1)));
            LOG(DEBUG,wxString::Format(wxT("value           : %10.6g"),
                                            value_in_x));
            LOG(DEBUG,wxString::Format(wxT("derivative      : %10.6g\n"),
                                            derivative_in_x));

            /* To wane podstawienie, bo nagle moe si okaza, ze wykonujemy
             * mikroskopijne kroki (np. gdy punkt startowy by daleko od minimum
             * i warto pierwszej pochodnej w nim bardzo dua).
             */
            alpha = mAlpha;

            ap_iteration.reset(new DerivativeIteration(x(1), alpha, mGamma));

            x_candidate             = x - alpha * derivative_in_x;
            value_in_x_candidate    = function.Evaluate(x_candidate);
            derivative_in_candidate = function.Gradient(x_candidate)(1);

            LOG(DEBUG, wxT("candidate     = ") << x_candidate(1));
            LOG(DEBUG, wxT("f(candidate)  = ") << value_in_x_candidate);
            LOG(DEBUG, wxT("f'(candidate) = ") << derivative_in_candidate);

            ap_iteration->rTrialPoints().push_back(x_candidate(1));

            /* W poniszej ptli radzimy skracamy krok w przyapdku gdy
             * przeskakujemy minimum z powodu jego nadmiernej dugoci.
             * Warunki stopu takie jak w duej ptli potrzebne s by
             * zmniejszanie wspczynnika alfa nie trwao w nieskoczono w
             * przypadku gdy algorytm wykona krok wikszy od epsilon i
             * wyldowa na tyle blisko minimum, ze poprawa nie jest moliwa w
             * warunkach ograniczonej dokadnoci.
             */
            while (fabs(derivative_in_candidate) >= mEpsilon &&
                   (value_in_x_candidate >= value_in_x ||
                    fabs(value_in_x_candidate - value_in_x) < 0.000000000001))
            {
                alpha   *= mGamma;

                LOG(DEBUG,wxT("reduction of alpha\nnew alpha   : ") << alpha);

                x_candidate             = x - alpha * derivative_in_x;
                value_in_x_candidate    = function.Evaluate(x_candidate);
                derivative_in_candidate = function.Gradient(x_candidate)(1);

                LOG(DEBUG, wxT("candidate     = ") << x_candidate(1));
                LOG(DEBUG, wxT("f(candidate)  = ") << value_in_x_candidate);
                LOG(DEBUG, wxT("f'(candidate) = ") << derivative_in_candidate);

                ap_iteration->rTrialPoints().push_back(x_candidate(1));

                NotifyYield();
            }

            x               = x_candidate;
            value_in_x      = value_in_x_candidate;
            derivative_in_x = derivative_in_candidate;

            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));

            LOG(DEBUG,wxT("-----------------------------------\n"));

            NotifyYield();
        }
        while (fabs(derivative_in_x) >= mEpsilon &&
               step_count < STEP_LIMIT);

        ap_result->rFinalResult() = x;

    } // try
    catch (BaseException)
    { // Wyjtek newmata.
      // TODO:  Tutaj raczej niepotrzebny, ale to i tak si zmieni.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve

wxString Derivative::mName(_("Derivative"));

// Utwrz i zarejestruj prototyp.
Derivative Derivative::mDerivative;
