#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "DichotomousDivision.h"  // class implemented
#include "DichotomousDivisionIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


DichotomousDivision::~DichotomousDivision()
{
}// ~DichotomousDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
DichotomousDivision::Clone() const
{
    return auto_ptr<Method>(new DichotomousDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
DichotomousDivision::Name() const
{
    return DichotomousDivision::mName;
}// Name

/* virtual */ wxString&
DichotomousDivision::rName()
{
    return DichotomousDivision::mName;
}// rName

/* virtual */ MethodIdType
DichotomousDivision::Id() const
{
    return DichotomousDivision::ClassId();
}// Id

/* static */ MethodIdType
DichotomousDivision::ClassId()
{
    const static MethodIdType DichotomousDivisionId = Method::GenerateId();

    return DichotomousDivisionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
DichotomousDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

DichotomousDivision::DichotomousDivision()
    : SectionLineSearch()
{
}// DichotomousDivision

DichotomousDivision::DichotomousDivision(const DichotomousDivision& from)
    : SectionLineSearch(from)
{
}// DichotomousDivision


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
DichotomousDivision::Solve(const UnivariateFunction&    function,
                           double                       a,
                           double                       b,
                           bool                         generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        if (a > b) swap(a, b);

        double  middle; // Aktualny rodek odcinka.
        double  left;   // Punkt w jednej czwartej dugoci odcinka
        double  right;  // Punkt w trzech czwartych dugoci odcinka
        double  value_in_middle;
        double  value_in_left;
        double  value_in_right;
        double  length = b - a; // Dugo przedziau nieokrelonoci.


        LOG(DEBUG,wxT("-----------------------------------"));

        middle          = (a + b) / 2;
        value_in_middle = function.Evaluate(middle);

        while (fabs(length) > mEpsilon && step_count < STEP_LIMIT)
        {
            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count);

            left            = a + length / 4;
            right           = b - length / 4;

            value_in_left   = function.Evaluate(left);
            value_in_right  = function.Evaluate(right);


            auto_ptr<DichotomousDivisionIteration>  ap_iteration
                    (new DichotomousDivisionIteration
                                (a, left, middle, right, b));

            if (value_in_left < value_in_right)
                ap_iteration->rBetterPoint()
                    = DichotomousDivisionIteration::LEFT;
            else
                ap_iteration->rBetterPoint()
                    = DichotomousDivisionIteration::RIGHT;


            LOG(DEBUG,wxString::Format(wxT("A      : %6g"),a));
            LOG(DEBUG,wxString::Format(wxT("Left   : %6g value: %6g"),
                            left,value_in_left));
            LOG(DEBUG,wxString::Format(wxT("Middle : %6g value: %6g"),
                            middle,value_in_middle));
            LOG(DEBUG,wxString::Format(wxT("Right  : %6g value: %6g"),
                            right,value_in_right));
            LOG(DEBUG,wxString::Format(wxT("B      : %6g"),b));

            LOG(DEBUG,wxString::Format(wxT("Length : %6g"),length));


            if (value_in_left < value_in_middle)
            {
                b               = middle;
                middle          = left;
                value_in_middle = value_in_left;

                ap_iteration->rNextMiddle() = DichotomousDivisionIteration::XL;
            }
            else if (value_in_right < value_in_middle)
            {
                a               = middle;
                middle          = right;
                value_in_middle = value_in_right;

                ap_iteration->rNextMiddle() = DichotomousDivisionIteration::XR;
            }
            else
            {
                a               = left;
                b               = right;

                ap_iteration->rNextMiddle() = DichotomousDivisionIteration::XM;
            }

            length  = b - a;

            if (generateResult)
            {
                ap_result->AddIteration
                                (auto_ptr<const IterationData>(ap_iteration));
            }

            // TODO: AddStep

            LOG(DEBUG,wxT("-----------------------------------"));

            NotifyYield();

        } // while

        ap_result->rFinalResult().resize(1);
        ap_result->rFinalResult()(1) = middle;

    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve


wxString DichotomousDivision::mName(_("Dichotomous Division"));

// Utwrz i zarejestruj prototyp.
DichotomousDivision DichotomousDivision::mDichotomousDivision;
