/**  Metoda podziau dychotomicznego.
 *
 * Metoda optymalizacji w kierunku, w ktrej w kadym kroku zmniejszamy
 * przedzia nieoznaczonoci o poow.
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ a \f$ - lewy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ b \f$ - prawy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ x_l \f$ - punkt w 1/4 przedziau nieoznaczonoci w danej iteracji<br>
 * \f$ x_m \f$ - punkt w 1/2 przedziau nieoznaczonoci w danej iteracji<br>
 * \f$ x_r \f$ - punkt w 3/4 przedziau nieoznaczonoci w danej iteracji<br>
 * \f$ F_l \f$ - warto funkcji celu w punkcie \f$ x_l \f$
 * \f$ F_m \f$ - warto funkcji celu w punkcie \f$ x_m \f$
 * \f$ F_r \f$ - warto funkcji celu w punkcie \f$ x_r \f$
 * \f$ L \f$ - dugo odcinka nieoznaczonoci<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ a^0 \f$ - lewy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ b^0 \f$ - prawy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 * <br>
 * W metodzie podziau dychotomicznego dokadno rozumiana jest jako
 * szeroko ostatniego przedziau nieokrelonoci.
 * Wymagamy by bya ona mniejsza od \f$ \varepsilon \f$.
 * <br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Podstawi \f$ x_m = (a^0+b^0)/2 \f$ i obliczy warto funkcji w tym punkcie.
 * Obliczy \f$ L = (b^0-a^0) \f$.
 *
 * <b>Krok 2</b>:<br>
 * Podstawi \f$ x_l = a + L/4 \f$ i \f$ x_r = b - L/4 \f$ i obliczy warto
 * funkcji w tych punktach.
 *
 * <b>Krok 3</b>:<br>
 * Jeli \f$ F_l < F_m \f$, to:
 * \f[ \left\{ \begin{array}{lll}
 *     b = x_m \\
 *     x_m = x_l \\
 *     F_m = F_l
 *     \end{array} \right. \f]
 *
 * oraz przej do kroku 6.
 * Jeli warunek nie jest speniony, przej do kroku 4.
 *
 * <b>Krok 4</b>:<br>
 * Jeli \f$ F_r < F_m \f$, to:
 * \f[ \left\{ \begin{array}{lll}
 *     a = x_m \\
 *     x_m = x_r \\
 *     F_m = F_r
 *     \end{array} \right. \f]
 *
 * i przej do kroku 6.
 * Jeli warunek nie jest speniony, przej do kroku 5.
 *
 * <b>Krok 5</b>:<br>
 * \f[ \left\{ \begin{array}{ll}
 *     a = x_l \\
 *     b = x_r \\
 *     \end{array} \right. \f]
 *
 * <b>Krok 6</b>:<br>
 * \f[ L = (b-a) \f]
 * jeli warunek \f$ |L| \leq 2\varepsilon \f$ jest speniony, zakoczy
 * dziaanie algorytmu. W przeciwnym wypadku przej do kroku 2.
 * <br><br>
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 *
 * @see Linesearch
 **/

#ifndef _DichotomousDivision_h
#define _DichotomousDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"
#include "../ui/SectionLineSearchPanel.h"

// FORWARD REFERENCES
//


class DichotomousDivision
    : public SectionLineSearch
{
public:
    friend class SectionLineSearchPanel<DichotomousDivision>;

// LIFECYCLE


    /** Destruktor.
     */
    ~DichotomousDivision(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    DichotomousDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    DichotomousDivision(const DichotomousDivision& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);


    /** Nazwa metody
     */
    static wxString       mName;
    /** Egzemplarz metody
     */
    static DichotomousDivision  mDichotomousDivision;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _DichotomousDivision_h_
