#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../Population.h"
#include "../text-util.h"
#include "../UnconstrainedProblem.h"
#include "../util.h"

#include "EvolutionaryMethod.h"  // class implemented
#include "EvolutionaryMethodIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////
//============================= LIFECYCLE ====================================

EvolutionaryMethod::~EvolutionaryMethod()
{
}// ~EvolutionaryMethod

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
EvolutionaryMethod::Clone() const
{
    return auto_ptr<Method>(new EvolutionaryMethod(*this));
}// Clone

/* virtual */ void
EvolutionaryMethod::UpdateStartingConditions(const Result* result) const
{
    const ColumnVector& centre = result->FinalResult();

    ColumnVector spans = mInitialRangeMaxBounds - mInitialRangeMinBounds;
    spans /= 4;

    mRangeMinBounds = centre - spans;
    mRangeMaxBounds = centre + spans;
}

/* virtual */ void
EvolutionaryMethod::ResetStartingConditions() const
{
    mRangeMinBounds = mInitialRangeMinBounds;
    mRangeMaxBounds = mInitialRangeMaxBounds;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
EvolutionaryMethod::Name() const
{
    return EvolutionaryMethod::mName;
}// Name

/* virtual */ wxString&
EvolutionaryMethod::rName()
{
    return EvolutionaryMethod::mName;
}// rName
 
/* virtual */ MethodIdType
EvolutionaryMethod::Id() const
{
    return EvolutionaryMethod::ClassId();
}// Id

/* static */ MethodIdType
EvolutionaryMethod::ClassId()
{
    const static MethodIdType EvolutionaryMethodId = Method::GenerateId();

    return EvolutionaryMethodId;
}// ClassId

/* virtual */ wxString
EvolutionaryMethod::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << wxT('\n');
    description << SelectionMethod::BaseClassName() << wxT(":\t");
    description << mcpSelectionMethod->Name() << wxT('\n');
    description << CrossoverMethod::BaseClassName() << wxT(":\t");
    description << mcpCrossoverMethod->Name() << wxT("\n\n");
    description << _("Genetic operators parameters") << wxT(":\n");
    description << _("Survival rate");
    description << wxString::Format(wxT(":  %g\n"),
                                    static_cast<double>(mSurvivalRate));
    description << _("Radioactivity");
    description << wxString::Format(wxT(":  %g\n"),
                                    static_cast<double>(mRadioactivity));
    description << _("Mutation rate");
    description << wxString::Format(wxT(":  %g\n"),
                                    static_cast<double>(mMutationRate));
    description << _("Crossover rate");
    description << wxString::Format(wxT(": %g\n\n"),
                                    static_cast<double>(mCrossoverRate));

    description << StopConditionDescription(mcpStopCondition->ToString());

    description << StartingRangeDescription(mRangeMinBounds, mRangeMaxBounds);
    description << wxT('\n');

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
EvolutionaryMethod::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL &&
            (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
EvolutionaryMethod::AllowedStopConditions() const
{
    StopConditions conditions
                        (StopConditionBase::EvolutionaryMethodStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

EvolutionaryMethod::EvolutionaryMethod()
    : Method(),
      mInitialRangeMinBounds(0),
      mInitialRangeMaxBounds(0),
      mRangeMinBounds(0),
      mRangeMaxBounds(0),
      mNumberOfChromosomes(100),
      mSurvivalRate(0.5, 0.0, true, 1.0, true),
      mRadioactivity(0.1, 0.0, true, 1.0, true),
      mMutationRate(1.0, 0.0, false),
      mCrossoverRate(0.5, 0.0, true, 1.0, true),
      mcpSelectionMethod(),
      mcpMutationMethod(),
      mcpCrossoverMethod(),
      mcpStopCondition()
{
}// EvolutionaryMethod

EvolutionaryMethod::EvolutionaryMethod(const EvolutionaryMethod& from)
    : Method(from),
      mInitialRangeMinBounds(from.mInitialRangeMinBounds),
      mInitialRangeMaxBounds(from.mInitialRangeMaxBounds),
      mRangeMinBounds(from.mRangeMinBounds),
      mRangeMaxBounds(from.mRangeMaxBounds),
      mNumberOfChromosomes(from.mNumberOfChromosomes),
      mSurvivalRate(from.mSurvivalRate),
      mRadioactivity(from.mRadioactivity),
      mMutationRate(from.mMutationRate),
      mCrossoverRate(from.mCrossoverRate),
      mcpSelectionMethod(from.mcpSelectionMethod),
      mcpMutationMethod(from.mcpMutationMethod),
      mcpCrossoverMethod(from.mcpCrossoverMethod),
      mcpStopCondition(from.mcpStopCondition)
{
}// EvolutionaryMethod

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
EvolutionaryMethod::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    LOG(DEBUG,wxT("Starting optimization"));

    auto_ptr<Result> ap_result(new Result(1));

    const unsigned int  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned int        step_count = 0;


    try
    {
        assert(mcpSelectionMethod.Get() != NULL);
        assert(mcpMutationMethod.Get() != NULL);
        assert(mcpCrossoverMethod.Get() != NULL);
        assert(mcpStopCondition.Get()   != NULL);

        const FunctionBase& function = problem.Function();

        const unsigned int dimensions = problem.VariableCount();

        assert(static_cast<unsigned int>
                            (mRangeMinBounds.nrows()) == dimensions &&
               static_cast<unsigned int>
                            (mRangeMaxBounds.nrows()) == dimensions);

        ColumnVector    best_so_far(NullPoint);
        ColumnVector    best_this_iteration(NullPoint);
        ColumnVector    best_previous(NullPoint);
        unsigned int    iterations_without_update = 0;

        // populacja.
        assert(mNumberOfChromosomes >= 2);
        Population  population(function,
                               mNumberOfChromosomes,
                               mRangeMinBounds,
                               mRangeMaxBounds);

        best_so_far = population.Best().mPoint;

        LOG(DEBUG,wxT("-------------------------------------"));

        // StopCondition
        while (step_count < STEP_LIMIT && !mcpStopCondition->IsMet(
                    best_previous, best_so_far, function,
                    population.ChromosomeStandardDeviation(),
                    population.ValueStandardDeviation(),
                    iterations_without_update))
        {
            ++step_count;

            LOG(DEBUG,wxString::Format(wxT("Iteracja nr %d\n"), step_count));

            assert(population.Size() == mNumberOfChromosomes);

            auto_ptr<EvolutionaryMethodIteration> ap_iteration
                                        (new EvolutionaryMethodIteration());
            ap_iteration->SetStartingStatistics
                                    (population.Best().mPoint,
                                     population.AverageChromosome(),
                                     population.AverageValue(),
                                     population.ChromosomeStandardDeviation(),
                                     population.ValueStandardDeviation());

//////////// pocztek transformacji populacji

            mcpSelectionMethod->Select(population,
                                       mSurvivalRate,
                                       *ap_iteration);

            mcpMutationMethod->Mutate(population,
                                      mRadioactivity,
                                      mMutationRate,
                                      *ap_iteration);

            mcpCrossoverMethod->Crossover(population,
                                          mCrossoverRate,
                                          *ap_iteration);

//////////// koniec transformacji populacji

            ap_iteration->SetEndStatistics
                                    (population.Best().mPoint,
                                     population.AverageChromosome(),
                                     population.AverageValue(),
                                     population.ChromosomeStandardDeviation(),
                                     population.ValueStandardDeviation());
            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));

            best_this_iteration = population.Best().mPoint;

            if (best_this_iteration == best_so_far)
                ++iterations_without_update;
            else
            {
                best_previous = best_so_far;
                best_so_far = best_this_iteration;
                iterations_without_update = 0;
            }

            LOG(DEBUG, wxT("Best this iteration: ")
                                << best_this_iteration.as_row());
            LOG(DEBUG, wxT("Value in best: ")
                                << function.Evaluate(best_this_iteration));
            LOG(DEBUG, wxT("Average chromosome: ")
                                << population.AverageChromosome().as_row());
            LOG(DEBUG, wxString::Format(wxT("Average value: %g"),
                                        population.AverageValue()));
            LOG(DEBUG, wxString::Format
                                (wxT("Chromosome standard deviation: %g"),
                                 population.ChromosomeStandardDeviation()));
            LOG(DEBUG, wxString::Format
                                (wxT("Value standard deviation: %g"),
                                 population.ValueStandardDeviation()));
            LOG(DEBUG, wxT("-------------------------------------"));

            NotifyYield();
        }// while

        ap_result->rFinalResult() = population.Best().mPoint;

    }// try
    catch (const BaseException& error)
    {   // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);

    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }

    LOG(INFO, wxT("Step count: ") << step_count);
    return ap_result;
}// Optimize

wxString EvolutionaryMethod::mName(_("Evolutionary method"));

// Utwrz i zarejestruj prototyp.
EvolutionaryMethod EvolutionaryMethod::mEvolutionaryMethod;
