/** Metoda oparta o algorytmy ewolucyjne.
 *
 * Algorytmy ewolucyjne daj si zastosowa w kadym zadaniu polegajcym na
 * przeszukiwaniu przestrzeni alternatywnych rozwiza w poszukiwaniu
 * najlepszego z nich. Takim zadaniem jest
 * poszukiwanie optimum funkcji w przestrzeni dowolnej liczby wymiarw. W tym
 * zastosowaniu algorytmy genetyczne charakteryzuj si najwiksz odpornoci
 * na wystpowanie minimw lokalnych. Z uwagi
 * jednak na spory narzut obliczeniowy zwizany z ich uyciem naley stosowa
 * je tylko w uzasadnionych przypadkach (np. gdy niecigo funkcji celu moe
 * stanowi problem dla klasycznych algorytmw).
 *
 * W algorytmach genetycznych operujcych w przestrzeni liczb rzeczywistych
 * problemem jest wybr odpowiedniego kodowania chromosomw. Klasyczne metody
 * krzyowania polegajce na wymianie genw (bitw) pomidzy chromosomami w
 * wersji z liczbami zmiennoprzecinkowymi staj si bardzo skomplikowane i
 * nieprzejrzyste. W niniejszej implementacji chromosom kodowany jest
 * w sposb intuicyjny, czyli za pomoc wektora liczb rzeczywistych.
 * Kodowanie to zdeterminowao uyte metody mutacji i krzyowania.
 *
 * Schemat dziaania algorytmu:<br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * <b>Inicializacja</b> - W zadanym obszarze tworzona jest losowo populacja o
 * okrelonej liczbie osobnikw (chromosmw).<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * <b>Selekcja</b> - Z pocztkowej populacji wybierane s osobniki najlepiej
 * przystosowane wg algorytmw zawartych w konkretnych metodach selekcji
 * (Narzucajce si od razu proste rozwizanie, czyli wybranie okrelonej
 * liczenie grupy najlepszych osobnikw nie sprzyjaoby zrnicowaniu populacji.
 * <br><br>
 * <b>Krok 2</b>:<br>
 * <b>Mutacja</b> - Losowo wybrane z populacji osobniki ulegaj mutacji, czyli
 * losowej modyfikacji wsprzdnych.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * <b>Krzyowanie</b> - Proces polegajcy na rozmnaaniu si osobnikw wg zasad
 * zawartych w konkretnych metodach krzyowania. Osobniki namnaane s do
 * momentu gdy populacja osignie pocztkow liczebno.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Sprawd czy zosta speniony warunek stopu. Jeli nie, to przejd do kroku 1,
 * jeli tak, to zakocz dziaanie algorytmu. Za miniumum funkcji brany jest
 * najlepszy osobnik z ostatniej populacji.<br>
 * <br>
 * We wszystkich zaimplementowanych metodach selekcji, mutacji i krzyowania
 * kierowano si zasada, e najlepszy osobnik nie moe zgin, ani zosta
 * zmodyfikowany.<br>
 * <br>
 * Zaimplementowano nastpujce metody manipulacji populacj:
 * <ol>
 * <li>Metody selekcji:
 * <ul>
 * <li>Selekcja proporcjonalna
 * <li>Selekcja turniejowa
 * </ul>
 * <li>Metody mutacji:
 * <ul>
 * <li>Mutacja ze zmienn losow o rozkadzie jednostajnym w przedziale
 *  \f$ [-1; 1] \f$.
 * <li>Mutacja ze zmienn losow o rozkadzie Cauchy'ego
 * </ul>
 * <li>Metody krzyowania:
 * <ul>
 * <li>Krzyowanie proste z dzieckiem w poowie drogi pomidzy rodzicami
 * <li>Krzyowanie tworzcy kopi trzeciego osobnika przesunit o odcinek
 * czcy rodzicw
 * </ul>
 * </ol>
 * Poszczeglne metody opisane s w klasach je reprezentujcych.<br>
 * <br>
 * W metodzie mona korzysta z nastepujych warunkw stopu:
 * <ul>
 * <li>Warunek \f$ \| x^k_b - x^{k-1}_b \| \leq \varepsilon \f$.
 * <li>Warunek \f$ | F^k_b - F^{k-1}_b | \leq \varepsilon \f$.
 * <li>Warunek \f$ \frac{| F^k_b - F^{k-1}_b |}{\| x^k_b - x^{k-1}_b \|}
 * \leq \varepsilon \f$.
 * <li>Warunek \f$ \sigma(x: x \in P) \leq \varepsilon \f$, czyli odchylenie
 * standardowe odlegoci chromosomow od chromosomu centralnego (zwarto
 * populacji).
 * <li>Warunek \f$ \sigma(f(x): x \in P) \leq \varepsilon \f$, czyli odchylenie
 * standardowe wartoci funkcji w chromosomach wzgldem redniej wartoci
 * funkcji.
 * <li>Warunek \f$ N \leq \varepsilon \f$, gdzie N oznacza liczb iteracji
 * </ul>
 * <br>
 * <b>Uwaga:</b> Nie istnieje jedna wybrana nomenklatura dotyczca algorytmw
 * genetycznych, mona si wic spotka z rnymi nazwami uytymi na okrelenie
 * tych samych etapw "ewolucji". W polskiej literaturze (np. [1]) selekcja
 * nazywana jest zwykle reprodukcj. W rdach angielskich reprodukcj czciej
 * okrela si nastpujce po sobie zjawiska krzyowania i mutacji.
 * W niniejszej pracy zdecydowano sie na podejcie zachodnie, gdy
 * zaimplementowane metody selekcji nie pozwalaj na utworzenie wicej ni
 * jednej kopii wybranego/wylosowanego chromosomu, dokonuj wic w
 * rzeczywistoci selekcji (wyboru) rozwiza, a nie ich namnoenia. Takie
 * podejcie w swoich konsekwencjach nie rni si niczym od podejcia z
 * utworzeniem wielu kopii lepszych osobnikw z nastpujcymi potem operacjami
 * mutacji i krzyowania.
 * <br>
 * Zaimplementowany algorytm zawdzicza wiele algorytmowi SADE (Simplified
 * Atavistic Differential Evolution), ktrego opis mona znale na stronie:
 * <a href="http://klobouk.fsv.cvut.cz/~ondra/sade/sade.html" target=blank>http://klobouk.fsv.cvut.cz/~ondra/sade/sade.html</a><br>
 *
 * Przy implementacji algorytmu korzystano z:<br>
 * [1] <i>Arabas, Jarosaw: Wykady z algorytmw ewolucyjnych, Wydawnictwa
 * Naukowo-Techniczne, Warszawa, 2004</i><br>
 * [2] <i>Goldberg, David E.: Algorytmy genetyczne i ich zastosowania,
 * Wydawnictwa Naukowo-Techniczne, Warszawa, 2003</i><br>
 */

#ifndef _EvolutionaryMethod_h
#define _EvolutionaryMethod_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CrossoverMethod.h"
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../Method.h"
#include "../MutationMethod.h"
#include "../SelectionMethod.h"
#include "../EMStopCondition.h"
//#include "EvolutionaryMethodIteration.h"

// FORWARD REFERENCES
//


class EvolutionaryMethod
    : public Method
{
public:

    friend class EvolutionaryMethodPanel;
    friend class InitialRangeValidator;

// LIFECYCLE

    /** Destruktor.
     */
    ~EvolutionaryMethod(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void                    UpdateStartingConditions
                                                (const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    /** Czy metoda nadaje si do rozwizania zadanego problemu
     */
    virtual bool                    CanHandle(const ProblemBase* pProblem)
                                                                        const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    EvolutionaryMethod(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    EvolutionaryMethod(const EvolutionaryMethod& from);

    /** Punkt startowy
     */
    ColumnVector                        mInitialRangeMinBounds;
    ColumnVector                        mInitialRangeMaxBounds;
    mutable ColumnVector                mRangeMinBounds;
    mutable ColumnVector                mRangeMaxBounds;
    unsigned                            mNumberOfChromosomes;

    /** Relatywna wielko populacji, ktra ma zosta po dokonaniu selekcji.
     */
    DoubleParameter                     mSurvivalRate;
    /** Prawdopodobiestwo zajcia mutacji osobnika.
     */
    DoubleParameter                     mRadioactivity;
    /** Wspczynnik mutacji decydujcy o tym jak bardzo zmieni si wsprzdne
     * mutowanego punktu.
     */
    DoubleParameter                     mMutationRate;
    /** Wspczynnik wykorzystywany w niektrych metodach krzyowania
     * (np. ThreeParticipantsCrossover).
     */
    DoubleParameter                     mCrossoverRate;

    CountedPtr<const SelectionMethod>   mcpSelectionMethod;
    CountedPtr<const MutationMethod>    mcpMutationMethod;
    CountedPtr<const CrossoverMethod>   mcpCrossoverMethod;

    /** Warunek stopu.
     */
    CountedPtr<const EMStopCondition> mcpStopCondition;

    virtual wxString&               rName();

private:

    virtual std::auto_ptr<Result>   Optimize(const ProblemBase& problem) const
        throw(OptimizationError);


    /** Nazwa metody
     */
    static wxString           mName;

    /** Egzemplarz metody
     */
    static EvolutionaryMethod       mEvolutionaryMethod;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _EvolutionaryMethod_h
