/** Metoda Fibonacciego.
 *
 * <p>Metoda Fibonacciego, opublikowana w 1953 roku przez Kiefera, jest
 * szczeglnie przydatna w sytuacjach gdy liczba oblicze, ktre moemy wykona
 * jest ograniczona (np. w systemach czasu
 * rzeczywistego). Metoda daje najmniejszy przedzia nieokrelonoci przy
 * zadanej z gry liczbie wylicze funkcji.</p>
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ a^k \f$ - lewy kraniec przedziau poszukiwa w <i>k</i>-tej iteracji<br>
 * \f$ b^k \f$ - prawy kraniec przedziau poszukiwa w <i>k</i>-tej iteracji<br>
 * \f$ \lambda^k \f$ - pierwszy punkt wyliczany w <i>k</i>-tej iteracji<br>
 * \f$ \mu^k \f$ - drugi punkt wyliczany w <i>k</i>-tej iteracji<br>
 * \f$ F_k \f$ - k-ty element cigu Fibonacciego<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ a^1 \f$ - lewy kraniec przedziau poszukiwa<br>
 * \f$ b^1 \f$ - prawy kraniec przedziau poszukiwa<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 *
 * W metodzie Fibonacciego dokadno rozumiana jest jako szeroko ostatniego
 * przdziau nieokrelonoci. Wymagamy by bya ona mniejsza od \f$ \varepsilon \f$.
 * Charakterystyczn cech metody Fibonacciego jest to, e liczb krokw
 * optymalizacji ustalamy przed rozpoczciem oblicze.
 * \f$ K \f$ musi spenia warunek: \f$ (b^1 - a^1) / F_K \leq \varepsilon \f$.
 * <br>
 * <b>Krok 1</b>:<br>
 * Tworzymy dwa nowe punkty (\f$ \lambda^k \f$ i \f$ \mu^k \f$) lece pomidzy
 * <code>a</code> i <code>b</code> zgodnie ze wzorami:
 * \f[ \lambda^k = a^k + \frac{F_{K-k-1}}{F_{K-k+1}}(b^k - a^k) \f]
 * \f[ \mu^k = a^k + \frac{F_{K-k}}{F_{K-k+1}}(b^k - a^k) \f]
 * W nowoutworzonych punktach wyliczamy warto funkcji.
 *
 * <b>Krok 2</b>:<br>
 * Jeli \f$ f(\lambda^k) > f(\mu^k)\f$, to podstawiamy:
 * \f[ \left\{ \begin{array}{llll}
 *     a^{k+1} = \lambda^k \\
 *     b_{k+1} = b^k \\
 *     \lambda^{k+1} = \mu^k \\
 *     \mu^{k+1} = a^{k+1} + \frac{F_{K-i}}{F_{K-k+1}}(b^{k+1} - a^{k+1})
 *     \end{array} \right. \f]
 * oraz obliczamy warto funkcji w punkcie \f$ \mu^{k+1} \f$. Nastpnie
 * przechodzimy do punktu 4.<br>
 * Jednak gdy \f$ f(\lambda^k) \leq f(\mu^k) \f$ przechodzimy do kroku 3.
 *
 * <b>Krok 3</b>:<br>
 * Podstawiamy
 * \f[ \left\{ \begin{array}{llll}
 *     a^{k+1} = a^k \\
 *     b^{k+1} = \mu^k \\
 *     \mu^{k+1} = \lambda^k \\
 *     \lambda^{k+1} = a^{k+1} + \frac{F_{K-k-1}}{F_{K-k+1}}(b^{k+1} - a^{k+1})
 *     \end{array} \right. \f]
 * i wyliczamy warto funkcji w punkcie \f$ \lambda^{k+1} \f$. Nastpnie
 * przechdzimy do punktu 4.
 *
 * <b>Krok 4</b>:<br>
 * Podstawiamy \f$ k=k+1 \f$.
 * Jeli \f$ i < K-1 \f$, to przej do kroku 2. W przeciwnym wypadku przej do
 * kroku 5.
 *
 * <b>Krok 5</b>:<br>
 * \f[ \left\{ \begin{array}{ll}
 *     \lambda^K = \lambda^{K-1} \\
 *     \mu^K = \lambda^K + \varepsilon
 *     \end{array} \right. \f]
 *
 * Jeli \f$ f(\lambda^K) > f(\mu^K) \f$, to podstawiamy:<br>
 * \f[ \left\{ \begin{array}{ll}
 *     a^K = \lambda^K \\
 *     b^K = b^{K-1}
 *     \end{array} \right. \f]
 *
 * Jeli natomiast \f$ f(\lambda^K) \leq f(\mu^K) \f$, to podstawiamy:<br>
 * \f[ \left\{ \begin{array}{ll}
 *     a^K = a^{K-1} \\
 *     b^K = \lambda^K
 *     \end{array} \right. \f]
 *
 * Wynikiem dziaania metody jest punkt w rodku odcinka \f$ [a, b] \f$.
 * <br><br>
 *
 * Algotytm zaimplementowany na podstawie // TODO: tej rosyjskiej ksiaki.
 */

#ifndef _FibonacciDivision_h
#define _FibonacciDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"
#include "../ui/SectionLineSearchPanel.h"

// FORWARD REFERENCES
//


class FibonacciDivision
    : public SectionLineSearch
{
public:
    friend class SectionLineSearchPanel<FibonacciDivision>;

// LIFECYCLE


    /** Destruktor.
     */
    ~FibonacciDivision(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    FibonacciDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    FibonacciDivision(const FibonacciDivision& from);

    /** Wygeneruj cig Fibonacciego.
     *
     * Wygeneruj cig Fibonacciego wystarczajco dugi by w wyniku dziaania
     * metody mona byo otrzyma przedzia nieokreslonoci o zadanej dugoci.
     *
     * @param a lewy kraniec pocztkowego przedziau poszukiwa
     * @param b prawy kraniec pocztkowego przedziau poszukiwa
     * @param epsilon wymagana dokadno
     * @return wygenerowany cig Fibonacciego
     */
    std::vector<unsigned int>   GenerateFibonacciSequence(double a,
                                                          double b,
                                                          double epsilon) const;
    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);


    /** Nazwa metody
     */
    static wxString       mName;
    /** Egzemplarz metody
     */
    static FibonacciDivision    mFibonacciDivision;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FibonacciDivision_h_
