#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "../text-util.h"
#include "../util.h"

#include "FletcherReeves.h"  // class implemented
#include "ConjugateGradientsMethodIteration.h"


LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FletcherReeves::~FletcherReeves()
{
}// ~FletcherReeves

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
FletcherReeves::Clone() const
{
    return auto_ptr<Method>(new FletcherReeves(*this));
}// Clone

//============================= ACCESS     ===================================


/* virtual */ const wxString&
FletcherReeves::Name() const
{
    return FletcherReeves::mName;
}// Name

/* virtual */ wxString&
FletcherReeves::rName()
{
    return FletcherReeves::mName;
}// rName

/* virtual */ MethodIdType
FletcherReeves::Id() const
{
    return FletcherReeves::ClassId();
}// Id

/* static */ MethodIdType
FletcherReeves::ClassId()
{
    const static MethodIdType FletcherReevesId = Method::GenerateId();

    return FletcherReevesId;
}// ClassId

/* virtual */ wxString
FletcherReeves::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << _("Reset direction after N iterations: ");
    description << (mModification ? _("Yes") : _("No"));
    description << wxT("\n\n");
    description << LineSearchDescription(mcpLineSearch->ToString(false));
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}

//============================= INQUIRY    ===================================

/* virtual */ bool
FletcherReeves::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL &&
            pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}// CanHandle

/* virtual */ StopConditions
FletcherReeves::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    return conditions;
}// AllowedStopConditions


/////////////////////////////// PROTECTED  ///////////////////////////////////

FletcherReeves::FletcherReeves()
    : MethodWithLineSearch(),
      mModification(false)
{
}// FletcherReeves

FletcherReeves::FletcherReeves(const FletcherReeves& from)
    : MethodWithLineSearch(from),
      mModification(from.mModification)
{
}// FletcherReeves


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
FletcherReeves::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result>                            ap_result(new Result(1));
    auto_ptr<ConjugateGradientsMethodIteration> ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpLineSearch.Get() != NULL);
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = dynamic_cast<const FunctionBase&>
                                                        (problem.Function());
        assert(function.IsDifferentiable());

        const unsigned int dimensions = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);



        // Biecy punkt.
        ColumnVector        x(mStartingPoint);
        // Punkt z poprzedniej iteracji.
        // (inicjowany wektorem zerowym, aby moliwe byo
        // sprawdzanie przed iteracj wszystkich warunkw stopu).
        ColumnVector        previous_x(NullPoint);
        // Kierunek poszukiwa jednowymiarowych
        ColumnVector        direction(dimensions);
        // Dugo kroku wykonanego w kierunku direction
        double              tau;
        // Gradient.
        ColumnVector        gradient(dimensions);
        // Gradient z poprzedniej iteracji.
        ColumnVector        previous_gradient(dimensions);

        // Norma gradientu.
        double              gradient_norm;
        // Norma poprzedniego gradientu. Jej warto nie ma znaczenia w
        // pierwszej iteracji.
        double              previous_gradient_norm = 0;
        // Wspczynnik wykorzystywany w obliczaniu kierunku sprzonego.
        // Jego warto nie ma znaczenia w pierwszej iteracji.
        double              gamma = 0;

        LOG(DEBUG,wxT("-------------------------------------"));


        /* Gradient wyliczamy przed pierwsz iteracj algorytmu, gdy moliwe
         * jest, e punkt startowy jest akceptowalnie blisko punktu minimum.
         */
        gradient = function.Gradient(x);
        gradient_norm   = gradient.norm_Frobenius();

        LOG(DEBUG,wxT("Iteracja wstepna\n"));
        LOG(DEBUG,wxT("Punkt:\n") << x);
        LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                << function.Evaluate(x) << wxT("\n"));
        LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
        LOG(DEBUG,wxT("Norma gradientu:\n") << gradient_norm);

        LOG(DEBUG,wxT("-------------------------------------"));


        while (step_count < STEP_LIMIT &&
               !mcpStopCondition->IsMet(previous_x, x, function, gradient_norm))
        {
            ++step_count;

            bool    direction_reset
                = (mModification && (step_count % dimensions) == 1) ||
                  (!mModification && step_count == 1);

            LOG(DEBUG,wxT("Iteracja ") << step_count << wxT("\n"));

            /* Jeli wprowadzono modyfikacj, to powracamy do kierunku
             * pocztkowego po wykonaniu liczby iteracji rwnej wymiarowi
             * problemu.
             */
            if (direction_reset)
            {
                direction = -1.0 * gradient;
                LOG(DEBUG,wxT("Kierunek przeciwny do gardientu:\n")
                                                            << direction);
            }
            else
            {
                // Taka sytuacja nie moe si zdarzy jeli algorytm dziaa
                // prawidowo.
                assert(previous_gradient_norm != 0);

                gamma = (gradient_norm * gradient_norm) /
                        (previous_gradient_norm * previous_gradient_norm);

                direction = -1.0 * gradient + gamma * direction;

                LOG(DEBUG,wxT("gamma: ") << gamma << wxT("\n"));
                LOG(DEBUG,wxT("Kierunek sprzezony z poprzednimi:\n")
                                                            << direction);
            }


            previous_x = x;
            x = StepInDirection(function, x, direction, &tau);

            ap_iteration.reset(new ConjugateGradientsMethodIteration
                       (previous_x,
                        gradient,
                        gamma,
                        direction_reset,
                        direction,
                        tau,
                        x));
            ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));

            previous_gradient       = gradient;
            previous_gradient_norm  = gradient_norm;

            gradient                = function.Gradient(x);
            gradient_norm           = gradient.norm_Frobenius();

            LOG(DEBUG,wxT("Nowy Punkt:\n") << x);
            LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                    << function.Evaluate(x) << wxT("\n"));
            LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
            LOG(DEBUG,wxT("Norma gradientu:\n") << gradient_norm << wxT("\n"));

            LOG(DEBUG,wxT("-------------------------------------"));

            NotifyYield();
        }

        ap_result->rFinalResult() = x;

    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString FletcherReeves::mName(_("Fletcher-Reeves'"));

// Utwrz i zarejestruj prototyp.
FletcherReeves FletcherReeves::mFletcherReeves;
