#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../text-util.h"
#include "../UnconstrainedProblem.h"

#include "HookeJeevesDiscrete.h"  // class implemented

LOGGER(INFO);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HookeJeevesDiscrete::~HookeJeevesDiscrete()
{
}// ~HookeJeevesDiscrete

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
HookeJeevesDiscrete::Clone() const
{
    return auto_ptr<Method>(new HookeJeevesDiscrete(*this));
}// Clone

/* virtual */ void
HookeJeevesDiscrete::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
HookeJeevesDiscrete::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
HookeJeevesDiscrete::Name() const
{
    return HookeJeevesDiscrete::mName;
}// Name

/* virtual */ wxString&
HookeJeevesDiscrete::rName()
{
    return HookeJeevesDiscrete::mName;
}// rName

/* virtual */ wxString
HookeJeevesDiscrete::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    parameter_descriptions.Add(TauDescriptionHJD(mTau));
    parameter_descriptions.Add(BetaDescriptionHJD(mBeta));

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);
    description << wxT('\n');
    description << StopConditionDescriptionHJD(mEpsilon);
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}// ToString

/* virtual */ MethodIdType
HookeJeevesDiscrete::Id() const
{
    return HookeJeevesDiscrete::ClassId();
}// Id

/* static */ MethodIdType
HookeJeevesDiscrete::ClassId()
{
    const static MethodIdType HookeJeevesDiscreteId = Method::GenerateId();

    return HookeJeevesDiscreteId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
HookeJeevesDiscrete::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

HookeJeevesDiscrete::HookeJeevesDiscrete()
    : Method(),
      mTau(1.0, 0, true),
      mBeta(0.5, 0, true, 1, true),
      mEpsilon(0.001, 0, true),
      mStartingPoint(0),
      mInitialStartingPoint(0)
{
}// HookeJeevesDiscrete

HookeJeevesDiscrete::HookeJeevesDiscrete(const HookeJeevesDiscrete& from)
    : Method(from),
      mTau(from.mTau),
      mBeta(from.mBeta),
      mEpsilon(from.mEpsilon),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint)
{
}// HookeJeevesDiscrete

double
HookeJeevesDiscrete::CoordinateSearch(
        const FunctionBase&             function,
        ColumnVector&                   rx,
        double                          tau,
        HookeJeevesDiscreteIteration&   rIterationResult) const
{
    double min_value;
    double value;
    ColumnVector x_prev(rx);

    min_value = function.Evaluate(rx);
    for (int i = 1; i <= rx.nrows(); ++i)
    {
        LOG(DEBUG, wxString::Format(wxT("x_prev(%d): %.60g"), i, x_prev(i)));
        // Krok wzdu osi
        rx(i) = x_prev(i) + tau;
        value = function.Evaluate(rx);

        if (value < min_value)
        {   // Poprawa, zostajemy
            rIterationResult.AddTrialPoint(rx, true);

            min_value = value;
            x_prev(i) = rx(i);
        }
        else
        {   // Prba w przeciwn stron
            rIterationResult.AddTrialPoint(rx, false);

            rx(i) = x_prev(i) - tau;
            value = function.Evaluate(rx);

            if (value < min_value)
            {   // Poprawa
                rIterationResult.AddTrialPoint(rx, true);

                min_value = value;
                x_prev(i) = rx(i);
            }
            else
            {   // Brak poprawy, pozostajemy przy pocztkowej wartoci
                rIterationResult.AddTrialPoint(rx, false);

                rx(i) = x_prev(i);
            }
        }
        LOG(DEBUG, wxString::Format(wxT("rx(%d):     %.60g"), i, rx(i)));
    }// for

    return min_value;
}// CoordinateSearch

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
HookeJeevesDiscrete::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik
    ColumnVector x(mStartingPoint); // Biecy punkt
    ColumnVector x_base(x);         // Punkt bazowy
    double tau = mTau;              // Bieca dugo kroku
    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    double value;
    double min = problem.Function().Evaluate(x);

    assert((mBeta > 0) && (mBeta < 1));
    assert(mStartingPoint.nrows() > 0);

    try
    {
        while ((tau >= mEpsilon) && (step_count < STEP_LIMIT))
        {
            ++step_count;

            auto_ptr<HookeJeevesDiscreteIteration> ap_result_iteration(
                    new HookeJeevesDiscreteIteration(x));
            ap_result_iteration->rOldBasePoint()    = x_base;

            LOG(DEBUG, wxT("Tau: ") << tau);
            value = CoordinateSearch(problem.Function(), x, tau,
                    *ap_result_iteration);
            LOG(DEBUG, wxT("CoordinateSearch: ") << x.as_row());

            // Zrb krok w wyznaczonym kierunku.
            ColumnVector step(x - x_base);

            // Zmniejsz may krok do 0 (kroki powinny by wielokrotnoci delty
            // w kadym kierunku, jeli s mniejsze, to znaczy e mamy do
            // czynienia z bdem zaokrgle).
            for(int i = 1; i <= step.nrows(); ++i)
            {
                if (fabs(step(i)) < 0.5 * tau) step(i) = 0;
            }

            if ((value < min) && (step.norm1() > 0))
            {   // Znaleziono mniejsz warto funkcji, dobrze.
                min = value;

                x_base = x;
                x = x_base + step;

                ap_result_iteration->rBasePoint()   = x_base;

                LOG(DEBUG, wxT("x_base: ") << x_base.as_row());
                LOG(DEBUG, wxT("Step to: ") << x.as_row());
            }
            else
            {   // Skr dugo kroku i powr do poprzedniego bazowego.
                ap_result_iteration->rStepCancelled()   = true;
                ap_result_iteration->rOldTau()        = tau;

                tau *= mBeta;

                ap_result_iteration->rNewTau() = tau;

                LOG(DEBUG, wxT("Contract"));
                x = x_base;

            }
            ap_result_iteration->rEndPoint() = x;

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));

            NotifyYield();
        }
    }
    catch (const Error& error)  // EvaluationError i Error
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x_base;

    return ap_result;
}// Optimize



wxString HookeJeevesDiscrete::mName(_("Hooke & Jeeves' Discrete"));

// Utwrz i zarejestruj prototyp.
HookeJeevesDiscrete HookeJeevesDiscrete::mHookeJeevesDiscrete;
