#ifndef _HookeJeevesDiscrete_h
#define _HookeJeevesDiscrete_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../Method.h"

#include "HookeJeevesDiscreteIteration.h"

// FORWARD REFERENCES
//

/**  Metoda Hooke'a i Jeevesa z krokiem dyskretnym.
 *
 * Metoda poszukiwa prostych.<br>
 * Metoda wyznacza kierunek za pomoc krokw prbnych wzdu osi, a nastpnie
 * wykonuje krok roboczy w tym kierunku.
 * Jeli w nowym punkcie, za pomoc kolejnych krokw prbnych,
 * uda si wyznaczy kierunek opadania funkcji, wykonywany jest kolejny
 * krok roboczy.
 * Jeli nie, to dugo kroku prbnego (\f$\tau\f$) jest
 * zmniejszana (mnoona razy wspczynnik \f$\beta\f$).
 *
 * Algorytm koczy si w momencie, gdy dugo kroku staje si mniejsza ni
 * parametr <code>epsilon</code> lub przekroczona zostaje dozwolona liczba
 * iteracji.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x^0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\tau\f$: - dugo kroku prbnego,<br>
 * \f$\beta\f$ - wspczynnik skracania kroku (\f$0 < \beta < 1\f$),<br>
 * \f$\varepsilon\f$ - warunek stopu - minimalna dugo kroku prbnego.<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$i\f$ - indeks aktualnego kierunku,<br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$D = d^{(i)},\: i=1,\ldots,n\f$ - aktualna baza kierunkw,<br>
 * \f$x^i\f$ - aktualny punkt prbny,<br>
 * \f$x^k_{min}\f$ - najlepszy punkt w k-tym etapie,<br>
 * \f$F^k_{min}\f$ - warto funkcji w punkcie \f$x^k_{min}\f$,<br>
 * \f$x_b^k\f$ - punkt bazowy dla krokw prbnych w k-tym etapie,<br>
 * \f$F^k_b\f$ - warto funkcji w punkcie \f$x^k_b\f$.<br>
 *
 * <br><b>Procedura.</b><br>
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ D = I(n), \mbox{ gdzie } I(n) - \mbox{ macierz jednostkowa }
 * n \times n , \f]
 * \f[ x^1_{min} = x^0, \f]
 * \f[ F^1_{min} = f(x^0) , \f]
 * \f[ x_b^1 = x^0, \f]
 * \f[ F_b^1 = f(x^0) , \f]
 * \f[ k = 1 , \f]
 * \f[ i = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Badamy warto funkcji w nowym punkcie prbnym po
 * wykonaniu kroku w i-tym kierunku (wzdu i-tej osi):
 * \f[ x^i = x^k_{min} + \tau d^{(i)} . \f]
 *
 * Jeli krok by udany (\f$f(x^i) < F^k_{min}\f$), to
 * zapamitujemy punkt prbny jako najlepszy:
 * \f[ x^k_{min} = x^i, \f]
 * \f[ F^k_{min} = f(x^i), \f]
 * a nastpnie przechodzimy do kroku 3.<br>
 * W przeciwnym wypadku (\f$f(x^i) \ge F^k_{min}\f$)
 * przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Wykonujemy krok prbny w przeciwnym kierunku:
 * \f[ x^i = x^k_{min} - \tau d^{(i)} . \f]
 * Jeli krok by udany (\f$f(x^i) < F^k_{min}\f$), to
 * zapamitujemy punkt prbny jako najlepszy:
 * \f[ x^k_{min} = x^i, \f]
 * \f[ F^k_{min} = f(x^i). \f]
 * Przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Sprawdzamy, czy wykonano kroki prbne we wszystkich kierunkach:<br>
 * Jeli \f$i < n\f$, podstawiamy \f$i = i + 1\f$ i ponownie przechodzimy
 * do kroku 1.<br>
 * Jeli \f$i = n\f$, przechodzimy do kroku 4.
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy, czy przy ostatnim przeszukiwaniu \f$n\f$
 * kierunkw nastpia poprawa:<br>
 * Jeli tak (\f$F^k_{min} < F_b^k\f$), to przechodzimy do kroku 5.<br>
 * W przeciwnym razie przechodzimy do kroku 6.
 *
 * <br><b>Krok 5:</b><br>
 * Zapamitujemy aktualny punkt jako punkt bazowy dla nastpnego
 * etapu:
 * \f[ x_b^{k+1} = x^k_{min}. \f]
 * Nastpnie wykonujemy krok roboczy w znalezionym w tym etapie
 * kierunku opadania funkcji:
 * \f[ x^{k+1}_{min} = x^k_{min} + (x^k_{min} - x^k_b). \f]
 * Przechodzimy do kroku 7.
 *
 * <br><b>Krok 6:</b><br>
 * Skracamy dugo kroku prbnego:
 * \f[ \tau = \beta \tau \f]
 * i cofamy si do punktu bazowego:
 * \f[ x^{k+1}_{min} = x^k_b. \f]
 * Przechodzimy do kroku 7.
 *
 * <br><b>Krok 7:</b><br>
 * Sprawdzamy warunek stopu:<br>
 * Jeli \f$\tau < \varepsilon\f$, koczymy dziaanie. Wynikiem jest
 * \f$ x^{k+1}_{min} \f$.<br>
 * W przeciwnym wypadku podstawiamy \f$k = k+1\f$, \f$i = 1\f$ i
 * przechodzimy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Metoda zaimplementowana na podstawie programu Marka G. Johnsona
 * (http://www.netlib.org/opt/hooke.c), ktry powsta na podstawie pseudokodu z
 * "Algorithm 178: Direct Search" Arthura F. Kaupe Jr. i zawiera poprawki Bella
 * i Pike'a (CACM v.9, str. 684, Wrzesie 1966) oraz Tomlina i Smitha ("Remark
 * on Algorithm 178", CACM v.12).
 *
 * Dugo krokw prbnych nie jest zalena od pozycji punktu startowego, w
 * przeciwiestwie do implementacji Johnsona.
 *
 * @see HookeJeevesOptimal
 */
class HookeJeevesDiscrete
    : public Method
{
public:

    /** Panel ustawie metody.
     */
    friend class HookeJeevesDiscretePanel;

// LIFECYCLE

    /** Destruktor.
     */
    ~HookeJeevesDiscrete(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void                    UpdateStartingConditions
                                                (const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    virtual wxString                ToString(bool standalone = true) const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    HookeJeevesDiscrete(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    HookeJeevesDiscrete(const HookeJeevesDiscrete& from);


    // Ponisza metoda nie jest prywatna wycznie ze wzgldu na testy
    /** Znajd kierunek.
     *
     * Wykonuje kroki prbne, po kolei dla wszystkich zmiennych, wybierajc te
     * kierunki, dla ktrych nastpuje poprawa.
     *
     * Kroki prbne s dodawane do wyniku.
     *
     * @param function Funkcja celu.
     * @param rx Referencja na biecy punkt, wok ktrego przeprowadzane jest
     * poszukiwanie. Zostaje ustawiony na punkt, ktry jest wynikiem
     * poszukiwania.
     * @param tau Dugo kroku prbnego.
     * @param rIterationData Referencja na dane biecej iteracji wyniku.
     *
     * @return Warto funkcji w <code>rx</code>.
     */
    double  CoordinateSearch
                (const FunctionBase&            function,
                 ColumnVector&                  rx,
                 double                         tau,
                 HookeJeevesDiscreteIteration&  rIterationData) const;

    DoubleParameter mTau;           ///< Dugo kroku (wersora) prbnego.
    DoubleParameter mBeta;          ///< Wspczynnik skracania kroku.
    DoubleParameter mEpsilon;       ///< Warunek stopu - minimalna dugo kroku.
    mutable ColumnVector    mStartingPoint; ///< Punkt startowy.
    ColumnVector            mInitialStartingPoint;

    virtual wxString&               rName();
private:
    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);


    static wxString      mName;                   ///< Nazwa metody.
    static HookeJeevesDiscrete mHookeJeevesDiscrete;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _HookeJeevesDiscrete_h_
