#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"

#include "HookeJeevesOptimal.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

HookeJeevesOptimal::~HookeJeevesOptimal()
{
}// ~HookeJeevesOptimal

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
HookeJeevesOptimal::Clone() const
{
    return auto_ptr<Method>(new HookeJeevesOptimal(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
HookeJeevesOptimal::Name() const
{
    return HookeJeevesOptimal::mName;
}// Name

/* virtual */ wxString&
HookeJeevesOptimal::rName()
{
    return HookeJeevesOptimal::mName;
}// rName

/* virtual */ MethodIdType
HookeJeevesOptimal::Id() const
{
    return HookeJeevesOptimal::ClassId();
}// Id

/* static */ MethodIdType
HookeJeevesOptimal::ClassId()
{
    const static MethodIdType HookeJeevesOptimalId = Method::GenerateId();

    return HookeJeevesOptimalId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
HookeJeevesOptimal::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
HookeJeevesOptimal::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::StandardStopConditions());

    return conditions;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

HookeJeevesOptimal::HookeJeevesOptimal()
    : MethodWithLineSearch()
{
}// HookeJeevesOptimal

HookeJeevesOptimal::HookeJeevesOptimal(const HookeJeevesOptimal& from)
    : MethodWithLineSearch(from)
{
}// HookeJeevesOptimal

double
HookeJeevesOptimal::CoordinateSearch(
        const FunctionBase&             function,
        ColumnVector&                   rx,
        HookeJeevesOptimalIteration&    rIteration) const
{
    LOG(DEBUG, wxT("CoordinateSearch at: ") << rx.as_row());
    for (int i = 1; i <= rx.nrows(); ++i)
    {
        ColumnVector d(rx.nrows());
        d    = 0.0;
        d(i) = 1.0;

        LOG(DEBUG, wxT("Direction: ") << d.as_row());

        // Zrb krok optymalny
        rx = StepInDirection(function, rx, d);

        LOG(DEBUG, wxT("Step to: ") << rx.as_row());

        rIteration.AddTrialPoint(rx);
    }

    return function.Evaluate(rx);
}// CoordinateSearch


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
HookeJeevesOptimal::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik
    ColumnVector x(mStartingPoint); // Biecy punkt
    ColumnVector x_base(x);         // Punkt bazowy
    ColumnVector x_prev(x);         // Punkt w poprzedniej iteracji
    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;             // Licznik iteracji
    const FunctionBase& function = problem.Function();

    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);
    assert(mStartingPoint.nrows() > 0);

    LOG(DEBUG, mcpStopCondition->Name());
    LOG(DEBUG, wxT("eps: ") << mcpStopCondition->Epsilon());

    try
    {
        do
        {
            auto_ptr<HookeJeevesOptimalIteration> ap_result_iteration(
                    new HookeJeevesOptimalIteration(x));
            ap_result_iteration->rOldBasePoint() = x_base;

            ++step_count;

            x_prev = x;
            CoordinateSearch(function, x, *ap_result_iteration);
            LOG(DEBUG, wxT("CoordinateSearch: ") << x.as_row());

            // Wyznacz kierunek
            ColumnVector d(x_base - x);

            // Nowy punkt bazowy oraz kolejny punkt
            x_base = x;
            x = StepInDirection(function, x, d);

            ap_result_iteration->rBasePoint() = x_base;

            LOG(DEBUG, wxT("x_base: ") << x_base.as_row());
            LOG(DEBUG, wxT("Step to: ") << x.as_row());

            ap_result_iteration->rEndPoint() = x;

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));

            NotifyYield();
        } while (!mcpStopCondition->IsMet(x_prev, x, function)
                && (step_count < STEP_LIMIT));
    }// try
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    // TODO: x, czy x_base?
    ap_result->rFinalResult() = x;

    return ap_result;
}// Optimize

wxString HookeJeevesOptimal::mName(_("Hooke & Jeeves' Optimal"));

// Utwrz i zarejestruj prototyp.
HookeJeevesOptimal HookeJeevesOptimal::mHookeJeevesOptimal;
