#ifndef _HookeJeevesOptimal_h
#define _HookeJeevesOptimal_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DistanceCondition.h"
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../MethodWithLineSearch.h"
#include "../ProblemBase.h"
#include "../Result.h"
#include "../ui/MethodWithLineSearchPanel.h"

#include "HookeJeevesOptimalIteration.h"

// FORWARD REFERENCES
//

/**  Metoda Hooke'a i Jeevesa z krokiem optymalnym.
 *
 * Metoda wyznacza kierunek za pomoc krokw prbnych wzdu osi, a nastpnie
 * wykonuje krok roboczy w tym kierunku. W odrnieniu od pierwotnej wersji
 * metody, w tym wariancie dugoci krokw prbnych i roboczego s wyznaczane
 * przez metod optymalizacji w kierunku. Oznacza to, e kady krok jest udany,
 * dlatego nigdy nie ma potrzeby cofania si do punktu bazowego.
 *
 * Warunek stopu jest oparty na porwnaniu dwch ostatnich krokw, zastosowana
 * moe by jedna z klas pochodnych StandardStopCondition.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x^0\f$ - dowolnie wybrany punkt startowy,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 * \f$\varepsilon\f$ - dokadno,<br>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami.<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$i\f$ - indeks aktualnego kierunku,<br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$D = d^{(i)},\: i=1,\ldots,n\f$ - aktualna baza kierunkw,<br>
 * \f$x^i\f$ - aktualny punkt prbny,<br>
 * \f$x^k_{min}\f$ - najlepszy punkt w k-tym etapie,<br>
 * \f$x_b^k\f$ - punkt bazowy dla krokw prbnych w k-tym etapie,<br>
 * \f$d\f$ - kierunek kroku roboczego.
 *
 * <br><b>Procedura.</b><br>
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ D = I(n), \mbox{ gdzie } I(n) - \mbox{ macierz jednostkowa }
 * n \times n , \f]
 * \f[ x^1_{min} = x^0, \f]
 * \f[ x_b^1 = x^0, \f]
 * \f[ k = 1 , \f]
 * \f[ i = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Przy pomocy metody optymalizacji w kierunku wyznaczamy optymaln dugo
 * kroku \f$\tau^{(i)}\f$ i wykonujemy krok prbny wzdu i-tej osi:
 * \f[ \tau^{(i)} = \arg \min_{\tau^{(i)}} f(x^k_{min}
 * + \tau^{(i)} d^{(i)}) . \f]
 * \f[ x^i = x^k_{min} + \tau^{(i)} d^{(i)} . \f]
 *
 * Zapamitujemy punkt prbny jako najlepszy:
 * \f[ x^k_{min} = x^i, \f]
 * a nastpnie przechodzimy do kroku 2.<br>
 *
 * <br><b>Krok 2:</b><br>
 * Sprawdzamy, czy wykonano kroki prbne we wszystkich kierunkach:<br>
 * Jeli \f$i < n\f$, podstawiamy \f$i = i + 1\f$ i ponownie przechodzimy
 * do kroku 1.<br>
 * Jeli \f$i = n\f$, przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Zapamitujemy aktualny punkt jako punkt bazowy dla nastpnego
 * etapu:
 * \f[ x_b^{k+1} = x^k_{min}. \f]
 * Wyznaczamy kierunek kroku roboczego:
 * \f[ d = x^k_{min} - x_b^k \f]
 * i wykonujemy krok roboczy o optymalnej dugoci w tym kierunku:
 * \f[ \tau = \arg \min_{\tau} f(x^k_{min} + \tau d) . \f]
 * \f[ x^{k+1}_{min} = x^k_{min} + \tau d . \f]
 * Przechodzimy do kroku 4.
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy warunek stopu:<br>
 * Jeli jest on speniony, koczymy dziaanie. Wynikiem jest
 * \f$ x^{k+1}_{min} \f$.<br>
 * W przeciwnym wypadku podstawiamy \f$k = k+1\f$, \f$i = 1\f$
 * i przechodzimy do kroku 1.
 *
 * @see HookeJeevesDiscrete, StandardStopCondition
 */
class HookeJeevesOptimal
    : public MethodWithLineSearch
{
public:

    /** Panel ustawie metody.
     */
    friend class MethodWithLineSearchPanel<HookeJeevesOptimal>;

// LIFECYCLE

    /** Destruktor.
     */
    ~HookeJeevesOptimal(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    HookeJeevesOptimal(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    HookeJeevesOptimal(const HookeJeevesOptimal& from);


    /** Znajd kierunek.
     *
     * Wykonuje kroki wzdu osi. Stosowana jest metoda optymalizacji w
     * kierunku, znajdowane s wic punkty o minimalnej wartoci funkcji.
     *
     * Kroki s dodawane do wyniku.
     *
     * @param function Funkcja celu.
     * @param rx Referencja na biecy punkt, wok ktrego przeprowadzane jest
     * poszukiwanie. Zostaje ustawiony na punkt, ktry jest wynikiem
     * poszukiwania.
     * @param iteration Iteracja, do ktrej dodane maj zosta kroki (w wyniku).
     * @param rResult Referencja na wynik.
     *
     * @return Warto funkcji w <code>rx</code>.
     */
    double                          CoordinateSearch
                                       (const FunctionBase&    function,
                                        ColumnVector&          rx,
                                        HookeJeevesOptimalIteration& rIteration)
                                    const;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    static wxString       mName;                  ///< Nazwa metody.
    static HookeJeevesOptimal   mHookeJeevesOptimal;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _HookeJeevesOptimal_h_
