#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../text-util.h"

#include "Marquardt.h"  // class implemented
#include "MarquardtIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Marquardt::~Marquardt()
{
}// ~Marquardt

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
Marquardt::Clone() const
{
    return auto_ptr<Method>(new Marquardt(*this));
}// Clone

/* virtual */ void
Marquardt::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
Marquardt::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
Marquardt::Name() const
{
    return Marquardt::mName;
}// Name

/* virtual */ wxString&
Marquardt::rName()
{
    return Marquardt::mName;
}// rName

/* virtual */ MethodIdType
Marquardt::Id() const
{
    return Marquardt::ClassId();
}// Id

/* static */ MethodIdType
Marquardt::ClassId()
{
    const static MethodIdType MarquardtId = Method::GenerateId();

    return MarquardtId;
}// ClassId

/* virtual */ wxString
Marquardt::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << LambdaDescriptionMarquardt(mInitialLambda);
    description << wxT('\n');
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
Marquardt::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL &&
            pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
Marquardt::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

Marquardt::Marquardt()
    : Method(),
      mInitialLambda(10000, 0),
      mStartingPoint(0),
      mInitialStartingPoint(0),
      mcpStopCondition()
{
}// Marquardt

Marquardt::Marquardt(const Marquardt& from)
    : Method(from),
      mInitialLambda(from.mInitialLambda),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mcpStopCondition(from.mcpStopCondition)
{
    LOG(ERROR,wxT("Co to ma znaczyc?!"))
}// Marquardt


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
Marquardt::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result>                ap_result(new Result(1));
    auto_ptr<MarquardtIteration>    ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = dynamic_cast<const FunctionBase&>
                                                       (problem.Function());
        assert(function.IsDifferentiable());

        const unsigned int dimensions = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);



        // Biecy punkt.
        ColumnVector        x(mStartingPoint);
        // Punkt z poprzedniej iteracji.
        // (inicjowany wektorem zerowym, aby moliwe byo
        // sprawdzanie przed iteracj wszystkich warunkw stopu).
        ColumnVector        previous_x(NullPoint);
        /* Kandydat na nastpny punkt x (akceptowany gdy warto funkcji celu
         * jest w nim mniejsza ni w punkcie biecym.
         */
        ColumnVector        trial_x(x);
        /* Wektor przesunicia punktu x wyliczany na podstawie wzoru:
         * step = -[H(x) + lambda * I]^(-1) * G(x), gdzie:
         * H - macierz hesjanu w punkcie x
         * I - macierz jednostkowa
         * G - gradient w punkcie x
         */
        ColumnVector        step(dimensions);
        /* Macierz tymczasowa obliczana ze wzoru:
         * -[H(x) + lambda * I]^(-1)
         */
        SquareMatrix        temp_matrix;

        // Macierz hesjanu.
        SymmetricMatrix     hessian(dimensions);
        // Gradient.
        ColumnVector        gradient(dimensions);
        // Macierz jednostkowa.
        IdentityMatrix      I(dimensions);

        // Wspczynnik lambda wystpujcy we wzorze na wektor przesunicia  x.
        double              lambda = mInitialLambda;
        bool                point_accepted;

        LOG(DEBUG,wxT("-------------------------------------"));



        /* Gradient wyliczamy przed pierwsz iteracj algorytmu, gdy moliwe
         * jest, e punkt startowy jest akceptowalnie blisko punktu minimum.
         */
        gradient = function.Gradient(x);
        hessian  = function.Hessian(x);

        ap_iteration.reset(new MarquardtIteration(x,
                                                  gradient,
                                                  hessian,
                                                  lambda));
        LOG(DEBUG, wxT("ITERATION DESCRIPTION"));
        LOG(DEBUG, wxT("Starting point: ")
                            << ap_iteration->StartingPoint().as_row());
        LOG(DEBUG, wxT("Gradient: ") << ap_iteration->Gradient().as_row())
        LOG(DEBUG, wxT("Hessian:\n") << ap_iteration->Hessian());

/*
        LOG(DEBUG,wxT("Iteracja wstepna\n"));
        LOG(DEBUG,wxT("lambda = ") << lambda << wxT("\n"));
        LOG(DEBUG,wxT("Punkt:\n") << x);
        LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                << function.Evaluate(x) << wxT("\n"));
        LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
        LOG(DEBUG,wxT("Hesjan:\n") << hessian);
        LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());
*/
        LOG(DEBUG,wxT("-------------------------------------"));



        while (step_count < STEP_LIMIT &&
               !mcpStopCondition->IsMet(previous_x, x, function,
                  gradient.norm_Frobenius()))
        {
            ++step_count;

            LOG(DEBUG,wxT("Iteracja ") << step_count << wxT("\n"));

            point_accepted  = false;


            while (!point_accepted)
            {
                temp_matrix = hessian + lambda * I;

                if (temp_matrix.determinant() == 0)
                   throw OptimizationError
                          (_("Matrix singular, unable to invert"), ap_result);

                step = -1.0 * (temp_matrix.i() * gradient);

                LOG(DEBUG,wxT("Wektor przesuniecia punktu:\n") << step);

                trial_x = x + step;

                ap_iteration->rMarquardtSteps().push_back(step);
                ap_iteration->rTrialPoints().push_back(trial_x);

                LOG(DEBUG, wxT("ITERATION DESCRIPTION"));
                LOG(DEBUG, wxT("Starting point: ")
                                    << ap_iteration->StartingPoint().as_row());
                LOG(DEBUG, wxT("Gradient: ") << ap_iteration->Gradient().as_row())
                LOG(DEBUG, wxT("Hesian:\n") << ap_iteration->Hessian());
/*
                LOG(DEBUG,wxT("Punkt probny:\n") << trial_x);
                LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                        << function.Evaluate(trial_x) << wxT("\n"));
*/
                /* Jeli po wykonaniu kroku nastpia poprawa, to akceptujemy
                 * nowy x oraz zmniejszamy wspczynnik lambda, czyli zwikszamy
                 * znaczenie wartoci hesjanu we wzorze na wektor przesunicia.
                 */
                if (function.Evaluate(trial_x) < function.Evaluate(x))
                {
                    lambda /= 2;

                    previous_x = x;
                    x = trial_x;
                    gradient = function.Gradient(x);
                    hessian  = function.Hessian(x);

                    point_accepted  = true;

                    ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));
/*
                    LOG(DEBUG,wxT("PUNKT ZAAKCEPTOWANY, zmniejszamy lambde\n"));
                    LOG(DEBUG,wxT("lambda = ") << lambda << wxT("\n"));
                    LOG(DEBUG,wxT("Nowy punkt:\n") << x);
                    LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
                    LOG(DEBUG,wxT("Hesjan:\n") << hessian);
                    LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());*/
                }
                /* Jeli wyliczony kierunek nie zapewniby poprawy, to
                 * zwikszamy wspczynnik lambda.
                 */
                else
                {
                    LOG(DEBUG,wxT("PUNKT ODRZUCONY, zwiekszamy lambde\n"));
                    LOG(DEBUG,wxT("lambda = ") << lambda << wxT("\n"));

                    lambda *= 2;
                }

                NotifyYield();
            }

            ap_iteration.reset(new MarquardtIteration(x,
                                                      gradient,
                                                      hessian,
                                                      lambda));

            LOG(DEBUG,wxT("-------------------------------------"));

            NotifyYield();
        }

        ap_result->rFinalResult() = x;
    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString Marquardt::mName(_("Marquardt's"));

// Utwrz i zarejestruj prototyp.
Marquardt Marquardt::mMarquardt;
