/** Metoda Marquardta.
 *
 * Metoda Marquardta jest udan prb poczenia korzystnych waciwoci metod
 * najwikszego spadku i Newtona. Metoda Newtona zbiega si szybko w pobliu
 * minimum, natomiast metoda Cauchy'ego w duym od niego oddaleniu.
 *
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ k \f$ - numer aktualnej iteracji<br>
 * \f$ \nabla f(x) \f$ - gradient w punkcie \f$ x \f$<br>
 * \f$ \nabla^2 f(x) \f$ - macierz hesjanu w punkcie \f$ x \f$<br>
 * \f$ I \f$ - macierz jednostkowa<br>
 * \f$ x_{kand} \f$ - kandydat na punkt w nastpnej iteracji (odrzucany po kroku
 * prbnym gdy nie nastpio zmniejszenie wartoci funkcji celu)<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja celu<br>
 * \f$ x^0 \f$ - punkt startowy<br>
 * \f$ \lambda \f$ - wspczynnik wagi macierzy jednostkowej<br>
 * \f$ \varepsilon \f$ - wymagana dokadno<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|\nabla f(x^k)\| < \varepsilon\f$</li>
 * <li>\f$\|\nabla f(x^k)\|^2 < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$.</li>
 * </ul>
 * <br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * Podstaw \f$ k=0 \f$.<br>
 * Wylicz wartoci funkcji, gradientu i macierzy hesjanu w punkcie
 * \f$ x^0 \f$.<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Sprawd czy speniony jest warunek stopu. Jeli tak, to przerwij dziaanie
 * algorytmu i przedstaw \f$ x^k \f$ jako wynik. W przeciwnym wypadku przejd do
 * kroku 2.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Oblicz:
 * \f[ x_{kand} = x^k + \tau^k \f]
 * gdzie:<br>
 * \f[ \tau^k = -[\nabla^2 f(x^k) + \lambda I]^{-1}
 * \nabla f(x^k) \f]
 * <br>
 * <b>Krok 3</b>:<br>
 * Jeli \f$ f(x_{kand}) < f(x^k) \f$, to przejd do kroku 4. W przeciwnym
 * wypadku przejd do kroku 5.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Podstaw:
 * \f[ \lambda = \frac{1}{2} \lambda \f]
 * \f[ x^{k+1} = x_{kand} \f]
 * \f[ k=k+1 \f]
 * , a nastpnie przejd do kroku 1.<br>
 * <br>
 * <b>Krok 5</b>:<br>
 * Podstaw \f$ \lambda = 2 \lambda \f$ i przejd do kroku 2.<br>
 * <br>
 * <br>
 * <b>Uwagi</b>:<br>
 * W algorytmnie nie ma minimalizacji w kierunku, gdy dugo kroku reguluje
 * ju wspczynnik \f$ \lambda \f$.
 * Na pocztku dziaania algorytmu warto wspczynnika \f$ \lambda \f$
 * musi by bardzo
 * dua aby wartoci na przektnej macierzy jednostkowej miay wiksze znaczenie
 * dla sumy we wzorze na kierunek. Z kadym udanym krokiem (tzn. takim,
 * dla ktrego warto funkcji w x maleje) lambda zmniejsza si o poow a
 * do momentu, w ktrym to wartoci hesjanu staj si waniejsze dla wyznaczania
 * kierunku.
 * Podobnie jak w innych metodach, na pocztku dziaania algorytmu zmienne
 * okrelajce odlego dwch ostatnich punktw i rnic wartoci funkcji w
 * tych punktach maj wartoci DBL_MAX. Wynika z tego, e algorytm moe si
 * zatrzyma bez wykonania adnej iteracji jedynie w przypadku uycia kryterium
 * "gradientowego". Jest to zachowanie jak najbardziej podane.
 *
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003<\i>
 */

#ifndef _Marquardt_h
#define _Marquardt_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../Method.h"
#include "../ProblemBase.h"
#include "../StandardStopCondition.h"
#include "../UnconstrainedProblem.h"

// FORWARD REFERENCES
//


class Marquardt
    : public Method
{
public:

    friend class MarquardtPanel;

// LIFECYCLE

    /** Destruktor.
     */
    ~Marquardt(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void        UpdateStartingConditions(const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem)
                                                                        const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    Marquardt(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Marquardt(const Marquardt& from);

    /** Wymagana dokadno
     */
    DoubleParameter mInitialLambda;

    /** Punkt startowy
     */
    mutable ColumnVector                        mStartingPoint;

    ColumnVector                                mInitialStartingPoint;

    /** Warunek stopu.
     */
    CountedPtr<const StandardStopCondition>     mcpStopCondition;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);


    /** Nazwa metody
     */
    static wxString   mName;
    /** Egzemplarz metody
     */
    static Marquardt        mMarquardt;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif // _Marquardt_h
