#include <precomp.h>

#include "../Configuration.h"
#include "../UnconstrainedProblem.h"
#include "../text-util.h"
#include "NelderMead.h"  // class implemented

#include <cassert>

LOGGER(WARNING);

using namespace std;

////////////////////////// class NelderMead //////////////////////////////////

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NelderMead::~NelderMead()
{
}// ~NelderMead

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
NelderMead::Clone() const
{
    return auto_ptr<Method>(new NelderMead(*this));
}// Clone

/* virtual */ void
NelderMead::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
NelderMead::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
NelderMead::Name() const
{
    return NelderMead::mName;
}

/* virtual */ wxString&
NelderMead::rName()
{
    return NelderMead::mName;
}// rName

/* virtual */ MethodIdType
NelderMead::Id() const
{
    return NelderMead::ClassId();
}// Id

/* static */ MethodIdType
NelderMead::ClassId()
{
    const static MethodIdType NelderMeadId = Method::GenerateId();

    return NelderMeadId;
}// ClassId

/* virtual */ wxString
NelderMead::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;
    wxArrayString starting_simplex_descriptions;

    parameter_descriptions.Add(ReflectionDescriptionNM(
                                mReflectionCoefficient));
    parameter_descriptions.Add(ExpansionDescriptionNM(
                                mExpansionCoefficient));
    parameter_descriptions.Add(ContractionDescriptionNM(
                                mContractionCoefficient));
    parameter_descriptions.Add(ShrinkageDescriptionNM(
                                mShrinkageCoefficient));

    starting_simplex_descriptions.Add(SideLengthDescriptionNM(
                                        mSideLength));
    starting_simplex_descriptions.Add(CenterDescriptionNM(
                                        mInitialStartingPoint));

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);
    description << wxT('\n');
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << AlignedParameterDescriptions(starting_simplex_descriptions);

    return description;
}// ToString
//============================= INQUIRY    ===================================
//
/* virtual */ bool
NelderMead::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

NelderMead::NelderMead()
    : Method(),
      mReflectionCoefficient(1.0, 0, true),
      mExpansionCoefficient(2.0, 0, true),
      mContractionCoefficient(0.5, 0, true, 1, true),
      mShrinkageCoefficient(0.5, 0, true, 1, true),
      mStartingPoint(0),
      mInitialStartingPoint(0),
      mSideLength(1.0, 0, true),
      mcpStopCondition()
{
}// NelderMead

NelderMead::NelderMead(const NelderMead& from)
    : Method(from),
      mReflectionCoefficient(from.mReflectionCoefficient),
      mExpansionCoefficient(from.mExpansionCoefficient),
      mContractionCoefficient(from.mContractionCoefficient),
      mShrinkageCoefficient(from.mShrinkageCoefficient),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mSideLength(from.mSideLength),
      mcpStopCondition(from.mcpStopCondition)
{
}// NelderMead

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
NelderMead::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = problem.Function();

        unsigned int dimensions      = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);
        assert(dimensions >= 1);

        if(mReflectionCoefficient >= mExpansionCoefficient)
                throw OptimizationError(_("Invalid optimization parameters"));



        // Tworzenie pocztkowego sympleksu.
        NelderMead::Simplex     current_simplex(mStartingPoint,
                                                mSideLength,
                                                &function);
        assert((current_simplex.Size() > 1) &&
               (current_simplex.Size() == dimensions + 1));


        double      circumference                   = DBL_MAX;

        /* centroid - Punkt punkt cikoci sympleksu z wyczeniem punktu
         * najgorszego.
         */
        PointWithValue  centroid(dimensions);
        /* expanded - Punkt przesunity wzgldem punktu Reflexted o odlego
         * rwn (reflected - centroid).
         */
        PointWithValue  expanded(dimensions);
        // reflected - Najgorszy punkt odbity wzgldem punktu M.
        PointWithValue  reflected(dimensions);
        /* contracted - Punkt lecy po rodku odcinka [rWorst centroid] lub
         * [centroid reflected].
         */
        PointWithValue  contracted(dimensions);
        /* worst_substitute - punkt, ktry w przebiebu iteracji zastpi punkt
         * najgorszy.
         */
        PointWithValue  worst_substitute(dimensions);
        /* previous_worst - najgoszy punkt poprzedniego sympleksu.
         */
        PointWithValue  previous_worst(dimensions);

        LOG(DEBUG,wxT("-------------------------------------"));



        /* Gwna ptla koczy si gdy osignita zostanie podana dokadno
         * rozwizania lub gdy przekroczony zostanie limit krkw.
         */
        do
        {
            ++step_count;

            auto_ptr<NelderMeadIteration> ap_iteration
               (new NelderMeadIteration(current_simplex.rPoints()));

            LOG(INFO,wxT("Step no ") << step_count);

            //previous_max = current_simplex.rWorst().mValue;
            previous_worst  = current_simplex.rWorst();

            // rBest - punkt w sympleksie z najnisz wartoci funkcji.
            const PointWithValue& rBest = current_simplex.rBest();
            // rWorst - punkt w sympleksie z najwysz waroci funkcji.
            const PointWithValue& rWorst = current_simplex.rWorst();
            // rNextToWorst - punkt w sympleksie lepszy jedynie od rWorst.
            const PointWithValue& rNextToWorst = current_simplex.rNextToWorst();
            LOG(DEBUG,wxT("Best: \n")           << rBest.mPoint.AsRow());
            LOG(DEBUG,wxT("Value in Best: ")    << rBest.mValue << endl);
            LOG(DEBUG,wxT("Worst: \n")          << rWorst.mPoint.AsRow());
            LOG(DEBUG,wxT("Value in Worst: ")   << rWorst.mValue << endl);
            LOG(DEBUG,wxT("Next To Worst: \n")  << rNextToWorst.mPoint.AsRow());
            LOG(DEBUG,wxT("Value in Next To Worst: ")
                                                << rNextToWorst.mValue << endl);

            /* Wyliczanie rodka ciekoci sympleksu (z wyczeniem najgorszego
             * punktu)
             */
            centroid = current_simplex.Centroid(0, current_simplex.Size() - 2);

            LOG(DEBUG,wxT("Centroid: \n") << centroid.mPoint.AsRow());

            /* Odbijanie najgorszego punktu (rWorst) wzgldem rodka (centroid).
             */
            reflected.mPoint = centroid.mPoint + mReflectionCoefficient *
                               (centroid.mPoint - rWorst.mPoint);
            reflected.mValue = function.Evaluate(reflected.mPoint);
            ap_iteration->AddReflected(reflected);

            LOG(DEBUG,wxT("Reflected: \n") << reflected.mPoint.AsRow());
            LOG(DEBUG,wxT("Value in Reflected: ") << reflected.mValue << '\n');

            /* Jeli w punkcie odbitym funkcja przyjmuje najnisz warto, to..
             */
            if (reflected.mValue < rBest.mValue)
            {
                /* ...sprawdzamy czy podaj dalej w tym kierunku nie uzyskamy
                 * jeszcze lepszych rezultatw.
                 */
                expanded.mPoint = centroid.mPoint + mExpansionCoefficient *
                                  (centroid.mPoint - rWorst.mPoint);
                expanded.mValue = function.Evaluate(expanded.mPoint);
                ap_iteration->AddExpanded(expanded);

                LOG(DEBUG,wxT("Expanded point: \n") << expanded.mPoint.AsRow());
                LOG(DEBUG,wxT("Value in Expanded: ")
                                            << expanded.mValue << '\n');

                /* Jeli rzeczywicie rozcignicie sympleksu przynioso
                 * dodatkow popraw, to podstawiamy expanded za punkt
                 * najgorszy.
                 */
                if (expanded.mValue < reflected.mValue)
                {
                    LOG(DEBUG,wxT("Substituting expanded for worst"));
                    worst_substitute    = expanded;
                    current_simplex.SubstituteNewForWorst(expanded);
                    ap_iteration->SetSimplexModification
                                        (NelderMeadIteration::EXPANSION);
                }
                /* Jeli natomiast dalsza poprawa nie nastpia, to zadowalamy
                 * si podstawieniem za najgorszy punktu odbitego.
                 */
                else
                {
                    LOG(DEBUG,wxT("Substituting reflected for worst"));
                    worst_substitute    = reflected;
                    current_simplex.SubstituteNewForWorst(reflected);
                    ap_iteration->SetSimplexModification
                                        (NelderMeadIteration::REFLECTION);
                }

            }
            /* Jeli w odbitym punkcie warto funkcji nie jest najnisza, ale
             * jednak nastpia poprawa i punkt w nastpnej iteracji nie bdzie
             * znowu najgorszy, to zadowalamy si nim i podstawiamy go za
             * najgorszy.
             */
            else if (reflected.mValue >= rBest.mValue &&
                     reflected.mValue <  rNextToWorst.mValue)
            {
                LOG(DEBUG,wxT("Substituting reflected for worst"));
                worst_substitute    = reflected;
                current_simplex.SubstituteNewForWorst(reflected);
                ap_iteration->SetSimplexModification
                                    (NelderMeadIteration::REFLECTION);
            }
            /* Jeli poprawa nie nastpia albo jest bardzo nieznaczna
             * (poprawiony punkt mimo wszystko byby najgorszy w nowym
             * sympleksie), to dokonujemy cinicia sympleksu i sprawdzamy czy
             * minimum nie znajduje si gdzie pomidzy punktem najgorszym,
             * a jego odbiciem (nie moemy sprawdza punktu cikoci,
             * gdy potrzebujemy sympleksu wypukego).
             */
            else
            {
                /* Jeli punkt odbity jest lepszy od najgorszego, to sprawdzamy
                 * punkt pomidzy nim a rodkiem cikoci.
                 */
                if (reflected.mValue >= rNextToWorst.mValue &&
                    reflected.mValue <  rWorst.mValue)
                {
                    contracted.mPoint = reflected.mPoint -
                                        mContractionCoefficient *
                                        (reflected.mPoint - centroid.mPoint);
                    contracted.mValue = function.Evaluate(contracted.mPoint);
                    ap_iteration->AddContracted(contracted);

                    LOG(DEBUG,wxT("Contracted: \n")
                            << contracted.mPoint.AsRow());
                    LOG(DEBUG,wxT("Value in Contracted : ")
                            << contracted.mValue << '\n');

                    /* Jeli nowy punkt obiecywaby wiksz popraw ni odbity,
                     * to podmieniamy go za dotychczas najgorszy.
                     */
                    if (contracted.mValue <= reflected.mValue)
                    {
                        LOG(DEBUG,wxT("Substituting contracted for worst"));
                        worst_substitute    = contracted;
                        current_simplex.SubstituteNewForWorst(contracted);
                        ap_iteration->SetSimplexModification
                                            (NelderMeadIteration::CONTRACTION);
                    }
                    /* Jeli nowy punkt nie obiecywaby wikszej poprawy,
                     * dokonujemy skurczenia sympleksu w kierunku najlepszego
                     * punktu.
                     */
                    else
                    {
                        worst_substitute.mPoint
                            = (current_simplex.rWorst().mPoint
                            + current_simplex.rBest().mPoint) / 2;
                        worst_substitute.mValue
                            = function.Evaluate(worst_substitute.mPoint);
                        current_simplex.ShrinkTowardBest(mShrinkageCoefficient);
                        ap_iteration->SetSimplexModification
                                            (NelderMeadIteration::SHRINKAGE);
                    }
                }
                /* Jeli w punkcie odbitym warto funkcji jest rwna lub
                 * wiksza ni w dotychczas najgorszym, to nastpnego punktu
                 * szukamy pomidzy najgorszym, a rodkiem cikoci.
                 */
                else
                {
                    contracted.mPoint = rWorst.mPoint +
                                        mContractionCoefficient *
                                        (centroid.mPoint - rWorst.mPoint);
                    contracted.mValue = function.Evaluate(contracted.mPoint);
                    ap_iteration->AddContracted(contracted);

                    LOG(DEBUG,wxT("Contracted: \n")
                            << contracted.mPoint.AsRow());
                    LOG(DEBUG,wxT("Value in Contracted : ")
                            << contracted.mValue << '\n');

                    /* Jeli nowy punkt obiecywaby pewn, minimaln popraw, to
                     * podmieniamy go za dotychczas najgorszy.
                     */
                    if (contracted.mValue < rWorst.mValue)
                    {
                        LOG(DEBUG,wxT("Substituting contracted for worst"));
                        worst_substitute    = contracted;
                        current_simplex.SubstituteNewForWorst(contracted);
                        ap_iteration->SetSimplexModification
                                            (NelderMeadIteration::CONTRACTION);
                    }
                    /* Jeli nowy punkt nie obiecywaby adnej poprawy,
                     * dokonujemy skurczenia sympleksu w kierunku najlepszego
                     * punktu.
                     */
                    else
                    {
                        worst_substitute.mPoint
                            = (current_simplex.rWorst().mPoint
                            + current_simplex.rBest().mPoint) / 2;
                        worst_substitute.mValue
                            = function.Evaluate(worst_substitute.mPoint);
                        current_simplex.ShrinkTowardBest(mShrinkageCoefficient);
                        ap_iteration->SetSimplexModification
                                            (NelderMeadIteration::SHRINKAGE);
                    }
                }

            }

            ap_iteration->AddNewSimplex(current_simplex.rPoints());
            ap_result->AddIteration(auto_ptr<const IterationData>
                                                            (ap_iteration));

            circumference               = current_simplex.Circumference();

            LOG(INFO,wxT("--------------------------------------"));

            NotifyYield();
        }
        while ((step_count < STEP_LIMIT)
               && !mcpStopCondition->IsMet(current_simplex.rBest(),
                                           current_simplex.rWorst(),
                                           previous_worst,
                                           worst_substitute,
                                           circumference));

        ap_result->rFinalResult() = current_simplex.rBest().mPoint;
        LOG(DEBUG,wxT("rFinalResult: ") << ap_result->FinalResult().as_row());
    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"));
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

/*static*/ wxString NelderMead::mName(_("Nelder-Mead's"));

// Utwrz i zarejestruj prototyp.
/*static*/ NelderMead NelderMead::mNelderMead;

//////////////////////////////////////////////////////////////////////////////





///////////////////////// class NelderMead::Simplex //////////////////////////

/////////////////////////////// PUBLIC    ////////////////////////////////////

//============================= LIFECYCLE ====================================

NelderMead::Simplex::Simplex()
                            : mPoints(),
                              mpFunction(NULL)
{
}

NelderMead::Simplex::Simplex(const FunctionBase* function)
                            : mPoints(),
                              mpFunction(function)
{
}

NelderMead::Simplex::Simplex(const ColumnVector&    centralPoint,
                             const DoubleParameter& sideLength,
                             const FunctionBase*    pFunction)
                            : mPoints(),
                              mpFunction(pFunction)
{
    GenerateVertices(centralPoint, sideLength);

    Sort();
}

NelderMead::Simplex::Simplex(const vector<ColumnVector>&  startingPoints,
                             const FunctionBase*          pFunction)
                            : mPoints(),
                              mpFunction(pFunction)
{
    vector<ColumnVector>::const_iterator iter;

    for (iter = startingPoints.begin();
         iter != startingPoints.end();
         ++iter)
    {
        mPoints.push_back
            (PointWithValue(*iter, mpFunction->Evaluate(*iter)));
    }

    Sort();

}

NelderMead::Simplex::Simplex(const NelderMead::Simplex& from)
                            : mPoints(from.mPoints),
                              mpFunction(from.mpFunction)
{
}

//============================= OPERATORS ====================================

NelderMead::Simplex&
NelderMead::Simplex::operator= (const NelderMead::Simplex& from)
{
    mPoints     = from.mPoints;
    mpFunction  = from.mpFunction;

    return *this;
}


//============================= OPERATIONS ===================================

/* static */ double
NelderMead::Simplex::Delta1(const   DoubleParameter&    sideLength,
                                    size_t              dimensions)
{
    return sideLength * ((sqrt(dimensions + 1.0) + dimensions - 1)
                                                / (dimensions * sqrt(2.0)));
}

/* static */ double
NelderMead::Simplex::Delta2(const   DoubleParameter&    sideLength,
                                    size_t              dimensions)
{
    return sideLength * ((sqrt(dimensions + 1.0) - 1)
                                                / (dimensions * sqrt(2.0)));
}

void
NelderMead::Simplex::Sort()
{
    assert(mPoints.size() > 1);

    /* Uycie procedury stable_sort umoliwia spenienie postulatu
     * rozwizywania sytuacji remisowych zamieszczonego w [2], ktry
     * gosi, e jeli nastpio skurczenie sympleksu w iteracji
     * k, to najlepszy punkt z tej iteracji pozostaje nim
     * iteracji k+1, chyba, e pojawi si punkty lepsze od
     * niego.
     */
    stable_sort(mPoints.begin(), mPoints.end());
}

const PointWithValue
NelderMead::Simplex::Centroid(size_t    firstIndex,
                              size_t    lastIndex) const
{
        PointWithValue  target_point(mPoints.size() - 1);
        size_t          i;

        target_point.mPoint = 0.0;

        for (i = firstIndex; i <= lastIndex; ++i)
        {
            target_point.mPoint += mPoints[i].mPoint;
        }

        target_point.mPoint /= (lastIndex - firstIndex + 1);
        target_point.mValue  = mpFunction->Evaluate(target_point.mPoint);

        return target_point;
}

void
NelderMead::Simplex::ShrinkTowardBest(double shrinkageCoefficient)
{
    LOG(DEBUG,wxT("Shrinking"));

    size_t  i;

    assert(mPoints.size() > 1);

    for (i = 1; i < mPoints.size(); ++i)
    {
        mPoints[i].mPoint -= shrinkageCoefficient *
                            (mPoints[i].mPoint - mPoints[0].mPoint);
        mPoints[i].mValue = mpFunction->Evaluate(mPoints[i].mPoint);
    }

    Sort();
}

void
NelderMead::Simplex::SubstituteNewForWorst(const PointWithValue& newPoint)
{
    LOG(DEBUG,wxT("Substituting point:\n") << newPoint.mPoint.as_row()
            << wxT("for:\n") << mPoints.back().mPoint.as_row());

    size_t          i = mPoints.size() - 1;
    PointWithValue  temp;

    assert(mPoints.size() > 1);

    mPoints.back() = newPoint;

    while (i > 0 && mPoints[i - 1].mValue > mPoints[i].mValue)
    {
        temp            = mPoints[i - 1];
        mPoints[i - 1]  = mPoints[i];
        mPoints[i]      = temp;

        --i;
    }

}

//============================= ACCESS     ===================================

const PointWithValue&
NelderMead::Simplex::rBest() const
{
    //return *mBestPosition;
    return mPoints[0];
}

const PointWithValue&
NelderMead::Simplex::rWorst() const
{
    //return *mWorstPosition;
    return mPoints.back();
}

const PointWithValue&
NelderMead::Simplex::rNextToWorst() const
{
    assert(mPoints.size() > 1);
    //return *mNextToWorstPosition;
    return mPoints[mPoints.size() - 2];
}

const vector<PointWithValue>&
NelderMead::Simplex::rPoints() const
{
    return mPoints;
}

//============================= INQUIRY    ===================================

double
NelderMead::Simplex::Circumference() const
{
    size_t          i, j;
    double          result = 0.0;
    ColumnVector    temp(mPoints[0].mPoint.nrows());

    for (i = 0; i < (mPoints.size() - 1); ++i)
        for (j = i + 1; j < mPoints.size(); ++j)
        {
            temp    = mPoints[i].mPoint - mPoints[j].mPoint;
            result  += temp.norm_Frobenius();
        }

    return result;
}

/* virtual */ StopConditions
NelderMead::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::NelderMeadStopConditions());

    return conditions;
}// AllowedStopConditions


double
NelderMead::Simplex::ValueSpan() const
{

    assert(mPoints.size() >= 2);
    assert(mPoints.back().mValue > mPoints[0].mValue);

    return mPoints.back().mValue - mPoints[0].mValue;
}

size_t
NelderMead::Simplex::Size() const
{
    return mPoints.size();
}

void
NelderMead::Simplex::GenerateVertices(const ColumnVector        centralPoint,
                                      const DoubleParameter&    sideLength)
{
    size_t dimensions = centralPoint.nrows();
    assert(dimensions > 0);
    assert(sideLength > 0);

    mPoints.clear();

    unsigned int    i, j;
    double          delta1, delta2;

    // Metoda generacji sympleksu w przestrzeni N-wymiarowej polega na
    // utworzeniu N punktw rwnoodlegych od punktu bazowego.

    // W pierwszym kroku tworzymy pierwszy punkt.
    for (i = 0; i <= dimensions; ++i)
        mPoints.push_back(PointWithValue(centralPoint));

    // Wsprzdne nastpnych punktw przesuwamy o warto <code>delta1</code>
    // lub <code>delta2</code> wzgldem wsprzdnych pierwszego punktu.
    delta1 = Delta1(sideLength, dimensions);
    delta2 = Delta2(sideLength, dimensions);

    for (i = 1; i <= dimensions; ++i)
    {
        for (j = 1; j <= dimensions; ++j)
        {
            if (i != j)
                mPoints[i].mPoint(j) += delta1;
            else
                mPoints[i].mPoint(j) += delta2;
        }
    }

    // W ostatnim kroku przesuwamy wszystkie punkty o odlego punktu
    // rodkowego od punktu bazowego.
    ColumnVector    centroid = Centroid(0, Size() - 1).mPoint;
    ColumnVector    shift = centroid - mPoints[0].mPoint;
    //shift2 = ((dimensions - 1) * delta1 + delta2) / (dimensions + 1);

    for (i = 0; i <= dimensions; ++i)
    {
        mPoints[i].mPoint -= shift;
        mPoints[i].mValue =  mpFunction->Evaluate(mPoints[i].mPoint);
    }

}

/////////////////////////////// PROTECTED ////////////////////////////////////
/////////////////////////////// PRIVATE   ////////////////////////////////////
