/** Algorytm optymalizacji metod Neldera-Meada.
 *
 * W metodzie Neldera-Meada po wykresie funkcji peza sympleks zoony z
 * <i>N+1</i> punktw, gdzie N oznacza liczb wymiarw problemu.
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ S^k \f$ - zbir punktw sympleksu w <i>k</i>-tej iteracji<br>
 * \f$ S_i \f$ - zbir indeksw punktw nalecych do \f$ S^k \f$.<br>
 * \f$ \check{S}^k = S^k \setminus \{x^k_w\} \f$<br>
 * \f$ \check{S}_i \f$ - zbir indeksw punktw nalecych do
 * \f$ \check{S}^k \f$<br>
 * \f$ x^k_b \f$ - najlepszy punkt sympleksu w <i>k</i>-tej iteracji<br>
 * \f$ x^k_w \f$ - najgorszy punkt sympleksu w <i>k</i>-tej iteracji<br>
 * \f$ x^k_v \f$ - prawie najgorszy punkt sympleksu w <i>k</i>-tej iteracji<br>
 * \f$ \bar{x} \f$ - rodek cikoci zbioru \f$ \check{S}^k \f$<br>
 * \f$ x_r \f$ - punkt powstay poprzez odbicie najgorszego punktu sympleksu
 * wzgldem punktu \f$ \bar{x} \f$<br>
 * \f$ x_e \f$ - punkt przesunity jeszcze dalej w tym samym kierunku, w ktrym
 * nastpio odbicie najgorszego punktu sympleksu przy tworzeniu punktu
 * \f$ x_r \f$<br>
 * \f$ x_c \f$ - punkt znajdujcy si pomidzy najgorszym punktem sympleksu, a
 * rodkiem cikoci pozostaych jego punktw lub tyme rodkiem cikoci, a
 * punktem \f$ x_r \f$<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - funkcja celu<br>
 * \f$ r \f$ - wspczynnik odbicia (\f$ r > 0 \f$, domylnie 1)<br>
 * \f$ e \f$ - wspczynnik rozcignicia (\f$ e > r \f$, domylnie 2)<br>
 * \f$ c \f$ - wspczynnik cinicia (\f$ 0 < c < 1 \f$, domylnie
 * \f$ \frac{1}{2} \f$)<br>
 * \f$ s \f$ - wspczynnik skurczenia (\f$ 0 < s < 1 \f$, domylnie
 * \f$ \frac{1}{2} \f$)<br>
 * \f$ x^0 \f$ - rodek pocztkowego sympleksu<br>
 * \f$ d \f$ - dugo krawdzi pocztkowego sympleksu<br>
 * <br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * W tym kroku wyliczamy wsprzdne wierzchokw pocztkowego sympleksu w ten
 * sposb, by punkt \f$ x^0 \f$ znalaz si w jego rodku.<br>
 * Ponisze wzory potrzebne do utworzenia regularnego sympleksu zaczerpnito
 * z [1].<br>
 * Wzr na wyliczenie <i>j</i>-tej wsprzdnej <i>i</i>-tego wierzchoka
 * sympleksu ma posta:<br>
 * \f[ x^{(j)}_i = \left\{ \begin{array}{ll}
 *                         x^{(j)}_0 + \delta_1 \mbox{ gdy } i \not= j;\\
 *                         x^{(j)}_0 + \delta_2 \mbox{ gdy } i = j.
 *                         \end{array} \right. \f]
 * Wartoci wspczynnika \f$ \sigma_1 \f$ i \f$ \sigma_2 \f$ wyliczamy ze
 * wzorw:<br>
 * \f[ \delta_1 = \left(\frac{\sqrt{N+1}+N-1}{N\sqrt{2}}\right)\frac{d}{2} \f]
 * \f[ \delta_2 = \left(\frac{\sqrt{N+1}-1}{N\sqrt{2}}\right)\frac{d}{2} \f]
 * W powyszy sposb otwrzymujemy zbir \f$ S^1 \f$.<br>
 * W wierzchokach aktualnego sympleksu wyliczamy wartoci funkcji.<br>
 * Wybieramy szczeglne punkty sympleksu: \f$ x^1_b \f$, \f$ x^1_w \f$
 * i \f$ x^1_v \f$.
 * Zapamitujemy rownie wartoci funkcji \f$ F_b = f(x^1_b) \f$,
 * \f$ F_w = f(x^1_w) \f$ oraz \f$ F_v = f(x^1_v) \f$<br>
 * Podstawiamy \f$ k = 1 \f$ i przechodzimy do kroku 1.<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Wyliczamy rodek cikoci punktw sympleksu z pominiciem punktu
 * najgorszego wg wzoru:
 * \f[ \bar{x} = \frac{ \sum_{i \in \check{S}_i} x_i }{N} \f]
 * Przechodzimy do kroku 2.
 * <br>
 * <b>Krok 2</b>:<br>
 * Wyznaczamy punkt \f$ x_r \f$ poprzez odbicie najgorszego punktu sympleksu
 * wzgldem rodka cikoci:
 * \f[ x_r = \bar{x} + r(\bar{x}-x^k_w) \f]
 * Wyliczamy warto funkcji celu w tym punkcie.<br>
 * Jeli \f$ F_r < F_b \f$, to przechodzimy do kroku 3. W przeciwnym przypadku
 * przechodzimy do kroku 5.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Wyznaczamy punkt \f$ x_e \f$ zgodnie ze wzorem:
 * \f[ x_e = \bar{x} + e(\bar{x}-x^k_w) \f]
 * Wyliczamy warto funkcji w tym punkcie.<br>
 * Jeli \f$ F_e < F_r \f$ to przechodzimy do kroku 4a. W przeciwnym wypadku
 * przechodzimy do kroku 4b.<br>
 * <br>
 * <b>Krok 4a</b>:<br>
 * W sympleksie zastpujemy punkt \f$ x^k_w \f$ punktem \f$ x_e \f$ i
 * przechodzimy do kroku 12.<br>
 * <br>
 * <b>Krok 4b</b>:<br>
 * W sympleksie zastpujemy punkt \f$ x^k_w \f$ punktem \f$ x_r \f$ i
 * przechodzimy do kroku 12.<br>
 * <br>
 * <b>Krok 5</b>:<br>
 * Jeli \f$ F_b \leq F_r < F_v \f$, to przechodzimy do kroku 6. W przeciwnym
 * wypadku przechodzimy do kroku 7.<br>
 * <br>
 * <b>Krok 6</b>:<br>
 * W sympleksie zastpujemy punkt \f$ x^k_w \f$ punktem \f$ x_r \f$ i
 * przechodzimy do kroku 12.<br>
 * <br>
 * <b>Krok 7</b>:<br>
 * Jeli \f$ F_v \leq F_r < F_w \f$, to przechodzimy do kroku 8. W przeciwnym
 * Wypadku przechodzimy do kroku 9.<br>
 * <br>
 * <b>Krok 8</b>:<br>
 * Wyznaczamy punkt \f$ x_c \f$ ze wzoru:
 * \f[ x_c = x_r - c(x_r - \bar{x}) \f]
 * Wyliczamy w tym punkcie warto funkcji.<br>
 * Jeli \f$ F_c \leq F_r \f$, przechodzimy do kroku 10. W przeciwnym wypadku
 * przechodzimy do kroku 11.<br>
 * <br>
 * <b>Krok 9</b>:<br>
 * Wyznaczamy punkt \f$ x_c \f$ ze wzoru:
 * \f[ x_c = x^k_w + c(\bar{x} - x^k_w) \f]
 * Wyliczamy w tym punkcie warto funkcji.<br>
 * Jeli \f$ F_c \leq F^k_w \f$, przechodzimy do kroku 10. W przeciwnym wypadku
 * przechodzimy do kroku 11.<br>
 * <br>
 * <b>Krok 10</b>:<br>
 * W sympleksie zastpujemy punkt \f$ x^k_w \f$ punktem \f$ x_c \f$ i
 * przechodzimy do kroku 12.<br>
 * <br>
 * <b>Krok 11</b>:<br>
 * Dokonujemy skurczenia sympleksu w kierunku punktu najlepszego zgodnie
 * z procedur:
 * \f[ x^{k+1}_i = x^k_i - s(x^k_i - x_b) \mbox{  dla } i \in S_i\f]
 * Oczywicie w wyniku dziaania tej procedury punkt najlepszy nie zmieni
 * swojego pooenia.<br>
 * W nowych punktach obliczamy wartoci funkcji.<br>
 * <br>
 * <b>Krok 12</b>:<br>
 * Podstawiamy \f$ k = k+1 \f$.<br>
 * Wybieramy szczeglne punkty sympleksu: \f$ x^{k+1}_b \f$, \f$ x^{k+1}_w \f$
 * i \f$ x^{k+1}_v \f$.
 * Zapamitujemy rownie wartoci funkcji \f$ F_b = f(x_b) \f$,
 * \f$ F_w = f(x_w) \f$ oraz \f$ F_v = f(x_v) \f$<br>
 * Przechodzimy do kroku 13.<br>
 * <br>
 * <b>Krok 13</b>:<br>
 * Jeli speniony jest warunek stopu, zakocz dziaanie algorytmu. Punktem
 * wynikowym jest najlepszy punkt ostatniego sympleksu.<br>
 * Jeli warunek niejest speniony przechodzimy do kroku 1.<br>
 * <br>
 * <b>Uwagi</b>:<br>
 * W metodzie mona zastosowa kilka ronych warunkw stopu
 * <ul>
 * <li>Warunek \f$ \| x^k_w - x'^k_w \| \leq \varepsilon \f$, gdzie poprzez
 * \f$ x'^k_w \f$ oznaczamy punkt, ktry zastpi w sympleksie punkt
 * \f$ x^k_w \f$. Ten warunek jest odpowiednikiem tradycyjnego warunku
 * sprawdzajcego dugo wykonanego kroku, wykorzystywanego czsto w
 * metodach gradientowych.
 * <li>Warunek \f$ f(x^k_w) - f(x'^k_w) \leq \varepsilon \f$, gdzie poprzez
 * \f$ x'^k_w \f$ oznaczamy punkt, ktry zastpi w sympleksie punkt
 * \f$ x^k_w \f$.
 * <li>Warunek \f$ \| x^k_w - x^{k-1}_w \| \leq \varepsilon \f$.
 * <li>Warunek \f$ F_w - F_b \leq \varepsilon \f$.
 * <li>Warunek \f$ \| x^k_w - x^k_b \| \leq \varepsilon \f$.
 * <li>Warunek \f$ D \leq \varepsilon \f$, gdzie \f$ D \f$ oznacza sum dugoci
 * wszystkich krawdzi sympleksu.
 * </ul>
 * <br>
 * Postpujc zgodnie z [2] do implementacji algorytmu wprowadzono
 * usprawnienie dotyczce jest uznany za gorszy od wszystkich punktw o takiej
 * samej wartoci funkcji celu.<br>
 *
 *
 * [1] Aleksander Ostanin - Metody i algorytmy optymalizacji, Wydawnictwo
 * Politechniki Biaostockiej, Biaystok 2003
 * oraz:
 * [2] J. C. Lagarias, J. A. Reeds, M. H. Wright, P. E. Wright - Convergence
 * Properties of the Nelder-Mead Simplex Function in low dimensions.
 * (SIAM Journal on Optimization, Vol. 1, No. 1, pp 112-147).
 *
 * W drugim rdle zaproponowano sposoby postpowania w przypadku gdy co
 * najmniej w dwch wierzchokach sympleksu warto funkcji jest taka sama.
 */

#ifndef _NelderMead_h
#define _NelderMead_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../Error.h"
#include "../Logger.h"
#include "../Method.h"
#include "../NMStopCondition.h"
#include "../PointWithValue.h"
#include "../ProblemBase.h"
#include "NelderMeadIteration.h"

// FORWARD REFERENCES
//


class NelderMead
      : public Method
{
public:

    friend class NelderMeadPanel;
    friend class NelderMeadIteration;

// LIFECYCLE

    /** Destruktor.
     */
    ~NelderMead(void);

// OPERATORS
// OPERATIONS

    /** Wygeneruj pocztkowy sympleks.
     *
     * Funkcja tworzy regularny wielokt, bry (itd. dla wyszych wymiarw)
     * zaczynajc od podanego punktu.
     * Algorytm wyliczania wsprzdnych punktw sympleksu pochodzi z [1].
     *
     * @param baseVertex Pierwszy punkt startowego sympleksu.
     * @param sideLength Wymagana dugo krawdzi sympleksu
     */
    /*NelderMead::Simplex CreateSimplex(const ColumnVector& baseVertex,
                                            double        sideLength = 1.0
                                      const FunctionBase& function);
    */
    /** Tworzy kopi obiektu.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void                    UpdateStartingConditions
                                                (const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    /** Czy metoda nadaje si do rozwizania zadanego problemu
     */
    virtual bool                    CanHandle(const ProblemBase* pProblem)
                                                                        const;

    virtual StopConditions          AllowedStopConditions() const;

    virtual wxString&               rName();
protected:

    /** Klasa wewntrzna, ktrej obiekt reprezentuje sympleks zoony
     * z obiektw klasy PointWithValue.
     * Punkty s posortowane zgodnie ze wzrostem wartoci funkcji celu.
     */
    class Simplex
    {
    public:
    //LIFECYCLE
    //

        /** Konstruktor domylny.
         */
        Simplex();

        /** Konstruktor
         *
         * @param function Referencja na funkcj celu
         */
        Simplex(const FunctionBase* function);

        /** Konstruktor
         *
         */
        Simplex(const   ColumnVector&       centralPoint,
                const   DoubleParameter&    sideLength,
                const   FunctionBase*       pFunction);

        /** Konstruktor
         *
         * Konstruktor tworzy sympleks z gotowego zestawu punktw.
         *
         * @param startingPoints Punkty, z ktrych tworzymy sympleks
         * @param function Wskanik na funkcj celu
         */
        Simplex(const vector<ColumnVector>&     startingPoints,
                const FunctionBase*             function);

        /** Kostruktor kopiujcy.
         *
         * @param from sympleks, z ktrego kopiujemy.
         */
        Simplex(const NelderMead::Simplex&  from);

        /** Destruktor
         */
        ~Simplex(){};

    //OPERATORS
    //

        NelderMead::Simplex&    operator=(const NelderMead::Simplex& from);

    //OPERATIONS
    //

        /** Wspczynnika \f$ \delta_1 \f$.
         *
         * Wyliczanie wspczynnika \f$ \delta_1 \f$ potrzebnego przy generacji
         * regularnego sympleksu.
         *
         * @param sideLength zadana dugo boku sympleksu
         * @param dimensions liczba wymiarw problemu
         *
         * @return warto wspczynnika
         */
        static  double  Delta1(const    DoubleParameter&    sideLength,
                                        size_t              dimensions);


        /** Wspczynnika \f$ \delta_2 \f$.
         *
         * Wyliczanie wspczynnika \f$ \delta_2 \f$ potrzebnego przy generacji
         * regularnego sympleksu.
         *
         * @param sideLength zadana dugo boku sympleksu
         * @param dimensions liczba wymiarw problemu
         *
         * @return warto wspczynnika
         */
        static  double  Delta2(const    DoubleParameter&    sideLength,
                                        size_t              dimensions);


        /** Znajd punkt ciekoci cigu punktw sympleksu.
         *
         * Funkcja zwraca punkt znajdujcy si porodku wybranych punktw
         * sympleksu.
         *
         * @param firstElem pierwszy element z cigu do wyliczenia rodka
         * cikoci
         * @param firstElem ostatni element z cigu do wyliczenia rodka
         * cikoci
         *
         * @return Punkt cikoci wybranych punktw sympleksu
         */
        const PointWithValue    Centroid(size_t firstIndex,
                                         size_t lastIndex) const;

        /** Zmniejsz sympleks w kierunku najlepszego punktu.
         *
         * Funkcja zmniejsza wszystkie krawdzie sympleksu o
         * w kierunku najlepszego punktu.
         *
         * @param shrinkageCoefficient wspczynnik, o ktry skracane s boki
         * sympleksu.
         */
        void        ShrinkTowardBest(double shrinkageCoefficient);

        /** Zastp najgorszy punkt nowym.
         *
         * Funkcja podstawia nowy punkt pod dotychczas najgorszy. Nowy punkt
         * jest od razu wstawiany na przysugujce mu miejsce.
         *
         * @param newPoint Nowy punkt
         */
        void        SubstituteNewForWorst(const PointWithValue& newPoint);

    //ACCESS
    //
        /** Referencja na najlepszy punkt sympleksu.
         *
         * Funkcja zwraca referencj na najlepszy punkt sympleksu czyli na ten
         * znajdujcy si na pierwszym miejscu w wektorze.
         *
         * @return Najlepszy punkt sympleksu.
         */
        const PointWithValue&                   rBest() const;

        /** Referencja na najgorszy punkt sympleksu.
         *
         * Funkcja zwraca referencj na najgorszy punkt sympleksu czyli na ten
         * znajdujcy si na ostatnim miejscu w wektorze.
         *
         * @return Referencja na najgorszy punkt sympleksu.
         */
        const PointWithValue&                   rWorst() const;

        /** Referencja na drugi najgorszy punkt sympleksu.
         *
         * Funkcja zwraca referencj na drugi najgorszy punkt sympleksu czyli
         * na ten znajdujcy si na przedostatnim miejscu w wektorze.
         *
         * @return Referencja na prawie najgorszy punkt sympleksu.
         */
        const PointWithValue&                   rNextToWorst() const;

        /** Referencja na punkty sympleksu.
         *
         * Funkcja zwraca referencj na wektor obiektw klasy PointWithValue
         * skadajcych si na sympleks.
         *
         * @return Punkty sympleksu
         */
        const vector<PointWithValue>&           rPoints() const;

    //INQUIRY
    //
        /** Obwd sympleksu.
         *
         * @return obwd sympleksu.
         */
        double Circumference() const;

        /** Szeroko przedziau wartoci przyjmowanych przez funkcj
         * celu w punktach sympleksu.
         *
         * Metoda zwraca rnic pomidzy wartoci funkcji celu w najgorszym
         * i najlepszym punkcie.
         *
         * @return Rnica pomidzy wartociami funkcji w najgorszym i
         * najlepszym punkcie
         */
        double ValueSpan() const;

        /** Liczba wierzchokw sympleksu
         *
         * @return liczba wierzchokw sympleksu
         */
        size_t                                  Size() const;

    protected:

        /** Wygeneruj wok punktu regularny sympleks o zadanej dugoci boku.
         *
         * @param centralPoint punkt, wok ktrego funkcja buduje sympleks
         * @param sideLength zadana dugo boku sympleksu
         */
        void             GenerateVertices(const ColumnVector     centralPoint,
                                          const DoubleParameter& sideLength);

        /** Posortuj punkty sympleksu
         *
         * Funkcja sortuje punkty sympleksu zgodnie ze wzrostem wartoci funkcji
         * celu. Z uwagi na mae prawdopodobiestwo wystpowania sympleksu
         * duych rozmiarw, uyto prostego algorytmu sortowania bbelkowego.
         */
        void                                    Sort();

        /** Punkty sympleksu
         */
        vector<PointWithValue>                  mPoints;
        /** Wskanik na funkcj celu
         */
        const FunctionBase*                     mpFunction;

    private:

    };

    /** Konstruktor domylny
     */
    NelderMead();

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NelderMead(const NelderMead& from);

protected:

    /** Wspczynnik odbicia
     */
    DoubleParameter         mReflectionCoefficient;
    /** Wspczynnik rozcignicia
     */
    DoubleParameter         mExpansionCoefficient;
    /** Wspczynnik cinicia
     */
    DoubleParameter         mContractionCoefficient;
    /** Wspczynnik skurczenia
     */
    DoubleParameter         mShrinkageCoefficient;
    /** Punkty startowe.
     */
    //NelderMead::Simplex     mInitialSimplex;
    /** Punkt startowy (rodek pierwszego sympleksu)
     */
    mutable ColumnVector    mStartingPoint;
    ColumnVector            mInitialStartingPoint;
    /** Dugo boku pocztkowego sympleksu
     */
    DoubleParameter         mSideLength;
    /** Promie sfery opisanej na sympleksie
     */
    double                  mSimplexRange;
    /** Warunek stopu.
     */
    CountedPtr<const NMStopCondition> mcpStopCondition;

private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    /** Nazwa metody.
     */
    static wxString   mName;
    /** Egzemplarz metody.
     */
    static NelderMead       mNelderMead;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NelderMead_h
