#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../text-util.h"
#include "../UnconstrainedProblem.h"

#include "Newton.h"  // class implemented
#include "NewtonIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Newton::~Newton()
{
}// ~Newton

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
Newton::Clone() const
{
    return auto_ptr<Method>(new Newton(*this));
}// Clone

/* virtual */ void
Newton::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
Newton::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
Newton::Name() const
{
    return Newton::mName;
}// Name

/* virtual */ wxString&
Newton::rName()
{
    return Newton::mName;
}// rName

/* virtual */ MethodIdType
Newton::Id() const
{
    return Newton::ClassId();
}// Id

/* static */ MethodIdType
Newton::ClassId()
{
    const static MethodIdType NewtonId = Method::GenerateId();

    return NewtonId;
}// ClassId

/* virtual */ wxString
Newton::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());
    description << wxT('\n');
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
Newton::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
Newton::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

Newton::Newton()
    : Method(),
      mStartingPoint(0),
      mInitialStartingPoint(0),
      mcpStopCondition()
{
}// Newton

Newton::Newton(const Newton& from)
    : Method(from),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mcpStopCondition(from.mcpStopCondition)
{
}// Newton


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
Newton::Optimize(const ProblemBase& problem) const
                                       throw(OptimizationError)
{
    auto_ptr<Result>            ap_result(new Result(1));
    auto_ptr<NewtonIteration>   ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = problem.Function();
        assert(function.IsDifferentiable());

        const unsigned int dimensions = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);



        // Biecy punkt.
        ColumnVector        x(mStartingPoint);
        // Punkt z poprzedniej iteracji.
        // (inicjowany wektorem zerowym, aby moliwe byo
        // sprawdzanie przed iteracj wszystkich warunkw stopu).
        ColumnVector        previous_x(NullPoint);
        /* Wektor przesunicia punktu x wyliczany z nastpujcego wzoru:
         * step = - H(x)^(-1) * G(x), gdzie
         * H(x) - macierz hesjanu w punkcie x
         * G(x) - Gradient w punkcie x
         */

        // Macierz hesjanu.
        SymmetricMatrix     hessian(dimensions);
        // Gradient.
        ColumnVector        gradient(dimensions);

        LOG(DEBUG,wxT("-------------------------------------"));

        /* Gradient wyliczamy przed pierwsz iteracj algorytmu, gdy moliwe
         * jest, e punkt startowy jest akceptowalnie blisko punktu minimum.
         */
        hessian  = function.Hessian(x);
        gradient = function.Gradient(x);


        LOG(DEBUG,wxT("Iteracja wstepna\n"));
        LOG(DEBUG,wxT("Punkt:\n") << x);
        LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                << function.Evaluate(x) << wxT("\n"));
        LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
        LOG(DEBUG,wxT("Hesjan:\n") << hessian);
        //LOG(DEBUG,wxT("Wyznacznik hesjanu ") << hessian.determinant());
        //LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());

        LOG(DEBUG,wxT("-------------------------------------"));


        while (step_count < STEP_LIMIT &&
               !mcpStopCondition->IsMet(previous_x,
                                        x,
                                        function,
                                        gradient.norm_Frobenius()))
        {
            ++step_count;

            LOG(DEBUG, wxT("Iteracja ") << step_count << wxT("\n"));

            if (hessian.determinant() == 0)
                throw Error(_("Hessian is singular"));

/*
            if (QuadraticForm(hessian,x) < 0)
                throw Error(_("Hessian is negatively defined, the method found a maximizer!"));
*/
/*
            LOG(DEBUG, wxT("Punkt:\n") << x);
            LOG(DEBUG, wxT("Wartosc funkcji w tym punkcie :")
                    << function.Evaluate(x) << wxT("\n"));
            LOG(DEBUG, wxT("Gradient w tym punkcie:\n") << gradient);
            LOG(DEBUG, wxT("Hesjan:\n") << hessian);
            //LOG(DEBUG,wxT("Wyznacznik hesjanu ") << hessian.determinant());
            //LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());
*/
            previous_x = x;
            x = x - (hessian.i() * gradient);

            ap_iteration.reset(new NewtonIteration
                       (previous_x,
                        gradient,
                        hessian,
                        x,
                        function.Evaluate(x) < function.Evaluate(previous_x)));

            hessian  = function.Hessian(x);
            gradient = function.Gradient(x);

            LOG(DEBUG, wxT("ITERATION DESCRIPTION"));
            LOG(DEBUG, wxT("Starting point: ")
                                    << ap_iteration->StartingPoint().as_row());
            LOG(DEBUG, wxT("Gradient: ") << ap_iteration->Gradient().as_row())
            LOG(DEBUG, wxT("Hesian:\n") << ap_iteration->Hessian());
            LOG(DEBUG, wxT("End point: ") << ap_iteration->EndPoint().as_row());
            LOG(DEBUG, wxT("Valid: ") << ap_iteration->StepValid());

            LOG(DEBUG,wxT("-------------------------------------"));

            ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));

            NotifyYield();
        }

        ap_result->rFinalResult() = x;

    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString Newton::mName(_("Newton's"));

// Utwrz i zarejestruj prototyp.
Newton Newton::mNewton;
