/** Metoda Newtona.
 *
 * Podstaw metody Newtona jest aproksymacja funkcji celu funkcj kwadratow.
 * Algorytm wyznacza kady nastpny punkt w taki sposb by gradient kwadratowej
 * funkcji aproksymujcej wynosi w nim 0.
 *
 * Przy zastosowaniu metody Newtona i przy niewystpowaniu niedokadnoci
 * numerycznych, funkcje kwadratowe minimalizowane s w jednej iteracji.
 * W przypadku gdy funkcja celu nie jest kwadratowa i gdy punkt startowy
 * znajduje si w znacznej odlegoci od minimum, metoda Newtona
 * charakteryzuje si woln zbienoci, lub nawet jej brakiem.
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ k \f$ - numer aktualnej iteracji<br>
 * \f$ \nabla f(x) \f$ - gradient w punkcie \f$ x \f$<br>
 * \f$ \nabla^2 f(x) \f$ - macierz hesjanu w punkcie \f$ x \f$<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja celu<br>
 * \f$ x^0 \f$ - punkt startowy<br>
 * \f$ \varepsilon \f$ - wymagana dokadno,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|\nabla f(x^k)\| < \varepsilon\f$</li>
 * <li>\f$\|\nabla f(x^k)\|^2 < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$.</li>
 * </ul>
 * <br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * Podstaw \f$ k=0 \f$.<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Sprawd czy speniony jest warunek stopu. Jeli tak, to przerwij dziaanie
 * algorytmu i przedstaw \f$ x^k \f$ jako wynik. W przeciwnym wypadku przejd do
 * kroku 2.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Sprawd czy:
 * \f[ | \nabla^2 f(x^k)) | = 0 \f]
 * Jeli tak to STOP: Hejsan jest macierz osobliw.<br>
 * W przeciwnym wypadku przejd do kroku 3.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Oblicz:
 * \f[ x^{k+1} = x^k - (\nabla^2 f(x^k))^{-1} \nabla f(x^k) \f]
 * Podstaw \f$ k=k+1 \f$ i przejd do kroku 1.<br>
 * <br>
 * <br>
 * <b>Uwagi</b>:<br>
 * W stosunku do wersji algorytmu zamieszczonej w [1] zmieniono kolejno
 * wykonywania
 * krokw. W zaimplementowanej wersji sprawdzanie warunku stopu dokonuje si
 * na pocztku kadej iteracji. Dziki temu algorytm moe zatrzyma si nie
 * wykonujc adnej iteracji w przypadku gdy punkt startowy znajduje sie w
 * minimum i wykorzystywany jest "gradientowy" warunek stopu. Pocztkowe
 * wartoci zmiennych oznaczajcych ronice pooenia dwch ostatnich punktw
 * oraz rnice wartoci funkcji w tych dwch punktach s rwne DBL_MAX, wic w
 * przypadku wykorzystaniu innego warunku stopu ni "gradientowy" modyfikacja
 * nie bdzie miaa adnego wpywu na dziaanie algorytmu.
 *
 *
 * [1] Aleksander Ostanin - Metody i algorytmy optymalizacji, Wydawnictwo
 * Politechniki Biaostockiej, Biaystok 2003
 *
 * @see NewtonModified
 */

#ifndef _Newton_h
#define _Newton_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../Method.h"
#include "../StandardStopCondition.h"
#include "../ui/BasicMethodPanel.h"

// FORWARD REFERENCES
//


class Newton
    : public Method
{
public:

    friend class BasicMethodPanel<Newton>;

// LIFECYCLE

    /** Destruktor.
     */
    ~Newton(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void                    UpdateStartingConditions
                                                (const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    /** Czy metoda nadaje si do rozwizania zadanego problemu
     */
    virtual bool                    CanHandle(const ProblemBase* pProblem)
                                                                        const;
    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    Newton(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Newton(const Newton& from);

    /** Punkt startowy
     */
    mutable ColumnVector    mStartingPoint;
    ColumnVector            mInitialStartingPoint;

    /** Warunek stopu.
     */
    CountedPtr<const StandardStopCondition> mcpStopCondition;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    /** Nazwa metody
     */
    static wxString   mName;
    /** Egzemplarz metody
     */
    static Newton           mNewton;

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Newton_h
