#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../util.h"

#include "NewtonModified.h"  // class implemented
#include "NewtonModifiedIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NewtonModified::~NewtonModified()
{
}// ~NewtonModified

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
NewtonModified::Clone() const
{
    return auto_ptr<Method>(new NewtonModified(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
NewtonModified::Name() const
{
    return NewtonModified::mName;
}// Name

/* virtual */ wxString&
NewtonModified::rName()
{
    return NewtonModified::mName;
}// rName

/* virtual */ MethodIdType
NewtonModified::Id() const
{
    return NewtonModified::ClassId();
}// Id

/* static */ MethodIdType
NewtonModified::ClassId()
{
    const static MethodIdType NewtonModifiedId = Method::GenerateId();

    return NewtonModifiedId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
NewtonModified::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
NewtonModified::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

NewtonModified::NewtonModified()
    : MethodWithLineSearch()
{
}// NewtonModified

NewtonModified::NewtonModified(const NewtonModified& from)
    : MethodWithLineSearch(from)
{
}// NewtonModified


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
NewtonModified::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result>                    ap_result(new Result(1));
    auto_ptr<NewtonModifiedIteration>   ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpLineSearch.Get() != NULL);
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = problem.Function();
        assert(function.IsDifferentiable());

        const unsigned int dimensions = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);



        // Biecy punkt.
        ColumnVector        x(mStartingPoint);
        // Punkt z poprzedniej iteracji.
        // (inicjowany wektorem zerowym, aby moliwe byo
        // sprawdzanie przed iteracj wszystkich warunkw stopu).
        ColumnVector        previous_x(NullPoint);
        // Kierunek poszukiwa jednowymiarowych
        ColumnVector        direction(dimensions);
        // Dugo kroku wykonanego w kierunku direction
        double              tau;

        // Macierz hesjanu.
        SymmetricMatrix     hessian(dimensions);
        // Gradient.
        ColumnVector        gradient(dimensions);

        LOG(DEBUG,wxT("-------------------------------------"));


        /* Gradient wyliczamy przed pierwsz iteracj algorytmu, gdy moliwe
         * jest, e punkt startowy jest akceptowalnie blisko punktu minimum.
         */
        hessian  = function.Hessian(x);
        gradient = function.Gradient(x);

        LOG(DEBUG,wxT("Iteracja wstepna\n"));
        LOG(DEBUG,wxT("Punkt:\n") << x);
        LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                << function.Evaluate(x) << wxT("\n"));
        LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
        LOG(DEBUG,wxT("Hesjan:\n") << hessian);
        LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());

        LOG(DEBUG,wxT("-------------------------------------"));

        while (step_count < STEP_LIMIT &&
               !mcpStopCondition->IsMet(previous_x, x, function,
                   gradient.norm_Frobenius()))
        {

            ++step_count;

            LOG(DEBUG,wxT("Iteracja ") << step_count << wxT("\n"));

            if (hessian.determinant() == 0)
                throw Error(_("Hessian is singular"));

            direction = -1.0 * hessian.i() * gradient;

            LOG(DEBUG,wxT("Kierunek poszukiwania:\n") << direction);

            previous_x = x;

            x = StepInDirection(function, x, direction, &tau);

            ap_iteration.reset(new NewtonModifiedIteration
                       (previous_x,
                        gradient,
                        hessian,
                        direction,
                        tau,
                        x));
            ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));

            hessian  = function.Hessian(x);
            gradient = function.Gradient(x);


            LOG(DEBUG,wxT("Nowy punkt:\n") << x);
            LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                    << function.Evaluate(x) << wxT("\n"));
            LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
            LOG(DEBUG,wxT("Hesjan:\n") << hessian);
            LOG(DEBUG,wxT("Odwrocony hesjan:\n") << hessian.i());

            LOG(DEBUG,wxT("-------------------------------------"));


            NotifyYield();
        }

        ap_result->rFinalResult() = x;
    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);

    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }

    return ap_result;
}// Optimize

wxString NewtonModified::mName(_("Newton's Modified"));

// Utwrz i zarejestruj prototyp.
NewtonModified NewtonModified::mNewtonModified;
