/** Zmodyfikowana metoda Newtona.
 *
 * Przy wykorzystaniu do minimalizacji funkcji niekwadratowych metoda Newtona
 * nie jest niezawodna. Gdy punkt startowy znajduje si daleko od minimum, krok
 * metody moe okaza si zbyt duy, co z kolei moe doprowadzi do
 * niezbienoci metody.
 * Najprostsz modyfikacj metody Newtona jest wprowadzenie etapu poszukiwania
 * optymalnej dugoci kroku w kierunku \f$ -(\nabla^2 f(x^k))^{-1}
 * \nabla f(x^k) \f$, podobnie jak w
 * metodzie Cauchy'ego (najwikszego spadku). Zapewnia to zmniejszanie si
 * wartoci funkcji w punkcie roboczym z kroku na krok.
 *
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ k \f$ - numer aktualnej iteracji<br>
 * \f$ \nabla f(x) \f$ - gradient w punkcie \f$ x \f$<br>
 * \f$ \nabla^2 f(x) \f$ - macierz hesjanu w punkcie \f$ x \f$<br>
 * \f$ \tau \f$ - wspczynnik dugoci kroku<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja celu<br>
 * \f$ x^0 \f$ - punkt startowy<br>
 * \f$ \varepsilon \f$ - wymagana dokadno<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|\nabla f(x^k)\| < \varepsilon\f$</li>
 * <li>\f$\|\nabla f(x^k)\|^2 < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami.<br>
 * <br>
 * <b>Krok 0 (wstpny)</b>:<br>
 * Podstaw \f$ k=0 \f$.<br>
 * <br>
 * <b>Krok 1</b>:<br>
 * Sprawd czy speniony jest warunek stopu. Jeli tak, to przerwij dziaanie
 * algorytmu i przedstaw \f$ x^k \f$ jako wynik. W przeciwnym wypadku przejd do
 * kroku 2.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Sprawd czy:
 * \f[ | \nabla^2 f(x^k)) | = 0 \f]
 * Jeli tak to STOP: Hesjan jest macierz osobliw.<br>
 * W przeciwnym wypadku przejd do kroku 3.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Wyznacz kierunek poszukiwa \f$d\f$:
 * \f[ d = -(\nabla^2 f(x^k))^{-1} \nabla f(x^k). \f]
 * Wykonaj krok w kierunku \f$d\f$ o optymalnej dugoci \f$\tau\f$:
 * \f[ \tau = \arg \min_{\tau} f(x^k + \tau d), \f]
 * \f[ x^{k+1} = x^k + \tau d. \f]
 * Podstaw \f$ k=k+1 \f$ i przejd do kroku 1.<br>
 * <br>
 * <b>Uwagi</b>:<br>
 * W stosunku do wersji algorytmu zamieszczonej w [1] zmieniono kolejno
 * wykonywania
 * krokw. W zaimplementowanej wersji sprawdzanie warunku stopu dokonuje si
 * na pocztku kadej iteracji. Dziki temu algorytm moe zatrzyma si nie
 * wykonujc adnej iteracji w przypadku gdy punkt startowy znajduje sie w
 * minimum i wykorzystywany jest "gradientowy" warunek stopu. Pocztkowe
 * wartoci zmiennych oznaczajcych ronice pooenia dwch ostatnich punktw
 * oraz rnice wartoci funkcji w tych dwch punktach s rwne DBL_MAX, wic w
 * przypadku wykorzystaniu innego warunku stopu ni "gradientowy" modyfikacja
 * nie bdzie miaa adnego wpywu na dziaanie algorytmu.
 *
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 *
 * @see Newton
 */

#ifndef _NewtonModified_h
#define _NewtonModified_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../MethodWithLineSearch.h"
#include "../ui/MethodWithLineSearchPanel.h"

// FORWARD REFERENCES
//

class NewtonModified
    : public MethodWithLineSearch
{
public:

    friend class MethodWithLineSearchPanel<NewtonModified>;

// LIFECYCLE

    /** Destruktor.
     */
    ~NewtonModified(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    /** Czy metoda nadaje si do rozwizania zadanego problemu
     */
    virtual bool                    CanHandle(const ProblemBase* pProblem)
                                                                        const;
    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    NewtonModified(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NewtonModified(const NewtonModified& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    static wxString  mName;   ///< Nazwa metody.
    static NewtonModified          mNewtonModified; ///< Egzemplarz metody.

};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NewtonModified_h
