#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "NewtonRaphson.h"  // class implemented
#include "NewtonRaphsonIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


NewtonRaphson::~NewtonRaphson()
{
}// ~NewtonRaphson


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
NewtonRaphson::Clone() const
{
    return auto_ptr<Method>(new NewtonRaphson(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
NewtonRaphson::Name() const
{
    return NewtonRaphson::mName;
}// Name

/* virtual */ wxString&
NewtonRaphson::rName()
{
    return NewtonRaphson::mName;
}// rName

/* virtual */ MethodIdType
NewtonRaphson::Id() const
{
    return NewtonRaphson::ClassId();
}// Id

/* static */ MethodIdType
NewtonRaphson::ClassId()
{
    const static MethodIdType NewtonRaphsonId = Method::GenerateId();

    return NewtonRaphsonId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
NewtonRaphson::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>
                            (&(pProblem->Function())) != NULL &&
            pProblem->Function().IsDifferentiable());
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

NewtonRaphson::NewtonRaphson()
    : SinglePointLineSearch()
{
}// NewtonRaphson

NewtonRaphson::NewtonRaphson(const NewtonRaphson& from)
    : SinglePointLineSearch(from)
{
}// NewtonRaphson

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
NewtonRaphson::Solve(const UnivariateFunction&   function,
                    const ColumnVector&         startingPoint,
                    bool                        generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result>                ap_result(new Result(1));
    auto_ptr<NewtonRaphsonIteration> ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        ColumnVector    x(1);               // Punkt w danej iteracji
        ColumnVector    previous_x(1);      // Punkt z poprzedniej iteracji
        double          first_derivative;   // Pierwsza pochodna w punkcie x
        double          second_derivative;  // Druga pochodna w punkcie x

        x                   = startingPoint;
        previous_x          = x;

        first_derivative    = function.Gradient(x)(1);
        second_derivative   = function.Hessian(x)(1,1);


        LOG(DEBUG,wxT("Krok nr ") << step_count);
        LOG(DEBUG,wxString::Format(wxT("point             : %10.6g\n\t"),
                                                x(1)));
        LOG(DEBUG,wxString::Format(wxT("first derivative  = %10.6g\n\t"),
                                                first_derivative));
        LOG(DEBUG,wxString::Format(wxT("second derivatice = %10.6g"),
                                                second_derivative));
        LOG(DEBUG,wxT("-----------------------------------"));


        while(fabs(first_derivative) > mEpsilon &&
              step_count < STEP_LIMIT)
        {
            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count);

            if (second_derivative == 0)
                throw Error(_("Second derivative equal to 0"));

            previous_x  = x;
            x           -= first_derivative / second_derivative;


            ap_iteration.reset(new NewtonRaphsonIteration(previous_x(1),
                                                         first_derivative,
                                                         second_derivative,
                                                         x(1)));
            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));


            first_derivative    = function.Gradient(x)(1);
            second_derivative   = function.Hessian(x)(1,1);


            LOG(DEBUG,wxString::Format(wxT("point             : %10.6g\n\t"),
                                                    x(1)));
            LOG(DEBUG,wxString::Format(wxT("first derivative  = %10.6g\n\t"),
                                                    first_derivative));
            LOG(DEBUG,wxString::Format(wxT("second derivatice = %10.6g"),
                                                    second_derivative));

            LOG(DEBUG,wxT("-----------------------------------"));

            NotifyYield();
        }

        ap_result->rFinalResult() = x;

    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve

wxString NewtonRaphson::mName(_("Newton-Raphson's"));

// Utwrz i zarejestruj prototyp.
NewtonRaphson NewtonRaphson::mNewtonRaphson;
