/** Metoda Newtona-Raphsona.
 *
 * Metoda Newtona-Raphsona jest oparta na kwadratowym modelu funkcji -
 * rozwiniciu funkcji celu w szereg Taylora drugiego rzdu. Dla
 * funkcji jednej zmiennej takie rozwinicie (z pominiciem reszty Lagrange'a)
 * ma posta:
 * \f[ q(x) = f(x_k) + f'(x_k)(x - x_k) + \frac{1}{2} f''(x_k)(x - x_k)^2 . \f]
 *
 * Chcc znale ekstremum powyszej funkcji, naley (z warunku koniecznego
 * na istnienie ekstremum) wyzerowa jej pochodn, ktra jest nastpujca:
 * \f[ q'(x) = f'(x_k) + f''(x_k)(x - x_k) . \f]
 *
 * Std bierze si wzr na kolejne punkty w metodzie Newtona-Raphsona:
 * \f[ x^k = x^{k-1}-\frac{f'(x^{k-1})}{f''(x^{k-1})} . \f]
 * W powyszy sposb metoda znajduje minimum funkcji kwadratowej
 * w zaledwie jednej iteracji. Minimalizacja funkcji wyszego rzdu
 * ni kwadratowa (czyli wyszego rzdu ni zastosowany model) moe
 * zaj wicej iteracji.<br>
 * Naley zauway, e powysza metoda pozwala znale ekstremum,
 * niekoniecznie minimum. Z tego powodu w implementacji algorytmu dodano
 * sprawdzanie znaku drugiej pochodnej - jeeli jest ona ujemna, metoda
 * koczy dziaanie.<br>
 * Ponadto, taka metoda wyznaczania kolejnych punktw wymaga oczywicie,
 * aby funkcja celu bya podwjnie rniczkowalna.
 *
 * Warunek stopu metody jest oparty na szacowanej odlegoci do ekstremum:
 * \f[ |f'(x)| < \varepsilon . \f]
 *
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ x^k \f$ - punkt w <i>k</i>-tej iteracji<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ x^0 \f$ - punkt pocztkowy<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 * <br>
 * <b>Krok wstpny</b>:<br>
 * Podstaw:
 * \f[ x^1 = x^0, \f]
 * \f[ k   = 1. \f]
 *
 * <b>Krok 1</b>:<br>
 * Oblicz \f$f'(x^k)$ i $f''(x^k)\f$.<br>
 * Przejd do kroku 2.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Sprawd warunek stopu.<br>
 * Jeli jest on speniony, to <i>STOP</i>.
 * Wynikiem jest \f$x^k\f$.<br>
 * W przeciwnym wypadku przejd do kroku 3.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Sprawd warto drugiej pochodnej w punkcie \f$x^k\f$:<br>
 * jeli \f$f''(x^k) \neq 0\f$, to przejd do kroku 4.<br>
 * W przeciwnym wypadku <i>STOP</i> - druga pochodna rwna zeru,
 * optymalizacji nie mona kontynuowa.<br>
 * <br>
 * <b>Krok 4</b>:<br>
 * Podstaw \f$k := k+1\f$.<br>
 * Wyznacz kolejny punkt:
 * \f[ x^k = x^{k-1} - \frac{f'(x^{k-1})}{f''(x^{k-1})} . \f]
 * Przejd do kroku~1.<br>
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 *
 * @see Linesearch
 **/

#ifndef _NewtonRaphson_h
#define _NewtonRaphson_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../SinglePointLineSearch.h"
#include "../ProblemBase.h"
#include "../UnivariateFunction.h"
#include "../ui/PointLineSearchPanel.h"

// FORWARD REFERENCES
//


class NewtonRaphson
    : public SinglePointLineSearch
{
public:
    friend class PointLineSearchPanel<NewtonRaphson>;

// LIFECYCLE


    /** Destruktor.
     */
    ~NewtonRaphson(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    NewtonRaphson(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    NewtonRaphson(const NewtonRaphson& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      const ColumnVector&       startingPoint,
                                      bool                      generateResult)
                          const throw(OptimizationError);

    /** Nazwa metody
     */
    static wxString           mName;
    /** Egzemplarz metody
     */
    static NewtonRaphson             mNewtonRaphson;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _NewtonRaphson_h_
