#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../util.h"

#include "PolakRibiere.h"  // class implemented
#include "ConjugateGradientsMethodIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PolakRibiere::~PolakRibiere()
{
}// ~PolakRibiere

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
PolakRibiere::Clone() const
{
    return auto_ptr<Method>(new PolakRibiere(*this));
}// Clone

//============================= ACCESS     ===================================


/* virtual */ const wxString&
PolakRibiere::Name() const
{
    return PolakRibiere::mName;
}// Name

/* virtual */ wxString&
PolakRibiere::rName()
{
    return PolakRibiere::mName;
}// rName

/* virtual */ MethodIdType
PolakRibiere::Id() const
{
    return PolakRibiere::ClassId();
}// Id

/* static */ MethodIdType
PolakRibiere::ClassId()
{
    const static MethodIdType PolakRibiereId = Method::GenerateId();

    return PolakRibiereId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
PolakRibiere::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
PolakRibiere::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::GradientStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

PolakRibiere::PolakRibiere()
    : MethodWithLineSearch()
{
}// PolakRibiere

PolakRibiere::PolakRibiere(const PolakRibiere& from)
    : MethodWithLineSearch(from)
{
}// PolakRibiere


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
PolakRibiere::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result>                            ap_result(new Result(1));
    auto_ptr<ConjugateGradientsMethodIteration> ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        assert(mcpLineSearch.Get() != NULL);
        assert(mcpStopCondition.Get() != NULL);

        const FunctionBase& function = dynamic_cast<const FunctionBase&>
                                                        (problem.Function());
        assert(function.IsDifferentiable());

        const unsigned int dimensions = problem.VariableCount();
        assert(static_cast<unsigned int>(mStartingPoint.nrows()) == dimensions);



        // Biecy punkt.
        ColumnVector        x(mStartingPoint);
        // Punkt z poprzedniej iteracji.
        // (inicjowany wektorem zerowym, aby moliwe byo
        // sprawdzanie przed iteracj wszystkich warunkw stopu).
        ColumnVector        previous_x(NullPoint);
        // Kierunek poszukiwa jednowymiarowych
        ColumnVector        direction(dimensions);
        // Dugo kroku wykonanego w kierunku direction
        double              tau;

        // Gradient.
        ColumnVector        gradient(dimensions);
        // Gradient z poprzedniej iteracji.
        ColumnVector           previous_gradient(dimensions);
        /* Rnica gradientw, zmienna pomocnicza we wzorze na wspczynnik
         * gamma.
         */
        ColumnVector        gradient_difference(dimensions);
        // Licznik we wzorze na wspczynnik gamma
        Matrix              numerator;
        // Norma gradientu.
        double              gradient_norm;
        // Norma poprzedniego gradientu. Jej warto nie ma znaczenia w
        // pierwszej iteracji.
        double              previous_gradient_norm = 0;
        // Wspczynnik wykorzystywany w obliczaniu kierunku sprzonego.
        // Jego warto nie ma znaczenia w pierwszej iteracji.
        double              gamma = 0;


        LOG(DEBUG,wxT("-------------------------------------"));



        /* Gradient wyliczamy przed pierwsz iteracj algorytmu, gdy moliwe
         * jest, e punkt startowy jest akceptowalnie blisko punktu minimum.
         */
        gradient = function.Gradient(x);
        gradient_norm   = gradient.norm_Frobenius();

        LOG(DEBUG,wxT("Iteracja wstepna\n"));
        LOG(DEBUG,wxT("Punkt:\n") << x);
        LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                << function.Evaluate(x) << wxT("\n"));
        LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
        LOG(DEBUG,wxT("Norma gradientu:\n") << gradient_norm);

        LOG(DEBUG,wxT("-------------------------------------"));



        while (step_count < STEP_LIMIT &&
               !mcpStopCondition->IsMet(previous_x, x, function, gradient_norm))
        {
            ++step_count;

            bool gradient_reset = (step_count == 1);

            LOG(DEBUG,wxT("Iteracja ") << step_count << wxT("\n"));

            if (gradient_reset)
            {
                direction = -1.0 * gradient.as_column();
                LOG(DEBUG,wxT("Kierunek przeciwny do gardientu:\n")
                                                            << direction);
            }
            else
            {
                LOG(DEBUG,wxT("Gradient:\n") << gradient.as_column());
                LOG(DEBUG,wxT("Poprzedni Gradient:\n") << previous_gradient);
                gradient_difference = gradient - previous_gradient;
                //LOG(DEBUG, wxT("Roznica pomiedzy gradientami:\n") << diff);
                numerator = gradient_difference.t() * gradient;
                //LOG(DEBUG, wxT("Licznik wzoru na gamma:\n")
                //                                  << product.as_scalar());

                // Taka sytuacja nie moe si zdarzy jeli algorytm dziaa
                // prawidowo.
                assert(previous_gradient_norm != 0);

                gamma = numerator.as_scalar() /
                         (previous_gradient_norm * previous_gradient_norm);

                direction = -1.0 * gradient + gamma * direction;

                LOG(DEBUG,wxT("gamma: ") << gamma);
                LOG(DEBUG,wxT("Kierunek sprzezony z poprzednimi:\n")
                                                            << direction);
            }

            previous_x = x;
            x = StepInDirection(function, x, direction, &tau);

            ap_iteration.reset(new ConjugateGradientsMethodIteration
                       (previous_x,
                        gradient,
                        gamma,
                        gradient_reset,
                        direction,
                        tau,
                        x));
            ap_result->AddIteration
                        (auto_ptr<const IterationData>(ap_iteration));

            previous_gradient       = gradient;
            previous_gradient_norm  = previous_gradient.norm_Frobenius();

            gradient        = function.Gradient(x);
            gradient_norm   = gradient.norm_Frobenius();

            LOG(DEBUG,wxT("Nowy Punkt:\n") << x);
            LOG(DEBUG,wxT("Wartosc funkcji w tym punkcie :")
                    << function.Evaluate(x) << wxT("\n"));
            LOG(DEBUG,wxT("Gradient w tym punkcie:\n") << gradient);
            LOG(DEBUG,wxT("Norma gradientu:\n") << gradient_norm << wxT("\n"));

            LOG(DEBUG,wxT("-------------------------------------"));

            NotifyYield();
        }

        ap_result->rFinalResult() = x;

    }
    catch (const BaseException& error)
    {
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                std::auto_ptr<Result>(ap_result));
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return auto_ptr<Result>(ap_result);
}// Optimize

wxString PolakRibiere::mName(_("Polak-Ribiere's"));

// Utwrz i zarejestruj prototyp.
PolakRibiere PolakRibiere::mPolakRibiere;

///////////////////// PolakRibiere::GradientMemory /////////////////////////
