#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "PowellVariant1.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


PowellVariant1::~PowellVariant1()
{
}// ~PowellVariant1


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
PowellVariant1::Clone() const
{
    return auto_ptr<Method>(new PowellVariant1(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
PowellVariant1::Name() const
{
    return PowellVariant1::mName;
}// Name

/* virtual */ wxString&
PowellVariant1::rName()
{
    return PowellVariant1::mName;
}// rName

/* virtual */ MethodIdType
PowellVariant1::Id() const
{
    return PowellVariant1::ClassId();
}// Id

/* static */ MethodIdType
PowellVariant1::ClassId()
{
    const static MethodIdType PowellVariant1Id = Method::GenerateId();

    return PowellVariant1Id;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
PowellVariant1::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
PowellVariant1::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::StandardStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

PowellVariant1::PowellVariant1()
    : MethodWithLineSearch()
{
}// PowellVariant1

PowellVariant1::PowellVariant1(const PowellVariant1& from)
    : MethodWithLineSearch(from)
{
}// PowellVariant1

void
PowellVariant1::SearchAllDirections (const FunctionBase&        function,
                                     ColumnVector&              rxBest,
                                     const SquareMatrix&        D,
                                     PowellVariant1Iteration&   rIterationData)
                                                                const
{
    double tau; // Optymalna dugo kroku w biecym kierunku.

    // Szukamy minimum dla aktualnych kierunkw,
    // wykonujc w kadym z nich krok o optymalnej dugoci.
    for (int i = 1; i <= rxBest.nrows(); ++i)
    {
        LOG(DEBUG, wxT("i: ") << i);

        rxBest = StepInDirection(function, rxBest, D.column(i), &tau);

        rIterationData.AddTrialPoint(rxBest, tau);
        LOG(DEBUG, wxT("x:") << endl << rxBest <<  wxT("Tau: ") <<
                tau << endl << wxT("f(x): ") << function.Evaluate(rxBest));
     } // for

} // SearchAllDirections


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
PowellVariant1::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(0));  // Wynik

    // Funkcja celu.
    const FunctionBase& function = problem.Function();
    // Aktualny punkt.
    ColumnVector x;
    // Punkt startowy aktualnej iteracji.
    ColumnVector x_start(mStartingPoint);
    // Odlego midzy ostatnim i startowym punktem aktualnej iteracji.
    ColumnVector x_diff;

    // Wektory (kolumnowe) kierunkw.
    SquareMatrix D;
    // Nowy kierunek sprzony.
    ColumnVector new_direction;

    // Optymalna dugo kroku w danym kierunku.
    double tau;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);

    try
    {
        // Tworzymy pierwsz baz kierunkw - zaczynamy
        // od kierunkw wzdu osi ukadu wsprzdnych.
        D = IdentityMatrix(x_start.nrows());

        auto_ptr<PowellVariant1Iteration> ap_result_iteration(
                    new PowellVariant1Iteration(x_start, D));

        // Faza wstpna - znajdujemy pierwszy punkt startowy.
        x_start = StepInDirection(function, x_start, D.column(x_start.nrows()),
                &tau);
        LOG(DEBUG, wxT("D:") << endl << D);
        LOG(DEBUG, wxT("x_start: ") << endl << x_start <<
                wxT("tau: ") << tau <<
                wxT("f(x_start): ") << function.Evaluate(x_start));
        x = x_start;

        ap_result_iteration->AddTrialPoint(x_start, tau);
        ap_result_iteration->rEndPoint() = x_start;
        ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));

        while (!finished && step_count < STEP_LIMIT)
        {
            ++step_count;

            auto_ptr<PowellVariant1Iteration> ap_result_iteration(
                    new PowellVariant1Iteration(x_start, D));

            LOG(DEBUG, wxT("---step ") << step_count);
            LOG(DEBUG, wxT("x_start: ") << endl << x_start <<
                    wxT("f(x_start): ") << function.Evaluate(x_start));
            LOG(DEBUG, wxT("D:") << endl << D);

            // Wykonujemy kroki we wszystkich aktualnych kierunkach.
            SearchAllDirections(problem.Function(), x, D, *ap_result_iteration);
            x_diff = x - x_start;

            ap_result_iteration->rEndPoint() = x;
            ap_result_iteration->rTrialDistance() = x_diff;

            // Sprawdzamy warunek stopu.
            finished = mcpStopCondition->IsMet(x_start, x, function);

            if (!finished)
            {
                // Wyznaczamy nowy kierunek sprzony.
                new_direction = x_diff / x_diff.norm_Frobenius();
                LOG(DEBUG, wxT("new direction: ") << endl << new_direction);

                ap_result_iteration->rNewDirection() = new_direction;

                // Znajdujemy kolejny punkt startowy wykonujc optymalny krok
                // w nowym kierunku.
                x = StepInDirection(function, x, new_direction, &tau);
                LOG(DEBUG, wxT("x_n+1:") << endl << x <<
                        wxT("tau: ") << tau);

                ap_result_iteration->rNewPoint() =
                    PowellIteration::TrialPoint(x, tau);

                // Dopisujemy nowy kierunek do bazy.
                for (int i = 1; i < D.ncols(); ++i)
                    D.column(i) = D.column(i + 1);
                D.column(D.ncols()) = new_direction;
                LOG(DEBUG, wxT("New D:") << endl << D);

                ap_result_iteration->rNewD() = D;

                x_start = x;
            }

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));
            NotifyYield();
        } // while
    } // try
    catch (BaseException)
    { // Wyjtek newmata.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x;
    LOG(INFO, wxT("Step count: ") << step_count);

    return auto_ptr<Result>(ap_result);
}// Optimize

wxString PowellVariant1::mName(_("Powell's Variant 1"));

// Utwrz i zarejestruj prototyp.
PowellVariant1 PowellVariant1::mPowellVariant1;
