#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "PowellVariant2.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


PowellVariant2::~PowellVariant2()
{
}// ~PowellVariant2


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
PowellVariant2::Clone() const
{
    return auto_ptr<Method>(new PowellVariant2(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
PowellVariant2::Name() const
{
    return PowellVariant2::mName;
}// Name

/* virtual */ wxString&
PowellVariant2::rName()
{
    return PowellVariant2::mName;
}// rName

/* virtual */ MethodIdType
PowellVariant2::Id() const
{
    return PowellVariant2::ClassId();
}// Id

/* static */ MethodIdType
PowellVariant2::ClassId()
{
    const static MethodIdType PowellVariant2Id = Method::GenerateId();

    return PowellVariant2Id;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
PowellVariant2::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
PowellVariant2::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::StandardStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

PowellVariant2::PowellVariant2()
    : MethodWithLineSearch()
{
}// PowellVariant2

PowellVariant2::PowellVariant2(const PowellVariant2& from)
    : MethodWithLineSearch(from)
{
}// PowellVariant2

void
PowellVariant2::SearchAllDirections (const FunctionBase&        function,
                                     ColumnVector&              rxBest,
                                     const SquareMatrix&        D,
                                     double&                    rTau_p,
                                     unsigned&                  rP,
                                     PowellVariant2Iteration&   rIterationData)
                                                                    const
{
    double tau; // Optymalna dugo kroku w biecym kierunku.
    int i;

    rTau_p = -DBL_MAX;
    rP = 0;

    // Szukamy minimum dla aktualnych kierunkw,
    // wykonujc w kadym z nich krok o optymalnej dugoci.
    for (i = 1; i <= rxBest.nrows(); ++i)
    {
        LOG(DEBUG, wxT("i: ") << i);

        // Wykonujemy optymalny krok.
        rxBest = StepInDirection(function, rxBest, D.column(i), &tau);

        rIterationData.AddTrialPoint(rxBest, tau);
        LOG(DEBUG, wxT("x:") << endl << rxBest << wxT("Tau: ") << tau <<
                endl << wxT("f(x): ") << function.Evaluate(rxBest));
        // Sprawdzamy, czy wykonany krok jest najduszy
        // w tym etapie.
        if (fabs(tau) > rTau_p)
        { // Zapamitujemy dugo kroku i indeks kierunku.
            rTau_p = fabs(tau);
            rP = i;
        }
    } // for

    assert(rP != 0);
} // SearchAllDirections


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
PowellVariant2::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(0));  // Wynik

    // Funkcja celu.
    const FunctionBase& function = problem.Function();
    // Aktualny punkt.
    ColumnVector x;
    // Punkt startowy aktualnej iteracji.
    ColumnVector x_start(mStartingPoint);
    // Odlego midzy ostatnim i startowym punktem aktualnej iteracji.
    ColumnVector x_diff;

    // Wektory (kolumnowe) kierunkw.
    SquareMatrix D;
    // Nowy kierunek sprzony.
    ColumnVector new_direction;

    // Optymalna dugo kroku w danym kierunku.
    double tau;
    // Najwiksza dugo kroku w aktualnej iteracji.
    double tau_p;
    // Indeks kierunku, w ktrym wykonano najduszy krok.
    unsigned p;

    // Wyznacznik macierzy kierunkw.
    double delta;
    // Wyznacznik macierzy kierunkw po wprowadzeniu do
    // niej nowego kierunku.
    double new_delta;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);

    try
    {
        // Tworzymy pierwsz baz kierunkw - zaczynamy
        // od kierunkw wzdu osi ukadu wsprzdnych.
        D = IdentityMatrix(x_start.nrows());
        delta = 1;

        auto_ptr<PowellVariant2Iteration> ap_result_iteration(
                    new PowellVariant2Iteration(x_start, D));

        // Faza wstpna - znajdujemy pierwszy punkt startowy.
        x_start = StepInDirection(function, x_start, D.column(x_start.nrows()),
                &tau);
        LOG(DEBUG, wxT("D:") << endl << D);
        LOG(DEBUG, wxT("x_start: ") << endl << x_start <<
                wxT("tau: ") << tau <<
                wxT("f(x_start): ") << function.Evaluate(x_start));
        x = x_start;

        ap_result_iteration->AddTrialPoint(x_start, tau);
        ap_result_iteration->rEndPoint() = x_start;
        ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));

        while (!finished && step_count < STEP_LIMIT)
        {
            ++step_count;

            auto_ptr<PowellVariant2Iteration> ap_result_iteration(
                    new PowellVariant2Iteration(x_start, D));

            LOG(DEBUG, wxT("---step ") << step_count);
            LOG(DEBUG, wxT("x_start: ") << endl << x_start <<
                    wxT("f(x_start): ") << function.Evaluate(x_start));
            LOG(DEBUG, wxT("D:") << endl << D);

            // Wykonujemy kroki we wszystkich aktualnych kierunkach.
            SearchAllDirections(problem.Function(), x, D, tau_p, p,
                                *ap_result_iteration);
            x_diff = x - x_start;

            ap_result_iteration->rEndPoint() = x;
            ap_result_iteration->rTrialDistance() = x_diff;

            // Sprawdzamy warunek stopu.
            finished = mcpStopCondition->IsMet(x_start, x, function);

            if (!finished)
            {
                // Wyznaczamy nowy kierunek sprzony.
                new_direction = x_diff / x_diff.norm_Frobenius();

                ap_result_iteration->rTrialDistance() = x_diff;
                ap_result_iteration->rNewDirection() = new_direction;

                LOG(DEBUG, wxT("x_diff: ") << endl << x_diff);
                LOG(DEBUG, wxT("||x_diff||: ") << x_diff.norm_Frobenius());
                LOG(DEBUG, wxT("new direction: ") << endl << new_direction);

                // Znajdujemy kolejny punkt startowy wykonujc optymalny krok
                // w nowym kierunku.
                x = StepInDirection(function, x, new_direction, &tau);
                LOG(DEBUG, wxT("x_n+1:") << endl << x <<
                        wxT("tau: ") << tau);

                ap_result_iteration->rNewPoint() =
                    PowellIteration::TrialPoint(x, tau);

                // Wyliczamy now delt i ewentualnie dopisujemy
                // nowy kierunek do bazy
                // (gdy poniszy warunek nie jest speniony, baza
                // nie zmienia si).
                LOG(DEBUG, wxT("tau_p: ") << tau_p << wxT(". p: ") << p);
                LOG(DEBUG, wxT("delta: ") << delta);
                LOG(DEBUG, wxT("||x_diff||: ") << x_diff.norm_Frobenius());
                new_delta = tau_p * delta / x_diff.norm_Frobenius();
                LOG(DEBUG, wxT("new delta: ") << new_delta);

                ap_result_iteration->rP()           = p;
                ap_result_iteration->rTauMax()      = tau_p;
                ap_result_iteration->rDelta()       = delta;
                ap_result_iteration->rNewDelta()    = new_delta;

                if (new_delta >= 0.8)
                {
                    D.column(p) = new_direction;
                    LOG(DEBUG, wxT("New D:") << endl << D);
                    delta = new_delta;

                    ap_result_iteration->rNewD() = D;
                }

                x_start = x;
            } // if

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));
            NotifyYield();
        } // while
    } // try
    catch (BaseException)
    { // Wyjtek newmata.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x;
    LOG(INFO, wxT("Step count: ") << step_count);

    return auto_ptr<Result>(ap_result);
}// Optimize

wxString PowellVariant2::mName(_("Powell's Variant 2"));

// Utwrz i zarejestruj prototyp.
PowellVariant2 PowellVariant2::mPowellVariant2;
