/**  Metoda Powella - wariant drugi.
 *
 * Bezgradientowa metoda kierunkw sprzonych.
 *
 * Od wariantu pierwszego odrnia j metoda modyfikacji bazy kierunkw
 * \f$D\f$ (krok 4 procedury). Metoda zaimplementowana w tym wariancie
 * gwarantuje zachowanie liniowej niezalenoci kierunkw.
 *
 * Metoda oparta na dwch twierdzeniach:<br>
 * 1. Dla wzajemnie sprzonych wzgldem dodatnio okrelonej macierzy
 * \f$A\f$ kierunkw
 * \f[ d^{(1)}, \: d^{(2)}, \: \ldots, \: d^{(n)} \f]
 * minimum formy kwadratowej
 * \f[ f(x) = q + b^T x + \frac{1}{2} x^T A x \f]
 * moe by znalezione w wyniku minimalizacji funkcji \f$f(x)\f$
 * wzdu kadego z kierunkw \f$d^{(i)}\f$ tylko raz.<br>
 * Kierunki \f$ d^{(1)}, \: d^{(2)}, \: \ldots, \: d^{(n)} \f$ s wzajemnie
 * sprzone wzgldem dodatnio okrelonej macierzy \f$A\f$, gdy
 * speniony jest warunek:
 * \f[ (d^{(i)})^T A d^{(j)} = 0 \mbox{ dla wszystkich } i \neq j. \f]
 * 2. Jeli \f$x^0\f$ jest minimum w kierunku \f$d\f$ nalecym do pewnej
 * rozmaitoci liniowej oraz jeli \f$x^1\f$ jest minimum w tym samym
 * kierunku \f$d\f$, ale w innej rozmaitoci zawierajcej ten kierunek
 * (np. \f$x^0\f$ i \f$x^1\f$ znaleziono w wyniku minimalizacji w kierunku
 * \f$d\f$ zaczynajc z rnych punktw startowych), to kierunek
 * \f[ (x^1 - x^0) \f]
 * czcy te dwa punkty jest sprzony z kierunkiem \f$d\f$.
 *
 * Twierdzenie 1 zapewnia, e metoda, przy wykorzystaniu dokadnych
 * poszukiwa w kierunku, bdzie znajdowa minimum wielomianu drugiego
 * stopnia w \f$n\f$ krokach, gdzie \f$n\f$ jest liczb zmiennych.<br>
 * Twierdzenie 2 dostarcza atwej metody  generowania kierunkw
 * sprzonych.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x^0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\varepsilon\f$ - wymagana dokadno oblicze,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami.
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$i\f$ - indeks aktualnego kierunku,<br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$ D = D^{(1)},\:  D^{(2)}, \: \ldots, \: D^{(n)} \f$  -
 * baza wzajemnie ortogonalnych kierunkw
 * (macierz utworzona z wektorw kolumnowych),<br>
 * \f$x\f$ - aktualny punkt,<br>
 * \f$x_s\f$ - punkt startowy etapu,<br>
 * \f$\tau\f$ - dugo optymalnego krok w danym kierunku \f$d\f$, wyznaczona
 * przez metod optymalizacji w kierunku,<br>
 * \f$\tau_p\f$ - najduszy krok wzdu jednego z \f$n\f$ kierunkw
 * w fazie przeszukiwania,<br>
 * \f$p\f$ - indeks kierunku, w ktrym wykonano najduszy krok,<br>
 * \f$\Delta^k\f$ - wyznacznik macierzy kierunkw \f$D\f$ w
 * \f$k\f$-tej iteracji,<br>
 * \f$\Delta_p\f$ - wyznacznik macierzy kierunkw \f$D\f$ po
 * ewentualnym umieszczeniu w~niej nowego kierunku sprzonego.<br>
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy \f$ D = I(n) \f$, gdzie \f$I(n)\f$ - macierz identycznociowa
 * o wymiarach \f$ n \times n \f$.<br>
 * Obliczamy pierwszy punkt pocztkowy:
 * \f[ \tau = \arg \min_{\tau} f(x + \tau D^{(n)}), \f]
 * \f[ x_s = x^0 + \tau  D^{(n)}. \f]
 * Podstawiamy:
 * \f[ x = x_s, \f]
 * \f[ \Delta^1 = 1, \f]
 * \f[ \tau_p = 0, \f]
 * \f[ k = 1 , \f]
 * \f[ i = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Przy pomocy metody optymalizacji w kierunku wyznaczamy optymaln dugo
 * kroku \f$\tau\f$ i wykonujemy krok:
 * \f[ \tau = \arg \min_{\tau} f(x + \tau D^{(i)}) . \f]
 * \f[ x = x + \tau D^{(i)} . \f]
 * Jeli wykonany krok jest najduszy w etapie (\f$|\tau| > \tau_p\f$),
 * to zapamitujemy jego dugo i indeks kierunku:
 * \f[ \tau_p = |\tau|, \f]
 * \f[ p = i. \f]
 *
 * Podstawiamy \f$i = i + 1\f$ i sprawdzamy, czy \f$i \le n \f$.
 * Jeli tak, ponownie wykonujemy krok 1.
 * Jeli nie, podstawiamy \f$i = 1\f$ i przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Sprawdzamy warunek stopu.
 * Jeli jest on speniony, to STOP.
 * Wynikiem jest \f$x\f$.<br>
 * W przeciwnym razie przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Wyznaczamy nowy kierunek sprzony:
 * \f[ D^{(n+1)} = \frac{x - x_s}{\| x - x_s \|} . \f]
 * Nastpnie wykonujemy krok o optymalnej dugoci w tym kierunku:
 * \f[ \tau = \arg \min_{\tau} f(x + \tau D^{(n+1)}) , \f]
 * \f[ x = x + \tau D^{(n+1)}. \f]
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy warunek modyfikacji bazy:<br>
 * \f[ \Delta_p = \frac{\tau_p \Delta^k}{\|x - x_s\|} \ge 0,8 \; , \f]
 * Jeli warunek ten jest speniony, kierunek, w ktrym wykonano
 * najduszy krok \f$\tau_p\f$, zostaje zastpiony nowym kierunkiem sprzonym:
 * \f[ D^{(p)} = D^{(n+1)} \f]
 * i zostaje zapamitany nowy wyznacznik bazy:
 * \f[ \Delta^{k+1} = \Delta_p. \f]
 * Jeli warunek nie jest speniony, baza kierunkw nie jest zmieniana.<br>
 * Podstawiamy aktualny punkt za punkt startowy dla kolejnego etapu:
 * \f[ x_s = x. \f]
 * Podstawiamy \f$k = k + 1\f$, \f$\tau_p = 0\f$ i wracamy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Wariant ten jest mniej wydajny od podstawowego, jednak gwarantuje
 * zachowanie liniowej niezalenoci kierunkw.
 *
 * Metoda zaimplementowana na podstawie opisu zamieszczonego w:<br>
 * Findeisen W., Szymanowski J., Wierzbicki A.: <i>Teoria i metody
 * obliczeniowe optymalizacji</i>. Warszawa, PWN 1977. <br>
 * W odrnieniu od tego opisu, w implementacji warunek stopu sprawdzany
 * jest przed obliczeniem nowego kierunku sprzonego, w celu uniknicia
 * ryzyka dzielenia przez 0 przy wyznaczaniu kierunku.
 *
 * @see MethodWithLineSearch
 * @see PowellVariant1
 */

#ifndef _PowellVariant2_h
#define _PowellVariant2_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../MethodWithLineSearch.h"
#include "../ui/MethodWithLineSearchPanel.h"
#include "PowellVariant2Iteration.h"

// FORWARD REFERENCES
//


class PowellVariant2
    : public MethodWithLineSearch
{
public:
// LIFECYCLE

    friend class MethodWithLineSearchPanel<PowellVariant2>;

    /** Destruktor.
     */
    ~PowellVariant2(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    PowellVariant2(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    PowellVariant2(const PowellVariant2& from);

    /**
     * Przeszukuje kierunki.
     *
     * Dla kadego kierunku \f$d^{(i)}, \: i=1,\ldots,n\f$
     * metoda wykonuje jeden krok prbny o optymalnej dugoci
     * \f$\tau\f$:
     * \f[ \tau = \arg \min_{\tau} f(x + \tau d^{(i)}) , \f]
     * \f[ x = x + \tau d^{(i)} . \f]
     * Ponadto, zapamitywany jest indeks \f$p\f$ kierunku,
     * w ktrym wykonano najduszy krok i dugo tego kroku
     * \f$\tau_p\f$:
     * \f[ \tau_p = \max_{i \in \overline{1,n}} |\tau_i| . \f]
     *
     * @param function Funkcja celu (\f$f\f$).
     * @param rxBest Referencja na aktualny punkt (\f$x\f$).
     * @param D Baza kierunkw \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * @param rTau_p Referencja na dugo najduszego kroku (\f$\tau_p\f$).
     * @param rP Referencja na indeks kierunku, w ktrym wykonano najduszy
     * krok (\f$p\f$).
     * @param rIterationData Referencja na dane biecej iteracji dla wyniku.
     */
    void SearchAllDirections(const FunctionBase&        function,
                             ColumnVector&              rxBest,
                             const SquareMatrix&        D,
                             double&                    rTau_p,
                             unsigned&                  rP,
                             PowellVariant2Iteration&   rIterationData) const;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);
    static wxString      mName; ///< Nazwa metody.
    static PowellVariant2 mPowellVariant2;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _PowellVariant2_h_
