#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "QuadraticApproximation.h"  // class implemented
#include "QuadraticApproximationIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


QuadraticApproximation::~QuadraticApproximation()
{
}// ~QuadraticApproximation


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
QuadraticApproximation::Clone() const
{
    return auto_ptr<Method>(new QuadraticApproximation(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString&
QuadraticApproximation::rName()
{
    return QuadraticApproximation::mName;
}// rName

/* virtual */ const wxString&
QuadraticApproximation::Name() const
{
    return QuadraticApproximation::mName;
}// Name

/* virtual */ MethodIdType
QuadraticApproximation::Id() const
{
    return QuadraticApproximation::ClassId();
}// Id

/* static */ MethodIdType
QuadraticApproximation::ClassId()
{
    const static MethodIdType QuadraticApproximationId = Method::GenerateId();

    return QuadraticApproximationId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
QuadraticApproximation::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

QuadraticApproximation::QuadraticApproximation()
    : SectionLineSearch()
{
}// QuadraticApproximation

QuadraticApproximation::QuadraticApproximation(const QuadraticApproximation& from)
    : SectionLineSearch(from)
{
}// QuadraticApproximation


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
QuadraticApproximation::Solve(const UnivariateFunction& function,
                              double                    initialA,
                              double                    initialB,
                              bool                      generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));
    auto_ptr<QuadraticApproximationIteration> ap_iteration;

    double a;           // Aktualny pocztek odcinka.
    double m = 0.0;     // Aktualny punkt prbny wewntrz odcinka.
    double b;           // Aktualny koniec odcinka.
    double x = 0.0;     // Punkt minimalizujcy funkcj aproksymujc.

    double length;  // Aktualna dugo odcinka.

    // Wartoci funkcji w punktach prbnych.
    double value_a;
    double value_m = 0.0;
    double value_b;
    double value_x = 0.0;

    // Informacja o tym, czy znalezione x jest minimum funkcji
    // aproksymujcej.
    bool success;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    try
    {
        if (initialA > initialB) swap(initialA, initialB);

        a = initialA;
        value_a = function.Evaluate(a);
        b = initialB;
        value_b = function.Evaluate(b);

        // Za pierwszy wewntrzny punkt prbny bierzemy
        // rodek odcinka.
        m = (a + b) / 2;
        value_m = function.Evaluate(m);

        while (!finished && step_count < STEP_LIMIT)
        {
            ++step_count;
            LOG(DEBUG, wxT("-----step") << step_count);
            LOG(DEBUG, wxT("a: ")       << a <<
                       wxT(". f(a): ")  << function.Evaluate(a) << endl <<
                       wxT("m: ")       << m <<
                       wxT(". f(m): ")  << function.Evaluate(m) << endl <<
                       wxT("b: ")       << b <<
                       wxT(". f(b): ")  << function.Evaluate(b));


            // Znajdujemy minimum funkcji aproksymujcej.
            x = FindApproximationMinimizer(a, m, b, value_a, value_m, value_b,
                    success);
            value_x = function.Evaluate(x);

            if (generateResult)
            {
                ap_iteration.reset(new QuadraticApproximationIteration(
                        a, b, m, x, value_x < value_m));

                if (!success)
                {
                    // TODO: no to chyba jest lekkie naduycie wyjtkw :)
                    throw Error(_("Approximation failed - found maximum \
or saddle point of the approximating function."));
                }
            }
            LOG(DEBUG, wxT("x: ")      << x <<
                       wxT(". f(x): ") << function.Evaluate(x));

            // Sprawdzamy pooenie znalezionego punktu.
            if (fabs(x - a) < mEpsilon || fabs(x - m) < mEpsilon ||
                    fabs(x - b) < mEpsilon)
            { // Minimum f. aproksymujcej jest wystarczajco blisko
              // jednego z punktw prbnych - koczymy dziaanie metody.
                finished = true;

                if (generateResult)
                {
                    ap_iteration->rNextSection() =
                        QuadraticApproximationIteration::NONE_OK;
                }

            }
            else if (x > a && x < m)
            { // Minimum f. aproksymujcej znajduje si przed punktem wewntrznym.
                if (value_x < value_m)
                {
                    // Redukujemy odcinek do [a, m].
                    b = m;
                    value_b = value_m;

                    if (generateResult)
                    {
                        ap_iteration->rNextSection() =
                            QuadraticApproximationIteration::AM;
                    }
                    // Nowym punktem wewntrznym staje si x.
                    m = x;
                    value_m = value_x;
                }
                else
                {
                    // Redukujemy odcinek do [x, b].
                    a = x;
                    value_a = value_x;

                    if (generateResult)
                    {
                        ap_iteration->rNextSection() =
                            QuadraticApproximationIteration::XB;
                    }
                }
            }
            else if (x > m && x < b)
            { // Minimum f. aproksymujcej znajduje si za punktem wewntrznym.
                 if (value_x < value_m)
                {
                    // Redukujemy odcinek do [m, b].
                    a = m;
                    value_a = value_m;

                    if (generateResult)
                    {
                        ap_iteration->rNextSection() =
                            QuadraticApproximationIteration::MB;
                    }

                    // Nowym punktem wewntrznym staje si x.
                    m = x;
                    value_m = value_x;
                }
                else
                {
                    // Redukujemy odcinek do [a, x].
                    b = x;
                    value_b = value_x;

                    if (generateResult)
                    {
                        ap_iteration->rNextSection() =
                            QuadraticApproximationIteration::AX;
                    }
                }
            }
            else if (x < a || x > b)
            { // Znaleziono minimum poza pocztkowym odcinkiem
              // - przebieg prawdopodobnie nie bdzie zbieny.

                throw Error(_("Found minimum outside of initial section - \
optimization aborted."));
            }

            length = b - a;
            finished = finished || length < mEpsilon;

            if (generateResult)
            {
                ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_iteration));
            }
            NotifyYield();
        } // while
    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const EvaluationError& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }
    catch (const Error& error)
    {
        if (generateResult)
         {
              ap_iteration->rNextSection() =
                  QuadraticApproximationIteration::NONE_ERROR;
              ap_result->AddIteration(auto_ptr<const IterationData>
                  (ap_iteration));
         }

        throw OptimizationError(error.What(), ap_result);
    }

    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }

    ColumnVector final_result(1);

    if (value_x < value_m)  final_result(1) = x;
    else                    final_result(1) = m;
    ap_result->rFinalResult() = final_result;

    return ap_result;
} // Solve

double QuadraticApproximation::FindApproximationMinimizer(
                                                    double    a,
                                                    double    m,
                                                    double    b,
                                                    double    valueA,
                                                    double    valueM,
                                                    double    valueB,
                                                    bool&     success) const
{
    double numerator;
    double denominator;

    numerator = (m*m - b*b) * valueA + (b*b - a*a) * valueM +
                (a*a - m*m) * valueB;

    denominator = (m - b) * valueA + (b - a) * valueM +
                  (a - m) * valueB;

    LOG(DEBUG, wxT("L = ") << numerator << wxT(", M = ") << denominator);

    // Nieujemny mianownik oznacza, e druga pochodna jest niedodatnia, czyli
    // e znaleziono maksimum lub punkt siodowy funkcji aproksymujcej.
    if (denominator >= 0)   success = false;
    else                    success = true;

    return numerator / (2 * denominator);
} // FindApproximationMinimizer


wxString QuadraticApproximation::mName(_("Quadratic Approximation"));

// Utwrz i zarejestruj prototyp.
QuadraticApproximation QuadraticApproximation::mQuadraticApproximation;
