/** Metoda aproksymacji kwadratowej.
 *
 * Metoda optymalizacji w kierunku polegajca na stopniowym
 * skracaniu zadanego odcinka (patrz SectionLineSearch).
 *
 * Opiera si na zaoeniu, e w otoczeniu minimum funkcja celu moe by
 * aproksymowana funkcj kwadratow.<br>
 * W kadej iteracji dokonywana jest minimalizacja funkcji kwadratowej
 * zbudowanej na podstawie wartoci funkcji celu w trzech punktach prbnych:
 * \f$a\f$, \f$m\f$ i \f$b\f$. Do skonstruowania takiej funkcji aproksymujcej
 * wykorzystywana jest interpolacja Lagrange'a:
 * \f[ f(x) = F_a \frac{(x - a)(x - b)}{(a - m)(a - b)} +
 * F_m \frac{(x - a)(x - b)}{(m - a)(m - b)} +
 * F_b \frac{(x - a)(x - m)}{(b - a)(b - m)} . \f]
 * Pierwsza pochodna tej funkcji zeruje si w punkcie:
 * \f[ x = \frac{1}{2} \frac{(m^2 - b^2)F_a + (b^2 - a^2)F_m + (a^2 - m^2)F_b}
 * {(m - b)F_a + (b - a)F_m + (a - m)F_b}.\f]
 * Algorytm polega na zastpowaniu jednego z punktw prbnych \f$a\f$,
 * \f$m\f$, \f$b\f$ punktem \f$x\f$ i skracaniu odcinka w taki sposb,
 * aby obejmowa on estymowane minimum.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f(x)\f$ - minimalizowana funkcja jednej zmiennej,<br>
 * \f$a^0\f$ - lewy kraniec pocztkowego przedziau poszukiwa,<br>
 * \f$b^0\f$ - prawy kraniec pocztkowego przedziau poszukiwa,<br>
 * \f$\varepsilon\f$ - wymagana dokadno (graniczna dugo odcinka,
 * minimalna odlego \f$x\f$ od punktw prbnych).
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$a^k\f$ - lewy kraniec aktualnego przedziau poszukiwa -
 * pierwszy punkt prbny,<br>
 * \f$m^k\f$ - drugi punkt prbny,<br>
 * \f$b^k\f$ - prawy kraniec aktualnego przedziau poszukiwa -
 * trzeci punkt prbny,<br>
 * \f$F_a\f$ - warto funkcji celu w punkcie \f$a^k\f$,<br>
 * \f$F_m\f$ - warto funkcji celu w punkcie \f$m^k\f$,<br>
 * \f$F_b\f$ - warto funkcji celu w punkcie \f$b^k\f$,<br>
 * \f$x^k\f$ - punkt zerowania si pierwszej pochodnej funkcji
 * aproksymujcej,<br>
 * \f$F_x\f$ - warto funkcji celu w punkcie \f$x^k\f$.<br>
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ a^1 = a^0, \f]
 * \f[ m^1 = \frac{a^0 + b^0}{2}, \f]
 * \f[ b^1 = b^0, \f]
 * \f[ F_a = f(a^1), \f]
 * \f[ F_m = f(m^1), \f]
 * \f[ F_b = f(b^1). \f]
 *
 * <br><b>Krok 1:</b><br>
 * Znajdujemy potencjalne minimum funkcji aproksymujcej:
 * \f[ x^k = \frac{1}{2} \frac{((m^k)^2 - (b^k)^2)F_a + ((b^k)^2 - (a^k)^2)F_m
 * + ((a^k)^2 - (m^k)^2)F_b}
 * {(m^k - b^k)F_a + (b^k - a^k)F_m + (a^k - m^k)F_b}.\f]
 * Jeli mianownik tego wyraenia jest liczb nieujemn, przechodzimy
 * do kroku 8.<br>
 * Podstawiamy \f$F_x = f(x^k)\f$ i przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Sprawdzamy, czy \f$x^k\f$ znajduje si wystarczajco blisko jednego z
 * punktw prbnych:<br>
 * Jeli \f$|x^k - a^k| < \varepsilon\f$
 * lub \f$|x^k - m^k| < \varepsilon\f$ lub \f$|x^k - b^k| < \varepsilon\f$,
 * to przechodzimy do kroku 7.<br>
 * Sprawdzamy, czy \f$x^k\f$ ley wewntrz badanego odcinka:<br>
 * Jeli \f$a^k < x^k < b^k\f$, to przechodzimy do kroku 3.
 * W przeciwnym wypadku przechodzimy do kroku 9.
 *
 * <br><b>Krok 3:</b><br>
 * Jeli \f$x^k < m^k\f$, przechodzimy do kroku 4. W przeciwnym wypadku
 * przechodzimy do kroku 5.
 *
 * <br><b>Krok 4:</b><br>
 * Jeli \f$F_x < F_m\f$, redukujemy odcinek do \f$[a; \: m]\f$:
 * \f[ a^{k+1} = a^k, \f]
 * \f[ m^{k+1} = x^k, \f]
 * \f[ b^{k+1} = m^k, \f]
 * \f[ F_b = F_m, \f]
 * \f[ F_m = F_x. \f]
 * Przechodzimy do kroku 6.<br>
 * Jeli \f$F_x \ge F_m\f$, redukujemy odcinek do \f$[x; \: b]\f$:
 * \f[ a^{k+1} = x^k, \f]
 * \f[ m^{k+1} = m^k, \f]
 * \f[ b^{k+1} = b^k, \f]
 * \f[ F_a = F_x, \f]
 * Przechodzimy do kroku 6.
 *
 * <br><b>Krok 5:</b><br>
 * Jeli \f$F_x < F_m\f$, redukujemy odcinek do \f$[m; \: b]\f$:
 * \f[ a^{k+1} = m^k, \f]
 * \f[ m^{k+1} = x^k, \f]
 * \f[ b^{k+1} = b^k, \f]
 * \f[ F_a = F_m, \f]
 * \f[ F_m = F_x. \f]
 * Przechodzimy do kroku 6.<br>
 * Jeli \f$F_x \ge F_m\f$, redukujemy odcinek do \f$[a; \: x]\f$:
 * \f[ a^{k+1} = a^k, \f]
 * \f[ m^{k+1} = m^k, \f]
 * \f[ b^{k+1} = x^k, \f]
 * \f[ F_b = F_x, \f]
 * Przechodzimy do kroku 6.
 *
 * <br><b>Krok 6:</b><br>
 * Sprawdzamy dugo odcinka po redukcji:<br>
 * Jeli \f$b^{k+1} - a^{k+1} \ge \varepsilon \f$, przechodzimy
 * do kroku 1. W przeciwnym wypadku przechodzimy do kroku 7.
 *
 * <br><b>Krok 7:</b><br>
 * Koczymy procedur. Jeli \f$F_x < F_m\f$, wynikiem jest
 * \f$x^k\f$. W przeciwnym wypadku wynikiem jest \f$m^k\f$.
 *
 * <br><b>Krok 8:</b><br>
 * Koczymy procedur z komunikatem bdu: "Optymalizacja nie powioda si -
 * znaleziono maksimum lub punkt siodowy funkcji aproksymujcej."
 *
 * <br><b>Krok 9:</b><br>
 * Koczymy procedur z komunikatem bdu: "Optymalizacja nie powioda si -
 * znalezione minimum funkcji aproksymujcej znajduje si poza badanym
 * odcinkiem."
 *
 * Metoda zaimplementowana na podstawie opisu zamieszczonego w:<br>
 * Schwefel H.-P.: <i>Evolution and Optimum Seeking</i>.
 * Nowy Jork, Wiley-Interscience 1995.
 *
 * @see SectionLineSearch
 */

#ifndef _QuadraticApproximation_h
#define _QuadraticApproximation_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"
#include "../ui/SectionLineSearchPanel.h"

// FORWARD REFERENCES
//

class QuadraticApproximation
    : public SectionLineSearch
{
public:
    friend class SectionLineSearchPanel<QuadraticApproximation>;

// LIFECYCLE

    /** Destruktor.
     */
    ~QuadraticApproximation(void);

// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

protected:

    /** Konstruktor domylny.
     */
    QuadraticApproximation(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    QuadraticApproximation(const QuadraticApproximation& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);

    /** Znajd minimum funkcji aproksymujcej.
     *
     * Metoda znajduje punkt \f$x^k\f$, w ktrym znajduje si minimum
     * funkcji kwadratowej aproksymujcej funkcj celu. Punkt wyznaczany
     * jest wedug wzoru:
     * \f[ x^k = \frac{1}{2} \frac{((m^k)^2 - (b^k)^2)F_a +
     * ((b^k)^2 - (a^k)^2)F_m + ((a^k)^2 - (m^k)^2)F_b}
     * {(m^k - b^k)F_a + (b^k - a^k)F_m + (a^k - m^k)F_b}.\f]
     * Jeli mianownik tego wyraenia jest liczb nieujemn, wyrzucany
     * jest wyjtek i cay algorytm koczy dziaanie.
     *
     * @param a Lewy kraniec aktualnego przedziau poszukiwa -
     * pierwszy punkt prbny (\f$a^k\f$).
     * @param m Drugi punkt prbny (\f$m^k\f$).
     * @param b Prawy kraniec aktualnego przedziau poszukiwa -
     * trzeci punkt prbny (\f$b^k\f$).
     * @param valueA Warto funkcji celu w punkcie \f$a^k\f$
     * (\f$F_a\f$).
     * @param valueM Warto funkcji celu w punkcie \f$m^k\f$
     * (\f$F_m\f$).
     * @param valueB Warto funkcji celu w punkcie \f$b^k\f$
     * (\f$F_b\f$).
     * @param success Informacja o tym, czy znaleziony punkt jest minimum
     * funkcji aproksymujcej.
     * @return Minimum funkcji aproksymujcej (\f$x^k\f$).
     */
    double          FindApproximationMinimizer(double a,
                                               double m,
                                               double b,
                                               double valueA,
                                               double valueM,
                                               double valueB,
                                               bool& success) const;

    static wxString      mName;                   ///< Nazwa metody.
    static QuadraticApproximation mQuadraticApproximation;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _QuadraticApproximation_h_
