#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "../text-util.h"
#include "RosenbrockDiscrete.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


RosenbrockDiscrete::~RosenbrockDiscrete()
{
}// ~RosenbrockDiscrete

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
RosenbrockDiscrete::Clone() const
{
    return auto_ptr<Method>(new RosenbrockDiscrete(*this));
}// Clone

/* virtual */ void
RosenbrockDiscrete::UpdateStartingConditions(const Result* result) const
{
    mStartingPoint = result->FinalResult();
}

/* virtual */ void
RosenbrockDiscrete::ResetStartingConditions() const
{
    mStartingPoint = mInitialStartingPoint;
}

//============================= ACCESS     ===================================

/* virtual */ const wxString&
RosenbrockDiscrete::Name() const
{
    return RosenbrockDiscrete::mName;
}// Name

/* virtual */ wxString&
RosenbrockDiscrete::rName()
{
    return RosenbrockDiscrete::mName;
}// rName

/* virtual */ MethodIdType
RosenbrockDiscrete::Id() const
{
    return RosenbrockDiscrete::ClassId();
}// Id

/* static */ MethodIdType
RosenbrockDiscrete::ClassId()
{
    const static MethodIdType RosenbrockDiscreteId = Method::GenerateId();

    return RosenbrockDiscreteId;
}// ClassId

/* virtual */ wxString
RosenbrockDiscrete::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    parameter_descriptions.Add(AlphaDescriptionRD(mAlpha));
    parameter_descriptions.Add(BetaDescriptionRD(mBeta));
    parameter_descriptions.Add(DeltaDescriptionRD(mDelta));
    parameter_descriptions.Add(TauDescriptionRD(mTau));

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);
    description << wxT('\n');
    description << StopConditionDescription(mcpStopCondition->ToString());
    description << StartingPointDescription(mInitialStartingPoint);

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
RosenbrockDiscrete::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
RosenbrockDiscrete::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::StandardStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

RosenbrockDiscrete::RosenbrockDiscrete()
    : Method(),
      mTau(0),
      mAlpha(3, 1, true),
      mBeta(0.5, 0, true, 1, true),
      mDelta(0.0001, 0, true),
      mStartingPoint(0),
      mInitialStartingPoint(0),
      mcpStopCondition()
{
}// RosenbrockDiscrete

RosenbrockDiscrete::RosenbrockDiscrete(const RosenbrockDiscrete& from)
    : Method(from),
      mTau(from.mTau),
      mAlpha(from.mAlpha),
      mBeta(from.mBeta),
      mDelta(from.mDelta),
      mStartingPoint(from.mStartingPoint),
      mInitialStartingPoint(from.mInitialStartingPoint),
      mcpStopCondition(from.mcpStopCondition)
{
}// RosenbrockDiscrete

void
RosenbrockDiscrete::SearchAllDirections(
        const FunctionBase&             function,
        ColumnVector&                   rxBest,
        double&                         rMin,
        const SquareMatrix&             D,
        ColumnVector&                   rTau,
        ColumnVector&                   rLambda,
        RosenbrockDiscreteIteration&    rIterationData) const
{
    ColumnVector x(rxBest.nrows()); // Biecy punkt prbny.
    double value;                   // Warto funkcji w punkcie x.
    double min_prev = DBL_MAX;      // Najmniejsza warto funkcji znaleziona
                                    // przy poprzednim przeszukiwaniu.
    double tau_prev;                // Dugo kroku w danym kierunku
                                    // przed zmian (poprzednia tylko dla
                                    // wynikowych danych iteracji).

    // Powtarzamy przeszukiwanie wszystkich kierunkw do momentu,
    // kiedy nie przyniesie ono adnej poprawy.
    while (rMin < min_prev)
    {
        min_prev = rMin;
        // Wykonujemy kroki prbne we wszystkich kierunkach.
        for (int i = 1; i <= x.nrows(); ++i)
        {
            LOG(DEBUG, wxT("i: ") << i);
            tau_prev = rTau(i);

            x = rxBest + rTau(i) * D.column(i);
            value = function.Evaluate(x);
            if (value < rMin)
            {   // Udany krok prbny, dopisujemy do sumy, wyduamy
                // i zapamitujemy punkt.
                rLambda(i) += rTau(i);
                rTau(i) *= mAlpha;
                rxBest = x;
                rMin = value;

                rIterationData.AddTrialPoint(x, true, tau_prev, rTau(i));
                LOG(DEBUG, wxT("Successful trial. x:") << endl << x <<
                        wxT("Tau: ") << rTau(i));
            }
            else
            {   // Nieudany krok prbny, skracamy, zmieniamy zwrot
                // i zostajemy przy poprzednim pukcie.
                rTau(i) *= -mBeta;

                rIterationData.AddTrialPoint(x, false, tau_prev, rTau(i));
                LOG(DEBUG, wxT("Failed trial. x:") << endl << x <<
                        wxT("Tau: ") << rTau(i));
                LOG(DEBUG, wxT("Value: ") << value);
            }
            LOG(DEBUG, wxT("Current minimum: ") << rMin);
        } // for
        LOG(DEBUG, wxT("-----------------------------"));
    } // while

} // SearchAllDirections


void
RosenbrockDiscrete::RotateBase (
        SquareMatrix&                   rD,
        const ColumnVector&             lambda,
        RosenbrockDiscreteIteration&    rIterationData) const
{
    int n = lambda.nrows(); // Liczba zmiennych
    // Wektory (kolumnowe) sum przesuni, od "najlepszego"
    // (uwzgldniajcego zmiany we wszystkich kierunkach) do
    // "najgorszego".
    // A(1) = lambda(1)*D(1) + lambda(2)*D(2) + ... + lambda(n)*D(n)
    // A(2) =                  lambda(2)*D(2) + ... + lambda(n)*D(n)
    // ...
    // A(n) =                                         lambda(n)*D(n)
    SquareMatrix A(n);
    // Wektor kwadratw norm A.
    // t(i) = ||A(i)||^2
    ColumnVector t(n);
    // Mianownik przy normalizacji wektorw ortogonalnych.
    double div;

    LOG(DEBUG, wxT("lambda: ") << endl << lambda);

    // Wypeniamy wektory A (od ostatniego do pierwszego).
    A.column(n) = lambda(n) * rD.column(n);

    for (int i = n - 1; i >= 1; --i)
    {
        A.column(i) = A.column(i + 1) + (lambda(i) * rD.column(i));
    }
    rIterationData.rA() = A;
    LOG(DEBUG, wxT("A:") << endl << A);

    // Ortogonalizujemy wektory A (metod Palmera).
    // Powstae wektory ortogonalne po normalizacji
    // staj si nowymi wektorami kierunkw.
    t(n) = lambda(n) * lambda(n);
    for (int i = n - 1; i >= 1; --i)
    {
        t(i) = t(i + 1) + (lambda(i) * lambda(i));
    }

    for (int i = n; i >= 2; --i)
    {
        div = sqrt(t(i - 1) * t(i));
        if (div != 0)
        {
            rD.column(i) = (lambda(i - 1) * A.column(i) -
                            rD.column(i - 1) * t(i)        ) / div;
        }

    } // for

    div = sqrt(t(1));
    rD.column(1) = A.column(1) / div;

    rIterationData.rNewD() = rD;
} // RotateBase

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
RosenbrockDiscrete::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    // Najlepszy punkt w obecnym etapie.
    ColumnVector x_best(mStartingPoint);
    // Najlepszy punkt w poprzednim etapie.
    ColumnVector x_prev_best(x_best);
    // Najmniejsza warto funkcji w obecnym etapie.
    double min;
    // Najmniejsza warto funkcji w poprzednim etapie.
    double min_prev;

    // Wektory (kolumnowe) kierunkw.
    SquareMatrix D(problem.VariableCount());
    // Roboczy wektor dugoci krokw (mTau nie jest zmieniany).
    ColumnVector tau(mTau);
    // Najmniejsza (po wszystkich kierunkach)
    // dugo kroku dla nastpnej iteracji.
    double tau_min;

    // Wektor sum dugoci udanych krokw prbnych.
    ColumnVector    lambda(problem.VariableCount());

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    assert(mAlpha > 1);
    assert((mBeta > 0) && (mBeta < 1));
    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpStopCondition.Get() != NULL);

    try
    {
        lambda = 0.0;
        // Wyliczamy i zapamitujemy warto funkcji w punkcie
        // startowym.
        min = problem.Function().Evaluate(x_best);
        min_prev = min;

        // Tworzymy pierwsz baz kierunkw - zaczynamy
        // od kierunkw wzdu osi ukadu wsprzdnych.
        D = IdentityMatrix(x_best.nrows());
        LOG(DEBUG, wxT("D: ") << endl << D);

        while (!finished && (step_count < STEP_LIMIT))
        {
            ++step_count;

            auto_ptr<RosenbrockDiscreteIteration> ap_result_iteration(
                    new RosenbrockDiscreteIteration(x_best, D));

            // Wykonujemy kroki prbne we wszystkich aktualnych kierunkach.
            SearchAllDirections(problem.Function(), x_best, min,
                    D, tau, lambda, *ap_result_iteration);

            ap_result_iteration->rLambda()   = lambda;
            ap_result_iteration->rEndPoint() = x_best;

            LOG(DEBUG, wxT("min: ") << min << wxT(". min_prev: ") << min_prev);
            // Sprawdzamy wstpny warunek stopu.
            tau_min = tau.minimum_absolute_value();
            finished = tau_min < mDelta;
            ap_result_iteration->rTauMin() = tau_min;

            // Dokonujemy obrotu bazy, jeli ma to sens
            // (aktualny etap przynis popraw).
            // TODO: lepsze radzenie sobie z niedokadnociami.
            if (min < min_prev || fabs(min - min_prev) < 0.0000001)
            {
                // Sprawdzamy gwny warunek stopu.
                finished = finished &&
                           mcpStopCondition->IsMet(
                                    x_prev_best, x_best, min_prev, min);

                // TODO: jak wyej
                if (!finished && min < min_prev)
                { // Obracamy baz i  przywracamy dugoci krokw.
                    RotateBase(D, lambda, *ap_result_iteration);
                    LOG(DEBUG, wxT("D:") << endl << D);

                    tau = mTau;
                    lambda = 0.0;

                    x_prev_best = x_best;
                    min_prev = min;
                 }

             } // top if

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));
            NotifyYield();
        } // while
    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x_best;
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString RosenbrockDiscrete::mName(_("Rosenbrock's Discrete"));

// Utwrz i zarejestruj prototyp.
RosenbrockDiscrete RosenbrockDiscrete::mRosenbrockDiscrete;
