/**  Metoda Rosenbrocka z krokiem dyskretnym.
 *
 * Metoda poszukiwa prostych.<br>
 * Podobnie jak algorytm Hooke'a i Jeevesa, oparta jest na wykonywaniu
 * w kadej iteracji krokw prbnych wzdu \f$n\f$ ortogonalnych kierunkw.
 * Kierunki te, inaczej ni w metodzie HJ, mog si zmienia - na kocu
 * kadej iteracji baza kierunkw \f$D^{(i)},\: i=1,\ldots,n\f$ jest obracana
 * w taki sposb, aby zawieraa kierunek najwikszej poprawy i pozostaych
 * \f$n - 1\f$ wzajemnie ortonormalnych kierunkw.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x^0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\tau_0^{(i)},\: i=1,\ldots,n\f$ - n-wymiarowy wektor pocztkowych
 * dugoci kroku,<br>
 * \f$\alpha\f$ - wspczynnik wyduania kroku (\f$\alpha > 1\f$),<br>
 * \f$\beta\f$ - wspczynnik skracania kroku (\f$0 < \beta < 1\f$),<br>
 * \f$\delta\f$ - wstpny warunek stopu - minimalna dugo kroku prbnego,<br>
 * \f$\varepsilon\f$ - dokadno waciwego warunku stopu,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$i\f$ - indeks aktualnego kierunku,<br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$D = D^{(i)},\: i=1,\ldots,n\f$ - aktualna baza kierunkw,<br>
 * \f$\lambda^{(i)},\: i=1,\ldots,n\f$ - wektor sum dugoci udanych
 * krokw prbnych w poszczeglnych kierunkach,<br>
 * \f$x^i\f$ - aktualny punkt prbny,<br>
 * \f$x^k_{min}\f$ - najlepszy punkt w k-tym etapie,<br>
 * \f$F^k_{min}\f$ - warto funkcji w punkcie \f$x^k_{min}\f$,<br>
 * \f$F^k_b\f$ - najlepsza warto funkcji przed rozpoczciem
 * przeszukiwania kierunkw z bazy,<br>
 * \f$A_i, \: i=1,\ldots,n\f$ - wektory sum przesuni
 * wykorzystywane przy obrocie bazy,<br>
 * \f$t_i, \: i=1,\ldots,n\f$ - kwadraty norm wektorw \f$A_i\f$
 * wykorzystywane przy normalizacji nowej bazy (\f$t_i = \|A_i\|^2\f$).
 *
 * <br><b>Procedura.</b><br>
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ D = I(n), \mbox{ gdzie } I(n) - \mbox{ macierz jednostkowa }
 * n \times n , \f]
 * \f[ \lambda = 0(n), \f]
 * \f[ x^1_{min} = x^0, \f]
 * \f[ F^0_{min} = f(x^0) , \f]
 * \f[ F^1_{min} = f(x^0) , \f]
 * \f[ F^1_b = f(x^0) , \f]
 * \f[ \tau = \tau_0 , \f]
 * \f[ k = 1 , \f]
 * \f[ i = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Badamy warto funkcji w nowym punkcie prbnym
 * (wykonujc krok w i-tym kierunku):
 * \f[ x^i = x^k_{min} + \tau^{(i)} D^{(i)} . \f]
 *
 * Jeli krok by udany (\f$f(x^i) < F^k_{min}\f$), to dodajemy
 * dugo kroku do sumy \f$\lambda^{(i)}\f$, wyduamy krok w biecym
 * kierunku i zapamitujemy aktualny punkt jako najlepszy:
 * \f[ \lambda^{(i)} = \lambda^{(i)} + \tau^{(i)}, \f]
 * \f[ \tau^{(i)} = \alpha \tau^{(i)}, \f]
 * \f[ x^k_{min} = x^i, \f]
 * \f[ F^k_{min} = f(x^i). \f]
 *
 * Jeli krok by nieudany (\f$f(x^i) \ge F^k_{min}\f$), to
 * skracamy krok w biecym kierunku i odwracamy jego zwrot:
 * \f[ \tau^{(i)} = -\beta \tau^{(i)}. \f]
 *
 * Podstawiamy \f$i = i + 1\f$ i sprawdzamy, czy \f$i \le n \f$.
 * Jeli tak, ponownie wykonujemy krok 1.
 * Jeli nie, podstawiamy \f$i = 1\f$ i przechodzimy do kroku 2.
 *
 * <br><b>Krok 2:</b><br>
 * Sprawdzamy, czy przy ostatnim przeszukiwaniu \f$n\f$
 * kierunkw nastpia poprawa.<br>
 * Jeli tak (\f$F^k_{min} < F^k_b\f$),
 * to podstawiamy:
 * \f[ F^k_b = F^k_{min} \f]
 * i przechodzimy do kroku 1.
 * W przeciwnym wypadku przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Sprawdzamy wstpny warunek stopu:
 * \f[ \min_{i \in \overline{1,\ldots,n}} |\tau^{(i)}| < \delta \f]
 * i zapamitujemy jego warto logiczn.
 *
 * Nastpnie sprawdzamy, czy w caej iteracji
 * nastpia poprawa - jeli tak (\f$F^k_{min} < F^{k-1}_{min}\f$),
 * to przechodzimy do kroku 4.
 *
 * W przeciwnym wypadku (\f$F^k_{min} = F^{k-1}_{min}\f$), jeli
 * wstpny warunek stopu jest speniony, to STOP.
 * Jeli nie jest, podstawiamy \f$k = k + 1\f$  i przechodzimy
 * do kroku 1.
 *
 * <b><br>Krok 4:</b><br>
 * Sprawdzamy gwny warunek stopu.
 * Jeli ten oraz wstpny warunek stopu z kroku 2 s spenione,
 * to STOP. W przeciwnym razie przechodzimy do kroku 5.
 *
 * <b><br>Krok 5:</b><br>
 * Dokonujemy obrotu bazy, wedug algorytmu Palmera:
 *
 * 1) Dla \f$ i = 1,\ldots,n \f$:<br>
 * Wyliczamy wektory sum przesuni:
 * \f[ A_i = \sum_{j=i}^n \lambda^{(j)} D^{(j)} \f]
 * oraz wektor \f$t\f$:
 * \f[ t_i = \|A_i\|^2 . \f]
 * 2) Tworzymy nowe kierunki ortonormalne \f$D^{(i)}\f$ :<br>
 * a) Dla \f$i = n,\ldots,2\f$ :
 * \f[ div = \sqrt{t_{i-1} t_i} . \f]
 * \f[ \mbox{Jesli } div \ne 0 :  D^{(i)} = \frac{\lambda^{(i-1)} A_i -
 * D^{(i-1)} t_i}{div} , \f]
 * b) Na koniec tworzymy kierunek najwikszej poprawy \f$D^{(1)}\f$.
 * \f[ div = \sqrt{t_1} , \f]
 * \f[ D^{(1)} = \frac{A_1}{div}. \f]
 *
 * W ten sposb otrzymujemy now baz kierunkw \f$D^{(i)},\: i=1,\ldots,n\f$.
 *
 * Nastpnie podstawiamy:
 * \f[ \tau = \tau_0 , \f]
 * \f[ \lambda = 0(n), \f]
 * \f[ x^{k+1}_{min} = x^k_{min}, \f]
 * \f[ F^{k+1}_{min} = F^k_{min}, \f]
 * \f[ F^{k+1}_b = F^k_{min}, \f]
 * \f[ k = k + 1 \f]
 * i przechodzimy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Oryginalna metoda Rosenbrocka wykorzystywaa przy ortonormalizacji
 * nowej bazy metod Grama-Schmidta. W tej implementacji uyto metody
 * Palmera, bardziej wydajnej i tworzcej poprawn baz rwnie w przypadku,
 * gdy w iteracji istniej kierunki, w ktrych nie wykonano adnego udanego
 * kroku.
 *
 * Metoda zaimplementowana na podstawie programu Franka Vanden Berghena
 * (http://iridia.ulb.ac.be/%7Efvandenb/download.php?id=33), ktry powsta
 * na podstawie publikacji:<br>
 * 1) Rosenbrock, H. H., <i>An Automatic Method for Finding the Greatest
 * or Least Value of a Function</i> (1960).
 * (http://iridia.ulb.ac.be/~fvandenb/optimization/paper_rosenbrock.zip)<br>
 * 2) Palmer, J. R., <i>An improved procedure for orthogonalising the search
 * vectors in Rosenbrock's and Swann's direct search optimisation methods</i>
 * (1969).
 * (http://iridia.ulb.ac.be/~fvandenb/optimization/paper_palmer.zip)
 *
 * @see RosenbrockOptimal
 */

#ifndef _RosenbrockDiscrete_h
#define _RosenbrockDiscrete_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../Method.h"
#include "../StandardStopCondition.h"
#include "RosenbrockDiscreteIteration.h"

// FORWARD REFERENCES
//


class RosenbrockDiscrete
    : public Method
{
public:
// LIFECYCLE

    friend class RosenbrockDiscretePanel;

    /** Destruktor.
     */
    ~RosenbrockDiscrete(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

    virtual void                    UpdateStartingConditions
                                                (const Result* result) const;

    virtual void                    ResetStartingConditions() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    RosenbrockDiscrete(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    RosenbrockDiscrete(const RosenbrockDiscrete& from);

    /**
     * Przeszukuje kierunki.
     *
     * Dla kadego kierunku \f$d^{(i)}, \: i=1,\ldots,n\f$
     * metoda wykonuje krok prbny o dugoci \f$\tau^{(i)}\f$:
     * \f[ x^i = x^k_{min} + \tau^{(i)} d^{(i)} , \f]
     * gdzie:<br>
     * \f$x^i\f$ - aktualny punkt prbny,<br>
     * \f$x^k_{min}\f$ - najlepszy punkt w biecym etapie.<br>
     * Jeli krok by udany (\f$f(x^i) < f(x^k_{min})\f$),
     * dugo \f$\tau^{(i)}\f$ jest dodawana do \f$\lambda^{(i)}\f$,
     * a nastpnie zwikszana:
     * \f[ \lambda^{(i)} = \lambda^{(i)} + \tau^{(i)} , \f]
     * \f[ \tau^{(i)} = \alpha \tau^{(i)} , \f]
     * gdzie:<br>
     * \f$\alpha > 1\f$ - wspczynnik wyduenia kroku.
     *
     * Punkt prbny zostaje rwnie zapamitany jako najlepszy:
     * \f[ x^k_{min} = x^i . \f]
     * Jeli krok nie powid si (\f$f(x^i) \ge f(x^k_{min})\f$),
     * \f$\tau^{(i)}\f$ jest zmniejszana, a zwrot kroku odwracany:
     * \f[ \tau^{(i)} = -\beta \tau^{(i)} , \f]
     * gdzie:<br>
     * \f$\beta \in (0,1) \f$ - wspczynnik skrcenia kroku.<br>
     *
     * Przebieg jest powtarzany do momentu, kiedy w adnym z kierunkw
     * nie wykonano udanego kroku.
     *
     * @param function Funkcja celu (\f$f\f$).
     * @param rxBest Referencja na najlepszy dotd punkt (\f$x^k_{min}\f$).
     * @param rMin Referencja na najmniejsz dotd znalezion
     * warto funkcji (\f$f(x^k_{min}) = F^k_{min}\f$).
     * @param D Baza kierunkw \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * @param rTau Referencja na wektor dugoci krokw w poszczeglnych
     * kierunkach (\f$\tau^{(i)}, i=1,\ldots,n\f$).
     * @param rLambda Referencja na wektor sum dugoci udanych krokw
     * w poszczeglnych kierunkach (\f$\lambda^{(i)}, \: i=1,\ldots,n\f$).
     * @param rIterationData Referencja na dane biecej iteracji dla wyniku.
     */
    void SearchAllDirections(const FunctionBase&            function,
                             ColumnVector&                  rxBest,
                             double&                        rMin,
                             const SquareMatrix&            D,
                             ColumnVector&                  rTau,
                             ColumnVector&                  rLambda,
                             RosenbrockDiscreteIteration&   rIterationData)
                                                                const;

    /** Dokonuje obrotu bazy kierunkw.
     *
     * Na podstawie sum dugoci udanych krokw \f$\lambda^{(i)}\f$,
     * zmienia baz wektorw kierunkw \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * Pierwszy wektor w nowej bazie jest kierunkiem najwikszej
     * poprawy znalezionym w danym etapie,  kolejny wektor - najlepszym
     * kierunkiem normalnym dla pierwszego, itd.
     *
     * Metoda wykorzystuje ortogonalizacj Palmera:<br>
     * na pocztku wyliczane s wektory sum przesuni
     * \f$A_i, i = 1,\ldots,n : \f$
     * \f[ A_i = \sum_{j=i}^n \lambda^{(j)} d^{(j)} , \f]
     * gdzie:<br>
     * \f$\lambda^{(j)}\f$ - suma dugoci udanych krokw w kierunku
     * \f$d^{(j)}\f$,
     *
     * oraz wektor \f$t\f$:
     * \f[ t^{(i)} = |A_i|^2 , \: i = 1,\ldots,n. \f]
     * Nastpnie wykonana zostaje procedura utworzenia nowych kierunkw
     * ortonormalnych \f$d^{(i)}\f$ :<br>
     * 1. Dla \f$i = n,\ldots,2\f$ :
     * \f[ div = \sqrt{t^{(i-1)} t^{(i)}} . \f]
     * \f[ \mbox{Jesli } div \ne 0 :  d^{(i)} = \frac{\lambda^{(i-1)} A_i -
     * d^{(i-1)} t^{(i)}}{div} , \f]
     * 2.
     * \f[ div = \sqrt{t^{(1)}} , \f]
     * \f[ d^{(1)} = \frac{A_1}{div}. \f]
     *
     * @param rD Referencja na zmienian baz kierunkw
     * \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * @param lambda Wektor sum dugoci udanych krokw w
     * poszczeglnych kierunkach (\f$\lambda^{(i)}, \: i=1,\ldots,n\f$).
     * @param rIterationData Referencja na dane biecej iteracji dla wyniku.
     */
    void                RotateBase (
            SquareMatrix&                   rD,
            const ColumnVector&             lambda,
            RosenbrockDiscreteIteration&    rIterationData) const;

    /** Pocztkowe (zadane przez uytkownika) dugoci kroku
     * prbnego, mog si rni dla poszczeglnych kierunkw.
     */
    ColumnVector            mTau;

    /** Wspczynnik wyduania kroku (\f$\alpha > 1\f$).
     */
    DoubleParameter         mAlpha;

    /** Wspczynnik skracania kroku  (\f$0 < \beta < 1\f$).
     */
    DoubleParameter         mBeta;

    /** Minimalna dugo kroku prbnego (\f$\delta\f$).
     */
    DoubleParameter         mDelta;

    /** Punkt startowy (\f$x^0\f$).
     */
    mutable ColumnVector    mStartingPoint;
    ColumnVector            mInitialStartingPoint;

    /** Warunek stopu.
     */
    CountedPtr<const StandardStopCondition> mcpStopCondition;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    static wxString      mName; ///< Nazwa metody.
    static RosenbrockDiscrete mRosenbrockDiscrete;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockDiscrete_h_
