#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "../util.h"
#include "RosenbrockOptimal.h"  // class implemented

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


RosenbrockOptimal::~RosenbrockOptimal()
{
}// ~RosenbrockOptimal


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
RosenbrockOptimal::Clone() const
{
    return auto_ptr<Method>(new RosenbrockOptimal(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
RosenbrockOptimal::Name() const
{
    return RosenbrockOptimal::mName;
}// Name

/* virtual */ wxString&
RosenbrockOptimal::rName()
{
    return RosenbrockOptimal::mName;
}// rName

/* virtual */ MethodIdType
RosenbrockOptimal::Id() const
{
    return RosenbrockOptimal::ClassId();
}// Id

/* static */ MethodIdType
RosenbrockOptimal::ClassId()
{
    const static MethodIdType RosenbrockOptimalId = Method::GenerateId();

    return RosenbrockOptimalId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
RosenbrockOptimal::CanHandle(const ProblemBase* pProblem) const
{
    return ((dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL)
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
RosenbrockOptimal::AllowedStopConditions() const
{
    StopConditions conditions(StopConditionBase::StandardStopConditions());

    return conditions;
}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

RosenbrockOptimal::RosenbrockOptimal()
    : MethodWithLineSearch()
{
}// RosenbrockOptimal

RosenbrockOptimal::RosenbrockOptimal(const RosenbrockOptimal& from)
    : MethodWithLineSearch(from)
{
}// RosenbrockOptimal

void
RosenbrockOptimal::SearchAllDirections (
        const FunctionBase&             function,
        ColumnVector&                   rxBest,
        const SquareMatrix&             D,
        ColumnVector&                   rLambda,
        RosenbrockOptimalIteration&     rIterationData) const
{
    double tau; // Optymalna dugo kroku w biecym kierunku.

    // Szukamy minimum dla aktualnych kierunkw,
    // wykonujc w kadym z nich krok o optymalnej dugoci.
    for (int i = 1; i <= rxBest.nrows(); ++i)
    {
        LOG(DEBUG, wxT("i: ") << i);

        // Wykonujemy optymalny krok.
        rxBest = StepInDirection(function, rxBest, D.column(i), &tau);
        // Zapamitujemy jego dugo.
        rLambda(i) = tau;

        rIterationData.AddTrialPoint(rxBest, tau);
        LOG(DEBUG, wxT("x:") << endl << rxBest
                << wxT("Tau: ") << tau);
    } // for

} // SearchAllDirections


void
RosenbrockOptimal::RotateBase(
        SquareMatrix&                   rD,
        const ColumnVector&             lambda,
        RosenbrockOptimalIteration&     rIterationData) const
{
    int n = lambda.nrows(); // Liczba zmiennych
    // Wektory (kolumnowe) sum przesuni, od "najlepszego"
    // (uwzgldniajcego zmiany we wszystkich kierunkach) do
    // "najgorszego".
    // A(1) = lambda(1)*D(1) + lambda(2)*D(2) + ... + lambda(n)*D(n)
    // A(2) =                  lambda(2)*D(2) + ... + lambda(n)*D(n)
    // ...
    // A(n) =                                         lambda(n)*D(n)
    SquareMatrix A(n);
    // Wektor kwadratw dugoci A.
    // t(i) = |A(i)|^2
    ColumnVector t(n);
    // Mianownik przy normalizacji wektorw ortogonalnych.
    double div;

    LOG(DEBUG, wxT("lambda: ") << endl << lambda);

    // Wypeniamy wektory A (od ostatniego do pierwszego).
    A.column(n) = lambda(n) * rD.column(n);

    for (int i = n - 1; i >= 1; --i)
    {
        A.column(i) = A.column(i + 1) + (lambda(i) * rD.column(i));
    }
    rIterationData.rA() = A;
    LOG(DEBUG, wxT("A:") << endl << A);

    // Ortogonalizujemy wektory A (metod Palmera).
    // Powstae wektory ortogonalne po normalizacji
    // staj si nowymi wektorami kierunkw.
    t(n) = lambda(n) * lambda(n);
    for (int i = n - 1; i >= 1; --i)
    {
        t(i) = t(i + 1) + (lambda(i) * lambda(i));
    }

    for (int i = n; i >= 2; --i)
    {
        div = sqrt(t(i - 1) * t(i));
        if (div != 0)
        {
            rD.column(i) = (lambda(i - 1) * A.column(i) -
                            rD.column(i - 1) * t(i)        ) / div;
        }

    } // for

    div = sqrt(t(1));
    rD.column(1) = A.column(1) / div;

    rIterationData.rNewD() = rD;
} // RotateBase

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
RosenbrockOptimal::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    // Funkcja celu.
    const FunctionBase& function = problem.Function();
    // Najlepszy punkt w obecnym etapie.
    ColumnVector x_best(mStartingPoint);
    // Najlepszy punkt w poprzednim etapie.
    ColumnVector x_prev_best(x_best);

    // Wektory (kolumnowe) kierunkw.
    SquareMatrix D(problem.VariableCount());
    // Wektor sum dugoci wykonanych krokw.
    ColumnVector lambda(problem.VariableCount());

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);

    try
    {
        lambda = 0.0;

        // Tworzymy pierwsz baz kierunkw - zaczynamy
        // od kierunkw wzdu osi ukadu wsprzdnych.
        D = IdentityMatrix(x_best.nrows());
        LOG(DEBUG, wxT("D: ") << endl << D);

        while (!finished && (step_count < STEP_LIMIT))
        {
            ++step_count;

            auto_ptr<RosenbrockOptimalIteration> ap_result_iteration(
                    new RosenbrockOptimalIteration(x_best, D));

            // Wykonujemy kroki we wszystkich aktualnych kierunkach.
            SearchAllDirections(problem.Function(), x_best, D, lambda,
                    *ap_result_iteration);

            ap_result_iteration->rLambda()   = lambda;
            ap_result_iteration->rEndPoint() = x_best;

            // Sprawdzamy warunek stopu.
            finished = mcpStopCondition->IsMet(x_prev_best, x_best, function);

            if (!finished)
            { // Obracamy baz.
                RotateBase(D, lambda, *ap_result_iteration);
                LOG(DEBUG, wxT("D:") << endl << D);

                lambda = 0.0;

                x_prev_best = x_best;
            }

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_result_iteration));
            NotifyYield();
        } // while
    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, std::auto_ptr<Result>(ap_result));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x_best;
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString RosenbrockOptimal::mName(_("Rosenbrock's Optimal"));

// Utwrz i zarejestruj prototyp.
RosenbrockOptimal RosenbrockOptimal::mRosenbrockOptimal;
