/**  Metoda Rosenbrocka z krokiem optymalnym.
 *
 * Metoda poszukiwa prostych.<br>
 * Podobnie jak metoda Rosenbrocka z krokiem dyskretnym, polega na
 * przeszukiwaniu w kadej iteracji \f$n\f$ ortogonalnych kierunkw
 * i obracaniu bazy w taki sposb, aby nowe kierunki byy kierunkami
 * najwikszej dotychczasowej poprawy. W odrnieniu od pierwotnego
 * wariantu metody Rosenbrocka, w metodzie z krokiem optymalnym wzdu
 * kadego z kierunkw wykonywany jest tylko jeden krok - krok o optymalnej
 * dugoci wyznaczonej przez metod optymalizacji w kierunku. Dziki temu
 * kady krok prbny jest krokiem udanym (koczcym si w punkcie
 * o mniejszej ni poprzednio wartoci funkcji celu). Upraszcza to znacznie
 * procedur.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x^0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\varepsilon\f$ - dokadno waciwego warunku stopu,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 *
 * \f$\varepsilon\f$ - minimalna dugo kroku w caym etapie,<br>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami.<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$i\f$ - indeks aktualnego kierunku,<br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$D = d^{(i)},\: i=1,\ldots,n\f$ - aktualna baza kierunkw,<br>
 * \f$\lambda^{(i)},\: i=1,\ldots,n\f$ - wektor sum dugoci udanych
 * krokw prbnych w poszczeglnych kierunkach,<br>
 * \f$x^i\f$ - aktualny punkt prbny,<br>
 * \f$x^k_{min}\f$ - najlepszy punkt w k-tym etapie,<br>
 * \f$\tau^{(i)}\f$ - optymalna dugo kroku w i-tym kierunku,<br>
 * \f$A_i, \: i=1,\ldots,n\f$ - wektory sum przesuni
 * wykorzystywane przy obrocie bazy,<br>
 * \f$t_i, \: i=1,\ldots,n\f$ - kwadraty norm wektorw \f$A_i\f$
 * wykorzystywane przy normalizacji nowej bazy (\f$t_i = \|A_i\|^2\f$).
 *
 * <br><b>Procedura.</b><br>
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ D = I(n), \mbox{ gdzie } I(n) - \mbox{ macierz jednostkowa }
 * n \times n , \f]
 * \f[ \lambda = 0(n), \f]
 * \f[ x^1_{min} = x^0, \f]
 * \f[ k = 1 , \f]
 * \f[ i = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Przy pomocy metody optymalizacji w kierunku wyznaczamy optymaln dugo
 * kroku \f$\tau^{(i)}\f$ i wykonujemy krok:
 * \f[ \tau^{(i)} = \arg \min_{\tau^{(i)}} f(x^k_{min}
 * + \tau^{(i)} D^{(i)}) . \f]
 * \f[ x^i = x^k_{min} + \tau^{(i)} D^{(i)} . \f]
 * Zapamitujemy dugo kroku i aktualny punkt:
 * \f[ \lambda^{(i)} = \tau^{(i)} , \f]
 * \f[ x^k_{min} = x^i . \f]
 *
 * Podstawiamy \f$i = i + 1\f$ i sprawdzamy, czy \f$i \le n \f$.
 * Jeli tak, ponownie wykonujemy krok 1.
 * Jeli nie, podstawiamy \f$i = 1\f$ i przechodzimy do kroku 2.
 *
 * <b><br>Krok 2:</b><br>
 * Sprawdzamy warunek stopu.
 * Jeli jest on speniony, to STOP.
 * Wynikiem jest \f$x^k_{min}\f$.<br>
 * W przeciwnym wypadku przechodzimy do kroku 3.
 *
 * <b><br>Krok 3:</b><br>
 * Dokonujemy obrotu bazy, wedug algorytmu Palmera:
 *
 * 1) Dla \f$ i = 1,\ldots,n \f$:<br>
 * Wyliczamy wektory sum przesuni:
 * \f[ A_i = \sum_{j=i}^n \lambda^{(j)} D^{(j)} \f]
 * oraz wektor \f$t\f$:
 * \f[ t_i = \|A_i\|^2 . \f]
 * 2) Tworzymy nowe kierunki ortonormalne \f$D^{(i)}\f$ :<br>
 * a) Dla \f$i = n,\ldots,2\f$ :
 * \f[ div = \sqrt{t_{i-1} t_i} . \f]
 * \f[ \mbox{Jesli } div \ne 0 :  D^{(i)} = \frac{\lambda^{(i-1)} A_i -
 * D^{(i-1)} t_i}{div} , \f]
 * b) Na koniec tworzymy kierunek najwikszej poprawy \f$D^{(1)}\f$.
 * \f[ div = \sqrt{t_1} , \f]
 * \f[ D^{(1)} = \frac{A_1}{div}. \f]
 *
 * W ten sposb otrzymujemy now baz kierunkw \f$D^{(i)},\: i=1,\ldots,n\f$.
 *
 * Nastpnie podstawiamy:
 * \f[ \lambda = 0(n), \f]
 * \f[ k = k + 1 \f]
 * i przechodzimy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Oryginalna metoda Rosenbrocka wykorzystywaa przy ortonormalizacji
 * nowej bazy metod Grama-Schmidta. W tej implementacji uyto metody
 * Palmera, bardziej wydajnej i tworzcej poprawn baz rwnie w przypadku,
 * gdy w iteracji istniej kierunki, w ktrych nie wykonano adnego udanego
 * kroku.
 *
 * @see MethodWithLineSearch
 * @see RosenbrockDiscrete
 */

#ifndef _RosenbrockOptimal_h
#define _RosenbrockOptimal_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../MethodWithLineSearch.h"
#include "../ui/MethodWithLineSearchPanel.h"
#include "RosenbrockOptimalIteration.h"

// FORWARD REFERENCES
//


class RosenbrockOptimal
    : public MethodWithLineSearch
{
public:
// LIFECYCLE

    friend class MethodWithLineSearchPanel<RosenbrockOptimal>;

    /** Destruktor.
     */
    ~RosenbrockOptimal(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    RosenbrockOptimal(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    RosenbrockOptimal(const RosenbrockOptimal& from);

    /**
     * Przeszukuje kierunki.
     *
     * Dla kadego kierunku \f$d^{(i)}, \: i=1,\ldots,n\f$
     * metoda wykonuje jeden krok prbny o optymalnej dugoci
     * \f$\tau^{(i)}\f$:
     * \f[ \tau^{(i)} = \arg \min_{\tau^{(i)}} f(x^k_{min} +
     * \tau^{(i)} d^{(i)}) , \f]
     * \f[ x^i = x^k_{min} + \tau^{(i)} d^{(i)} . \f]
     * Dugo kroku jest zapamitywana jako \f$\lambda^{(i)}\f$ :
     * \f[ \lambda^{(i)} = \tau^{(i)} . \f]
     *
     * @param function Funkcja celu (\f$f\f$).
     * @param rxBest Referencja na najlepszy dotd punkt (\f$x^k_{min}\f$).
     * @param D Baza kierunkw \f$d^{(i)}, \:  i=1,\ldots,n\f$.
     * @param rLambda Referencja na wektor sum dugoci udanych krokw
     * w poszczeglnych kierunkach (\f$\lambda^{(i)}, \: i=1,\ldots,n\f$).
     * @param rIterationData Referencja na dane biecej iteracji dla wyniku.
     */
    void SearchAllDirections(const FunctionBase&                function,
                             ColumnVector&                      rxBest,
                             const SquareMatrix&                D,
                             ColumnVector&                      rLambda,
                             RosenbrockOptimalIteration&        rIterationData)
                                                                    const;


    /** Dokonuje obrotu bazy kierunkw.
     *
     * Na podstawie sum dugoci udanych krokw \f$\lambda^{(i)}\f$,
     * zmienia baz wektorw kierunkw \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * Pierwszy wektor w nowej bazie jest kierunkiem najwikszej
     * poprawy znalezionym w danym etapie,  kolejny wektor - najlepszym
     * kierunkiem normalnym dla pierwszego, itd.
     *
     * Metoda wykorzystuje ortogonalizacj Palmera:<br>
     * na pocztku wyliczane s wektory sum przesuni
     * \f$A_i, i = 1,\ldots,n : \f$
     * \f[ A_i = \sum_{j=i}^n \lambda^{(j)} d^{(j)} , \f]
     * gdzie:<br>
     * \f$\lambda^{(j)}\f$ - suma dugoci udanych krokw w kierunku
     * \f$d^{(j)}\f$,
     *
     * oraz wektor \f$t\f$:
     * \f[ t^{(i)} = |A_i|^2 , \: i = 1,\ldots,n. \f]
     * Nastpnie wykonana zostaje procedura utworzenia nowych kierunkw
     * ortonormalnych \f$d^{(i)}\f$ :<br>
     * 1. Dla \f$i = n,\ldots,2\f$ :
     * \f[ div = \sqrt{t^{(i-1)} t^{(i)}} . \f]
     * \f[ \mbox{Jesli } div \ne 0 :  d^{(i)} = \frac{\lambda^{(i-1)} A_i -
     * d^{(i-1)} t^{(i)}}{div} , \f]
     * 2.
     * \f[ div = \sqrt{t^{(1)}} , \f]
     * \f[ d^{(1)} = \frac{A_1}{div}. \f]
     *
     * @param rD Referencja na zmienian baz kierunkw
     * \f$d^{(i)}, \: i=1,\ldots,n\f$.
     * @param lambda Wektor sum dugoci udanych krokw w
     * poszczeglnych kierunkach (\f$\lambda^{(i)}, \: i=1,\ldots,n\f$).
     * @param rIterationData Referencja na dane biecej iteracji dla wyniku.
     */
    void                RotateBase (
            SquareMatrix&                   rD,
            const ColumnVector&             lambda,
            RosenbrockOptimalIteration&     rIterationData) const;

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);
    static wxString      mName; ///< Nazwa metody.
    static RosenbrockOptimal mRosenbrockOptimal;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _RosenbrockOptimal_h_
