#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "../UnconstrainedProblem.h"
#include "SecantDivision.h"  // class implemented
#include "SecantDivisionIteration.h"

LOGGER(DEBUG);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


SecantDivision::~SecantDivision()
{
}// ~SecantDivision


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
SecantDivision::Clone() const
{
    return auto_ptr<Method>(new SecantDivision(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
SecantDivision::Name() const
{
    return SecantDivision::mName;
}// Name

/* virtual */ wxString&
SecantDivision::rName()
{
    return SecantDivision::mName;
}// rName

/* virtual */ MethodIdType
SecantDivision::Id() const
{
    return SecantDivision::ClassId();
}// Id

/* static */ MethodIdType
SecantDivision::ClassId()
{
    const static MethodIdType SecantDivisionId = Method::GenerateId();

    return SecantDivisionId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
SecantDivision::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL &&
            pProblem->Function().IsDifferentiable());
}

/* virtual */ bool
SecantDivision::CanAssist(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem *>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable());
}// CanAssist

/////////////////////////////// PROTECTED  ///////////////////////////////////

SecantDivision::SecantDivision()
    : SectionLineSearch()
{
}// SecantDivision

SecantDivision::SecantDivision(const SecantDivision& from)
    : SectionLineSearch(from)
{
}// SecantDivision

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
SecantDivision::Solve(const UnivariateFunction&    function,
                      double                       a,
                      double                       b,
                      bool                         generateResult)
        const throw(OptimizationError)
{

    auto_ptr<Result>                    ap_result(new Result(1));
    auto_ptr<SecantDivisionIteration>   ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        if (a > b) swap(a, b);

        ColumnVector    left(1);   // Lewy kraniec przedziau
        ColumnVector    x(1);      // Biece przyblienie punktu stacjonarnego
        ColumnVector    right(1);  // Prawy kraniec przedziau

        double          derivative_in_left;     // warto pochodnej w left
        double          derivative_in_x;        // warto pochodnej w x
        double          derivative_in_right;    // warto pochodnej w right

        left                = a;
        right               = b;

        derivative_in_left  = function.Gradient(left)(1);
        derivative_in_right = function.Gradient(right)(1);

/*
        LOG(DEBUG,wxT("Krok nr ") << step_count);

        LOG(DEBUG,wxString::Format(wxT("Left   : %6g, derivative: %6g"),
                                    left(1),function.Gradient(left)(1)));
        LOG(DEBUG,wxString::Format(wxT("Right  : %6g, derivative: %6g"),
                                    right(1),function.Gradient(right)(1)));
*/

        x = right - derivative_in_right * (right - left) /
                        (derivative_in_right - derivative_in_left);
        derivative_in_x = function.Gradient(x)(1);

        ap_iteration.reset(new SecantDivisionIteration(left(1),
                                                       x(1),
                                                       right(1)));

        // To tu sprawdzamy czy mamy w odcinku bbel.
        if ((function.Gradient(left)(1) > 0 &&
                    function.Gradient(right)(1) < 0))
        {
            throw Error(_("The initial section contains a maximum."));
        }

        if (x(1) < a || x(1) > b)
        {
            throw Error(_("Found minimum outside of initial section - \
optimization aborted."));
        }

        if (fabs(derivative_in_x) <= mEpsilon)
            ap_iteration->rPartChosen() = SecantDivisionIteration::NONE;


        LOG(DEBUG,wxString::Format(
                        wxT("a = %10.5g, derivative: %10.5g"),
                        left(1), function.Gradient(left)(1)));
        LOG(DEBUG,wxString::Format(
                        wxT("x = %10.5g, derivative: %10.5g"),
                        x(1) ,derivative_in_x));
        LOG(DEBUG,wxString::Format(
                        wxT("b = %10.5g, derivative: %10.5g"),
                        right(1), function.Gradient(right)(1)));


        while (right(1) - left(1) >= mEpsilon &&
               fabs(derivative_in_x) >= mEpsilon &&
               step_count < STEP_LIMIT)
        {

            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count);

            if (derivative_in_x < 0)
            {
                left                = x;
                derivative_in_left  = derivative_in_x;

                ap_iteration->rPartChosen() = SecantDivisionIteration::RIGHT;
            }
            else // derivative_in_middle > 0 (==0 w warunku ptli)
            {
                right               = x;
                derivative_in_right = derivative_in_x;

                ap_iteration->rPartChosen() = SecantDivisionIteration::LEFT;
            }


            x = right - derivative_in_right * (right - left) /
                            (derivative_in_right - derivative_in_left);
            derivative_in_x = function.Gradient(x)(1);


            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));
            ap_iteration.reset(new SecantDivisionIteration(left(1),
                                                           x(1),
                                                           right(1)));

            if (x(1) < a || x(1) > b)
            {
                throw Error(_("Found minimum outside of initial section - \
    optimization aborted."));
            }

            if (fabs(derivative_in_x) <= mEpsilon)
                ap_iteration->rPartChosen() = SecantDivisionIteration::NONE;

            LOG(DEBUG,wxString::Format(
                            wxT("a = %10.5g, derivative: %10.5g"),
                            left(1),function.Gradient(left)(1)));
            LOG(DEBUG,wxString::Format(
                            wxT("x = %10.5g, derivative: %10.5g"),
                            x(1),derivative_in_x));
            LOG(DEBUG,wxString::Format(
                            wxT("b = %10.5g, derivative: %10.5g"),
                            right(1),function.Gradient(right)(1)));


            LOG(DEBUG,wxT("-----------------------------------"));

            NotifyYield();
        }

        ap_result->AddIteration
                       (auto_ptr<const IterationData>(ap_iteration));

        ap_result->rFinalResult() = x;

    } // try
    catch (const BaseException& error)
    { // Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const EvaluationError& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }
    catch (const Error& error)
    {
        if (generateResult)
        {
            ap_iteration->rPartChosen() = SecantDivisionIteration::NONE;
            ap_result->AddIteration(auto_ptr<const IterationData>
                (ap_iteration));
        }

        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve

wxString SecantDivision::mName(_("Secant Division"));

// Utwrz i zarejestruj prototyp.
SecantDivision SecantDivision::mSecantDivision;
