/** Metoda siecznych.
 *
 * Metoda siecznych jest podobna do metod Bolzano i Newtona-Raphsona. Tak samo
 * jak w metodzie Bolzano wykluczamy cz przedziau i uywamy do tego cely
 * pochodnej, ale tym razem interesuje nas jej warto. Kade kolejne
 * przyblienie punktu stacjonarnego wyliczamy ze wzoru:
 * \f[ x_{k+1} = x_k - f'(x_k) \frac{x_k-x_{k-1}}{f'(x_k)-f'(x_{k-1})} \f]
 * Jak mona zauway drugi czynnik iloczynu jest odwrotnoci numerycznego
 * przyblienia drugiej pochodnej funkcji celu, a wic wzr w zasadzie
 * przyblia metod Newtona-Rowsona.
 *
 *
 * <b>Algorytm metody</b>:<br>
 * <br>
 * <b>Oznaczenia</b>:<br>
 * \f$ a \f$ - lewy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ b \f$ - prawy kraniec przedziau poszukiwa w danej iteracji<br>
 * \f$ x_m \f$ - biece przyblienie punktu stacjonarnego<br>
 * <br>
 * <b>Dane potrzebne do oblicze</b>:<br>
 * \f$ f(x) \f$ - minimalizowana funkcja jednej zmiennej<br>
 * \f$ a^0 \f$ - lewy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ b^0 \f$ - prawy kraniec pocztkowego przedziau poszukiwa<br>
 * \f$ \varepsilon \f$ - wymagana dokadno rozwizania<br>
 * <br>
 *
 * <b>Krok 1</b>:<br>
 * Obliczy \f$ f'(a^0) \f$ i \f$ f'(b^0) \f$.<br>
 * <br>
 * <b>Krok 2</b>:<br>
 * Jeli \f$ f'(a^0)>0 \f$, a \f$ f'(b^0)<0 \f$, zakoczy dziaanie algorytmu
 * stwierdzajc, e punkt stacjonarny nie znajduje si w przedziale
 * pocztkowym.<br>
 * <br>
 * <b>Krok 3</b>:<br>
 * Obliczy:
 * \f[ x=b-f'(b) \frac{b-a}{f'(b)-f'(a)} \f]
 * Obliczy pochodn w punkcie \f$ x \f$<br>
 * <br>
 * <b>Krok 4a</b>:<br>
 * Jeli \f$ f'(x)<0 \f$, to podstawi \f$ a=x \f$ i przej do kroku 5.<br>
 * W przeciwnym wypadku przej do kroku 4b.<br>
 * <br>
 * <b>Krok 4b</b>:<br>
 * Jeli \f$ f'(x)>0 \f$, to podstawi \f$ b=x \f$.<br>
 * Przej do kroku 5.<br>
 * <br>
 * <b>Krok 5</b>:<br>
 * Jeli \f$ |f'(x)|>\varepsilon \f$, przej do kroku 3.<br>
 * W przeciwnym wypadku zakoczy algorytm. Wynikiem jest ostatni punkt
 * \f$ x \f$.
 *
 *
 * Algorytm zaimplementowano na podstawie:<br>
 * <i>Ostanin. A: Metody i algorytmy optymalizacji, Wydawnictwo Politechniki
 * Biaostockiej, Biaystok, 2003</i>
 *
 * @see SectionLineSearch
 **/

#ifndef _SecantDivision_h
#define _SecantDivision_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../DoubleParameter.h"
#include "../ProblemBase.h"
#include "../SectionLineSearch.h"
#include "../UnivariateFunction.h"
#include "../ui/SectionLineSearchPanel.h"

// FORWARD REFERENCES
//


class SecantDivision
    : public SectionLineSearch
{
public:
    friend class SectionLineSearchPanel<SecantDivision>;

// LIFECYCLE


    /** Destruktor.
     */
    ~SecantDivision(void);


// OPERATORS

// OPERATIONS


    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>   Clone() const;

// ACCESS
    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;
    virtual bool                    CanAssist(const ProblemBase* pProblem) const;


protected:

    /** Konstruktor domylny.
     */
    SecantDivision(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SecantDivision(const SecantDivision& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result>   Solve(
                                      const UnivariateFunction& function,
                                      double                    a,
                                      double                    b,
                                      bool                      generateResult)
                          const throw(OptimizationError);

    /** Nazwa metody
     */
    static wxString       mName;
    /** Egzemplarz metody
     */
    static SecantDivision       mSecantDivision;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SecantDivision_h_
