#include <precomp.h>

#include "../Configuration.h"
#include "../Logger.h"
#include "../text-util.h"
#include "SignOfDerivative.h"  // class implemented
#include "SignOfDerivativeIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


SignOfDerivative::~SignOfDerivative()
{
}// ~SignOfDerivative


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================


/* virtual */ std::auto_ptr<Method>
SignOfDerivative::Clone() const
{
    return auto_ptr<Method>(new SignOfDerivative(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ const wxString&
SignOfDerivative::Name() const
{
    return SignOfDerivative::mName;
}// Name

/* virtual */ wxString&
SignOfDerivative::rName()
{
    return SignOfDerivative::mName;
}// rName

/* virtual */ MethodIdType
SignOfDerivative::Id() const
{
    return SignOfDerivative::ClassId();
}// Id

/* static */ MethodIdType
SignOfDerivative::ClassId()
{
    const static MethodIdType SignOfDerivativeId = Method::GenerateId();

    return SignOfDerivativeId;
}// ClassId

/* virtual */ wxString
SignOfDerivative::ToString(bool standalone) const
{
    wxString description;
    wxArrayString parameter_descriptions;

    parameter_descriptions.Add(AlphaDescriptionDerivative(mAlpha));
    parameter_descriptions.Add(GammaDescriptionDerivative(mGamma));
    AddCommonParameterDescriptions(parameter_descriptions, standalone);

    description << MethodName(Name());
    description << AlignedParameterDescriptions(parameter_descriptions);

    return description;
}// ToString
//============================= INQUIRY    ===================================

/* virtual */ bool
SignOfDerivative::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnivariateFunction*>(
                &(pProblem->Function())) != NULL &&
            pProblem->Function().IsDifferentiable());
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

SignOfDerivative::SignOfDerivative()
    : SinglePointLineSearch(),
      mAlpha(1.0, 0, true),
      mGamma(0.5, 0, true, 1, true)
{
}// SignOfDerivative

SignOfDerivative::SignOfDerivative(const SignOfDerivative& from)
    : SinglePointLineSearch(from),
      mAlpha(from.mAlpha),
      mGamma(from.mGamma)
{
}// SignOfDerivative

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ std::auto_ptr<Result>
SignOfDerivative::Solve(const UnivariateFunction&   function,
                        const ColumnVector&         startingPoint,
                        bool                        generateResult)
        const throw(OptimizationError)
{
    auto_ptr<Result>                    ap_result(new Result(1));
    auto_ptr<SignOfDerivativeIteration> ap_iteration;

    const unsigned  STEP_LIMIT = Configuration::GetInstance().StepLimit();
    unsigned        step_count = 0;

    try
    {
        ColumnVector    x(1);                   // Punkt w danej iteracji
        ColumnVector    x_candidate(1);         // Punkt z poprzedniej iteracji
        double          value_in_x;             // Warto funkcji w punkcie
        double          value_in_candidate;
        double          derivative_in_x;        // Pierwsza pochodna w punkcie x
        double          derivative_in_candidate;
        //double          step_length = DBL_MAX;  // Dugo wykonanaego kroku
        double          alpha = mAlpha;

        x               = startingPoint;
        value_in_x      = function.Evaluate(x);
        derivative_in_x = function.Gradient(x)(1);

        do
        {
            ++step_count;

            LOG(DEBUG,wxT("Krok nr ") << step_count << endl);

            LOG(DEBUG,wxString::Format(wxT("point           : %10.6g"),
                                            x(1)));
            LOG(DEBUG,wxString::Format(wxT("value           : %10.6g"),
                                            value_in_x));
            LOG(DEBUG,wxString::Format(wxT("derivative      : %10.6g"),
                                            derivative_in_x));


            ap_iteration.reset
                        (new SignOfDerivativeIteration(x(1), alpha, mGamma));

            x_candidate             = x - alpha * Sgn(derivative_in_x);
            value_in_candidate      = function.Evaluate(x_candidate);
            derivative_in_candidate = function.Gradient(x_candidate)(1);

            ap_iteration->rTrialPoints().push_back(x_candidate(1));


            LOG(DEBUG,wxString::Format(wxT("candidate       : %10.6g"),
                                            x_candidate(1)));
            LOG(DEBUG,wxString::Format(wxT("value           : %10.6g"),
                                            value_in_candidate));

            /* W poniszej ptli radzimy skracamy krok w przyapdku gdy
             * przeskakujemy minimum z powodu jego nadmiernej dugoci.
             * Warunki stopu takie jak w duej ptli potrzebne s by
             * zmniejszanie wspczynnika alfa nie trwao w nieskoczono w
             * przypadku gdy algorytm wykona krok wikszy od epsilon i
             * wyldowa na tyle blisko minimum, ze poprawa nie jest moliwa w
             * warunkach ograniczonej dokadnoci.
             * TODO: Pamita, e tu tez jest warunek stopu.
             */
            while (fabs(derivative_in_candidate) >= mEpsilon &&
                   //step_length >= mEpsilon &&
                   value_in_candidate >= value_in_x)
            {

                alpha   *= mGamma;

                LOG(DEBUG,wxT("Reduction of alpha\nnew alpha      : ")
                                                                    << alpha);

                x_candidate             = x - alpha * Sgn(derivative_in_x);
                value_in_candidate      = function.Evaluate(x_candidate);
                derivative_in_candidate = function.Gradient(x_candidate)(1);

                ap_iteration->rTrialPoints().push_back(x_candidate(1));

                //step_length             = fabs((x_candidate - x).as_scalar());


                LOG(DEBUG,wxString::Format(wxT("candidate       : %10.6g"),
                                                x_candidate(1)));
                LOG(DEBUG,wxString::Format(wxT("value           : %10.6g"),
                                                value_in_candidate));

                NotifyYield();
            }


            //step_length = fabs((x_candidate - x).as_scalar());

            x               = x_candidate;
            value_in_x      = value_in_candidate;
            derivative_in_x = derivative_in_candidate;

            ap_result->AddIteration
                            (auto_ptr<const IterationData>(ap_iteration));

            //LOG(DEBUG,wxString::Format(wxT("step_length     : %10.6g"),
            //                            step_length));
            LOG(DEBUG,wxT("-----------------------------------\n"));

            NotifyYield();
        }
        while (fabs(derivative_in_x) >= mEpsilon &&
               //step_length > mEpsilon &&
               step_count < STEP_LIMIT);

        ap_result->rFinalResult() = x;

    }// try
    catch (const BaseException& error)
    {// Wyjtek newmata.
        RethrowNewmatError(error, ap_result);
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(), ap_result);
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"), ap_result);
    }
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
} // Solve

wxString SignOfDerivative::mName(_("Sign of derivative"));

// Utwrz i zarejestruj prototyp.
SignOfDerivative SignOfDerivative::mSignOfDerivative;
