#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../text-util.h"

#include "Simplex.h"  // class implemented
#include "SimplexIteration.h"

LOGGER(WARNING);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

Simplex::~Simplex()
{
}// ~Simplex


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
Simplex::Clone() const
{
    return std::auto_ptr<Method>(new Simplex(*this));
}// Clone

//============================= ACCESS      ===================================

/* virtual */ const wxString&
Simplex::Name() const
{
    return Simplex::mName;
}// Name

/* virtual */ wxString&
Simplex::rName()
{
    return Simplex::mName;
}// rName

/* virtual */ MethodIdType
Simplex::Id() const
{
    return Simplex::ClassId();
}// Id

/* static */ MethodIdType
Simplex::ClassId()
{
    const static MethodIdType SimplexId = Method::GenerateId();

    return SimplexId;
}// ClassId

/* virtual */ wxString
Simplex::ToString(bool standalone) const
{
    wxString description;

    description << MethodName(Name());

    return description;
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

Simplex::Simplex()
    : LinearMethod()
{
}// Simplex

Simplex::Simplex(const Simplex& from)
    :LinearMethod(from)
{
}// Simplex

/////////////////////////////// PRIVATE    ///////////////////////////////////

std::auto_ptr<Result>
Simplex::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    // Indeksy zmiennych bazowych.
    vector<int>     iB;
    // Ideksy zmiennych niebazowych.
    vector<int>     iN;
    // Oglny licznik iteracji;
    int step_count = 0;

    const LinearProblem& linearProblem =
        dynamic_cast<const LinearProblem&>(problem);

    LOG(INFO, wxT("-----------Optimize-----------"));

    try
    {
        // Tworzymy standardow macierzow posta problemu.
        CountedPtr<const AugmentedForm> cpMainProblemData(
                new AugmentedForm(linearProblem));

        // Znajdujemy pierwsz baz.
        if (TrivialFirstBase(cpMainProblemData, iB, iN) == false)
        { // Jeli baza nie jest trywialna do znalezienia
          // (macierz A nie jest w postaci kanonicznej),
          // naley znale pierwsze indeksy bazowe
          // poprzez rozwizanie problemu rozszerzonego.
            SolveExtendedProblem(cpMainProblemData, iB, iN, step_count,
                    ap_result.get());
        }

        // Majc poprawne indeksy pierwszej bazy,
        // przeprowadzamy optymalizacj.
        Solve(cpMainProblemData, CountedPtr<const AugmentedForm>(),
                iB, iN, step_count, ap_result.get());
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }

    return ap_result;
}// Optimize

bool
Simplex::TrivialFirstBase(CountedPtr<const AugmentedForm>   cpMainProblemData,
                          vector<int>&                      riB,
                          vector<int>&                      riN) const
{
    int i;
    const AugmentedForm& problem_data = *cpMainProblemData;

    LOG(INFO, wxT("-----------TrivialFirstBase-----------"));
    LOG(INFO, wxT("-----------Basic problem-----------"));
    LOG(INFO, wxT("A: ") << endl << problem_data.A());

    // Sprawdzamy, czy macierz A ma posta kanoniczn
    // (czy m ostatnich kolumn tworzy macierz jednostkow).
    int row_count    = problem_data.A().nrows();
    int column_count = problem_data.A().ncols();
    int first_basic_column  = column_count - row_count + 1;

    if (first_basic_column > 0 &&
        problem_data.A().columns(first_basic_column, column_count)
            == IdentityMatrix(row_count))
    {
        // Tworzymy pierwsze tablice indeksw bazowych i niebazowych.
        for (i = 1; i < first_basic_column; ++i)
        {
            riN.push_back(i);
        }

        for (     ; i <= column_count; ++i)
        {
            riB.push_back(i);
        }

        return true;
    }
    else
    {
        return false;
    }
} // TrivialFirstBase


void
Simplex::SolveExtendedProblem(CountedPtr<const AugmentedForm>   cpMainProblemData,
                              vector<int>&                      riB,
                              vector<int>&                      riN,
                              int&                              stepCount,
                              Result*                           pResult) const
    throw(Error)
{
    const AugmentedForm& problem_data = *cpMainProblemData;

    // Wsp. funkcji ogranicze problemu rozszerzonego [m x (n + m)].
    Matrix            A_Ext;
    // Koszty problemu rozszerzonego [n + m].
    ColumnVector      c_Ext(problem_data.c().nrows() + problem_data.A().nrows());
    // Macierz rozszerzajca [m].
    IdentityMatrix    I(problem_data.A().nrows());
    // Macierzowa posta problemu rozszerzonego.
    CountedPtr<const AugmentedForm> extended_problem_data;
    // Bazowe wsp. funkcji ogranicze problemu rozszerzonego [m x m].
    SquareMatrix      B_Ext(problem_data.A().nrows());
    // Niebazowe wsp. funkcji ogranicze problemu rozszerzonego [m x n].
    Matrix            N_Ext(problem_data.A().nrows(), problem_data.A().ncols());

    int               i;
    std::vector<int>::iterator iter;

    LOG(INFO, wxT("-----------SolveExtendedProblem-----------"));

    // Wypeniamy macierz ogranicze problemu rozszerzonego
    A_Ext = problem_data.A() | I;
    LOG(INFO, wxT("-----------Extended problem-----------"));
    LOG(INFO, wxT("A^: ") << endl << A_Ext);

    // Wypeniamy wektor kosztw problemu rozszerzonego
    for (i = 1; i <= problem_data.c().nrows(); i++)
    {
        c_Ext(i) = 0;
    }

    for (     ; i <= problem_data.c().nrows() + problem_data.A().nrows(); i++)
    {
        c_Ext(i) = 1;
    }
    LOG(INFO, wxT("c^: ") << endl << c_Ext);

    // Tworzymy macierzow form problemu rozszerzonego
    CountedPtr<const AugmentedForm> cp_extended_problem_data =
        CountedPtr<const AugmentedForm>(new AugmentedForm(
                A_Ext, problem_data.b(), c_Ext,problem_data.ActualVariables()));

    // Tworzymy pierwsze tablice indeksw bazowych i niebazowych
    for (i = 1; i <= problem_data.A().ncols(); i++)
    {
        riN.push_back(i);
    }

    for (     ; i <= problem_data.A().ncols() + problem_data.A().nrows(); i++)
    {
        riB.push_back(i);
    }
    LOG(INFO, wxT("iB: ") << riB);
    LOG(INFO, wxT("iN: ") << riN);

    // Rozwizujemy problem rozszerzony.
    // Interesuje nas tylko otrzymane iB i iN.
    try
    {
        Solve(cpMainProblemData, cp_extended_problem_data, riB, riN,
               stepCount, pResult);
        LOG(INFO, wxT("iB: ") << riB);
        LOG(INFO, wxT("iN: ") << riN);

        // Sprawdzamy, czy otrzymalimy poprawn baz.
        for (iter = riB.begin(); iter != riB.end(); iter++)
        {
            if (*iter > problem_data.A().ncols())
            {
                throw Error(_("Problem unsolvable - extending variables have not been eliminated."));
            }
        }
    }
    catch (Error& error)
    {
        throw Error(_("Failed to create base - error occurred while solving the extended problem:\n") + error.What());
    }

    // Eliminujemy niebazowe zmienne dodane przy rozszerzeniu.
    for (iter = riN.begin(); iter != riN.end(); )
    {
        if (*iter > problem_data.A().ncols())
        {
            iter = riN.erase(iter);
        }
        else
        {
            iter++;
        }
    } // for

} // SolveExtendedProblem

void
Simplex::InitializeBase(const Matrix&           A,
                        const vector<int>&      iB,
                        const vector<int>&      iN,
                        SquareMatrix&           rB,
                        Matrix&                 rN) const
{
    int i;

    LOG(INFO, wxT("-----------InitializeBase-----------"));

    // Majc indeksy zmiennych bazowych, wypeniamy macierz B.
    rB.resize(A.nrows());
    for (i = 1; i <= rB.ncols(); i++)
    {
        rB.column(i) = A.column(iB[i - 1]);
    }
    LOG(INFO, wxT("B:") << endl << rB);

    // Analogicznie, wypeniamy macierz N.
    rN.resize(A.nrows(), A.ncols() - rB.ncols());
    for (i = 1; i <= (A.ncols() - rB.ncols()); i++)
    {
        rN.column(i) = A.column(iN[i - 1]);
    }
    LOG(INFO, wxT("N:") << endl << rN);
    LOG(DEBUG, wxString::Format(wxT("size: %dx%d"), rN.nrows(), rN.ncols()));
} // InitializeBase

ColumnVector
Simplex::Calculate_d(const ColumnVector&        c,
                     const vector<int>&         iB,
                     const vector<int>&         iN,
                     const SquareMatrix&        BInv,
                     const Matrix&              N) const
{
    // Koszty zredukowane [n - m]
    ColumnVector    d;
    // Koszty bazowe [m]
    ColumnVector    c_B(BInv.ncols());
    // Koszty niebazowe [n - m]
    ColumnVector    c_N(c.nrows() - BInv.ncols());

    int             i;

    LOG(INFO, wxT("-----------Calculate_d-----------"));

    for (i = 1; i <= BInv.ncols(); i++)
    {
       c_B(i) = c(iB[i - 1]);
    }
    // LOG (INFO, wxT("c_B: ") << endl << c_B);


    for (i = 1; i <= (c.nrows() - BInv.ncols()); i++)
    {
       c_N(i) = c(iN[i - 1]);
    }
    // LOG (INFO, wxT("c_N: ") << endl << c_N);

    d = c_N - N.t() * (BInv.t() * c_B);

    return d;

} // Calculate_d

void
Simplex::Solve(CountedPtr<const AugmentedForm>  cpMainProblemData,
               CountedPtr<const AugmentedForm>  cpExtendedProblemData,
               vector<int>&                     riB,
               vector<int>&                     riN,
               int&                             stepCount,
               Result*                          pResult) const
    throw(Error)
{
    bool main_phase = cpExtendedProblemData.Get() == NULL;
    const AugmentedForm& problem_data = (main_phase ?
            *cpMainProblemData : *cpExtendedProblemData);
    // Macierz bazowych wspczynnikw funkcji ogranicze.
    SquareMatrix            B;
    // Macierz pozostaych, niebazowych wspczynnikw funkcji ogranicze.
    Matrix                  N;
    // Bazowa cz punktu, pozostae wspczynniki x s zerowe [m].
    ColumnVector            x_B;
    // Wynikowa cz punktu.
    ColumnVector            x;
    // Koszty zredukowane [n - m].
    ColumnVector            d;
    // Odwrcona macierz B.
    SquareMatrix            B_inv;
    // Wektor przesunicia.
    ColumnVector            h;
    // Indeks zmiennej wprowadzanej do bazy.
    int                     s = 0;
    // Indeks zmiennej wyprowadzanej z bazy.
    int                     q = 0;
    // Wskaniki spadku zmiennych bazowych.
    ColumnVector            sigma;
    // Najmniejsze sigma.
    float                   alpha;

    // Aktualny krok.
    const int               STEP_LIMIT = Configuration::GetInstance().StepLimit();
    bool                    finished = false;

    auto_ptr<SimplexIteration> ap_iteration(new SimplexIteration(
                cpMainProblemData, cpExtendedProblemData, riB, riN));
    if (stepCount == 0)
    {
        if (main_phase) ap_iteration->rType() = SimplexIteration::FIRST_TRIVIAL;
        else            ap_iteration->rType() = SimplexIteration::FIRST_EXTENDED;
    }
    else
    {
        ap_iteration->rType() = SimplexIteration::FIRST_MAINPHASE;
    }

    try
    {
        // Wypeniamy macierze B i N na podstawie otrzymanych
        // indeksw.
        InitializeBase(problem_data.A(), riB, riN, B, N);

        ap_iteration->AddMatricesData(B, N);

        try
        {
            B_inv = B.i();
        }
        catch (SingularException)
        {
            throw Error(_("Invalid base - base matrix is singular."));
        }

        // Wyliczamy pierwszy punkt bazowy.
        x_B = B_inv * problem_data.b();
        assert(x_B.minimum() >= 0);

        ap_iteration->rX() = x_B;
        LOG(INFO, wxT("-----------First point-----------"));
        LOG(INFO, wxT("x_B: ") << endl << x_B);

        // Wyliczamy pierwszy wektor kosztw zredukowanych d.

        if (N.ncols() == 0)
        {// Jeli macierz niebazowa nie istnieje (macierz
         // A ma m kolumn i caa jest baz), to nie da si
         // oczywicie wykona adnego kroku - optymalizacja
         // koczy si w aktualnym punkcie.
            LOG(DEBUG, wxT("N = 0"));
            ap_iteration->AddReducedCostData(NullPoint, -1, -1);
            if (main_phase) pResult->rFinalResult() =
                ap_iteration->ActualPoint();
            pResult->AddIteration(auto_ptr<const IterationData>(
                        ap_iteration));
            return;
        }

        d = Calculate_d(problem_data.c(), riB, riN, B_inv, N);

        // Sprawdzamy warunek stopu
        // i wyznaczamy s - indeks zmiennej wchodzcej do bazy.
        finished = (d.minimum1(s) >= 0);

        ap_iteration->AddReducedCostData(d, s, 0);
        LOG(INFO, wxT("d:") << endl << d);

        while (!finished && stepCount < STEP_LIMIT)
        {
            ++stepCount;

            LOG(INFO, wxT("-----------Entering variable-----------"));
            LOG(INFO, wxT("s: ") << s << wxT(" column: ") << riN[s - 1]);

            // Wyliczamy wektor przesunicia
            // i sprawdzamy, czy problem jest ograniczony.
            h = B_inv * N.column(s);
            LOG(INFO, wxT("h:") << endl << h);
            if (h.maximum() <= 0)
            {
                ap_iteration->AddAllowedShiftData(h, 0, 0, NullPoint);
                throw Error(_("Cannot find the minimum - the problem is unbounded."));
            }

            sigma.resize(h.nrows());
            // Wyznaczamy indeks zmiennej opuszczajcej baz.
            for (int i = 1; i <= h.nrows(); ++i)
            {
                if (h(i) > 0)   sigma(i) = x_B(i) / h(i);
                else            sigma(i) = DBL_MAX;
            }
            int prev_q = q;
            alpha = sigma.minimum1(q);

            ap_iteration->AddAllowedShiftData(h, q, prev_q, sigma);
            LOG(INFO, wxT("-----------Leaving variable-----------"));
            LOG(INFO, wxT("q: ") << q << wxT(" column: ") << riB[q - 1]);
            LOG(DEBUG, wxT("alpha: ") << alpha);

            // Obliczamy kolejne x.
            x_B = x_B - alpha * h;
            x_B(q) = alpha;

            ap_iteration->rNewX() = x_B;
            pResult->AddIteration(auto_ptr<const IterationData>(
                        ap_iteration));
            LOG(INFO, wxT("-----------New point-----------"));
            LOG(INFO, wxT("x_B: ") << endl << x_B);

            // Tworzymy now macierz bazow.
            swap(riB[q - 1], riN[s - 1]);
            LOG(INFO, wxT("-----------New base-----------"));
            LOG(INFO, wxT("iB: ") << riB);
            LOG(INFO, wxT("iN: ") << riN);
            InitializeBase(problem_data.A(), riB, riN, B, N);

            ap_iteration.reset(new SimplexIteration(
                cpMainProblemData, cpExtendedProblemData, riB, riN));
            ap_iteration->AddMatricesData(B, N);
            ap_iteration->rX() = x_B;
            try
            {
                B_inv = B.i();
            }
            catch (SingularException)
            {
                // To nie powinno si nigdy zdarzy.
                throw Error(_("Invalid base - base matrix is singular."));
            }

            // Wyliczamy kolejny wektor kosztw zredukowanych d.
            d = Calculate_d(problem_data.c(), riB, riN, B_inv, N);

            int prev_s = s;
            // Sprawdzamy warunek stopu
            // i wyznaczamy s - indeks zmiennej wchodzcej do bazy.
            finished = (d.minimum1(s) >= 0);

            ap_iteration->AddReducedCostData(d, s, prev_s);
            LOG(INFO, wxT("d:") << endl << d);

            NotifyYield();
        } // while

        ap_iteration->AddAllowedShiftData(NullPoint, 0, q, NullPoint);
        if (main_phase) pResult->rFinalResult() = ap_iteration->ActualPoint();
        else            ap_iteration->rType() = SimplexIteration::LAST_EXTENDED;

        pResult->AddIteration(auto_ptr<const IterationData>(
                        ap_iteration));
    } // try
    catch (const BaseException& error)
    {
        pResult->AddIteration(auto_ptr<const IterationData>(
                        ap_iteration));
        throw Error(_("Internal computation error:\n") +
                wxString::FromAscii(error.what()));
    }
    catch (const Error& error)
    {
        pResult->AddIteration(auto_ptr<const IterationData>(
                        ap_iteration));
        throw;
    }

    assert(stepCount <= STEP_LIMIT);
    if (stepCount == STEP_LIMIT)
    {
        throw Error(_("Step limit reached - the problem is probably degenerate."));
    }

    LOG(INFO, wxT("Finished"));
} // Solve

/* static */ Simplex Simplex::mSimplex;
/* static */ wxString Simplex::mName(_("Simplex"));
