/**  Algorytm programowania liniowego "simplex".
 *
 * Metoda programowania liniowego suca do rozwizywania
 * problemw w postaci standardowej:
 * \f[ \min \: \lbrace c^T x \; : \; Ax = b, \; x \ge 0 \rbrace, \f]
 * gdzie:<br>
 * \f$A\f$ - macierz wspczynnikw funkcji ogranicze,<br>
 * \f$b\f$ - wektor ogranicze,<br>
 * \f$c\f$ - wektor kosztw.
 *
 * Metoda simplex oparta jest na spostrzeeniu, e punkt, w ktrym
 * znajduje si minimum, jest jednym z punktw wierzchokowych
 * obszaru dopuszczalnego i e da si go osign w skoczonej
 * liczbie krokw, przechodzc midzy punktami wierzchokowymi w taki
 * sposb, aby przy kadym przejciu warto funkcji celu zmniejszaa
 * si.
 *
 * Aby znajdowa punkty wierzchokowe, algorytm w kolejnych iteracjach
 * wyrnia w macierzy \f$A\f$ cz bazow i niebazow
 * oraz rozwizuje ukady rwna \f$A x = b \f$
 * wzgldem bazowych wspczynnikw punktu \f$x\f$
 * (niebazowe wspczynniki traktuje si jako zerowe).<br>
 * W ten sposb w kadym etapie wyznaczany jest jeden punkt znajdujcy si
 * na przeciciu ogranicze zwizanych z aktualn baz.
 * Na kocu kadego etapu w bazie wymienia si jedn zmienn:
 * wprowadza si t ze zmiennych niebazowych, ktrej wzrost
 * prowadzi do najszybszego spadku funkcji celu, a wyprowadza
 * t ze zmiennych bazowych, ktra po wprowadzeniu nowej zmiennej
 * najszybciej osiga warto 0 (pamitajmy, e standardowe sformuowanie
 * problemu narzuca ograniczenie \f$x \ge 0\f$).
 *
 * Algorytm koczy si, gdy wprowadzenie adnej ze zmiennych niebazowych
 * do bazy nie bdzie mogo prowadzi do spadku wartoci funkcji celu.
 *
 * Zaimplementowano dwufazow wersj algorytmu - jeli wyznaczenie pierwszej
 * bazy nie jest trywialne (macierz \f$A\f$ nie jest w postaci kanonicznej),
 * metoda w celu znalezienia bazy tworzy problem rozszerzony (macierze
 * \f$\hat{A}, \: \hat{b}, \: \hat{c}\f$) i rozwizuje go
 * przy pomocy algorytmu simplex.
 *
 * <br><b>Informacje wejciowe:</b><br>
 * \f$A_0\f$ - macierz wspczynnikw funkcji ogranicze [\f$m \times n\f$],<br>
 * \f$b_0\f$ - wektor ogranicze [\f$m\f$],<br>
 * \f$c_0\f$ - wektor kosztw (wspczynnikw funkcji celu) [\f$n\f$].<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$m\f$ - liczba ogranicze (liczba wierszy macierzy \f$A_0\f$),<br>
 * \f$n\f$ - liczba zmiennych (uwzgldnia zmienne dopeniajce, nie uwzgldnia
 * zmiennych rozszerzajcych, liczba kolumn macierzy \f$A_0\f$),<br>
 * \f$\hat{A}\f$ - macierz wsp. f. ogranicze problemu rozszerzonego
 * [\f$m \times (n + m)\f$],<br>
 * \f$\hat{b}\f$ - wektor ogranicze problemu rozszerzonego [\f$m\f$],<br>
 * \f$\hat{c}\f$ - wektor kosztw problemu rozszerzonego [\f$n + m\f$],<br>
 * \f$A, \: b, \: c\f$ - macierze aktualnie rozwizywanego problemu,<br>
 * \f$x_B\f$ - wektor bazowych wsprzdnych aktualnego punktu [\f$m\f$],<br>
 * \f$i_B\f$ - wektor indeksw bazowych (indeksw kolumn macierzy \f$A\f$,
 * ktre tworz aktualn baz),<br>
 * \f$i_N\f$ - wektor indeksw niebazowych,<br>
 * \f$B\f$ - aktualna baza (macierz zbudowana z kolumn macierzy \f$A\f$
 * o indeksach z \f$i_B\f$) [\f$m \times m\f$],<br>
 * \f$N\f$ - macierz zbudowana z pozostaych, niebazowych kolumn \f$A\f$
 * [\f$m \times (n - m) \mbox{ gdy } A = A_0; \; m \times n \mbox{ gdy }
 * A = \hat{A}\f$],<br>
 * \f$d\f$ - wektor kosztw zredukowanych (zawiera informacj o wpywie
 * ewentualnego wprowadzenia danej zmiennej niebazowej na warto funkcji
 * celu) [\f$(n - m) \mbox{ gdy } A = A_0; \; n \mbox{ gdy }
 * A = \hat{A}\f$],<br>
 * \f$h\f$ - wektor przesunicia (zawiera informacj o tym, jak proporcjonalnie
 * zmieni si poszczeglne zmienne bazowe po wprowadzeniu nowej zmiennej
 * do bazy) [\f$m\f$],<br>
 * \f$s\f$ - indeks zmiennej wprowadzanej do bazy,<br>
 * \f$q\f$ - indeks zmiennej wyprowadzanej z bazy,<br>
 * \f$\alpha\f$ - dugo kroku przy przejciu do nowego punktu
 * wierzchokowego,<br>
 * \f$p\f$ - informacja o aktualnie przeprowadzanej fazie algorytmu
 * (\f$p = 1\f$ - faza wstpna: rozwizywanie problemu rozszerzonego w celu
 * znalezienia pierwszej bazy; \f$p = 2\f$ - rozwizywanie problemu waciwego)
 * .<br>
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok 1:</b><br>
 * Sprawdzamy, czy ostatnich \f$m\f$ kolumn macierzy \f$A_0\f$ tworzy
 * macierz identycznociow. Jeli tak, przechodzimy do kroku 2.
 * Jeli nie, przechodzimy do kroku 3.
 *
 * <br><b>Krok 2:</b><br>
 * Macierz bazowa bdzie utworzona z ostatnich \f$m\f$ kolumn
 * macierzy \f$A_0\f$.<br>
 * Podstawiamy:
 * \f[ i_B = \lbrack n - m + 1, \: n - m + 2, \: \ldots, \: n \rbrack , \f]
 * \f[ i_N = \lbrack 1, \: 2, \: \ldots, \: n - m \rbrack . \f]
 * Przechodzimy do kroku 5.
 *
 * <br><b>Krok 3:</b><br>
 * W celu znalezienia pierwszej bazy rozwizujemy problem rozszerzony.<br>
 * Tworzymy macierze problemu rozszerzonego:
 * \f[ A = \lbrack A_0 \; | \; I(m) \rbrack = \hat{A} , \f]
 * \f[ b = b_0 = \hat{b} , \f]
 * \f[ c = \lbrack 0(n) \; | \; 1(m) \rbrack = \hat{c} , \f]
 * gdzie:<br>
 * \f$I(m)\f$ - macierz jednostkowa \f$m \times m\f$,<br>
 * \f$0(n)\f$ - wektor zerowy o \f$n\f$ wersach,<br>
 * \f$1(m)\f$ - wektor jednostkowy o \f$m\f$ wersach.<br>
 * Podstawiamy:
 * \f[ i_B = \lbrack n + 1, \: n + 2, \: \ldots, \: n + m \rbrack , \f]
 * \f[ i_N = \lbrack 1, \: 2, \: \ldots, \: n \rbrack . \f]
 * Na podstawie \f$i_B\f$ i \f$i_N\f$ oraz \f$A\f$ tworzymy macierze \f$B\f$
 * i \f$N\f$.<br>
 * Podstawiamy \f$p = 1\f$ i przechodzimy do kroku 6.
 *
 * <br><b>Krok 4:</b><br>
 * Sprawdzamy, czy otrzymane po rozwizaniu problemu rozszerzonego
 * indeksy bazowe s poprawne (nie dotycz kolumn rozszerzajcych):<br>
 * Jeli \f$ \exists \: i \in \overline{1,\ldots,m} \: . \: (i_B^{(i)} > n) \f$,
 * to STOP - zmiennych rozszerzajcych nie udao si wyeliminowa,
 * problemu nie da si  rozwiza.<br>
 * W przeciwnym wypadku usuwamy niebazowe indeksy zmiennych rozszerzajcych
 * :<br>
 * dla \f$i = 1, \: \ldots, n\f$ jeli \f$i_N^{(i)} > n\f$, element ten jest
 * usuwany z wektora.<br>
 * Przechodzimy do kroku 5.
 *
 * <br><b>Krok 5:</b><br>
 * Podstawiamy:
 * \f[ A = A_0, \f]
 * \f[ b = b_0, \f]
 * \f[ c = c_0,  \f]
 * \f[ p = 2 . \f]
 * Na podstawie \f$i_B\f$ i \f$i_N\f$ oraz \f$A\f$ tworzymy macierze \f$B\f$
 * i \f$N\f$.<br>
 * Przechodzimy do kroku 6.
 *
 * <br><b>Krok 6:</b><br>
 * Obliczamy macierz odwrotn do bazowej \f$B^{-1}\f$.<br>
 * Wyznaczamy pierwszy punkt wierzchokowy:
 * \f[ x_B = B^{-1} b .\f]
 * Przechodzimy do kroku 7.
 *
 * <br><b>Krok 7:</b><br>
 * Obliczamy wektor kosztw zredukowanych:
 * \f[ d = c_N - N^T B^{-T} c_B . \f]
 * Sprawdzamy warunek stopu:<br>
 * Jeli \f$d \ge 0\f$, to:
 * jeli \f$p = 2\f$ to STOP - wynikiem jest \f$x_B\f$ uzupeniony o pozostae,
 * niebazowe zmienne; jeli \f$p = 1\f$, to przechodzimy do kroku 4.<br>
 * Jeli \f$d < 0\f$, to zapamitujemy indeks najmniejszej skadowej tego
 * wektora - bdzie to indeks zmiennej wprowadzanej do bazy:
 * \f[ s = \arg \min_{i \in \overline{1,\ldots,dim(d)}} d^{(i)} . \f]
 * Przechodzimy do kroku 8.
 *
 * <br><b>Krok 8:</b><br>
 * Obliczamy wektor przesunicia:
 * \f[ h = B^{-1} N^{(s)} , \f]
 * gdzie:<br>
 * \f$N^{(s)}\f$ - s-ta kolumna macierzy \f$N\f$.<br>
 * Sprawdzamy, czy \f$h \le 0\f$. Jeli tak, to STOP - problem jest
 * nieograniczony.<br>
 * W przeciwnym wypadku szukamy zmiennej, ktr naley wyprowadzi
 * z bazy:
 * \f[ \mbox{Dla } i = 1, \: \ldots, \: m \mbox{: } \;
 * \sigma^{(i)} = \left\{ \begin{array}{ll}
 *                \frac{x_B^{(i)}}{h^{(i)}} & \mbox{ dla $h^{(i)} > 0$} \\
 *                +\infty & \mbox{ dla $h^{(i)} \le 0$}
 *                \end{array} \right. \f]
 * \f[ q = \arg \min_{i \in \overline{1,\ldots,m}} \sigma^{(i)} , \f]
 * \f[ \alpha = \sigma^{(q)} . \f]
 * Przechodzimy do kroku 9.
 *
 * <br><b>Krok 9:</b><br>
 * Zamieniamy indeksy zmiennej wychodzcej i wchodzcej do bazy:
 * \f[ i_B^{(q)} \leftrightarrow i_N^{(s)}. \f]
 * Na podstawie nowych indeksw tworzymy macierze \f$B\f$
 * i \f$N\f$.<br>
 * Obliczamy now macierz \f$B^{-1}\f$.<br>
 * Wyznaczamy nowy punkt wierzchokowy:
 * \f[ x_B = x_B - \alpha h, \f]
 * \f[ x_B^{(q)} = \alpha. \f]
 * Przechodzimy do kroku 7.
 *
 * <br><b>Uwagi.</b><br>
 * Zaimplementowano metod niezmodyfikowan, bez algorytmu zapobiegajcego
 * zaptleniu przy wystpowaniu punktw zdegenerowanych.
 *
 * Metoda zaimplementowana na podstawie opisu zamieszczonego w:<br>
 * Bruun H. N.: <i>Algorithms for Linear Optimization - an Introduction</i>.
 * Lyngby, Technical University of Denmark 1999.<br>
 * (http://www2.imm.dtu.dk/courses/02611/ALO.pdf).
 *
 * @see AugmentedForm
 */

#ifndef _Simplex_h
#define _Simplex_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../AugmentedForm.h"
#include "../LinearProblem.h"
#include "../LinearMethod.h"
#include "../util.h"

// FORWARD REFERENCES
//


class Simplex
    : public LinearMethod
{
public:
// LIFECYCLE

   /** Destruktor.
    */
   ~Simplex(void);


// OPERATORS

// OPERATIONS

    // TODO: Co tu zrobi? Tej metody nie powinno by w sympleksie,
    // ale abstrakcyjna w Method jest bezpieczeniejsza.
    virtual void        UpdateStartingConditions(const Result* result) const
    {}

    virtual void        ResetStartingConditions() const
    {}

public:

    /** Wykonuje kopi siebie.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&             Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType                Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType                 ClassId();

    virtual wxString                ToString(bool standalone = true) const;

// INQUIRY
// LIFECYCLE
protected:

    /** Konstruktor domylny.
     */
    Simplex(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    Simplex(const Simplex& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);

    /** Sprawdza, czy macierz \f$A\f$ jest w postaci kanonicznej. Jeli tak,
     * tworzy pierwsze indeksy kolumn bazowych i niebazowych.
     *
     * Na pocztku metoda sprawdza, czy macierz \f$A\f$ ma posta kanoniczn
     * (czy ostatnich \f$m\f$ kolumn tej macierzy tworzy macierz
     * identycznociow).
     * Jeli tak jest, metoda wypenia indeksy kolumn bazowych indeksami
     * ostatnich \f$m\f$ kolumn macierzy i indeksy kolumn niebazowych
     * pozostaymi.
     * W przeciwnym wypadku metoda nie wypenia indeksw i zwraca "false".
     *
     * @param cpMainProblemData CountedPtr na macierzow posta problemu
     * waciwego (\f$A_0, \: b_0, \: c_0\f$).
     * @param riB Referencja na wypeniany przez metod wektor
     * indeksw zmiennych bazowych (\f$i_B\f$).
     * @param riN Referencja na wypeniany przez metod wektor
     * indeksw zmiennych niebazowych (\f$i_N\f$).
     * @return Informacja, czy wypeniono indeksy bazowe i niebazowe.
     * @see AugmentedForm
     */
    bool            TrivialFirstBase(
                CountedPtr<const AugmentedForm>     cpMainProblemData,
                vector<int>&                        riB,
                vector<int>&                        riN) const;

    /** Znajduje pocztkow baz rozwizujc problem rozszerzony.
     *
     * Metoda wywoywana przez <code>Optimize</code> wtedy, gdy
     * macierz \f$A\f$ nie jest w postaci kanonicznej. Tworzy form macierzow
     * problemu rozszerzonego i rozwizuje go za pomoc algorytmu simplex
     * (metoda <code>Solve</code>). Nastpnie sprawdza, czy otrzymana
     * w ten sposb baza jest prawidowa (czy zmienne rozszerzajce zostay
     * wyeliminowane). Jeli tak, baza jest wykorzystywana do rozwizania
     * waciwego problemu. W przeciwnym razie, cay algorytm simplex koczy
     * si - waciwego problemu nie da si rozwiza.
     *
     * @param cpMainProblemData CountedPtr na macierzow posta
     * problemu waciwego (\f$A_0, \: b_0, \: c_0\f$).
     * @param riB Rererencja na wypeniany przez metod wektor
     * indeksw zmiennych bazowych (\f$i_B\f$).
     * @param riN Referencja na wypeniany przez metod wektor
     * indeksw zmiennych niebazowych (\f$i_N\f$).
     * @param stepCount Liczba iteracji, jak zajo rozwizywanie
     * problemu rozszerzonego.
     * @param pResult Wskanik na obiekt wyniku.
     */
    void            SolveExtendedProblem(
                CountedPtr<const AugmentedForm>     cpMainProblemData,
                vector<int>&                        riB,
                vector<int>&                        riN,
                int&                                stepCount,
                Result*                             pResult) const
                                                     throw(Error);


    /** Wypenia macierze bazowych i niebazowych wspczynnikw
     * ogranicze.
     *
     * Metoda tworzy macierz \f$B\f$ z kolumn macierzy \f$A\f$ o indeksach
     * zapamitanych w wektorze \f$i_B\f$ oraz macierz \f$N\f$ z pozostaych
     * kolumn \f$A\f$.
     *
     * @param A Macierz wspczynnikw funkcji ogranicze (\f$A\f$).
     * @param iB Wektor indeksw zmiennych bazowych (\f$i_B\f$).
     * @param iN Wektor indeksw zmiennych niebazowych (\f$i_N\f$).
     * @param rB Referencja na wypenian przez metod macierz
     * bazowych wspczynnikw funkcji ogranicze (\f$B\f$).
     * @param rN Referencja na wypenian przez metod macierz
     * niebazowych wspczynnikw funkcji ogranicze (\f$N\f$).
     */
    void            InitializeBase(const Matrix&            A,
                                   const vector<int>&       iB,
                                   const vector<int>&       iN,
                                   SquareMatrix&            rB,
                                   Matrix&                  rN) const;

    /** Wylicza wektor kosztw zredukowanych.
     *
     * @param c Wektor kosztw (\f$c\f$).
     * @param iB Wektor indeksw zmiennych bazowych (\f$i_B\f$).
     * @param iN Wektor indeksw zmiennych niebazowych (\f$i_N\f$).
     * @param BInv Odwrcona macierz bazowych wspczynnikw
     * funkcji ogranicze (\f$B^{-1}\f$).
     * @param N Macierz niebazowych wspczynnikw
     * funkcji ogranicze (\f$N\f$).
     *
     * @return Wektor kosztw zredukowanych (\f$d\f$).
     */
    ColumnVector    Calculate_d(const ColumnVector&         c,
                                const vector<int>&          iB,
                                const vector<int>&          iN,
                                const SquareMatrix&         BInv,
                                const Matrix&               N) const;

    /** Realizuje algorytm simplex (kroki 6-9 procedury).
     *
     * @param cpMainProblemData CountedPtr na macierzow posta
     * problemu waciwego (\f$A, \: b, \: c\f$).
     * @param cpExtendedProblemData CountedPtr na macierzow posta
     * problemu rozszerzonego (\f$\hat{A}, \: \hat{b}, \: \hat{c}\f$)
     * lub na NULL, gdy rozwizywany jest problem waciwy.
     * @param riB Referencja na wektor indeksw zmiennych bazowych (\f$i_B\f$).
     * @param riN Referencja na wektor indeksw zmiennych niebazowych
     * (\f$i_N\f$).
     * @param stepCount Referencja na liczb iteracji (pozwala sumowa
     * liczb iteracji wykonywanych przy rozwizywaniu problemu
     * rozszerzonego i waciwego).
     * @param pResult Wskanik na obiekt wyniku.
     */
    void    Solve(CountedPtr<const AugmentedForm>   cpMainProblemData,
                  CountedPtr<const AugmentedForm>   cpExtendedProblemData,
                  vector<int>&                      riB,
                  vector<int>&                      riN,
                  int&                              stepCount,
                  Result*                           pResult) const
                                                throw(Error);



    static Simplex          mSimplex;
    static wxString   mName;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _Simplex_h_
