#include <precomp.h>

#include "SimplexIteration.h"  // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================
/*
SimplexIteration::SimplexIteration()
    : mcpMainProblemData(),
      mcpExtendedProblemData(),
      miB(),
      miN(),
      mX(NullPoint),
      mD(NullPoint),
      mS(0),
      mPrevS(0),
      mH(NullPoint),
      mSigma(NullPoint),
      mQ(0),
      mAlpha(0.0)
{
}// SimplexIteration
*/

SimplexIteration::SimplexIteration(
            CountedPtr<const AugmentedForm>    cpMainProblemData,
            CountedPtr<const AugmentedForm>    cpExtendedProblemData,
            const vector<int>&                 iB,
            const vector<int>&                 iN)
    : mcpMainProblemData(cpMainProblemData),
      mcpExtendedProblemData(cpExtendedProblemData),
      mType(SimplexIteration::OTHER),
      miB(iB),
      miN(iN),
      mX(NullPoint),
      mD(NullPoint),
      mS(0),
      mPrevS(0),
      mH(NullPoint),
      mSigma(NullPoint),
      mQ(0),
      mPrevQ(0)
{
}// SimplexIteration

SimplexIteration::~SimplexIteration()
{
}// ~SimplexIteration

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
void
SimplexIteration::AddMatricesData(const SquareMatrix&       B,
                                  const Matrix&             N)
{
    assert(B.ncols() != 0);

    mB = B;
    mN = N;
}// AddMatricesData

void
SimplexIteration::AddReducedCostData(const ColumnVector&    d,
                                     int                    s,
                                     int                    prevS)
{
    mD     = d;
    mS     = s;
    mPrevS = prevS;
}// AddReducedCostData

void
SimplexIteration::AddAllowedShiftData(const ColumnVector&   h,
                                      int                   q,
                                      int                   prevQ,
                                      const ColumnVector&   sigma)
{
    mH     = h;
    mQ     = q;
    mPrevQ = prevQ;
    mSigma = sigma;
}// AddAllowedShiftData

//============================= ACCESS     ===================================

/* virtual */ vector<IterationData::IterationDataIdType>
SimplexIteration::ClassIds() const
{
    // Bierzemy identyfikatory od bezporedniego rodzica
    vector<IterationDataIdType> base_ids(IterationData::ClassIds());

    base_ids.insert(base_ids.begin(), SimplexIteration::ClassId());

    return base_ids;
}// ClassIds

/* static */ IterationData::IterationDataIdType
SimplexIteration::ClassId()
{
    static IterationDataIdType class_id = GenerateId();

    return class_id;
}// ClassId

/* virtual */ ColumnVector
SimplexIteration::EndValuePoint() const
{
    return ActualPoint();
}// EndValue


SimplexIteration::SimplexIterationType
SimplexIteration::Type() const
{
    return mType;
}// Type

SimplexIteration::SimplexIterationType&
SimplexIteration::rType()
{
    return mType;
}// rType


const Matrix&
SimplexIteration::MainA() const
{
    assert(mcpMainProblemData.Get() != NULL);

    return mcpMainProblemData->A();
}// MainA

const ColumnVector&
SimplexIteration::MainB() const
{
    assert(mcpMainProblemData.Get() != NULL);

    return mcpMainProblemData->b();
}// MainB

const ColumnVector&
SimplexIteration::MainC() const
{
    assert(mcpMainProblemData.Get() != NULL);

    return mcpMainProblemData->c();
}// MainC


const Matrix&
SimplexIteration::ExtendedA() const
{
    assert(mcpExtendedProblemData.Get() != NULL);

    return mcpExtendedProblemData->A();
}// ExtendedA

const ColumnVector&
SimplexIteration::ExtendedB() const
{
    assert(mcpExtendedProblemData.Get() != NULL);

    return mcpExtendedProblemData->b();
}// ExtendedB

const ColumnVector&
SimplexIteration::ExtendedC() const
{
    assert(mcpExtendedProblemData.Get() != NULL);

    return mcpExtendedProblemData->c();
}// ExtendedC


const vector<int>&
SimplexIteration::BasicIndices() const
{
    return miB;
}// BasicIndices

const vector<int>&
SimplexIteration::NonbasicIndices() const
{
    return miN;
}// NonbasicIndices


const SquareMatrix&
SimplexIteration::Base() const
{
    return mB;
}// Base

const Matrix&
SimplexIteration::N() const
{
    return mN;
}// N


const ColumnVector&
SimplexIteration::X() const
{
    return mX;
}// X

ColumnVector&
SimplexIteration::rX()
{
    return mX;
}// rX


const ColumnVector&
SimplexIteration::D() const
{
    return mD;
}// D

int
SimplexIteration::S() const
{
    return mS;
}// S

int
SimplexIteration::PreviousS() const
{
    return mPrevS;
}// PreviousS


const ColumnVector&
SimplexIteration::H() const
{
    return mH;
}// H

const ColumnVector&
SimplexIteration::Sigma() const
{
    return mSigma;
}// Sigma

int
SimplexIteration::Q() const
{
    return mQ;
}// Q

int
SimplexIteration::PreviousQ() const
{
    return mPrevQ;
}// PreviousQ

double
SimplexIteration::Alpha() const
{
    assert(mSigma != NullPoint);
    assert(mQ != 0);

    return mSigma(mQ);
}// Alpha

const ColumnVector&
SimplexIteration::NewX() const
{
    return mNewX;
}// NewX

ColumnVector&
SimplexIteration::rNewX()
{
    return mNewX;
}// rNewX

//============================= INQUIRY    ===================================
bool
SimplexIteration::MainPhase() const
{
    return mcpExtendedProblemData.Get() == NULL;
}// MainPhase

ColumnVector
SimplexIteration::ActualPoint() const
{
    assert(mcpMainProblemData.Get() != NULL);
    assert(mX != NullPoint);

    ColumnVector x(mcpMainProblemData->ActualVariables());
    std::vector<int>::const_iterator iter;

    for(int i = 1; i <= x.nrows(); i++)
    {
        iter = find(miB.begin(), miB.end(), i);
        if (iter != miB.end())
        {
            int pos = iter - miB.begin() + 1;
            x(i) = mX(pos);
        }
        else
        {
            x(i) = 0;
        }
    }

    return x;
}// ActualPoint

ColumnVector
SimplexIteration::FullPoint() const
{
    assert(mcpMainProblemData.Get() != NULL);
    assert(mX.nrows() != 0);
    assert(mX.nrows() == mB.ncols());

    ColumnVector x(mB.ncols() + mN.ncols());
    std::vector<int>::const_iterator iter;

    for(int i = 1; i <= x.nrows(); i++)
    {
        iter = find(miB.begin(), miB.end(), i);
        if (iter != miB.end())
        {
            int pos = iter - miB.begin() + 1;
            x(i) = mX(pos);
        }
        else
        {
            x(i) = 0;
        }
    }

    return x;
}// ActualPoint

int
SimplexIteration::ActualVariables()  const
{
    assert(mcpMainProblemData.Get() != NULL);

    return mcpMainProblemData->ActualVariables();
}// ActualVariables
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
