#include <precomp.h>

#include "../Configuration.h"
#include "../Error.h"
#include "../Logger.h"
#include "../Symbols.h"
//#include "../UnconstrainedProblem.h"
#include "../util.h"

#include "SteepestDescent.h"  // class implemented
#include "SteepestDescentIteration.h"

LOGGER(WARNING);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


SteepestDescent::~SteepestDescent()
{
}// ~SteepestDescent


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ std::auto_ptr<Method>
SteepestDescent::Clone() const
{
    return auto_ptr<Method>(new SteepestDescent(*this));
}// Clone

//============================= ACCESS     ===================================

/* virtual */ wxString&
SteepestDescent::rName()
{
    return SteepestDescent::mName;
}// rName

/* virtual */ const wxString&
SteepestDescent::Name() const
{
    return SteepestDescent::mName;
}// Name

/* virtual */ MethodIdType
SteepestDescent::Id() const
{
    return SteepestDescent::ClassId();
}// Id

/* static */ MethodIdType
SteepestDescent::ClassId()
{
    const static MethodIdType SteepestDescentId = Method::GenerateId();

    return SteepestDescentId;
}// ClassId

//============================= INQUIRY    ===================================

/* virtual */ bool
SteepestDescent::CanHandle(const ProblemBase* pProblem) const
{
    return (dynamic_cast<const UnconstrainedProblem*>(pProblem) != NULL
            && pProblem->Function().IsDifferentiable()
            && (pProblem->Variables().GetCount() >= 2));
}

/* virtual */ StopConditions
SteepestDescent::AllowedStopConditions() const
{
    StopConditions conditions;

    // Metoda najwikszego spadku jest mniej dokadna od pozostaych
    // gradientowych, dlatego domylnym warunkiem stopu jest
    // ||grad f(x k)||^2 < epsilon.
    conditions.push_back(
            CountedPtr<StopConditionBase>(
                new SquaredCustomValueCondition(wxT("|| grad f(x k) ||^2"))));
    conditions.push_back(
            CountedPtr<StopConditionBase>(
               new CustomValueCondition(wxT("|| grad f(x k) ||"))));

    const vector< CountedPtr<StopConditionBase> >& standard_conditions =
        StopConditionBase::StandardStopConditions();

    conditions.insert(conditions.end(),
            standard_conditions.begin(), standard_conditions.end());

    return conditions;

}// AllowedStopConditions

/////////////////////////////// PROTECTED  ///////////////////////////////////

SteepestDescent::SteepestDescent()
    : MethodWithLineSearch()
{
}// SteepestDescent

SteepestDescent::SteepestDescent(const SteepestDescent& from)
    : MethodWithLineSearch(from)
{
}// SteepestDescent


/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ auto_ptr<Result>
SteepestDescent::Optimize(const ProblemBase& problem) const
    throw(OptimizationError)
{
    auto_ptr<Result> ap_result(new Result(1));  // Wynik

    // Funkcja celu.
    const FunctionBase& function = problem.Function();
    // Aktualny punkt.
    ColumnVector x(mStartingPoint);
    // Poprzedni punkt (inicjowany wektorem zerowym, aby moliwe byo
    // sprawdzanie przed iteracj wszystkich warunkw stopu).
    ColumnVector x_prev(NullPoint);

    // Aktualny kierunek.
    ColumnVector d;
    // Dugo wykonanego kroku.
    double tau;

    const int STEP_LIMIT = Configuration::GetInstance().StepLimit();
    int step_count = 0;
    bool finished = false;

    assert(mStartingPoint.nrows() == problem.VariableCount());
    assert(mcpLineSearch.Get() != NULL);
    assert(mcpStopCondition.Get() != NULL);
    assert(function.IsDifferentiable());


    try
    {
        while (!finished && step_count < STEP_LIMIT)
        {
            ++step_count;
            LOG(DEBUG, wxT("---step ") << step_count);

            // Wyznaczamy kierunek przeciwny do gradientu.
            d = -(function.Gradient(x));

            auto_ptr<SteepestDescentIteration> ap_iteration(
                    new SteepestDescentIteration(x, -d));
            ap_iteration->rDirection() = d;

            // Sprawdzamy warunek stopu.
            finished = mcpStopCondition->IsMet(x_prev, x, function, d.norm_Frobenius());

            LOG(DEBUG, wxT("x: ") << endl << x);
            LOG(DEBUG, wxT("f(x): ") << endl << function.Evaluate(x));
            LOG(DEBUG, wxT("d: ") << endl << d);
            LOG(DEBUG, wxT("||d||^2: ") << endl << d.norm_Frobenius()
                * d.norm_Frobenius());
            LOG(DEBUG, wxT("d*d: ") << endl << (d.t() * d).as_scalar());


            if (!finished)
            { // Wykonujemy optymalny krok w kierunku
              // przeciwnym do gradientu.
                x_prev = x;
                x = StepInDirection(function, x_prev, d, &tau);

                ap_iteration->rTau() = tau;
                ap_iteration->rEndPoint() = x;
            } // if

            ap_result->AddIteration(auto_ptr<const IterationData>
                    (ap_iteration));
            NotifyYield();
        } // while

    } // try
    catch (BaseException)
    { // Wyjtek newmata.
        LOG(ERROR,wxString::FromAscii(BaseException::what()));
    }
    catch (const Error& error)
    {
        throw OptimizationError(error.What(),
                                std::auto_ptr<Result>(ap_result));
    }


    assert(step_count <= STEP_LIMIT);
    if (step_count == STEP_LIMIT)
    {
        throw OptimizationError(_("Step limit reached!"),
                                std::auto_ptr<Result>(ap_result));
    }

    ap_result->rFinalResult() = x;
    LOG(INFO, wxT("Step count: ") << step_count);

    return ap_result;
}// Optimize

wxString SteepestDescent::mName(_("Steepest Descent"));

// Utwrz i zarejestruj prototyp.
SteepestDescent SteepestDescent::mSteepestDescent;
