/**  Metoda najwikszego spadku (Cauchy'ego).
 * Gradientowa metoda kierunkw poprawy.<br>
 *
 * Podstaw teoretyczn metody s dwa twierdzenia:
 *
 * 1. Jeli istnieje taki kierunek \f$d\f$, e
 * \f[ (\nabla f(x))^T d < 0 , \f]
 * to \f$d\f$ jest kierunkiem poprawy:
 * \f[ f(x + \tau d) < f(x) , \f]
 * gdzie \f$ \tau \in (0; \sigma), \: \sigma > 0 \f$.
 *
 * 2. Dla liniowego modelu funkcji - rozwinicia w szereg
 * Taylora pierwszego rzdu:
 * \f[ f(x + \tau d) = f(x) + \tau d^T f'(x) + O(\tau^2) ,\f]
 * spord wszystkich kierunkw speniajcych warunek z twierdzenia
 * 1, kierunkiem najszybszego spadku jest
 * \f[ d = -\nabla f(x) . \f]
 *
 * <b>Informacje wejciowe:</b><br>
 * \f$f\f$ - minimalizowana funkcja \f$n\f$ zmiennych,<br>
 * \f$x_0\f$ - dowolnie wybrany punkt startowy,<br>
 * \f$\varepsilon\f$ - dokadno warunku stopu,<br>
 * warunek stopu - do wyboru:
 * <ul>
 * <li>\f$\|\nabla f(x^k)\|^2 < \varepsilon\f$</li>
 * <li>\f$\|\nabla f(x^k)\| < \varepsilon\f$</li>
 * <li>\f$\|x^{k+1} - x^k\| < \varepsilon\f$,</li>
 * <li>\f$|f(x^{k+1}) - f(x^k)| < \varepsilon\f$,</li>
 * <li>\f$\frac{|f(x^{k+1}) - f(x^k)|}{\|x^{k+1} - x^k\|} < \varepsilon\f$,</li>
 * </ul>
 * wybrana metoda minimalizacji w kierunku wraz z parametrami.<br>
 *
 * <br><b>Oznaczenia:</b><br>
 * \f$k\f$ - numer aktualnej iteracji,<br>
 * \f$d\f$ - aktualny kierunek poszukiwa,<br>
 * \f$x^k\f$ - aktualny punkt prbny,<br>
 * \f$\tau\f$ - dugo wykonywanego aktualnie kroku.<br>
 *
 * <br><b>Procedura.</b><br>
 *
 * <br><b>Krok wstpny:</b><br>
 * Podstawiamy:
 * \f[ x^1 = x^0 , \f]
 * \f[ k = 1 . \f]
 *
 * <br><b>Krok 1:</b><br>
 * Wyznaczamy kierunek przeciwny do gradientu:
 * \f[ d = -\nabla f(x^k) . \f]
 *
 * <br><b>Krok 2:</b><br>
 * Sprawdzamy warunek stopu: <br>
 * Jeli \f$ \left<\nabla f(x^k), \nabla f(x^k)\right> < \varepsilon , \f$
 * to STOP. W przeciwnym wypadku przechodzimy do kroku 3.
 *
 * <br><b>Krok 3:</b><br>
 * Wykonujemy krok w kierunku \f$d\f$ o optymalnej dugoci
 * \f$\tau\f$:
 * \f[ \tau = \arg \min_{\tau} f(x^k + \tau d) , \f]
 * \f[ x^{k+1} = x^k + \tau d . \f]
 * Podstawiamy \f$ k = k + 1 \f$ i przechodzimy do kroku 1.
 *
 * <br><b>Uwagi.</b><br>
 * Metoda opiera si na liniowym modelu funkcji (patrz twierdzenie 2)
 * i dla funkcji wyszego rzdu jest najczciej wolno zbiena.
 * Podstawow wad, wpywajc na woln zbieno,
 * jest "zygzakowaty" charakter przebiegw optymalizacji. Dokadne
 * metody minimalizacji w kierunku (a taka jest tu wykorzystywana w kroku 3)
 * zatrzymuj si w punkcie, w ktrym gradient funkcji \f$\nabla f(x^k)\f$
 * jest prostopady do aktualnego kierunku poszukiwa \f$d^k\f$.
 * Kolejny kierunek \f$d^{k+1}\f$ bdzie, zgodnie z podstawowym zaoeniem
 * metody, rwnolegy do gradientu. W ten sposb metoda w caym swym przebiegu
 * operuje tylko dwoma prostopadymi do siebie kierunkami, silnie
 * zalenymi od punktu pocztkowego.
 *
 * Metoda zaimplementowana na podstawie opisu zamieszczonego w:<br>
 * Findeisen W., Szymanowski J., Wierzbicki A.: <i>Teoria i metody
 * obliczeniowe optymalizacji</i>. Warszawa, PWN 1977.
 *
 */

#ifndef _SteepestDescent_h
#define _SteepestDescent_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CustomValueCondition.h"
#include "../DoubleParameter.h"
#include "../FunctionBase.h"
#include "../ProblemBase.h"
#include "../MethodWithLineSearch.h"
#include "../SquaredCustomValueCondition.h"
#include "../ui/MethodWithLineSearchPanel.h"

// FORWARD REFERENCES
//


class SteepestDescent
    : public MethodWithLineSearch
{
public:
// LIFECYCLE

    /** Panel ustawie metody.
     */
    friend class MethodWithLineSearchPanel<SteepestDescent>;

    /** Destruktor.
     */
    ~SteepestDescent(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi metody.
     */
    virtual std::auto_ptr<Method>       Clone() const;

// ACCESS

    /** Zwraca nazw metody.
     */
    virtual const wxString&         Name() const;

    /** Zwraca ID metody.
     */
    virtual MethodIdType            Id() const;

    /** Zwraca ID metody.
     */
    static MethodIdType             ClassId();

// INQUIRY

    virtual bool                    CanHandle(const ProblemBase* pProblem) const;

    virtual StopConditions          AllowedStopConditions() const;

protected:

    /** Konstruktor domylny.
     */
    SteepestDescent(void);

    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    SteepestDescent(const SteepestDescent& from);

    virtual wxString&               rName();
private:

    virtual std::auto_ptr<Result> Optimize(const ProblemBase& problem) const
        throw(OptimizationError);
    static wxString      mName; ///< Nazwa metody.
    static SteepestDescent mSteepestDescent;    ///< Egzemplarz metody.
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _SteepestDescent_h_
