#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Logger.h"

#include "AttributeDatabase.h"
#include "ConstraintsLayer.h"  // class implemented
#include "ConstraintsLegendAction.h"
#include "ContourPlotter.h"
#include "PlotControl.h"

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ConstraintsLayer::ConstraintsLayer(const PlotControl&           plotControl,
                                   const Constraint::Container  constraints)
    : PlotLayer(plotControl),
      mConstraints(constraints),
      mLegend(true)
{
    LOG(DEBUG, wxT("Constraint number in ctor: ") << mConstraints.size());
}// ConstraintsLayer

ConstraintsLayer::ConstraintsLayer(const ConstraintsLayer& from)
    : PlotLayer(from),
      mConstraints(from.mConstraints),
      mLegend(from.mLegend)
{
}// ConstraintsLayer

ConstraintsLayer::~ConstraintsLayer()
{
}// ~ConstraintsLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ vector<ToolbarTool>
ConstraintsLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(PlotLayer::CreateTools(pPlotControl));

    ToolbarTool legend_tool(
            auto_ptr<ToggleToolAction>
                (new ConstraintsLegendAction(pPlotControl, this)),
            ToolbarTool::LAYER_SPECIFIC,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_LEGEND")),
            _("Legend"), _("Show/Hide constraints legend"));

    tools.push_back(legend_tool);

    return tools;
}// CreateTools

//============================= ACCESS     ===================================

bool&
ConstraintsLayer::rLegend()
{
    return mLegend;
}// rLegend

//============================= INQUIRY    ===================================

bool
ConstraintsLayer::IsLegend() const
{
    return mLegend;
}// IsLegend

/////////////////////////////// PROTECTED  ///////////////////////////////////

void
ConstraintsLayer::DrawLegend(wxDC& dc) const
{
    // Sprawd, czy w ogle co robi.
    if (!mLegend)   return;

    // TODO: przenie w lepsze miejsce:
    wxFont font(8, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    const wxCoord EXAMPLE_LINE_LENGTH = 20;
    const wxCoord SPACING = 5;
    const wxCoord OFFSET = 10; // Od prawego grnego rogu
    const wxCoord OUTLINE_DISTANCE = 4;
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    // Ustal rozmiar najwikszego tekstu
    wxSize max_text_extent;

    // TODO: pamita i przywraca stary?
    dc.SetFont(font);

    Constraint::Container::const_iterator constraint_iterator;
    for (constraint_iterator = mConstraints.begin();
         constraint_iterator != mConstraints.end();
         ++constraint_iterator)
    {
        wxCoord width;
        wxCoord height;

        dc.GetTextExtent(constraint_iterator->ToString(false), &width, &height);

        max_text_extent.IncTo(wxSize(width, height));

    }

    wxCoord pixel_width;
    wxCoord pixel_height;
    dc.GetSize(&pixel_width, &pixel_height);

    unsigned constraint_count = mConstraints.size();

    // Rysuj prostokt
    dc.SetBrush(*wxWHITE_BRUSH);
    dc.SetPen(*wxBLACK_PEN);

    wxCoord rectangle_width =
        3*SPACING + max_text_extent.GetWidth() + EXAMPLE_LINE_LENGTH;
    wxCoord rectangle_height =
        constraint_count * (max_text_extent.GetHeight() + SPACING) + SPACING;

    dc.DrawRectangle(
            pixel_width - OFFSET - 3*SPACING - max_text_extent.GetWidth() - EXAMPLE_LINE_LENGTH,
            OFFSET,
            rectangle_width,
            rectangle_height);

    wxCoord text_x =
        pixel_width - OFFSET - SPACING - max_text_extent.GetWidth();
    wxCoord text_y = OFFSET + SPACING;
    wxCoord line_x = text_x - SPACING - EXAMPLE_LINE_LENGTH;
    wxCoord line_y_offset = max_text_extent.GetHeight() / 2;

    // Ograniczenia nierwnociowe
    unsigned index = 0;         // indeks koloru rysowanego ograniczenia
    unsigned dummy_index = 0;   // indeks koloru drugiego typu ogranicze
    for (constraint_iterator = mConstraints.begin();
         constraint_iterator != mConstraints.end();
         ++constraint_iterator)
    {
        if (constraint_iterator->Type() != Constraint::EQUAL)
        {
            // Tekst
            dc.SetPen(attributes.Pen(AttributeDatabase::DEFAULT));

            dc.DrawText(constraint_iterator->ToString(false), text_x, text_y);

            // Przykadowa linia
            wxPen normal = GetConstraintPen(*constraint_iterator,
                                            dummy_index, index);
            wxPen dashed = normal;
            dashed.SetStyle(ContourPlotter::DASH_STYLE);

            dc.SetPen(dashed);
            dc.DrawLine(line_x,
                text_y + line_y_offset - OUTLINE_DISTANCE / 2,
                line_x + EXAMPLE_LINE_LENGTH,
                text_y + line_y_offset - OUTLINE_DISTANCE / 2);

            dc.SetPen(normal);
            dc.DrawLine(line_x,
                text_y + line_y_offset + OUTLINE_DISTANCE / 2,
                line_x + EXAMPLE_LINE_LENGTH,
                text_y + line_y_offset + OUTLINE_DISTANCE / 2);

            text_y += max_text_extent.GetHeight() + SPACING;
        }
    }

    // Ograniczenia rwnociowe
    index = 0;
    for (constraint_iterator = mConstraints.begin();
         constraint_iterator != mConstraints.end();
         ++constraint_iterator)
    {
        if (constraint_iterator->Type() == Constraint::EQUAL)
        {
            // Tekst
            dc.SetPen(attributes.Pen(AttributeDatabase::DEFAULT));

            dc.DrawText(constraint_iterator->ToString(false), text_x, text_y);

            // Przykadowa linia
            dc.SetPen(GetConstraintPen(*constraint_iterator,
                                       index, dummy_index));
            dc.DrawLine(line_x, text_y + line_y_offset,
                    line_x + EXAMPLE_LINE_LENGTH, text_y + line_y_offset);

            text_y += max_text_extent.GetHeight() + SPACING;
        }
    }

    dc.SetPen(wxNullPen);
}// DrawLegend


const wxPen&
ConstraintsLayer::GetConstraintPen(const Constraint&  constraint,
                                   unsigned&          rEqualityIndex,
                                   unsigned&          rInequalityIndex) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    static bool first_time = true;
    static wxPen constraint_pens[] = {
        wxPen(attributes.Pen(AttributeDatabase::PLOT_CONSTRAINT1)),
        wxPen(attributes.Pen(AttributeDatabase::PLOT_CONSTRAINT2)),
        wxPen(attributes.Pen(AttributeDatabase::PLOT_CONSTRAINT3)),
        wxPen(attributes.Pen(AttributeDatabase::PLOT_CONSTRAINT4)),
        wxPen(attributes.Pen(AttributeDatabase::PLOT_CONSTRAINT5))
    };
    const unsigned constraint_max = 5;      // Poprawi, jak si zmieni!

    if (first_time)
    {
        for (unsigned i = 0; i < constraint_max; ++i)
        {
            constraint_pens[i].SetWidth(2);
        }
        first_time = false;
    }

    if (constraint.Type() == Constraint::EQUAL)
    {   // Od koca, eby si kolory nie pokryway przy kilku ograniczeniach
        if (rEqualityIndex == 0)    rEqualityIndex = constraint_max;

        --rEqualityIndex;

        return constraint_pens[rEqualityIndex];
    }
    else
    {
        unsigned index = rInequalityIndex;

        ++rInequalityIndex;
        if (rInequalityIndex == constraint_max) rInequalityIndex = 0;

        return constraint_pens[index];
    }
}// GetConstraintPen

/////////////////////////////// PRIVATE    ///////////////////////////////////

