#include <precomp.h>


#include "../Logger.h"
#include "../ProblemBase.h"
#include "ConstraintsPanel.h"
#include "ConstraintsValidator.h"  // class implemented


LOGGER(DEBUG);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

ConstraintsValidator::ConstraintsValidator(
        CountedPtr<const FunctionBase>&     rcpFunction,
        CountedPtr<const VariableNames>&    rcpVariables,
        Constraint::Container&              rConstraints)
    : mrcpFunction(rcpFunction),
      mrcpVariables(rcpVariables),
      mrConstraints(rConstraints)
{
}// ConstraintsValidator

ConstraintsValidator::ConstraintsValidator(
        const ConstraintsValidator& from)
    : mrcpFunction(from.mrcpFunction),
      mrcpVariables(from.mrcpVariables),
      mrConstraints(from.mrConstraints)
{
}// ConstraintsValidator

ConstraintsValidator::~ConstraintsValidator()
{
}// ~ConstraintsValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
ConstraintsValidator::Clone() const
{
    return new ConstraintsValidator(*this);
}// Clone

/* virtual */ bool
ConstraintsValidator::TransferToWindow()
{
    ConstraintsPanel* p_panel = dynamic_cast<ConstraintsPanel*>(GetWindow());

    LOG(DEBUG, wxT("TransferToWindow"));

    p_panel->mcpFunction    = mrcpFunction;
    p_panel->mcpVariables   = mrcpVariables;
    p_panel->mConstraints   = mrConstraints;
    p_panel->InitFunctionDirty();

    p_panel->RefreshList();

    p_panel->TransferDataToWindow();

    return true;
}// TransferToWindow

/* virtual */ bool
ConstraintsValidator::TransferFromWindow()
{
    ConstraintsPanel* p_panel = dynamic_cast<ConstraintsPanel*>(GetWindow());

    p_panel->TransferDataFromWindow();

    mrcpFunction    = p_panel->mcpFunction;
    mrcpVariables   = p_panel->mcpVariables;
    mrConstraints   = p_panel->mConstraints;

    return true;
}// TransferFromWindow

/* virtual */ bool
ConstraintsValidator::Validate(wxWindow* pParent)
{
    ConstraintsPanel* p_panel = dynamic_cast<ConstraintsPanel*>(GetWindow());
    bool result = GetWindow()->Validate();

    if (result && (p_panel->mConstraints.size() > 0))
    {
        const FunctionValidator *p_function_validator =
            dynamic_cast<FunctionValidator *>
                (p_panel->mpFunctionCombo->GetValidator());

        // Sprawd, czy sa jakie ograniczenia zawierajce zmienne, ktre nie
        // wystpuj we wzorze funkcji celu.
        // Poniewa jeszcze nie zostao wywoane TransferDataFromWindow, musimy
        // pobra zmienne nieoficjalnie.
        wxArrayInt invalid_indexes =
            FindInvalidConstraints(
                    p_panel->mConstraints,
                    p_function_validator->PeekTemporaryVariables());

        LOG(DEBUG, wxT("Number of invalid constraints: ")
                << invalid_indexes.GetCount());

        if (invalid_indexes.GetCount() > 0)
        {
            // Przygotuj komunikat do okienka dialogowego
            wxString error(_("The following constraints contain variables that are not present in objective function:\n"));

            for (unsigned i = 0; i < invalid_indexes.GetCount(); ++i)
            {
                error <<
                    p_panel->mConstraints[invalid_indexes[i]].ToString(false);
                error << wxT('\n');
            }

            error << wxT('\n') << _("Remove them?");

            // Wywietl okienko i zadziaaj zgodnie z odpowiedzi
            wxMessageDialog message_box(pParent,
                    error,
                    _("Invalid constraints!"),
                    wxYES_NO | wxNO_DEFAULT | wxICON_ERROR);

            if (message_box.ShowModal() == wxID_YES)
            {
                RemoveInvalidConstraints(p_panel->mConstraints,
                        invalid_indexes);

                p_panel->RefreshList();
                result = true;
            }
            else    result = false;
        } // end if (invalid_indexes.GetCount() > 0)
        else    result = true;
    }

    if (result && (p_panel->mConstraints.size() == 0))
    {
        wxMessageDialog message_box(pParent, _("Enter constraints!"),
                _("No constraints!"), wxOK | wxICON_ERROR);
        message_box.ShowModal();
//        GetWindow()->SetFocus();

        result = false;
    }

    return result;
}// Validate

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

wxArrayInt
ConstraintsValidator::FindInvalidConstraints(
        const Constraint::Container& constraints,
        const VariableNames&         variables) const
{
    wxArrayInt indexes;

    for (unsigned i = 0; i < constraints.size(); ++i)
    {
        if (constraints[i].Function().ContainsUnknown(variables))
        {
            indexes.Add(i);
        }
    }
    return indexes;
}// FindInvalidConstraints

void
ConstraintsValidator::RemoveInvalidConstraints(
        Constraint::Container&  rConstraints,
        const wxArrayInt&       indexes) const
{
    for (int i = indexes.GetCount() - 1; i >= 0; --i)
    {
        rConstraints.erase(rConstraints.begin() + indexes[i]);
    }
}// RemoveInvalidConstraints

