#include <precomp.h>

#include "../Logger.h"

#include "ContourPlotter.h"  // class implemented

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

const /* static */ unsigned ContourPlotter::DASH_STYLE = wxSHORT_DASH;
const /* static */ unsigned ContourPlotter::OUTLINE_DISTANCE = 9;

//============================= LIFECYCLE ====================================

ContourPlotter::ContourPlotter(unsigned levelCount)
    : mLevels(levelCount)
{
}// ContourPlotter



//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void
ContourPlotter::AddSegment(double      x1,
                           double      y1,
                           double      x2,
                           double      y2,
                           unsigned    level)
{
    assert (level < mLevels.size());

    wxPoint a(lround(x1), lround(y1));
    wxPoint b(lround(x2), lround(y2));

    ContourContainer& rContour = mLevels[level];
    ContourContainer::iterator iContour = rContour.begin();

    bool is_connected = false;
    while (iContour != rContour.end() && !is_connected)
    {
        is_connected = iContour->AddSegment(a, b);
        ++iContour;
    }

    if (!is_connected && (a != b))
    {   // Dodaj nowy, niepoczony kontur
        rContour.push_back(Contour(a, b));
    }
}// ContourPlotter

void
ContourPlotter::Collapse()
{
    for (LevelContainer::iterator iLevel = mLevels.begin();
         iLevel != mLevels.end();
         ++iLevel)
    {
        *iLevel = Collapse(*iLevel);
    }
}


void
ContourPlotter::Draw(wxDC& dc, bool shading) const
{
    unsigned shade_distance = 0;
    if (shading)
    {
        shade_distance = static_cast<unsigned int>(ceil(
                                (LIGHTEST_SHADE - DARKEST_SHADE) /
                                        mLevels.size()));

        LOG(INFO, wxString::Format(wxT("%d levels, shade distance = %d"),
                    mLevels.size(), shade_distance));
    }
    DoDraw(dc, NULL, 0, 0, 0, NULL, false, shade_distance);
}// Draw

void
ContourPlotter::Draw(wxDC&                 dc,
                     const FunctionBase&   function,
                     unsigned              variableIndexX,
                     unsigned              variableIndexY,
                     unsigned              variableCount,
                     const PlotControl&    plotControl,
                     bool                  greaterThanConstraint) const
{
    DoDraw(dc,
           &function,
           variableIndexX,
           variableIndexY,
           variableCount,
           &plotControl,
           greaterThanConstraint,
           0);
}// Draw

//============================= ACCESS     ===================================

void
ContourPlotter::DumpStats() const
{
    for (unsigned level = 0; level < mLevels.size(); ++level)
    {
        LOG(INFO, wxT("Level: ") << level);
        LOG(INFO, wxT(" Contours: ") << mLevels[level].size());
        for (unsigned contour = 0; contour < mLevels[level].size(); ++contour)
        {
            LOG(INFO, wxT("  segments: ")
                    << mLevels[level][contour].SegmentCount()
                    << (mLevels[level][contour].IsClosed() ? wxT(" closed") : wxT(" open")));
        }
    }
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
ContourPlotter::DoDraw(wxDC& dc,
                       const FunctionBase*     pFunction,
                       unsigned                variableIndexX,
                       unsigned                variableIndexY,
                       unsigned                variableCount,
                       const PlotControl*      pPlotControl,
                       bool                    greaterThanConstraint,
                       unsigned                shadeDistance) const
{
    wxPoint points[MaxSegmentLength()];
    unsigned current_shade = LIGHTEST_SHADE;

    for (LevelContainer::const_iterator iLevel = mLevels.begin();
         iLevel != mLevels.end();
         ++iLevel)
    {
        const ContourContainer& rContour = *iLevel;

        if (shadeDistance > 0)
        {   // cieniujemy poziomice
            dc.SetPen(wxPen(wxColour(current_shade, current_shade,
                            current_shade)));
            current_shade -= shadeDistance;
        }

        for (ContourContainer::const_iterator iContour = rContour.begin();
             iContour != rContour.end();
             ++iContour)
        {
            iContour->ToPointArray(points);


            dc.DrawLines(iContour->SegmentCount(), points);

            if (pFunction != NULL)
            {   // Rysujemy ograniczenie
                assert (pPlotControl != NULL);

                unsigned count;
                bool outline_counter_clockwise = iContour->FindOuterSide(
                        *pFunction,
                        variableIndexX,
                        variableIndexY,
                        variableCount,
                        *pPlotControl);

                iContour->ShiftedToArray(points, count,
                        greaterThanConstraint ?
                            !outline_counter_clockwise :
                             outline_counter_clockwise);

                wxPen old_pen = dc.GetPen();
                wxPen pen = old_pen;
                pen.SetStyle(DASH_STYLE);
                dc.SetPen(pen);
                dc.DrawLines(count, points);
                dc.SetPen(old_pen);
            }
        }
    }
}// DoDraw

unsigned
ContourPlotter::MaxSegmentLength() const
{
    unsigned max = 0;
    for (LevelContainer::const_iterator iLevel = mLevels.begin();
         iLevel != mLevels.end();
         ++iLevel)
    {
        const ContourContainer& rContour = *iLevel;
        for (ContourContainer::const_iterator iContour = rContour.begin();
             iContour != rContour.end();
             ++iContour)
        {
            if (iContour->SegmentCount() > max) max = iContour->SegmentCount();
        }
    }
    return max;
}// MaxSegmentLength

ContourPlotter::ContourContainer
ContourPlotter::Collapse(const ContourPlotter::ContourContainer& contours)
{
    ContourContainer new_contours;

    for (ContourContainer::const_iterator iContour = contours.begin();
         iContour != contours.end();
         ++iContour)
    {
        ContourContainer::iterator iNewContour = new_contours.begin();

        bool is_connected = false;
        while (iNewContour != new_contours.end() && !is_connected)
        {
            is_connected = iNewContour->AddContour(*iContour);
            ++iNewContour;
        }

        // Skopiuj kontur bez poczenia
        if (!is_connected)      new_contours.push_back(*iContour);
    }

    return new_contours;
}

//--------------------------- Contour class ----------------------------------

//============================= LIFECYCLE ====================================
ContourPlotter::Contour::Contour()
    : mPolyline(), mBegin(), mEnd(), mClosed(false)
{
}// Contour::Contour

ContourPlotter::Contour::Contour(const Contour& from)
    : mPolyline(from.mPolyline),
      mBegin(from.mBegin),
      mEnd(from.mEnd),
      mClosed(from.mClosed)
{
}// Contour::Contour

ContourPlotter::Contour::Contour(wxPoint a, wxPoint b)
    : mPolyline(), mBegin(), mEnd(), mClosed(false)

{
    AddSegment(a, b);
}// Contour::Contour

//============================= OPERATIONS ====================================

// Ta wersja niby mogaby by zrobiona na maym i prostym konturze, ale skoro
// ju ona jest i wymaga mniej operacji i kopiowania, to zostawiam (cho nie
// testowaem, na ile bez niej byoby wolniej).
bool
ContourPlotter::Contour::AddSegment(wxPoint a, wxPoint b)
{
    bool result = false;

    if (! mClosed && (a != b))
    {
        if (mPolyline.size() == 0)
        {
            mPolyline.push_back(a);
            mPolyline.push_back(b);
            mBegin = a;
            mEnd = b;
            result = true;
        }
        else
        {
//            LOG(DEBUG, wxT("a: (" << a.x << wxT(", ") << a.y <<wxT(")"))
//                    << wxT('\t')
//                    << wxT("b: (" << b.x << wxT(", ") << b.y <<wxT(")")));
//            LOG(DEBUG, wxT("begin: (" << mBegin.x << wxT(", ") << mBegin.y <<wxT(")"))
//                    << wxT('\t')
//                    << wxT("end:   (" << mEnd.x << wxT(", ") << mEnd.y <<wxT(")")));
            if (a == mBegin)
            {
//                LOG(DEBUG, wxT("Connected beginning"));
                mBegin = b;
                mPolyline.insert(mPolyline.begin(), b);
                result = true;
            }
            else if (b == mBegin)
            {
//                LOG(DEBUG, wxT("Connected beginning"));
                mBegin = a;
                mPolyline.insert(mPolyline.begin(), a);
                result = true;
            }

            if (a == mEnd)
            {
                mEnd = b;
                mPolyline.insert(mPolyline.end(), b);

                // Jeli poprzedni koniec rwnie zosta dodany, to ten odcinek
                // zamyka aman.
                mClosed = result;
                result = true;
//                LOG(DEBUG, wxT("Connected ending"));
            }
            else if (b == mEnd)
            {
                mEnd = a;
                mPolyline.insert(mPolyline.end(), a);

                // Jeli poprzedni koniec rwnie zosta dodany, to ten odcinek
                // zamyka aman.
                mClosed = result;
                result = true;
//                LOG(DEBUG, wxT("Connected ending"));
            }
        }
    }

    return result;
}// Contour::AddSegment


bool
ContourPlotter::Contour::AddContour(const ContourPlotter::Contour& contour)
{
    bool result = false;

    if ((! mClosed) && (! contour.mClosed) && (contour.mBegin != contour.mEnd))
    {
        if (mPolyline.size() == 0)
        {
            mPolyline.insert(mPolyline.begin(),
                             contour.mPolyline.begin(),
                             contour.mPolyline.end());
            mBegin = contour.mBegin;
            mEnd = contour.mEnd;
            result = true;
        }
        else
        {
            assert (SegmentCount() >= 2);
            if (contour.mBegin == mBegin)
            {
                mBegin = contour.mEnd;

                // Pomi wsplny punkt
                Polyline::const_reverse_iterator rend =
                    contour.mPolyline.rend();
                --rend;
                mPolyline.insert(mPolyline.begin(),
                                 contour.mPolyline.rbegin(),
                                 rend);
                result = true;
            }
            else if (contour.mEnd == mBegin)
            {
                mBegin = contour.mBegin;

                // Pomi wsplny punkt
                Polyline::const_iterator end = contour.mPolyline.end();
                --end;
                mPolyline.insert(mPolyline.begin(),
                                 contour.mPolyline.begin(),
                                 end);
                result = true;
            }

            if (contour.mBegin == mEnd)
            {
                mEnd = contour.mEnd;

                // Jeli poprzedni koniec rwnie zosta dodany, to ten
                // odcinek zamyka aman.
                mClosed = result;

                if (!mClosed)
                {
                    Polyline::const_iterator begin = contour.mPolyline.begin();
                    ++begin;

                    mPolyline.insert(mPolyline.end(),
                                     begin,
                                     contour.mPolyline.end());
                    result = true;
                }
            }
            else if (contour.mEnd == mEnd)
            {
                mEnd = contour.mBegin;
                // Jeli poprzedni koniec rwnie zosta dodany, to ten
                // odcinek zamyka aman.
                mClosed = result;

                if (!mClosed)
                {
                    Polyline::const_reverse_iterator rbegin =
                        contour.mPolyline.rbegin();

                    ++rbegin;

                    mPolyline.insert(mPolyline.end(),
                                     rbegin,
                                     contour.mPolyline.rend());
                    result = true;
                }
            }
        }
    }

    return result;
}// Contour::AddContour

bool
ContourPlotter::Contour::FindOuterSide(const FunctionBase& function,
                                        unsigned            variableIndexX,
                                        unsigned            variableIndexY,
                                        unsigned            variableCount,
                                        const PlotControl&  plotControl) const
{
    if (SegmentCount() < 2)     return false;   // Olewamy :)

    wxPoint a = mPolyline.front();
    wxPoint contour_vector;

    Polyline::const_iterator iNext = mPolyline.begin();
    ++iNext;
    while (iNext != mPolyline.end())
    {
        // Krtkie odcinki s pomijane i traktowane jako jeden duszy.
        if (((iNext->x - a.x)*(iNext->x - a.x) +
                (iNext->y - a.y)*(iNext->y - a.y) >= 9)
            || (iNext == mPolyline.end()))
        {
            contour_vector = *iNext - a;
            break;  //  EXIT LOOP
        }
        else    ++iNext;
    }

    ColumnVector point(variableCount);
    point = 0;
    point(variableIndexX) = plotControl.DcToPlotX(a.x);
    point(variableIndexY) = plotControl.DcToPlotY(a.y);
    ColumnVector gradient = function.Gradient(point);

    LOG(DEBUG, wxT("gradient ") << gradient.as_row());
    LOG(DEBUG, wxT("vector   ") << contour_vector.x << wxT(", ") << contour_vector.y);

    // Sprawdzamy znak iloczynu wektorowego wektora prostopadego do fragmentu
    // konturu oraz gradientu.
    double dot_perp_product = (
            -static_cast<double>(contour_vector.y) * gradient(variableIndexX)
            // Minus zamiast plusa, bo Y jest do gry nogami na wykresie
            -static_cast<double>(contour_vector.x) * gradient(variableIndexY));
    LOG(DEBUG, wxT("dot_perp_product ") << dot_perp_product);

    return dot_perp_product > 0;
}// FindOuterSide

void
ContourPlotter::Contour::ToPointArray(wxPoint points[]) const
{
    unsigned i;
    Polyline::const_iterator iPolyline;
    for (i = 0, iPolyline = mPolyline.begin();
         iPolyline != mPolyline.end();
         ++i, ++iPolyline)
    {
        points[i] = *iPolyline;
    }
}// Contour::ToPointArray

void
ContourPlotter::Contour::ShiftedToArray(wxPoint    points[],
                                        unsigned&  rCount,
                                        bool       counterClockwise) const
{
    rCount = 0;

    if (SegmentCount() < 2)     return;     // EXIT


    // Wektor prostopady do odcinka
    ColumnVector perpendicular(2);

    Polyline::const_iterator i = mPolyline.begin();
    Polyline::const_iterator iLast = mPolyline.end();
    Polyline::const_iterator iNext = i;
    --iLast;
    ++iNext;
    while (iNext != mPolyline.end())
    {
        // Krtkie odcinki s pomijane i traktowane jako jeden duszy.
        if (((iNext->x - i->x)*(iNext->x - i->x) +
                (iNext->y - i->y)*(iNext->y - i->y) >= 9)
            || (iNext == iLast))
        {
            if (counterClockwise)
            {
                perpendicular << -static_cast<double>(iNext->y - i->y)
                              <<  static_cast<double>(iNext->x - i->x);
            }
            else
            {
                perpendicular <<  static_cast<double>(iNext->y - i->y)
                              << -static_cast<double>(iNext->x - i->x);
            }

            perpendicular = perpendicular / perpendicular.norm_Frobenius()
                * static_cast<double>(OUTLINE_DISTANCE);

            points[rCount].x = lround(perpendicular(1)) + i->x;
            points[rCount].y = lround(perpendicular(2)) + i->y;
            ++rCount;
            i = iNext;
        }

        ++iNext;
    }
}// Contour::ShiftedToArray

unsigned
ContourPlotter::Contour::SegmentCount() const
{
    return mPolyline.size();
}// Contour::SegmentCount

bool
ContourPlotter::Contour::IsClosed() const
{
    return mClosed;
}// Contour::IsClosed

const /* static */ unsigned
ContourPlotter::LIGHTEST_SHADE = 230;

const /* static */ unsigned
ContourPlotter::DARKEST_SHADE = 40;

