#include <precomp.h>

#include "../methods/DerivativeIteration.h"

#include "DerivativeLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DerivativeLayer::DerivativeLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// DerivativeLayer

DerivativeLayer::DerivativeLayer(const DerivativeLayer& from)
    : UnivariateResultLayer(from)
{
}// DerivativeLayer

DerivativeLayer::~DerivativeLayer()
{
}// ~DerivativeLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
DerivativeLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
DerivativeLayer::Create(const PlotControl&          plotControl,
                        const Result&               result,
                              CountedPtr<PlotLayer> cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new DerivativeLayer(plotControl,
                                                     result,
                                                     cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
DerivativeLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const DerivativeIteration& iteration =
        dynamic_cast<const DerivativeIteration&>
            (mResult.Iteration(iterationNumber));

    double  left    = iteration.StartingPoint();
    double  right   = iteration.TrialPoints().front();

    if (left > right)
        swap(left, right);

    return Region(left, 0, right, 0);

}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
DerivativeLayer::DrawIteration(wxDC&             dc,
                               unsigned          iterationNumber,
                               bool              highlight) const
{
    const DerivativeIteration& iteration =
        dynamic_cast<const DerivativeIteration&>
            (mResult.Iteration(iterationNumber));
    AttributeDatabase::AttributeType   attribute;

    const std::vector<double>& trial_points = iteration.TrialPoints();

    if (highlight)
        attribute = AttributeDatabase::NORMAL;
    else
        attribute = AttributeDatabase::INACTIVE;

    DrawVerticalLine(dc, iteration.StartingPoint(), attribute);
    DrawVerticalLine(dc, trial_points.back(), attribute);

    if (highlight)
    {
        DrawBigPoint(dc, iteration.StartingPoint(),
                                AttributeDatabase::POINT_START);
        DrawBigPoint(dc, trial_points.back(), AttributeDatabase::POINT_BEST);

        for (size_t i = 0; i < (trial_points.size() - 1); ++i)
        {
            DrawVerticalLine(dc,
                             trial_points[i],
                             AttributeDatabase::INACTIVE);
            DrawBigPoint(dc, trial_points[i], AttributeDatabase::POINT_WORST);
        }
    }

}// DrawIteration

void
DerivativeLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const DerivativeIteration& iteration =
        dynamic_cast<const DerivativeIteration&>
            (mResult.Iteration(mHighlightedIteration));


    if (iteration.StartingPoint() < iteration.TrialPoints().front())
    {
        DrawRangeBackground(dc,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            dc.GetSize().GetWidth(),
                            AttributeDatabase::PLOT_BACKGROUND);
    }
    else
    {
        DrawRangeBackground(dc,
                            0,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            AttributeDatabase::PLOT_BACKGROUND);
    }

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* DerivativeLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            DerivativeIteration::ClassId(),
            DerivativeLayer::Create);
