#include <precomp.h>

#include "../methods/DichotomousDivisionIteration.h"

#include "DichotomousDivisionLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

DichotomousDivisionLayer::DichotomousDivisionLayer
                        (const PlotControl&             plotControl,
                         const Result&                  result,
                               CountedPtr<PlotLayer>    cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// DichotomousDivisionLayer

DichotomousDivisionLayer::DichotomousDivisionLayer
                       (const DichotomousDivisionLayer& from)
    : UnivariateResultLayer(from)
{
}// DichotomousDivisionLayer


DichotomousDivisionLayer::~DichotomousDivisionLayer()
{
}// ~DichotomousDivisionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
DichotomousDivisionLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);

}// Draw

/* static */ std::auto_ptr<ResultLayer>
DichotomousDivisionLayer::Create(const PlotControl&     plotControl,
                                 const Result&          result,
                                 CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new DichotomousDivisionLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
DichotomousDivisionLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const DichotomousDivisionIteration& iteration =
        dynamic_cast<const DichotomousDivisionIteration&>
            (mResult.Iteration(iterationNumber));

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
DichotomousDivisionLayer::DrawIteration(wxDC&             dc,
                                        unsigned          iterationNumber,
                                        bool              highlight) const
{
    const DichotomousDivisionIteration& iteration =
        dynamic_cast<const DichotomousDivisionIteration&>
            (mResult.Iteration(iterationNumber));


    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if(!highlight) return;


    AttributeDatabase::AttributeType attr_left(AttributeDatabase::DEFAULT),
                                     attr_middle(AttributeDatabase::DEFAULT),
                                     attr_right(AttributeDatabase::DEFAULT);

    if (iteration.NextMiddle() == DichotomousDivisionIteration::XL)
    {
        attr_left   = AttributeDatabase::POINT_BEST;
        attr_middle = AttributeDatabase::POINT_WORST;
    }
    else if (iteration.NextMiddle() == DichotomousDivisionIteration::XR)
    {
        attr_left   = AttributeDatabase::POINT_WORST;
        attr_middle = AttributeDatabase::POINT_MIDDLE;
        attr_right  = AttributeDatabase::POINT_BEST;
    }
    else
    {
        attr_middle = AttributeDatabase::POINT_BEST;

        if (iteration.BetterPoint() == DichotomousDivisionIteration::LEFT)
        {
            attr_left   = AttributeDatabase::POINT_MIDDLE;
            attr_right  = AttributeDatabase::POINT_WORST;
        }
        else
        {
            attr_left   = AttributeDatabase::POINT_WORST;
            attr_right  = AttributeDatabase::POINT_MIDDLE;
        }
    }

    DrawBigPoint(dc, iteration.LeftStep(), attr_left);
    DrawBigPoint(dc, iteration.MiddleStep(), attr_middle);

    if (iteration.NextMiddle() != DichotomousDivisionIteration::XL)
        DrawBigPoint(dc, iteration.RightStep(), attr_right);

}// DrawIteration

void
DichotomousDivisionLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const DichotomousDivisionIteration& iteration =
        dynamic_cast<const DichotomousDivisionIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ void* DichotomousDivisionLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            DichotomousDivisionIteration::ClassId(),
            DichotomousDivisionLayer::Create);
