#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Logger.h"

#include "DrawDeceasedAction.h"
#include "EvolutionaryMethodLayer.h"  // class implemented
#include "OnlyBestAction.h"
#include "VisualizerFactory.h"

LOGGER(DEBUG);

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

EvolutionaryMethodLayer::EvolutionaryMethodLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyResultLayer(plotControl, result, cpFunctionLayer),
      mDrawDeceased(true),
      mDrawOnlyBest(false)
{
}// EvolutionaryMethodLayer

EvolutionaryMethodLayer::EvolutionaryMethodLayer
                                        (const EvolutionaryMethodLayer& from)
    : XyResultLayer(from),
      mDrawDeceased(from.mDrawDeceased),
      mDrawOnlyBest(from.mDrawOnlyBest)
{
}// EvolutionaryMethodLayer

EvolutionaryMethodLayer::~EvolutionaryMethodLayer()
{
}// ~EvolutionaryMethodLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
EvolutionaryMethodLayer::Draw(wxDC& dc) const
{

    // Wszystko rysujemy na wykresie funkcji.
    mcpFunctionLayer->Draw(dc);

    if (mDrawOnlyBest)
    {
        for (unsigned i = mResult.FirstIterationNumber();
             i <= mResult.LastIterationNumber();
             ++i)
        {
            if (i != mHighlightedIteration)
                DrawIteration(dc, i, false);
        }
    }

    DrawIteration(dc, mHighlightedIteration, true);

    //DrawFinalResult(dc);

}// Draw

/* virtual */ vector<ToolbarTool>
EvolutionaryMethodLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(XyResultLayer::CreateTools(pPlotControl));

    ToolbarTool only_best_tool(
            auto_ptr<ToggleToolAction>
                (new OnlyBestAction(pPlotControl, this)),
            ToolbarTool::LAYER_SPECIFIC,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_SUBSEQUENT")),
            _("Only show improvements"), _("Show improvement/Show generation"));
    ToolbarTool draw_deceased_tool(
            auto_ptr<ToggleToolAction>
                (new DrawDeceasedAction(pPlotControl, this)),
            ToolbarTool::LAYER_SPECIFIC,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_DECEASED")),
            _("Draw Deceased"), _("Toggle drawing of deceased chromosomes"));

    tools.push_back(only_best_tool);
    tools.push_back(draw_deceased_tool);

    return tools;
}// CreateTools

/* static */ std::auto_ptr<ResultLayer>
EvolutionaryMethodLayer::Create(const PlotControl&          plotControl,
                                const Result&               result,
                                      CountedPtr<PlotLayer> cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new EvolutionaryMethodLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
EvolutionaryMethodLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const EvolutionaryMethodIteration& iteration =
        dynamic_cast<const EvolutionaryMethodIteration&>
            (mResult.Iteration(iterationNumber));
    const EvolutionaryMethodIteration::ChromosomeContainer&
                                    deceased    = iteration.Deceased();
    const EvolutionaryMethodIteration::ChromosomeContainer&
                                    survivors   = iteration.Survivors();
    const EvolutionaryMethodIteration::ChromosomeContainer&
                                    mutants     = iteration.Mutants();
    const EvolutionaryMethodIteration::ChromosomeContainer&
                                    children    = iteration.Children();
    //Region initial_bounding_box;
    Region final_bounding_box;

    if (mDrawOnlyBest)
    {
        final_bounding_box.Extend((iteration.StartingBest())(mVariableXIndex),
                                  (iteration.StartingBest())(mVariableYIndex));
        final_bounding_box.Extend((iteration.EndBest())(mVariableXIndex),
                                  (iteration.EndBest())(mVariableYIndex));

        return final_bounding_box;
    }

    unsigned number_of_points = deceased.size() +
                                survivors.size() +
                                mutants.size() +
                                children.size();

    EvolutionaryMethodIteration::ChromosomeContainer::const_iterator iter;

    //vector<EvolutionaryMethodLayer::PointInRegion>                  points;
    //vector<EvolutionaryMethodLayer::PointInRegion>::const_iterator  pir_iter;

    //points.reserve(number_of_points);
    if (mDrawDeceased)
    {
        for (iter = deceased.begin();
             iter != deceased.end();
             ++iter)
        {
            final_bounding_box.Extend((*iter)(mVariableXIndex),
                                      (*iter)(mVariableYIndex));
            //points.push_back(PointInRegion(*iter));
        }
    }
    for (iter = survivors.begin();
         iter != survivors.end();
         ++iter)
    {
        final_bounding_box.Extend((*iter)(mVariableXIndex),
                                  (*iter)(mVariableYIndex));
        //points.push_back(PointInRegion(*iter));
    }
    for (iter = mutants.begin();
         iter != mutants.end();
         ++iter)
    {
        final_bounding_box.Extend((*iter)(mVariableXIndex),
                                  (*iter)(mVariableYIndex));
        //points.push_back(PointInRegion(*iter));
    }
    for (iter = children.begin();
         iter != children.end();
         ++iter)
    {
        final_bounding_box.Extend((*iter)(mVariableXIndex),
                                  (*iter)(mVariableYIndex));
        //points.push_back(PointInRegion(*iter));
    }
/*
    //TODO: W tym miejscu trzeba rozpoznawa punkty, ktre przeszy na tyle
    //radykaln mutacj, e za bardzo powiksz ramk - np. takie, ktrych
    //odlego od ramki bez nich jest wiksza kilkukrotnie od szerokoci ramki
    //bez nich.
    for (pir_iter = points.begin();
         pir_iter != points.end();
         ++pir_iter)
    {
        initial_bounding_box.Extend((pir_iter->mPoint)(mVariableXIndex),
                                    (pir_iter->mPoint)(mVariableYIndex));
    }

    RemoveOutliers(initial_bounding_box, points);

    for (pir_iter = points.begin();
         pir_iter != points.end();
         ++pir_iter)
    {
        final_bounding_box.Extend((pir_iter->mPoint)(mVariableXIndex),
                                  (pir_iter->mPoint)(mVariableYIndex));
    }
*/
    return final_bounding_box;

}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
/////////////////////////////// PRIVATE    ///////////////////////////////////

void
EvolutionaryMethodLayer::DrawIteration(wxDC&        dc,
                                       unsigned     iterationNumber,
                                       bool         highlight) const
{

    const EvolutionaryMethodIteration& iteration =
        dynamic_cast<const EvolutionaryMethodIteration&>
            (mResult.Iteration(iterationNumber));

    if (mDrawOnlyBest)
        DrawImprovementLine(dc, iteration, highlight);

    if (!mDrawOnlyBest)
    {
        if (mDrawDeceased)
            DrawDeceased(dc, iteration);
        DrawSurvivors(dc, iteration);
        DrawMutants(dc, iteration);
        DrawChildren(dc, iteration);
    }

    DrawStartingBest(dc, iteration, highlight);
    DrawEndBest(dc, iteration, highlight);

}// DrawIteration

void
EvolutionaryMethodLayer::DrawDeceased
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const
{
    DrawChromosomeContainer(dc,
                            iteration.Deceased(),
                            AttributeDatabase::CH_DECEASED);
}

void
EvolutionaryMethodLayer::DrawSurvivors
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const
{
    DrawChromosomeContainer(dc,
                            iteration.Survivors(),
                            AttributeDatabase::CH_SURVIVOR);
}

void
EvolutionaryMethodLayer::DrawMutants
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const
{
    DrawChromosomeContainer(dc,
                            iteration.Mutants(),
                            AttributeDatabase::CH_MUTANT);
}

void
EvolutionaryMethodLayer::DrawChildren
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration) const
{
    DrawChromosomeContainer(dc,
                            iteration.Children(),
                            AttributeDatabase::CH_CHILD);
}

void
EvolutionaryMethodLayer::DrawStartingBest
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const
{
    AttributeDatabase::AttributeType    attribute;

    attribute = highlight ? AttributeDatabase::CH_STARTING_BEST :
                            AttributeDatabase::NORMAL;

    DrawBigPoint(dc,
                 iteration.StartingBest(),
                 attribute);
}

void
EvolutionaryMethodLayer::DrawEndBest
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const
{
    AttributeDatabase::AttributeType    attribute;

    attribute = highlight ? AttributeDatabase::CH_END_BEST :
                            AttributeDatabase::NORMAL;

    DrawBigPoint(dc, iteration.EndBest(), attribute);
}


void
EvolutionaryMethodLayer::DrawImprovementLine
                        (      wxDC&                        dc,
                         const EvolutionaryMethodIteration& iteration,
                               bool                         highlight) const
{
    AttributeDatabase::AttributeType    attribute;

    attribute = highlight ? AttributeDatabase::CH_STARTING_BEST :
                            AttributeDatabase::NORMAL;

    DrawLine(dc,
             iteration.StartingBest(),
             iteration.EndBest(),
             attribute);
}

void
EvolutionaryMethodLayer::DrawChromosomeContainer(wxDC&              dc,
            const EvolutionaryMethodIteration::ChromosomeContainer& container,
                  AttributeDatabase::AttributeType                  attribute)
                                                                        const
{
    PointList   points_to_draw;

    EvolutionaryMethodIteration::ChromosomeContainer::const_iterator    iter;

    for (iter = container.begin(); iter != container.end(); ++iter)
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(*iter),
                                                 attribute,
                                                 SMALL_CIRCLE));

    for_each(points_to_draw.begin(),
             points_to_draw.end(),
             PointPainter(*this, dc));
}

/* Nie kasuj tej funkcji, bo moe si przyda.
void
EvolutionaryMethodLayer::RemoveOutliers(const Region&           boundingBox,
                                        vector<PointInRegion>&  points) const
{

    double x1 = boundingBox.X1();
    double x2 = boundingBox.X2();
    double y1 = boundingBox.Y1();
    double y2 = boundingBox.Y2();
    double width = x2 - x1;
    double height = y1 - y2;

    const int chromosome_count = points.size();
    int i;
    int empty_count_x; //w ilu obszarach nie znalazy si adne punkty (wym. x)
    int empty_count_y; //w ilu obszarach nie znalazy si adne punkty (wym. y)
    int resolution = 10; //na ile podobszarw dzielimy boundingBox w 1 wymiarze
    int histogram_x[resolution];
    int histogram_y[resolution];

    bool remove_outliers_x = false;
    bool remove_outliers_y = false;

    std::vector<EvolutionaryMethodLayer::PointInRegion>::iterator  iter;

    for (iter = points.begin();
         iter != points.end();
         ++iter)
    {
        double  relative_placement_x =
                            ((iter->mPoint)(mVariableXIndex) - x1) / width;
        double  relative_placement_y =
                            ((iter->mPoint)(mVariableYIndex) - y2) / height;

        //Obszar dzielimy na liczb obszarw rwna resolution i zaokrglamy
        //wzgldne pooenie punktu tak by mwio, do ktrej czci obsaru
        //punkt naley.
        relative_placement_x = floor(relative_placement_x * resolution);
        relative_placement_y = floor(relative_placement_y * resolution);

        //W tabeli zapamitujemy, do ktrej czci obszaru nalea i-ty w
        //kolejnoci punkt.
        iter->mBoxPartX = (int)relative_placement_x;
        iter->mBoxPartY = (int)relative_placement_y;

        //Powikszamy o 1 liczb w komrce tabeli reprezentujcej cz obszaru,
        //do ktrej nalea punkt.
        histogram_x[iter->mBoxPartX] += 1;
        histogram_y[iter->mBoxPartY] += 1;
    }

    for (i = 0; i < resolution; ++i)
    {
        if (histogram_x[i] == 0)    empty_count_x++;
        if (histogram_y[i] == 0)    empty_count_y++;
    }

    //Jeli wicej ni poowa obszarw jest pusta, naley usun
    //punkty odstajce
    if (empty_count_x > resolution / 2)
        remove_outliers_x = true;
    if (empty_count_y > resolution / 2)
        remove_outliers_y = true;

    for (iter = points.begin();
         iter != points.end();
         ++iter)
    {
        bool already_removed = false;

        if (remove_outliers_x)
        {
            if (histogram_x[iter->mBoxPartX] <= chromosome_count / 50)
            {
                *iter = points.back();
                points.erase(points.end() - 1);
                already_removed = true;
            }
        }

        if (remove_outliers_y && !already_removed)
        {
            if (histogram_y[iter->mBoxPartY] <= chromosome_count / 50)
            {
                *iter = points.back();
                points.erase(points.end() - 1);
            }
        }
    }
}
*/

const /* static */ void* EvolutionaryMethodLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            EvolutionaryMethodIteration::ClassId(),
            EvolutionaryMethodLayer::Create);
