#include <precomp.h>

#include <wx/xrc/xmlres.h>

#include "../Logger.h"
#include "../ProblemBase.h"
#include "AttributeDatabase.h"
#include "FitIterationAction.h"
#include "FitResultAction.h"
#include "FullPlotVisualizer.h"  // class implemented
#include "MultivariateFunctionLayer.h"
#include "MultivariatePlot.h"
#include "ToolbarTool.h"
#include "UnivariateFunctionLayer.h"
#include "UnivariatePlot.h"


BEGIN_EVENT_TABLE(FullPlotVisualizer, Visualizer)
    EVT_BUTTON(ID_BUTTON_NEXT,      FullPlotVisualizer::OnNext)
    EVT_BUTTON(ID_BUTTON_PREVIOUS,  FullPlotVisualizer::OnPrevious)
    EVT_LISTBOX(ID_LIST_ITERATIONS, FullPlotVisualizer::OnIterationSelect)
END_EVENT_TABLE()

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FullPlotVisualizer::FullPlotVisualizer(
        wxWindow*                                   pParent,
        const Result&                               result,
        const ResultLayer::Creator                  resultLayerCreator,
        auto_ptr<const TextIterationVisualizer>     apTextIterationVisualizer)
    : Visualizer(pParent, result),
      mapTextIterationVisualizer(apTextIterationVisualizer),
      mFitIteration(false)
{
    wxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);

    // ---- Tworzenie splitterw ----
    // Panele s potrzebne, aby uy sizerw i eby splittery ustawiy sobie
    // adnie minimalne rozmiary na podstawie zawartych w nich kontrolek.
    mpTopBottomSplitter = new wxSplitterWindow(this, wxID_ANY,
            wxDefaultPosition, wxDefaultSize, 0);

    wxPanel* p_top_panel = new wxPanel(mpTopBottomSplitter, wxID_ANY);
    mpPlotListSplitter  = new wxSplitterWindow(p_top_panel, wxID_ANY,
            wxDefaultPosition, wxDefaultSize, 0);

    wxPanel* p_bottom_panel = new wxPanel(mpTopBottomSplitter, wxID_ANY);
    mpStatusTextSplitter= new wxSplitterWindow(p_bottom_panel, wxID_ANY,
            wxDefaultPosition, wxDefaultSize, 0);

    // ---- Tworzenie okienek ----
    mpListIterations    = new wxListBox(mpPlotListSplitter, ID_LIST_ITERATIONS);

    // Status wykresu i kontrolki iteracji
    wxPanel* p_status_and_iteration_control_panel =
        new wxPanel(mpStatusTextSplitter, wxID_ANY);
    mpPlotStatus        = new PlotStatus(p_status_and_iteration_control_panel,
            wxID_ANY, mResult.Problem().Variables().GetCount() == 1);
    mpStaticIteration   = new wxStaticText(p_status_and_iteration_control_panel,
            wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize,
            wxALIGN_CENTRE | wxST_NO_AUTORESIZE);
    SetUpStatusAndIterationControlPanel(p_status_and_iteration_control_panel,
                                        mpPlotStatus,
                                        mpStaticIteration);
    wxSize status_panel_size =
        p_status_and_iteration_control_panel->GetSize();

    // Tworzy kontrolk wykresu, pasek narzdzi i warstw wykresu.
    wxPanel* p_plot_panel =
        SetUpPlotPanel(mpPlotListSplitter, resultLayerCreator);

    // Okienko z tekstowym opisem iteracji
    mpTextIteration     = new wxTextCtrl(mpStatusTextSplitter, wxID_ANY,
            wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE |
            wxTE_READONLY | wxTE_RICH2 | wxTE_DONTWRAP);

    // ---- Podzia i ustawienia splitterw ----
    mpPlotListSplitter->SplitVertically(p_plot_panel,
                                        mpListIterations,
                                        -mIterationListWidth);
    mpPlotListSplitter->SetMinimumPaneSize(mIterationListWidth);
    mpPlotListSplitter->SetSashGravity(1.0);

    mpStatusTextSplitter->SplitVertically(mpTextIteration,
                                          p_status_and_iteration_control_panel,
                                          status_panel_size.GetWidth());
    mpStatusTextSplitter->SetMinimumPaneSize(-status_panel_size.GetWidth());
    mpStatusTextSplitter->SetSashGravity(1.0);

    // Wstaw grny i dolny splitter do sizerw i paneli
    wxSizer* p_top_sizer = new wxBoxSizer(wxVERTICAL);
    wxSizer* p_bottom_sizer = new wxBoxSizer(wxVERTICAL);

    p_top_sizer->Add(mpPlotListSplitter, 1, wxEXPAND);
    p_bottom_sizer->Add(mpStatusTextSplitter, 1, wxEXPAND);

    p_top_panel->SetSizer(p_top_sizer);
    p_top_sizer->SetSizeHints(p_top_panel);
    p_bottom_panel->SetSizer(p_bottom_sizer);
    p_bottom_sizer->SetSizeHints(p_bottom_panel);

    mpTopBottomSplitter->SplitHorizontally(p_top_panel,
                                           p_bottom_panel,
                                           -status_panel_size.GetHeight());
    mpTopBottomSplitter->SetMinimumPaneSize(1);
    mpTopBottomSplitter->SetSashGravity(1.0);

    // ---- Wypenienie listy iteracji ----
    FillIterationList(mpListIterations, mResult);

    // ---- Ustawienie stylu podstawowego w okienku tekstowym ----
    mpTextIteration->SetDefaultStyle(AttributeDatabase::GetInstance().
            TextAttribute(AttributeDatabase::DEFAULT));

    // ---- Dodanie gwnego splittera i sizera do okna ----
    p_main_sizer->Add(mpTopBottomSplitter, 1, wxEXPAND);
    SetSizer(p_main_sizer);
    p_main_sizer->SetSizeHints(this);

    // ---- Zaznaczenie pierwszej iteracji i ustawienie widoku ----
    SelectIteration(0, true);

    wxCommandEvent event(PLOT_FITVIEW_REQUEST_EVENT, 0);
    mpPlotControl->AddPendingEvent(event);
}// FullPlotVisualizer

FullPlotVisualizer::~FullPlotVisualizer()
{
}// ~FullPlotVisualizer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

void
FullPlotVisualizer::OnIterationSelect(wxCommandEvent& event)
{
    if (event.IsSelection())
    {
        // SelectIteration samo czyci kontrolk tekstow.
        SelectIteration(event.GetSelection(), false);
    }
    else mpTextIteration->Clear();
}// OnIterationSelect

void
FullPlotVisualizer::OnPrevious(wxCommandEvent& event)
{
    unsigned selectionIndex = mpListIterations->GetSelection();

    assert (selectionIndex > 0);

    SelectIteration(selectionIndex - 1, true);
}// OnPrevious

void
FullPlotVisualizer::OnNext(wxCommandEvent& event)
{
    unsigned selectionIndex = mpListIterations->GetSelection();

    assert (selectionIndex <
            mResult.LastIterationNumber() - mResult.FirstIterationNumber());

    SelectIteration(selectionIndex + 1, true);
}// OnNext

void
FullPlotVisualizer::SetFitIteration(bool toggle)
{
    mFitIteration = toggle;
    if (mFitIteration)  FitIteration();
}// SetFitIteration


//============================= ACCESS     ===================================

/* virtual */ wxString
FullPlotVisualizer::Name() const
{
    return wxGetTranslation(mName);
}// Name

//============================= INQUIRY    ===================================

bool
FullPlotVisualizer::IsFitIteration() const
{
    return mFitIteration;
}// IsFitIteration

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
FullPlotVisualizer::SelectIteration(unsigned selectionIndex, bool select)
{
    unsigned iteration_number =
        mResult.FirstIterationNumber() + selectionIndex;

    mpTextIteration->Clear();
    mpTextIteration->SetDefaultStyle(AttributeDatabase::GetInstance().
            TextAttribute(AttributeDatabase::DEFAULT));
    if (mapTextIterationVisualizer.get() != NULL)
    {
        mapTextIterationVisualizer->Print(mpTextIteration,
                mResult.Iteration(iteration_number),
                mResult.Problem().BaseFunction(),
                true);
    }

    mpStaticIteration->SetLabel(wxString::Format(_("Iteration: %u"),
                iteration_number));

    // Wcz/wycz przyciski
    mpButtonNext->Enable(
            iteration_number != mResult.LastIterationNumber());
    mpButtonPrevious->Enable(
            iteration_number != mResult.FirstIterationNumber());

    mcpResultPlotLayer->Highlight(iteration_number);

    if (mFitIteration)      FitIteration();
    else
    {   // Przesu, jeli wystaje poza obszar
        mpPlotControl->AdjustView(
                mcpResultPlotLayer->IterationBoundingBox(iteration_number));
    }
    mpPlotControl->RefreshPlot();

    if (select) mpListIterations->SetSelection(selectionIndex);
}// SelectIteration

void
FullPlotVisualizer::FitIteration()
{
    mpPlotControl->SetView(mcpResultPlotLayer->IterationBoundingBox(
                mcpResultPlotLayer->HighlightedIterationNumber()));
}// FitIteration

void
FullPlotVisualizer::FillIterationList(wxListBox* pListBox,
                                    const Result& result) const
{
    pListBox->Clear();

    const Result::IterationContainer& iterations = result.Iterations();
    unsigned number = result.FirstIterationNumber();
    for (Result::IterationContainer::const_iterator i = iterations.begin();
         i != iterations.end();
         ++i, ++number)
    {
        pListBox->Append(wxString::Format(wxT("%3u"), number));
    }
}// FillIterationList

void
FullPlotVisualizer::SetUpStatusAndIterationControlPanel(
        wxPanel*        pPanel,
        PlotStatus*     pPlotStatus,
        wxStaticText*   pStaticIteration)
{
    wxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);

    wxSizer* p_iteration_control_sizer = new wxBoxSizer(wxVERTICAL);
    wxSizer* p_buttons_sizer = new wxBoxSizer(wxHORIZONTAL);

    mpButtonPrevious = new wxButton(pPanel, ID_BUTTON_PREVIOUS, wxT("<-"));
    mpButtonNext = new wxButton(pPanel, ID_BUTTON_NEXT, wxT("->"));

    // Wsad kontrolki iteracji do sajzerw
    p_buttons_sizer->Add(mpButtonPrevious, 1, wxALL, 2);
    p_buttons_sizer->Add(mpButtonNext, 1, wxALL, 2);

    p_iteration_control_sizer->Add(pStaticIteration, 1, wxEXPAND | wxALL, 2);

    p_iteration_control_sizer->Add(p_buttons_sizer, 1, wxEXPAND);

    // Wsad wszystko do sajzera gwnego
    p_main_sizer->Add(pPlotStatus, 0, wxEXPAND | wxALIGN_TOP);
    p_main_sizer->AddStretchSpacer();
    p_main_sizer->Add(new wxStaticLine(pPanel, wxID_ANY), 0, wxEXPAND);
    p_main_sizer->AddStretchSpacer();
    p_main_sizer->Add(p_iteration_control_sizer, 0, wxEXPAND | wxALIGN_BOTTOM);

    pPanel->SetSizer(p_main_sizer);
    p_main_sizer->SetSizeHints(pPanel);
}// SetUpStatusAndIterationControlPanel


wxPanel*
FullPlotVisualizer::SetUpPlotPanel(wxWindow*            pParent,
                                   ResultLayer::Creator resultLayerCreator)
{
    wxPanel* p_panel = new wxPanel(pParent);
    wxSizer* p_sizer = new wxBoxSizer(wxVERTICAL);

    PlotToolbar* p_toolbar;

    if (mResult.Problem().Variables().GetCount() == 1)
    {
        const UnivariateFunction& univariate_function =
                dynamic_cast<const UnivariateFunction&>
                        (mResult.Problem().BaseFunction());

        // TODO: przenie ustawianie rozmiaru do PlotControl
        UnivariatePlot* p_univariate_plot =
            new UnivariatePlot(p_panel, wxID_ANY,
                    univariate_function, mpPlotStatus,
                    mResult.Problem().Variables()[0],
                    wxDefaultPosition, wxSize(400, 300));

        LOG(DEBUG, wxT("Plot height: ") << p_univariate_plot->GetBestSize().GetHeight());

        mcpFunctionPlotLayer.Reset(
                new UnivariateFunctionLayer(*p_univariate_plot,
                    univariate_function));
        mcpResultPlotLayer = resultLayerCreator(*p_univariate_plot,
                                                mResult,
                                                mcpFunctionPlotLayer);

        p_univariate_plot->AppendLayer(mcpResultPlotLayer);

        PlotToolbar* p_univariate_toolbar =
            new PlotToolbar(p_panel, wxID_ANY, p_univariate_plot);

        mpPlotControl = p_univariate_plot;
        p_toolbar = p_univariate_toolbar;
    }
    else
    {
        MultivariatePlot* p_multivariate_plot =
            new MultivariatePlot(p_panel, wxID_ANY,
                mResult.Problem().BaseFunction(), mResult.Problem().Variables(),
                mpPlotStatus, wxDefaultPosition, wxSize(400, 300));

        mcpFunctionPlotLayer.Reset(
                new MultivariateFunctionLayer(*p_multivariate_plot,
                    mResult.Problem().BaseFunction()));
        mcpResultPlotLayer = resultLayerCreator(*p_multivariate_plot,
                                                mResult,
                                                mcpFunctionPlotLayer);

        p_multivariate_plot->AppendLayer(mcpResultPlotLayer);

        PlotToolbar* p_multivariate_toolbar =
            new PlotToolbar(p_panel, wxID_ANY, p_multivariate_plot);

        mpPlotControl = p_multivariate_plot;
        p_toolbar = p_multivariate_toolbar;
    }

    SetUpToolbarTools(p_toolbar);

    // TODO: moe da si jako adniej?
    // Obejcie problemu z przyciskami zachodzcymi na wykres. Gruboci
    // marginesw dobrane eksperymentalnie.
#ifdef __WXMSW__
    p_sizer->Add(p_toolbar, 0, wxEXPAND | wxBOTTOM, 3);
#else
    p_sizer->Add(p_toolbar, 0, wxEXPAND);
#endif

    p_sizer->Add(mpPlotControl, 1, wxEXPAND);

    p_panel->SetSizer(p_sizer);
    p_sizer->SetSizeHints(p_panel);

    return p_panel;
}// SetUpPlotPanel

void
FullPlotVisualizer::SetUpToolbarTools(PlotToolbar *pToolbar)
{
    // ---- Narzdzia wsplne dla wszystkich wykresw ----

    // Dodaj przycisk: "dopasuj do wyniku"
    ToolbarTool fit_result_tool(
            auto_ptr<ClickToolAction>(new FitResultAction(mpPlotControl)),
            ToolbarTool::SET_VIEW,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_FIT")),
            _("Fit"), _("Fit\nFit view to whole result"));

    // Dodaj przycisk: "dopasuj do iteracji"
    ToolbarTool fit_iteration_tool(
            auto_ptr<ToggleToolAction>(new FitIterationAction(this)),
            ToolbarTool::SET_VIEW,
            wxXmlResource::Get()->LoadBitmap(wxT("TOOL_FIT_ITERATION")),
            _("Fit Iteration"),
            _("Fit Iteration\nAutomatically zoom to selected iteration"));

    pToolbar->AddCustomTool(fit_result_tool);
    pToolbar->AddCustomTool(fit_iteration_tool);

    vector<ToolbarTool>::const_iterator i;
    vector<ToolbarTool> tools = mcpResultPlotLayer->CreateTools(mpPlotControl);
    for (i = tools.begin(); i != tools.end(); ++i)
    {
        pToolbar->AddCustomTool(*i);
    }

    pToolbar->Realize();
}/// SetUpToolbarTools

const /* static */ wxString FullPlotVisualizer::mName(_("Plot"));
const /* static */ unsigned FullPlotVisualizer::mIterationListWidth = 50;
const /* static */ unsigned FullPlotVisualizer::mTextIterationDefaultHeight = 150;
