#include <precomp.h>

#include "../Logger.h"
#include "../ParseError.h"
#include "../ProblemBase.h"

#include "FunctionValidator.h"  // class implemented
#include "ParseErrorDialog.h"

LOGGER(INFO);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

FunctionValidator::FunctionValidator(
        CountedPtr<const Parser>            cpParser,
        CountedPtr<const FunctionBase>&     rcpFunction,
        CountedPtr<const VariableNames>&    rcpVariables,
        const ProblemBase*                  pProblem,
        bool                                resetVariables,
        History&                            rHistory,
        FunctionValidator::Type             type)
    : mcpParser(cpParser),
      mrcpFunction(rcpFunction),
      mrcpVariables(rcpVariables),
      mpProblem(pProblem),
      mResetVariables(resetVariables),
      mType(type),
      mrHistory(rHistory)
{
}// FunctionValidator

FunctionValidator::FunctionValidator(const FunctionValidator& from)
    : mcpParser(from.mcpParser),
      mrcpFunction(from.mrcpFunction),
      mrcpVariables(from.mrcpVariables),
      mpProblem(from.mpProblem),
      mResetVariables(from.mResetVariables),
      mType(from.mType),
      mrHistory(from.mrHistory)
{
}// FunctionValidator

FunctionValidator::~FunctionValidator()
{
}// ~FunctionValidator


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ wxObject*
FunctionValidator::Clone() const
{
    return new FunctionValidator(*this);
}// Clone

/* virtual */ bool
FunctionValidator::Validate(wxWindow* parent)
{
    wxString error_message;
    bool incorrect_input = false;

    try
    {
        const wxWindow* p_control = GetWindow();
        wxString value;

        // Nieporadne wykrywanie typu kontrolki
        if (dynamic_cast<const wxComboBox *>(p_control) != NULL)
        {
            value = dynamic_cast<const wxComboBox *>(p_control)->GetValue();
        }
        else if (dynamic_cast<const wxTextCtrl *>(p_control) != NULL)
        {
            value = dynamic_cast<const wxTextCtrl *>(p_control)->GetValue();
        } else {
            // Moe dziaa tylko z wxTextCtrl lub wxComboBox
            assert(false);
        }

        if (value.Trim().IsEmpty())
        {
            error_message = _("Please enter a function");
            incorrect_input = true;
        }
        else
        {
            // Ustaw/zainicjuj zmienne
            if ((!mResetVariables) && (mrcpVariables.Get() != NULL))
            {
                mapVariablesCache.reset(new VariableNames(*mrcpVariables));
            } else {
                mapVariablesCache.reset(new VariableNames);
            }

            // Zapamietaj wynik, pewnie sie przyda do TransferFromWindow
            mapFunctionCache = mcpParser->CreateFunction(value,
                    *mapVariablesCache,
                    mResetVariables);

            // Sprawd liczb zmiennych
            if ((mType == ALLOW_UNIVARIATE)
                    && (mapVariablesCache->GetCount() < 1))
            {
                error_message = _("The function must have at least 1 variable");
                incorrect_input = true;
            }
            else if ((mType == MULTIVARIATE_ONLY)
                    && (mapVariablesCache->GetCount() < 2))
            {
                error_message =
                    _("The function must have at least 2 variables");
                incorrect_input = true;
            }
            else if ((mType == CONSTRAINT)
                    && (mapFunctionCache->PresentVariables().GetCount() < 1))
            {
                error_message =
                    _("The constraint must have at least 1 variable");
                incorrect_input = true;
            }
            else
            {
                // Ustaw wskanik na problem
                mapFunctionCache->SetProblem(mpProblem);
            }
        }// else (przypadek, gdy wprowadzono funkcj)

        if (incorrect_input)
        {
            mapFunctionCache.reset(NULL);

            wxMessageDialog message_box(parent, error_message,
                    _("Invalid function"), wxOK | wxICON_ERROR);
            message_box.ShowModal();
            GetWindow()->SetFocus();

            return false;
        }
    }
    catch (const ParseError& error)
    {
        // TODO: porzdne okienko z porzdnym komunikatem i wskazaniem miejsca
        // bdu
        LOG(DEBUG, error.What());

        ParseErrorDialog error_dialog(parent, wxID_ANY, error);
        error_dialog.ShowModal();
        GetWindow()->SetFocus();

        return false;
    }


    return true;
}// Validate

/* virtual */ bool
FunctionValidator::TransferToWindow()
{
    LOG(DEBUG, wxT("TransferToWindow"));

    wxWindow* p_control = GetWindow();

    wxComboBox *p_combo = dynamic_cast<wxComboBox *>(p_control);

    if (p_combo != NULL)
    {
        p_combo->Clear();
        p_combo->Append(mrHistory.Strings());
    }

    if (mrcpFunction.Get() != NULL) {
        LOG(DEBUG, wxT("Function is not null: ") << mrcpFunction->ToString(false));
        // Nieporadne wykrywanie typu kontrolki
        if (p_combo != NULL)
        {
            LOG(DEBUG, wxT("combo: ") << mrcpFunction->ToString(false));
            p_combo->SetValue(mrcpFunction->ToString(false));
        }
        else if (dynamic_cast<wxTextCtrl *>(p_control) != NULL)
        {
            LOG(DEBUG, wxT("textctrl: ") << mrcpFunction->ToString(false));
            dynamic_cast<wxTextCtrl *>(p_control)->SetValue(
                    mrcpFunction->ToString(false));
        } else {
            // Moe dziaa tylko z wxTextCtrl lub wxComboBox
            assert(true);
        }
    }
    else
    {   // Obejcie brzydkiej przypadoci w wxGTK (wypenia pole tekstowe
        // ComboBoxu jednym z elementw listy) poprzez wyzerowanie napisu.

        // TODO: unikn powtarzania tego fragmentu
        if (p_combo != NULL)
        {
            p_combo->SetValue(wxT(""));
        }
        else if (dynamic_cast<wxTextCtrl *>(p_control) != NULL)
        {
            dynamic_cast<wxTextCtrl *>(p_control)->SetValue(
                    wxT(""));
        } else {
            // Moe dziaa tylko z wxTextCtrl lub wxComboBox
            assert(true);
        }
    }

    return true;
}// TransferToWindow

/* virtual */ bool
FunctionValidator::TransferFromWindow()
{
    // Wczeniej powinna by udana walidacja
    assert(mapFunctionCache.get() != NULL);

    // Przenie zmienne i funkcj na zewntrz
    if (mResetVariables)
    {
        mrcpVariables = CountedPtr<const VariableNames>(mapVariablesCache);
    }

    mrcpFunction = auto_ptr<const FunctionBase>(mapFunctionCache);

    LOG(INFO, wxT("TransferFromWindow: ") << mrcpFunction->ToString(false));

    wxComboBox* p_combo = dynamic_cast<wxComboBox *>(GetWindow());
    if (p_combo != NULL)
    {
        mrHistory.InsertUserString(mrcpFunction->ToString(false));
        p_combo->Clear();
        // wxMSW czyci ca kontrolk, wic trzeba string doda ponownie.
        p_combo->SetValue(mrcpFunction->ToString(false));
        p_combo->Append(mrHistory.Strings());
    }

    return true;
}// TransferFromWindow

//============================= ACCESS     ===================================

const VariableNames&
FunctionValidator::PeekTemporaryVariables() const
{
    return *mapVariablesCache;
}// PeekTemporaryVariables

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
