/** wxValidator dla funkcji matematycznych.
 */

#ifndef _FunctionValidator_h
#define _FunctionValidator_h

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

// LOCAL INCLUDES
//
#include "../CountedPtr.h"
#include "../FunctionBase.h"
#include "../History.h"
#include "../Parser.h"
#include "../VariableNames.h"

// FORWARD REFERENCES
//

class FunctionValidator
    : public wxValidator
{
public:
    /// Typ wprowadzanej funkcji
    enum Type {
        /// Pozwalaj na wprowadzanie f. jednej (i wicej) zmiennej.
        ALLOW_UNIVARIATE,
        MULTIVARIATE_ONLY,  ///< Tylko wielu zmiennych (przynajmniej 2).
        CONSTRAINT          ///< Ograniczenie (przynajmniej 1 zmienna).
    };

// LIFECYCLE

    /** Konstruktor.
     *
     * @param cpParser <code>CountedPtr</code> na parser funkcji.
     * @param rcpFunction Referncja na <code>CountedPtr</code> na funkcj, ktra
     * bdzie utworzona w trakcie parsowania.
     * @param rcpVariables Referencja na <code>CountedPtr</code> na zmienne,
     * ktre bd utworzone/wykorzystane w trakcie parsowania (patrz
     * <code>resetVariables</code>).
     * @param pProblem Wskanik na problem, ktrego funkcja i zmienne maj by
     * ustawione przez walidator.
     * @param resetVariables Czy zmienne maja byc tworzone na podstawie funkcji,
     * czy wykorzystywa te zastane w <code>rcpVariables</code>.
     * @param rHistory Referencja na histori funkcji w kontrolce (jeli to
     * wxComboBox).
     * @param type Rodzaj funkcji zatwierdzanej przez walidator.
     */
    FunctionValidator(CountedPtr<const Parser>          cpParser,
                      CountedPtr<const FunctionBase>&   rcpFunction,
                      CountedPtr<const VariableNames>&  rcpVariables,
                      const ProblemBase*                pProblem,
                      bool                              resetVariables,
                      History&                          rHistory,
                      Type                              type);


    /** Konstruktor kopiujcy.
     *
     * @param from Obiekt, ktrego warto jest kopiowana.
     */
    FunctionValidator(const FunctionValidator& from);


    /** Destruktor.
     */
    ~FunctionValidator(void);


// OPERATORS

// OPERATIONS

    /** Tworzy kopi walidatora.
     */
    virtual wxObject*       Clone() const;

    /** Sprawdza poprawnosc wprowadzonej funkcji.
     *
     * Funkcja jest parsowana, jesli nie wystapia bledy, to zwracane jest
     * <code>true</code>. Wynik parsowania jest zapamietywany.
     */
    virtual bool            Validate(wxWindow* parent);

    /** Nic nie robi.
     */
    virtual bool            TransferToWindow();

    /** Przekazuje dane z kontrolki do skojarzonej z walidatorem funkcji i
     * zmiennych.
     *
     * Wymaga wczesniejszego wywolania Validate.
     */
    virtual bool            TransferFromWindow();

// ACCESS

    /** Zwraca zmienne scache'owane po walidacji.
     *
     * Wywoywa TYLKO po Validate.
     *
     * Poniewa wywoanie TransferFromWindow zmienia zadanie optymalizacji,
     * dla ktrego jest funkcja, to nie mona sobie pozwoli na
     * przetransferowanie funkcji i zmiennych do obiektw tymczasowych.
     * A e nie ma czasu tego przerobi, to potrzebna jest ta metoda.
     */
    const VariableNames&    PeekTemporaryVariables() const;

// INQUIRY

protected:

private:
    CountedPtr<const Parser>            mcpParser;      ///< Stosowany parser.
    /// Referencja na funkcj.
    CountedPtr<const FunctionBase>&     mrcpFunction;
    /// Referencja na zmienne.
    CountedPtr<const VariableNames>&    mrcpVariables;
    /// Problem, ktry jest ustawiany w funkcjach.
    const ProblemBase*              mpProblem;
    bool                            mResetVariables;///< Czy czyci zmienne.
    Type                            mType;          ///< Typ funkcji.

    History&                        mrHistory;      ///< Referencja na histori.

    /// Tymczasowe zmienne, przechowywane do czasu pobrania (TransferFromWindow)
    std::auto_ptr<VariableNames>    mapVariablesCache;
    /// Tymczasowa funkcja, przechowywana do czasu pobrania (TransferFromWindow)
    std::auto_ptr<FunctionBase>     mapFunctionCache;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

#endif  // _FunctionValidator_h
