#include <precomp.h>

#include "../methods/MarquardtIteration.h"

#include "MarquardtLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

MarquardtLayer::MarquardtLayer(
        const PlotControl&            plotControl,
        const Result&                 result,
              CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// MarquardtLayer

MarquardtLayer::MarquardtLayer(const MarquardtLayer& from)
    : XyUnconstrainedLayer(from)
{
}// MarquardtLayer

MarquardtLayer::~MarquardtLayer()
{
}// ~MarquardtLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
MarquardtLayer::Create(const PlotControl&           plotControl,
                       const Result&                result,
                             CountedPtr<PlotLayer>  cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new MarquardtLayer(plotControl,
                                                    result,
                                                    cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
MarquardtLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const MarquardtIteration& iteration =
        dynamic_cast<const MarquardtIteration&>
            (mResult.Iteration(iterationNumber));

    Region  bounding_box;

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));
    bounding_box.Extend(iteration.TrialPoints().front()(mVariableXIndex),
                        iteration.TrialPoints().front()(mVariableYIndex));

    return bounding_box;

}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/* virtual */ void
MarquardtLayer::DrawIteration(wxDC&             dc,
                              unsigned          iterationNumber,
                              bool              highlight) const
{
    const MarquardtIteration& iteration =
        dynamic_cast<const MarquardtIteration&>
            (mResult.Iteration(iterationNumber));
    //AttributeDatabase::AttributeType   attribute = AttributeDatabase::NORMAL;

    const std::vector<ColumnVector>& trial_points = iteration.TrialPoints();
    PointList           points_to_draw;
    const ColumnVector& starting_point = iteration.StartingPoint();


    if (highlight)
    {
        for (size_t i = 0; i < (trial_points.size() - 1); ++i)
        {
            points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(trial_points[i]),
                         AttributeDatabase::POINT_WORST));
            DrawLine(dc,
                     starting_point,
                     trial_points[i],
                     AttributeDatabase::POINT_WORST,
                     wxDOT);
        }
    }


    points_to_draw.push_back(ColourPointType
            (PlotToDcPoint(starting_point),
             highlight ?
                AttributeDatabase::POINT_START :
                AttributeDatabase::INACTIVE));
    points_to_draw.push_back(ColourPointType
            (PlotToDcPoint(trial_points.back()),
             highlight ?
                AttributeDatabase::POINT_BEST :
                AttributeDatabase::INACTIVE));

    DrawLine(dc,
             starting_point,
             trial_points.back(),
             highlight ?
                 AttributeDatabase::STEP_IN_DIRECTION :
                 AttributeDatabase::INACTIVE,
             wxSOLID);


    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));

}// DrawIteration

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* MarquardtLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            MarquardtIteration::ClassId(),
            MarquardtLayer::Create);
