#include <precomp.h>

#include <newmat/newmatrc.h>

#include "../util.h"
#include "../Symbols.h"

#include "AttributeDatabase.h"
#include "MultilineString.h"

MultilineString::MultilineString(const wxTextAttr& style)
    : mStrings(), mLength(0),
      mDefaultStyle(AttributeDatabase::GetInstance().TextAttribute(AttributeDatabase::DEFAULT))
{
    AddToDefaultStyle(style);
}// MultilineString

void
MultilineString::Append(const wxString& text)
{
    operator<<(text);
}// Append

void
MultilineString::Append(const wxString& text, const wxTextAttr& style)
{
    assert(text.Find(wxT('\n')) == wxNOT_FOUND);
    Allocate(1);    // jedna linia wysokoci.

    const unsigned first_row = FirstRow(1);

    mTextStyles[first_row].push_back(TextStyleRange(
                mLength,
                text.Length(),
                style));
    mStrings[first_row].Append(text);

    mLength += text.Length();
}// Append

MultilineString&
MultilineString::operator<<(const wxString& text)
{
    assert(text.Find(wxT('\n')) == wxNOT_FOUND);
    Allocate(1);    // zwyky tekst ma jedn lini wysokoci.

    mStrings[FirstRow(1)].Append(text);

    mLength += text.Length();
    return *this;
}// operator<<

MultilineString&
MultilineString::operator<<(const BaseMatrix& X)
{
    GeneralMatrix* gm = ((BaseMatrix&)X).Evaluate();
    operator<<(*gm);
    gm->tDelete();

    return *this;
}// operator<<

MultilineString&
MultilineString::operator<<(const GeneralMatrix& matrix)
{
    MatrixRow mr((GeneralMatrix*)&matrix, LoadOnEntry);
    int w = 10;
    int nr = matrix.Nrows();

    wxString number_format = MatrixNumberFormat(matrix);

    Allocate(nr);

    int index_offset = FirstRow(nr) - 1;
    for (int i=1; i<=nr; i++)
    {
        int skip = mr.skip;
        int storage = mr.storage;
        Real* store = mr.data;
        skip *= w;

        wxString& r_current_row = mStrings[index_offset + i];

        r_current_row.Append(wxT("[ "));
        r_current_row.Append(wxT(' '), skip);
        while (storage-- > 0)
        {
            r_current_row.Append(
                    wxString::Format(number_format, *(store++)));
            r_current_row.Append(wxT(' '));
        }
        r_current_row.Append(wxT(']'));
        mr.Next();

        if (mLength < r_current_row.Length()) mLength = r_current_row.Length();
    }

    return *this;
}// operator<<

void
MultilineString::Append(const GeneralMatrix&    matrix,
                        const MatrixStyle&      style,
                        const wxString&         upperIndex)
{
    MatrixRow mr((GeneralMatrix*)&matrix, LoadOnEntry);
    int w = 10;
    int nr = matrix.Nrows();

    wxString number_format = MatrixNumberFormat(matrix, true);
    wxString inf_symbol    = PaddedInfinitySymbol(matrix);

    Allocate(nr);

    int index_offset = FirstRow(nr) - 1;
    for (int i=1; i<=nr; i++)
    {
        // TODO: co ze skipem?
        int skip = mr.skip;
        int storage = mr.storage;
        Real* store = mr.data;
        skip *= w;

        wxString& r_current_row = mStrings[index_offset + i];

        r_current_row.Append(wxT("[ "));
        r_current_row.Append(wxT(' '), skip);
        while (storage-- > 0)
        {
            wxString number_string;
            if (*store != DBL_MAX)
                number_string = wxString::Format(number_format, *store);
            else
                number_string = inf_symbol;

            store++;

            assert (skip == 0);
            unsigned column_index = matrix.Ncols() - skip - storage;

            if (!style.Style(column_index, i).IsDefault())
            {   // Styl zawiera jakie ustawienia
                mTextStyles[index_offset + i].push_back(
                        TextStyleRange(r_current_row.Length(),
                                       number_string.Length(),
                                       style.Style(column_index, i)));
            }
            r_current_row.Append(number_string);
            r_current_row.Append(wxT(' '));
        }
        r_current_row.Append(wxT(']'));

        if ((i == 1) && (upperIndex.Length() > 0))
        {
            r_current_row.Append(upperIndex);
        }

        mr.Next();

        if (mLength < r_current_row.Length()) mLength = r_current_row.Length();
    }
}// Append

void
MultilineString::Print(wxTextCtrl*  pTextCtrl) const
{
    pTextCtrl->Freeze();
    wxTextAttr  old_style = pTextCtrl->GetDefaultStyle();

    pTextCtrl->SetDefaultStyle(mDefaultStyle);
    for (unsigned i = 0; i < mStrings.GetCount(); ++i)
    {
        unsigned character_index;
        TextStyleRanges::const_iterator style_iterator;
        const wxString& text_line = mStrings[i];

        style_iterator = mTextStyles[i].begin();
        character_index = 0;

        while(style_iterator != mTextStyles[i].end())
        {
            // Wydrukuj tekst do pocztku najbliszego zakresu z atrybutami.
            if (character_index < style_iterator->mFirst)
            {
                pTextCtrl->AppendText(text_line.Mid(character_index,
                            style_iterator->mFirst - character_index));
                character_index = style_iterator->mFirst;
            }

            assert(character_index == style_iterator->mFirst);

            // Wydrukuj zakres z okrelonym stylem.
            pTextCtrl->SetDefaultStyle(style_iterator->mStyle);
            pTextCtrl->AppendText(text_line.Mid(character_index,
                                                style_iterator->mLength));
            pTextCtrl->SetDefaultStyle(mDefaultStyle);

            character_index += style_iterator->mLength;

            style_iterator++;
        }

        if (character_index < text_line.Length())
        {
            pTextCtrl->AppendText(
                    text_line.Right(text_line.Length() - character_index));
        }
        if (i < mStrings.GetCount() - 1)    pTextCtrl->AppendText(wxT('\n'));
    }

    pTextCtrl->SetDefaultStyle(old_style);
    pTextCtrl->Thaw();
}// Print

wxString
MultilineString::ToString() const
{
    wxString string;

    for (unsigned i = 0; i < mStrings.GetCount(); ++i)
    {
        string << mStrings[i];
        if (i < mStrings.GetCount() - 1)    string << wxT('\n');
    }

    return string;
}// ToString

void
MultilineString::Clear()
{
    mStrings.Clear();
    mLength = 0;
    mTextStyles.clear();
    mDefaultStyle = AttributeDatabase::GetInstance().TextAttribute(
                        AttributeDatabase::DEFAULT);
}// Clear

void
MultilineString::Flush(wxTextCtrl* pTargetCtrl)
{
    Print(pTargetCtrl);
    Clear();
}// Flush

void
MultilineString::AddToDefaultStyle(const wxTextAttr& style)
{
    mDefaultStyle.Merge(style);
}// AddToDefaultStyle

const wxTextAttr&
MultilineString::DefaultStyle() const
{
    return mDefaultStyle;
}// DefaultStyle

void
MultilineString::Dump() const
{
    for (unsigned i = 0; i < mStrings.GetCount(); ++i)
    {
        cout << mStrings[i].mb_str() << endl;
    }
}// Dump


///////////////////////////////// PRIVATE ////////////////////////////////////

unsigned
MultilineString::FirstRow(unsigned rowCount) const
{
    assert(rowCount > 0);
    assert(mStrings.GetCount() >= rowCount);

    // Powoduje przesunicie o 1 wiersz w d w przypadku gdy liczba wszystkich
    // wierszy jest parzysta, a wymaganych nieparzysta
    if (mStrings.GetCount() % 2 == 0)   rowCount -= 1;

    return (mStrings.GetCount() - rowCount) / 2;
}// FirstRow

void
MultilineString::Allocate(unsigned rowCount)
{
    assert(rowCount > 0);

    // TODO: doda zmienn dla mStrings.GetCount()
    if (rowCount > mStrings.GetCount())
    {
        // Liczba wierszy do dodania od gry i od dou.
        const int add_row_count = (rowCount - mStrings.GetCount()) / 2;

        if ((mStrings.GetCount() - rowCount) % 2 == 1)
        {
            if (mStrings.GetCount() % 2 == 0)
            {
                mStrings.Add(wxT(""));
                mTextStyles.insert(mTextStyles.end(), TextStyleRanges());
            }
            else
            {
                mStrings.Insert(wxT(""), 0);
                mTextStyles.insert(mTextStyles.begin(), TextStyleRanges());
            }
        }

        mTextStyles.insert(mTextStyles.begin(), add_row_count,
                TextStyleRanges());
        mStrings.Insert(wxT(""), 0, add_row_count);

        mTextStyles.insert(mTextStyles.end(), add_row_count, TextStyleRanges());
        mStrings.Add(wxT(""), add_row_count);
    }

    for (unsigned i = FirstRow(rowCount); i < FirstRow(rowCount) + rowCount; ++i)
    {
        mStrings[i].Append(wxT(' '), mLength - mStrings[i].Length());
    }
}// Allocate
