#include <precomp.h>

#include "../Logger.h"

#include "AttributeDatabase.h"
#include "PlotControl.h"
#include "MultivariateFunctionLayer.h"  // class implemented
#include "conrec.h"

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

MultivariateFunctionLayer::MultivariateFunctionLayer(
        const PlotControl&  targetPlot,
        const FunctionBase& function,
        unsigned            variable1Index,
        unsigned            variable2Index)
    : PlotLayer(targetPlot),
      mFunction(function),
      mVariableXIndex(variable1Index + 1),
      mVariableYIndex(variable2Index + 1)
{
}// MultivariateFunctionLayer

MultivariateFunctionLayer::MultivariateFunctionLayer(
        const MultivariateFunctionLayer& from)
    : PlotLayer(from),
      mFunction(from.mFunction),
      mVariableXIndex(from.mVariableXIndex),
      mVariableYIndex(from.mVariableYIndex)
{
}// MultivariateFunctionLayer

MultivariateFunctionLayer::~MultivariateFunctionLayer()
{
}// ~MultivariateFunctionLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
MultivariateFunctionLayer::Draw(wxDC& dc) const
{
    // Wyzeruj wszystkie zmienne, bdziemy operowa tylko na tych, ktre s
    // przedstawiane na wykresie.
    // TODO: zrobi jako adniej to pobieranie.
//    ColumnVector point(mFunction.Variables().GetCount());
    ColumnVector point(2);
    point = 0;

    wxCoord pixel_width;
    wxCoord pixel_height;

    dc.GetSize(&pixel_width, &pixel_height);

    assert (pixel_width > 1);
    assert (pixel_height > 1);

//    LOG(DEBUG, wxT("mPlotY1: ") <<mPlotY1 << wxT("; mPlotY2: ") << mPlotY2);
//    assert (mPlotX2 > mPlotX1);
//    assert (mPlotY2 > mPlotY1);

    // TODO: zrobi sta klasy.
    // Odlego midzy kolejnymi prbkami (w pikslach).
    const int SAMPLE_BOX_SIZE = 5;
    // Siatka musi mie o 1 wicej krawd ni komrek.
    int column_count =
        static_cast<int>(
                ceil(static_cast<double>(pixel_width) / SAMPLE_BOX_SIZE)) + 1;
    int row_count    =
        static_cast<int>(
                ceil(static_cast<double>(pixel_height) / SAMPLE_BOX_SIZE)) + 1;

    double columns[column_count];
    double rows[row_count];
    double** samples;

    // A nie da si lepiej?
    samples = new double*[column_count];
    for (int i = 0; i < column_count; ++i) samples[i] = new double[row_count];


    LOG(DEBUG, wxT("column_count: ") << column_count <<
            wxT("; row_count: ") << row_count);

    mFunction.BeginFixed();

    double plot_min =  DBL_MAX;
    double plot_max = -DBL_MIN;
    try
    {
        for (int i = 0; i < column_count; ++i)
        {
            columns[i] = i * SAMPLE_BOX_SIZE;
            point(mVariableXIndex) = mTargetPlot.DcToPlotX(i * SAMPLE_BOX_SIZE);
            for (int j = 0; j < row_count; ++j)
            {
                rows[j] = j * SAMPLE_BOX_SIZE;
                point(mVariableYIndex) =
                    mTargetPlot.DcToPlotY(j * SAMPLE_BOX_SIZE);
                samples[i][j] = mFunction.EvaluateNoThrow(point);

                if (plot_min > samples[i][j])   plot_min = samples[i][j];
                if (plot_max < samples[i][j])   plot_max = samples[i][j];
            }
        }
    }
    catch (const Exception& error)
    {
        LOG(ERROR, wxString::FromAscii(error.what()));
    }

    mFunction.EndFixed();

    // Potrzebne do skalowania okienka z wartociami funkcji.
    mTargetPlot.SetFunctionValueExtent(plot_min, plot_max);

    if (plot_min != plot_max)
    {
        LOG(DEBUG, wxT("Calling conrec"));
        LOG(DEBUG, wxT("Bitmap size: ") << pixel_width
            << wxT("x") << pixel_height);
        LOG(DEBUG, wxT("Plot size:   ") << columns[column_count - 1]
            << wxT("x") << rows[row_count - 1]);

        unsigned level_count;
        double* levels = ComputeContourLevels(plot_min, plot_max, level_count);

        ContourPlotter contour(level_count);
        ::conrec(samples,
                 0, column_count - 1,
                 0, row_count - 1,
                 static_cast<double *>(columns),
                 static_cast<double *>(rows),
                 level_count,
                 levels,
                 contour);

        dc.SetPen(AttributeDatabase::GetInstance().Pen(
                    AttributeDatabase::PLOT_FUNCTION));
        contour.Collapse();
        contour.Draw(dc, true);

        delete levels;
    }
    // Dealokacja pamici
    for (int i = 0; i < column_count; ++i) delete[] samples[i];
    delete[] samples;

}// Draw

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
double*
MultivariateFunctionLayer::ComputeContourLevels(
        double      plotMin,
        double      plotMax,
        unsigned&   levelCount) const
{
    double plot_width = mTargetPlot.X2() - mTargetPlot.X1();
    double plot_height = mTargetPlot.Y2() - mTargetPlot.Y1();

    FixedCountLimiter interval_count_limiter(50);

    double level_interval = FindBestInterval(plotMax - plotMin,
            static_cast<long long>(ceil(plot_width > plot_height ?
                                                plot_width : plot_height)),
                                            interval_count_limiter);
    levelCount = lround((plotMax - plotMin) / level_interval);
    double* levels = new double[levelCount];

    // Wyznacz poziomice
    double first_level;
    // Oblicz wysoko pierwszej poziomicy
    if (plotMin < 0)
    {
        first_level = ceil(plotMin / level_interval) * level_interval;
    }
    else
    {
        first_level = floor(plotMin / level_interval + 1) * level_interval;
    }

    for (unsigned i = 0; i < levelCount; ++i)
    {
        levels[i] = first_level + i*level_interval;
    }

    return levels;
}// ComputeContourLevels

///////////////////////// FixedCountLimiter ///////////////////////////////

MultivariateFunctionLayer::FixedCountLimiter::FixedCountLimiter(
        unsigned maximumCount)
    : mMaximumCount(maximumCount)
{
}// FixedCountLimiter::FixedCountLimiter

/* virtual */ unsigned
MultivariateFunctionLayer::FixedCountLimiter::operator()(double intervalLength)
    const
{
    return mMaximumCount;
}// FixedCountLimiter::()
