#include <precomp.h>

#include "../Logger.h"
#include "AttributeDatabase.h"
#include "PlotControl.h"
#include "MultivariateFunctionLayer.h"
#include "MultivariatePlot.h" // implemented class
#include "ui_util.h"


LOGGER(INFO)

MultivariatePlot::MultivariatePlot(
        wxWindow*               pParent,
        wxWindowID              id,
        const FunctionBase&     function,
        const VariableNames&    variables,
        PlotStatus*             pPlotStatus,
        const wxPoint&          position,
        const wxSize&           size)
    : PlotControl(pParent, id, function, pPlotStatus, position, size),
      // mVariableXIndex(variable1Index + 1),
      // mVariableYIndex(variable2Index + 1),
      mVariableXIndex(1),
      mVariableYIndex(2),
      mVariables(variables),
      mKeepAspectRatio(true)
{
//    assert (mVariables.GetCount() > 1);

    // Na razie (czyli ju na zawsze) nie obsugujemy wikszej liczby zmiennych.
    assert (mVariables.GetCount() == 2);

    mXAxis.SetLabel(mVariables[mVariableXIndex - 1]);
    mYAxis.SetLabel(mVariables[mVariableYIndex - 1]);

    // TODO: usun
//    AppendLayer(CountedPtr<const PlotLayer>
//            (new MultivariateFunctionLayer(*this, mFunction)));

    // To wymaga ustawienia skal, ale nie jest tu potrzebne, bo bdzie i tak
    // wywoane z OnResize na pocztku.
//    EnforceAspectRatio();
}// MultivariatePlot

MultivariatePlot::~MultivariatePlot()
{
}// ~MultivariatePlot

bool
MultivariatePlot::KeepAspectRatio() const
{
    return mKeepAspectRatio;
}// KeepAspectRatio

bool&
MultivariatePlot::rKeepAspectRatio()
{
    return mKeepAspectRatio;
}// rKeepAspectRatio

const VariableNames&
MultivariatePlot::Variables() const
{
    return mVariables;
}// Variables

int
MultivariatePlot::VariableXIndex() const
{
    return mVariableXIndex - 1;
}// VariableXIndex

int
MultivariatePlot::VariableYIndex() const
{
    return mVariableYIndex - 1;
}// VariableYIndex

void
MultivariatePlot::SetVariables(int variableXIndex, int variableYIndex)
{
    assert(variableXIndex != variableYIndex);
    assert(variableXIndex < static_cast<int>(mVariables.GetCount()));
    assert(variableYIndex < static_cast<int>(mVariables.GetCount()));

    mVariableXIndex = variableXIndex + 1;
    mVariableYIndex = variableYIndex + 1;

    mXAxis.SetLabel(mVariables[variableXIndex]);
    mYAxis.SetLabel(mVariables[variableYIndex]);

    // TODO: Zamie zakresy miejscami, nie zmieniaj skali
//    double middle_x = (mPlotX2 - mPlotX1) / 2;
//    int pixel_half_width = mPlotBitmap.GetWidth() / 2;
//
//    mPlotX1 = middle_x + pixel_half_width * mScaleY;
//    mPlotX2 = middle_x - pixel_half_width * mScaleY;
//    [...]

//    mXAxis.Zoom(mPlotX1, mPlotX2);
//    mYAxis.Zoom(mPlotY1, mPlotY2);

    Redraw();
    Refresh();
}// SetVariables


/* virtual */ void
MultivariatePlot::OnAspectChange(bool redraw)
{
    EnforceAspectRatio();

    if (redraw)
    {
        Redraw();
        Refresh();
    }
}// AspectChange

////////////////////////////////////// PROTECTED //////////////////////////////

/* virtual */ void
MultivariatePlot::Pan(wxCoord x, wxCoord y)
{
    assert(mScaleX > 0);
    assert(mScaleY > 0);

    double plot_shift_x = (x - mDragStartX) * mScaleX;
    // Y jest do gry nogami
    double plot_shift_y = -(y - mDragStartY) * mScaleY;

    mPlotX1 -= plot_shift_x;
    mPlotX2 -= plot_shift_x;
    mPlotY1 -= plot_shift_y;
    mPlotY2 -= plot_shift_y;

    mXAxis.Zoom(mPlotX1, mPlotX2);
    mYAxis.Zoom(mPlotY1, mPlotY2);
}// Pan

/* virtual */ void
MultivariatePlot::ZoomRegionIn(wxCoord x,
                               wxCoord y,
                               wxCoord width,
                               wxCoord height)
{
    int pixel_width  = mPlotBitmap.GetWidth();
    int pixel_height = mPlotBitmap.GetHeight();

    double shift_on_left;
    double shift_on_right;
    double shift_on_top;
    double shift_on_bottom;

    wxCoord drag_end_x = x + width;
    wxCoord drag_end_y = y + height;

    if (x < drag_end_x)
    {
        shift_on_left = x * mScaleX;
        shift_on_right = (pixel_width - drag_end_x) * mScaleX;
    }
    else
    {
        shift_on_left = drag_end_x * mScaleX;
        shift_on_right = (pixel_width - x) * mScaleX;
    }

    if (y < drag_end_y)
    {
        shift_on_top    = y * mScaleY;
        shift_on_bottom = (pixel_height - drag_end_y) * mScaleY;
    }
    else
    {
        shift_on_top    = (drag_end_y * mScaleY);
        shift_on_bottom = (pixel_height - y) * mScaleY;
    }

    mPlotX1 += shift_on_left;
    mPlotX2 -= shift_on_right;

    // Y jest do gry nogami
    mPlotY1 += shift_on_bottom;
    mPlotY2 -= shift_on_top;

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);
    LimitZoom(mPlotY1, mPlotY2, mMinZoomY, mMaxZoomY);

    EnforceAspectRatio();
}// ZoomRegionIn

/* virtual */ void
MultivariatePlot::ZoomIn(wxCoord x, wxCoord y)
{
    double plot_half_width = (mPlotX2 - mPlotX1)/2;
    double plot_half_height = (mPlotY2 - mPlotY1)/2;

    double middle_x = mPlotX1 + x * mScaleX;
    // Y jest do gry nogami
    double middle_y = mPlotY2 - y * mScaleY;

    mPlotX1 = middle_x - plot_half_width * ZOOM_RATIO;
    mPlotX2 = middle_x + plot_half_width * ZOOM_RATIO;

    mPlotY1 = middle_y - plot_half_height * ZOOM_RATIO;
    mPlotY2 = middle_y + plot_half_height * ZOOM_RATIO;

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);
    LimitZoom(mPlotY1, mPlotY2, mMinZoomY, mMaxZoomY);
    EnforceAspectRatio();
}// ZoomIn

// TODO: to by trzeba byo jako przenie do warstwy rysujcej wykres...
/* virtual */ void
MultivariatePlot::UpdateStatusOnMove(long x, long y)
{
    if (mpPlotStatus != NULL)
    {
        if ((x >= mPlotAreaX) && (x <= mPlotAreaX + mPlotBitmap.GetWidth())
            && (y >= mPlotAreaY) && (y <= mPlotAreaY + mPlotBitmap.GetHeight()))
        {
            ColumnVector point(mVariables.GetCount());

            point(mVariableXIndex) = mPlotX1 + WindowToPlotAreaX(x) * mScaleX;
            point(mVariableYIndex) = mPlotY2 - WindowToPlotAreaY(y) * mScaleY;

            mpPlotStatus->UpdatePosition(mFunction.EvaluateNoThrow(point),
                    point(mVariableXIndex),
                    point(mVariableYIndex));
        }
        else
        {
            mpPlotStatus->ClearPosition();
        }
    }
}// UpdateStatusOnMove

/* virtual */ void
MultivariatePlot::UpdateScale()
{
    int pixel_width  = mPlotBitmap.GetWidth();
    int pixel_height = mPlotBitmap.GetHeight();

    mScaleX     = (mPlotX2 - mPlotX1) / pixel_width;
    mScaleY     = (mPlotY2 - mPlotY1) / pixel_height;
    mScaleValues= (mPlotMax - mPlotMin) / max(pixel_width, pixel_height);

//    mpPlotStatus->UpdateFormatStrings(mScaleValues, mScaleX, mScaleY);
}// UpdateScale

/* virtual */ void
MultivariatePlot::ZoomOut(wxCoord x, wxCoord y)
{
    double plot_half_width = (mPlotX2 - mPlotX1)/2;
    double plot_half_height = (mPlotY2 - mPlotY1)/2;
    double middle_x = mPlotX1 + x * mScaleX;
    // Y jest do gry nogami
    double middle_y = mPlotY2 - y * mScaleY;

    mPlotX1 = middle_x - plot_half_width * (1/ZOOM_RATIO);
    mPlotX2 = middle_x + plot_half_width * (1/ZOOM_RATIO);

    LOG(DEBUG, wxT("y1: ") << mPlotY1 << wxT("; y2: ") << mPlotY2);
    mPlotY1 = middle_y - plot_half_height * (1/ZOOM_RATIO);
    mPlotY2 = middle_y + plot_half_height * (1/ZOOM_RATIO);
    LOG(DEBUG, wxT("y1: ") << mPlotY1 << wxT("; y2: ") << mPlotY2);

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);
    LimitZoom(mPlotY1, mPlotY2, mMinZoomY, mMaxZoomY);

    EnforceAspectRatio();

    mXAxis.Zoom(mPlotX1, mPlotX2);
    mYAxis.Zoom(mPlotY1, mPlotY2);

}// ZoomOut

/* virtual */ void
MultivariatePlot::DrawDisplaced(long x, long y)
{
    // Kopiuje bitmap z wykresem odpowiednio przycit i przesunit. Czyci
    // obszar dookoa.
    wxClientDC dc(this);

    dc.SetBrush(AttributeDatabase::GetInstance().
            Brush(AttributeDatabase::PLOT_BACKGROUND));
    dc.SetPen(AttributeDatabase::GetInstance().
            Pen(AttributeDatabase::PLOT_BACKGROUND));

    // Lewy grny rg obszaru, na ktry bdzie kopiowana bitmapa
    long plot_area_x;
    long plot_area_y;
    // Rozmiar kopiowanego fragmentu bitmapy
    long plot_area_width;
    long plot_area_height;
    // Przycicie bitmapy z lewej i z gry
    long plot_cut_x;
    long plot_cut_y;

    // Pozycja i szeroko prostokta czyszczcego z boku
    long clear_x;
    long clear_width;

    // Pozycja i wysoko prostokta czyszczcego u gry/dou
    long clear_y;
    long clear_height;

    if (x > 0)
    {
        plot_area_x     = mPlotAreaX + x;
        plot_area_width = mPlotBitmap.GetWidth() - x;
        plot_cut_x      = 0;
        clear_x         = mPlotAreaX;
        clear_width     = x;
    }
    else
    {   // x <= 0
        plot_area_x     = mPlotAreaX;
        plot_area_width = mPlotBitmap.GetWidth() + x;
        plot_cut_x      = -x;
        clear_x         = mPlotAreaX + mPlotBitmap.GetWidth() + x;
        clear_width     = -x;
    }

    if (y > 0)
    {
        plot_area_y     = mPlotAreaY + y;
        plot_area_height= mPlotBitmap.GetHeight() - y;
        plot_cut_y      = 0;
        clear_y         = mPlotAreaY;
        clear_height    = y;
    }
    else
    {   // y <= 0
        plot_area_y     = mPlotAreaY;
        plot_area_height= mPlotBitmap.GetHeight() + y;
        plot_cut_y      = -y;
        clear_y         = mPlotAreaY + mPlotBitmap.GetHeight() + y;
        clear_height    = -y;
    }

    dc.Blit(plot_area_x, plot_area_y,
            plot_area_width, plot_area_height, &mPlotDc,
            plot_cut_x, plot_cut_y);
    dc.DrawRectangle(clear_x, mPlotAreaY, clear_width, mPlotBitmap.GetHeight());
    dc.DrawRectangle(mPlotAreaX, clear_y, mPlotBitmap.GetWidth(), clear_height);

    mXAxis.Displace(x);
    // Y jest do gry nogami
    mYAxis.Displace(-y);

    RedrawAxes(dc);
}// DrawDisplaced

/////////////////////////////////// PRIVATE ////////////////////////////////

void
MultivariatePlot::EnforceAspectRatio()
{
    if (!mKeepAspectRatio) return;      //  EXIT

    long pixel_width  = mPlotBitmap.GetWidth();
    long pixel_height = mPlotBitmap.GetHeight();

    // najczciej jest nieakualna
    UpdateScale();

    if (mScaleX < mScaleY)
    {
        mScaleX = mScaleY;
        FitToWidth(mPlotX1, mPlotX2, pixel_width * mScaleX);
    }
    else
    {
        mScaleY = mScaleX;
        FitToWidth(mPlotY1, mPlotY2, pixel_height * mScaleY);
    }
}// EnforceAspectRatio

void
MultivariatePlot::FitToWidth(double&    rStart,
                             double&    rEnd,
                             double     newWidth) const
{
    double middle = (rStart + rEnd) / 2;

    LOG(DEBUG, wxT("old width: ") << rEnd - rStart << wxT("; new: ")
            << newWidth);

    rStart  = middle - newWidth / 2;
    rEnd    = middle + newWidth / 2;
}// FitToWidth

