#include <precomp.h>

#include "../methods/NelderMeadIteration.h"

#include "NelderMeadLayer.h"  // class implemented
#include "VisualizerFactory.h"

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NelderMeadLayer::NelderMeadLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// NelderMeadLayer

NelderMeadLayer::NelderMeadLayer(const NelderMeadLayer& from)
    : XyUnconstrainedLayer(from)
{
}// NelderMeadLayer

NelderMeadLayer::~NelderMeadLayer()
{
}// ~NelderMeadLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
NelderMeadLayer::Create(const PlotControl&      plotControl,
                        const Result&           result,
                        CountedPtr<PlotLayer>   cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new NelderMeadLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
NelderMeadLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{

    ColumnVector point;
    Region bounding_box;

    const NelderMeadIteration& iteration =
        dynamic_cast<const NelderMeadIteration&>
            (mResult.Iteration(iterationNumber));

    vector<PointWithValue>::const_iterator  iter;
    iter    = iteration.InitialSimplex().begin();

    bounding_box.Extend(iter->mPoint(mVariableXIndex),
                        iter->mPoint(mVariableYIndex));

    ++iter;

    for (; iter != iteration.InitialSimplex().end(); ++iter)
    {
        point = iter->mPoint;
        bounding_box.Extend(point(mVariableXIndex), point(mVariableYIndex));
    }

    point = iteration.Reflected().mPoint;
    bounding_box.Extend(point(mVariableXIndex), point(mVariableYIndex));


    if (iteration.Expanded().mPoint != NullPoint)
    {
        point = iteration.Expanded().mPoint;
        bounding_box.Extend(point(mVariableXIndex), point(mVariableYIndex));
    }

    if (iteration.Contracted().mPoint != NullPoint)
    {
        point = iteration.Contracted().mPoint;
        bounding_box.Extend(point(mVariableXIndex), point(mVariableYIndex));
    }

    /* Ponisze punkty te teoretycznie powinny by uwzgldnione, ale kociwy
     * sympleks iteracji jest poczatkowym nastpnej (z wyjtkiem ostatniego, ale
     * ten jest dostatecznie may by go pomin
     */
    /*
    for (iter  = iteration.NewSimplex().begin();
         iter != iteration.NewSimplex().end();
         ++iter)
    {
        point = iter->mPoint;
        bounding_box.Extend(point(mVariableXIndex), point(mVariableYIndex));
    }
    */

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
NelderMeadLayer::DrawIteration(wxDC&       dc,
                               unsigned    iterationNumber,
                               bool        highlight) const
{

    const NelderMeadIteration& iteration =
        dynamic_cast<const NelderMeadIteration&>
            (mResult.Iteration(iterationNumber));

    AttributeDatabase::AttributeType    attribute;

    if (highlight)      attribute = AttributeDatabase::NORMAL;
    else                attribute = AttributeDatabase::INACTIVE;

    NelderMeadIteration::SimplexModificationType modification;
    modification = iteration.SimplexModification();

    PointList       points_to_draw;

    ColumnVector    point;

    const std::vector<PointWithValue>&  simplex = iteration.InitialSimplex();
    size_t  i;
    for (i = 0; i < simplex.size(); ++i)
    {
        DrawLine(dc,
                 simplex[i].mPoint,
                 simplex[(i+1) % simplex.size()].mPoint,
                 attribute,
                 wxSOLID);
    }

    if (highlight)
    {
        if (modification != NelderMeadIteration::REFLECTION)
        {
            for (i = 0; i < (simplex.size() - 1); ++i)
            {
                points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(iteration.Reflected().mPoint),
                         AttributeDatabase::NORMAL));
                DrawLine(dc,
                         iteration.Reflected().mPoint,
                         simplex[i].mPoint,
                         attribute,
                         wxDOT);
            }
        }

        if (modification != NelderMeadIteration::EXPANSION &&
            iteration.Expanded().mPoint != NullPoint)
        {
            for (i = 0; i < (simplex.size() - 1); ++i)
            {
                points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(iteration.Expanded().mPoint),
                         AttributeDatabase::NORMAL));
                DrawLine(dc,
                         iteration.Expanded().mPoint,
                         simplex[i].mPoint,
                         attribute,
                         wxDOT);
            }
        }

        if (modification != NelderMeadIteration::CONTRACTION &&
            iteration.Contracted().mPoint != NullPoint)
        {
            for (i = 0; i < (simplex.size() - 1); ++i)
            {
                points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(iteration.Contracted().mPoint),
                         AttributeDatabase::NORMAL));
                DrawLine(dc,
                         iteration.Contracted().mPoint,
                         simplex[i].mPoint,
                         attribute,
                         wxDOT);
            }
        }


        const std::vector<PointWithValue>&
                                       new_simplex = iteration.NewSimplex();

        for (i = 0; i < new_simplex.size(); ++i)
        {
            points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(new_simplex[i].mPoint),
                         AttributeDatabase::NORMAL));
            DrawLine(dc,
                     new_simplex[i].mPoint,
                     new_simplex[(i+1) % new_simplex.size()].mPoint,
                     attribute,
                     wxDOT);
        }


        points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(simplex.front().mPoint),
                         AttributeDatabase::POINT_BEST));
        points_to_draw.push_back(ColourPointType
                        (PlotToDcPoint(simplex.back().mPoint),
                         AttributeDatabase::POINT_WORST));
        if (simplex.size() > 2)
            points_to_draw.push_back(ColourPointType
                            (PlotToDcPoint(simplex[simplex.size() - 2].mPoint),
                             AttributeDatabase::POINT_MIDDLE));



    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));

}// DrawIteration

const /* static */ void* NelderMeadLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            NelderMeadIteration::ClassId(),
            NelderMeadLayer::Create);
