#include <precomp.h>

#include "../methods/NewtonRaphsonIteration.h"

#include "NewtonRaphsonLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

NewtonRaphsonLayer::NewtonRaphsonLayer(
                const PlotControl&      plotControl,
                const Result&           result,
                CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// NewtonRaphsonLayer

NewtonRaphsonLayer::NewtonRaphsonLayer(const NewtonRaphsonLayer& from)
    : UnivariateResultLayer(from)
{
}// NewtonRaphsonLayer

NewtonRaphsonLayer::~NewtonRaphsonLayer()
{
}// ~NewtonRaphsonLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
NewtonRaphsonLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
NewtonRaphsonLayer::Create(const PlotControl&            plotControl,
                          const Result&                 result,
                                CountedPtr<PlotLayer>   cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new NewtonRaphsonLayer(plotControl,
                                                     result,
                                                     cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
NewtonRaphsonLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const NewtonRaphsonIteration& iteration =
        dynamic_cast<const NewtonRaphsonIteration&>
            (mResult.Iteration(iterationNumber));

    double  left    = iteration.StartingPoint();
    double  right   = iteration.EndPoint();

    if (left > right)
        swap(left, right);

    return Region(left, 0, right, 0);

}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
NewtonRaphsonLayer::DrawIteration(wxDC&             dc,
                                 unsigned          iterationNumber,
                                 bool              highlight) const
{
    const NewtonRaphsonIteration& iteration =
        dynamic_cast<const NewtonRaphsonIteration&>
            (mResult.Iteration(iterationNumber));
    AttributeDatabase::AttributeType    line_attribute,
                                        starting_attribute,
                                        end_attribute;

    //line_attribute = starting_attribute = end_attribute = AttributeDatabase::NORMAL;

    line_attribute      = highlight ?
                            AttributeDatabase::NORMAL :
                            AttributeDatabase::INACTIVE;
    starting_attribute  = highlight ?
                            AttributeDatabase::POINT_START :
                            AttributeDatabase::NORMAL;
    end_attribute       = highlight ?
                            AttributeDatabase::POINT_BEST :
                            AttributeDatabase::NORMAL;

    DrawVerticalLine(dc, iteration.StartingPoint(), line_attribute);
    DrawVerticalLine(dc, iteration.EndPoint(), line_attribute);

    DrawBigPoint(dc, iteration.StartingPoint(), starting_attribute);
    DrawBigPoint(dc, iteration.EndPoint(), end_attribute);

}// DrawIteration

void
NewtonRaphsonLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const NewtonRaphsonIteration& iteration =
        dynamic_cast<const NewtonRaphsonIteration&>
            (mResult.Iteration(mHighlightedIteration));


    if (iteration.StartingPoint() < iteration.EndPoint())
        DrawRangeBackground(dc,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            dc.GetSize().GetWidth(),
                            AttributeDatabase::PLOT_BACKGROUND);
    else
        DrawRangeBackground(dc,
                            0,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* NewtonRaphsonLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            NewtonRaphsonIteration::ClassId(),
            NewtonRaphsonLayer::Create);
