#include <precomp.h>

#include "../Logger.h"

#include "AttributeDatabase.h"
#include "PlotAxis.h"  // class implemented

LOGGER(INFO)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PlotAxis::PlotAxis(bool vertical, bool limitToIntegers)
    : mVertical(vertical),
      mPixelLength(1),  // Za maa szeroko te nie byaby rewelacyjna
      mStubInterval(1),
      mAxisBitmap(),
      mAxisFont(10, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_LIGHT),
      mLabelFont(12, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL),
      mLimitToIntegers(limitToIntegers)
{
}// PlotAxis

PlotAxis::~PlotAxis()
{
}// ~PlotAxis


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void
PlotAxis::Resize(wxCoord newLength)
{
    assert (newLength > 0);

    mPixelLength = newLength;

    if (mVertical)  mAxisBitmap = wxBitmap(Breadth(), mPixelLength);
    else            mAxisBitmap = wxBitmap(mPixelLength, Breadth());

    UpdateStubInterval();
    Redraw();
}// Resize

void
PlotAxis::Zoom(double start, double end)
{
    assert (start < end);

    mPlotStart = start;
    mPlotEnd = end;

    mDisplacement = 0;

    UpdateStubInterval();
    Redraw();
}// Zoom

void
PlotAxis::Displace(wxCoord displacement)
{
    mDisplacement = displacement;

    Redraw();
}// Displace

//============================= ACCESS     ===================================

wxCoord
PlotAxis::Breadth() const
{
    // TODO: oblicza
    if (mVertical)          return 60;
    else                    return 50;
}// Breadth

wxCoord
PlotAxis::Length() const
{
    return mPixelLength;
}// GetLength

const wxBitmap&
PlotAxis::Draw() const
{
    return mAxisBitmap;
}// Draw

void
PlotAxis::SetLabel(const wxString& label)
{
    mLabel = label;

    Redraw();
}// SetLabel

bool&
PlotAxis::IntegersOnly()
{
    return mLimitToIntegers;
}// IntegersOnly

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////


void
PlotAxis::Redraw()
{
    if (mAxisBitmap.Ok())
    {
        wxMemoryDC dc;

        LOG(DEBUG, wxT("Redraw"));

        dc.SelectObject(mAxisBitmap);

        dc.SetBackground(AttributeDatabase::GetInstance().
                Brush(AttributeDatabase::PLOT_BACKGROUND));
        dc.SetPen(AttributeDatabase::GetInstance().
                Pen(AttributeDatabase::NORMAL));
        dc.SetFont(mAxisFont);

        dc.Clear();

        double scale = mPixelLength / (mPlotEnd - mPlotStart);

        // Uwzgldnij przesunicie (podane w pikslach)
        // TODO: czemu dziaa odejmowanie?
        double axis_start = mPlotStart - (mDisplacement / scale);
        double axis_end = mPlotEnd - (mDisplacement / scale);

        double first_stub;

        LOG(DEBUG, wxT("Stub interval: ") << mStubInterval);

        // Oblicz pozycj pierwszego stuba
        if (axis_start < 0)
        {
            first_stub = ceil(axis_start / mStubInterval) * mStubInterval;
        }
        else
        {
            first_stub = floor(axis_start / mStubInterval + 1) * mStubInterval;
        }

        wxString number_format = NumberFormat(mStubInterval);

        LOG(DEBUG, axis_start << wxT(", ") << axis_end);

        // Narysuj podziak. Dodawanie mStubInterval nie jest jakie genialne,
        // ale daje zadowalajce wyniki.
        for (double plot_value = first_stub;
                plot_value < axis_end;
                plot_value += mStubInterval)
        {
            // Zaokrglaj bliskie zera liczby do 0.
            if (abs(plot_value) < 1e-14)    plot_value = 0;

            wxCoord pixel_value = lround((plot_value - axis_start) * scale);
            if (mVertical) pixel_value = mPixelLength - pixel_value;

            if (mVertical)
            {
                dc.DrawLine(Breadth() - 1, pixel_value,
                            Breadth() - 7, pixel_value);
            }
            else           dc.DrawLine(pixel_value, 0, pixel_value, 6);

            // liczba (TODO: do osobnej funkcji)
            wxCoord width, height;
            wxString number_string = wxString::Format(number_format, plot_value);
            dc.GetTextExtent(number_string, &width, &height);
            LOG(DEBUG, wxT("Text extent: ") << width << wxT(", ") << height);
            if (mVertical)
            {
                dc.DrawText(number_string, Breadth() - 8 - width,
                        pixel_value - height/2);
            }
            else
            {
                dc.DrawText(number_string, pixel_value - width/2, 8);
            }
            // koniec pisania liczby
        }

        // O
        if (mVertical)      dc.DrawLine(Breadth()-1, 0, Breadth()-1, mPixelLength);
        else                dc.DrawLine(0, 0, mPixelLength, 0);

        // Etykieta
        DrawLabel(dc);

        dc.SelectObject(wxNullBitmap);
    }
}// Redraw

void
PlotAxis::DrawLabel(wxDC& dc)
{
    int width;
    int height;
    int descent;
    dc.GetTextExtent(mLabel, &width, &height, &descent);

    height += descent;

    dc.SetFont(mLabelFont);

    if (mVertical)
    {
        if (mLabel.Length() > 1)
        {
            dc.DrawRotatedText(mLabel, 0, (mPixelLength + width) / 2, 90);
        }
        else
        {
            dc.DrawText(mLabel, 0, (mPixelLength - width) / 2);
        }
    }
    else dc.DrawText(mLabel, (mPixelLength - width) / 2, Breadth() - height);
}// DrawLabel

void
PlotAxis::UpdateStubInterval()
{
    TextSizeIntervalCountLimiter
        interval_count_limiter(mPlotStart, mPlotEnd, mPixelLength, mVertical,
                mLimitToIntegers);

    mStubInterval = FindBestInterval(mPlotEnd - mPlotStart,
                            mPixelLength,
                            interval_count_limiter);
}// UpdateStubInterval

//////////////////////// TextSizeIntervalCountLimiter ////////////////////////

PlotAxis::TextSizeIntervalCountLimiter::TextSizeIntervalCountLimiter(
        double      plotStart,
        double      plotEnd,
        unsigned    pixelLength,
        bool        vertical,
        bool        limitToIntegers)
    : mPlotStart(plotStart),
      mPlotEnd(plotEnd),
      mPixelLength(pixelLength),
      mVertical(vertical),
      mLimitToIntegers(limitToIntegers)
{
}// TextSizeIntervalCountLimiter::TextSizeIntervalCountLimiter


/* virtual */ unsigned
PlotAxis::TextSizeIntervalCountLimiter::operator()(double intervalLength) const
{
    const wxCoord PADDING = 5;
    long max_interval_count;

    wxMemoryDC dc;
    wxBitmap dummy_bitmap = wxBitmap(1, 1);
    dc.SelectObject(dummy_bitmap);  // Nie kasowa, wymagane np. na Macu

    // Get number with largest number of digits (though not necessarily with the
    // longest string representation, but it'll do).
    double number = -max<double>(
        floor(abs(mPlotStart) / intervalLength) * intervalLength,
        floor(abs(mPlotEnd) / intervalLength) * intervalLength);

    wxString number_format = NumberFormat(intervalLength);

    wxCoord width;
    wxCoord height;

    dc.GetTextExtent(wxString::Format(number_format, number), &width, &height);

    LOG(DEBUG, wxT("Width of: '") << wxString::Format(number_format, number)
            << wxT("': ") <<  static_cast<int>(width));
    LOG(DEBUG, wxT("Height of: '") << wxString::Format(number_format, number)
            << wxT("': ") <<  static_cast<int>(height));
    LOG(DEBUG, static_cast<int>(mPixelLength));

    if (mVertical)  max_interval_count = mPixelLength / (height + PADDING);
    else            max_interval_count = mPixelLength / (width + PADDING);

    if ((mLimitToIntegers) && (intervalLength < 1.0))   max_interval_count = 0;

    LOG(DEBUG, wxT("Max interval count: ") <<
            static_cast<int>(max_interval_count));

    return max_interval_count;
}// TextSizeIntervalCountLimiter::()
