#ifndef _PlotControl_h
#define _PlotControl_h

#include "../CountedPtr.h"
#include "../Region.h"
#include "PlotAxis.h"
#include "PlotLayer.h"
#include "PlotStatus.h"

DECLARE_EVENT_TYPE(PLOT_FITVIEW_REQUEST_EVENT, -1);

class PlotControl
    : public wxWindow
{
public:
    enum ToolType
    {
        PAN,
        ZOOM_IN,
        ZOOM_OUT
    };

    PlotControl(
        wxWindow*               pParent,
        wxWindowID              id,
        const FunctionBase&     function,
        PlotStatus*             pPlotStatus = NULL,
        const wxPoint&          position = wxDefaultPosition,
        const wxSize&           size = wxDefaultSize);

    virtual ~PlotControl();

    /** Wybiera biece narzdzie.
     */
    void    SetTool(ToolType tool);

    /** Obsuga zdarzenia czyszczenia okna (brak akcji).
     */
    void    OnEraseEvent(wxEraseEvent& event);

    /** Obsuga zdarzenia przerysowania okna.
     */
    void    OnPaint(wxPaintEvent& event);

    /** Obsuga zdarzenia zmiany rozmiaru okna.
     */
    void    OnResize(wxSizeEvent& event);

    /** Obsuga proby o dopasowanie widoku.
     */
    void    OnFitViewRequest(wxCommandEvent& event);

    // Mysz
    void    OnMouseLeftDown(wxMouseEvent& event);
    void    OnMouseLeftUp(wxMouseEvent& event);
    void    OnMouseRightUp(wxMouseEvent& event);
    void    OnMouseMove(wxMouseEvent& event);
    void    OnMouseWheel(wxMouseEvent& event);

    void    OnMouseEnter(wxMouseEvent& event);
    void    OnMouseLeave(wxMouseEvent& event);


    /** Zapisuje bitmap z wykresem do pliku.
     */
    void    ExportBitmap();

    /** Zmiana proporcji.
     *
     * Wywoywane przy zmianie rozmiaru okna (to nie jest event handler z
     * wxWidgets).
     *
     * @param redraw Czy przerysowa wykres.
     */
    virtual void    OnAspectChange(bool redraw = false) = 0;

    /** Przesuwa wykres, aby pokaza cay podany obszar.
     */
    bool            AdjustView(const Region& region);

    /** Ustawia widok tak, aby obejmowa cay <code>region</code> (z pewnym
     * marginesem).
     */
    void    SetView(const Region& region);

    /** Dostosowuje obszar wykresu, aby pokaza wszystko.
     *
     * Warstwy s odpytywane w celu uzyskania informacji o istotnych obszarach.
     */
    bool            FitView();

    /** Daje zna, e naley przerysowa wykres.
     */
    void            RefreshPlot();

// ACCESS

    inline double       X1() const;
    inline double       X2() const;
    inline double       Y1() const;
    inline double       Y2() const;
    inline double       ScaleX() const;
    inline double       ScaleY() const;

    inline double       DcToPlotX(wxCoord x) const;
    inline double       DcToPlotY(wxCoord y) const;

    inline wxCoord      PlotToDcX(double x) const;
    inline wxCoord      PlotToDcY(double y) const;

    void        SetFunctionValueExtent(double min, double max) const;

    void        AppendLayer(CountedPtr<PlotLayer> cpLayer);

protected:
    /** Inicjuje pamiciowy kontekst graficzny.
     */
    void    InitPlotDc();

    /** Rysuje osie.
     */
    void    RedrawAxes(wxDC& dc);

    /** Przerysowuje bitmap.
     */
    void    Redraw();

    void    StartDragRectangle(wxCoord x, wxCoord y);
    void    UpdateDragRectangle(wxCoord x, wxCoord y);
    void    StopDragRectangle();

    wxSize  ComputePlotSize() const;

    wxCoord WindowToPlotAreaX(wxCoord x) const;
    wxCoord WindowToPlotAreaY(wxCoord y) const;

//    void    LimitMaximalZoom(double& rX1, double& rX2) const;

    /// Ogranicza wartoci rA i rB, aby: min <= rA - rB <= max
    void    LimitZoom(double& rA, double& rB, double min, double max) const;

// Metody wirtualne wywoywane z powyszych.


    /** Rysuje wykres funkcji przesunity o (x, y) piksli.
     *
     * Rysowana jest tylko bitmapa z wykresem odp. przesunita.
     */
    virtual void    DrawDisplaced(long x, long y) = 0;

    /** Przesuwa wykres o (x, y).
     */
    virtual void    Pan(wxCoord x, wxCoord y) = 0;

    /** Rysuje krzyw funkcji.
     */
//    virtual void    PlotFunction(wxDC& dc) = 0;

    /** Powikszenie wykresu (centrum w (x, y)).
     */
    virtual void    ZoomIn(wxCoord x, wxCoord y) = 0;

    /** Powikszenie zaznaczonego regionu.
     */
    virtual void    ZoomRegionIn(wxCoord x,
                                 wxCoord y,
                                 wxCoord width,
                                 wxCoord height) = 0;

    /** Pomniejszenie wykresu (centrum w (x, y)).
     */
    virtual void    ZoomOut(wxCoord x, wxCoord y) = 0;

    /** Uaktualnia pozycj kursora myszy na wykresie wywietlan w okienku
     * statusowym.
     */
    virtual void    UpdateStatusOnMove(long x, long y) = 0;

    /** Oblicza na nowo skale (np. przy zmianie rozmiaru okna).
     */
    virtual void    UpdateScale() = 0;

// Skadowe

    const FunctionBase&     mFunction;

    wxMemoryDC              mPlotDc;
    wxBitmap                mPlotBitmap;
    bool                    mDirty;

    PlotStatus*             mpPlotStatus;

    PlotAxis                mXAxis;
    PlotAxis                mYAxis;

    ToolType                mTool;

    // Wielkoci potrzebne dla rysowania wykresu
    double          mPlotX1;
    double          mPlotX2;
    double          mPlotY1;
    double          mPlotY2;
    mutable double  mPlotMin;   ///< Najmniejsza warto funkcji.
    mutable double  mPlotMax;   ///< Najwiksza warto funkcji.
    double          mScaleX;
    double          mScaleY;
    double          mScaleValues;

    wxCoord     mPlotAreaX; ///< Pozycja lewego-grnego rogu obszaru wykresu
    wxCoord     mPlotAreaY; ///< Pozycja lewego-grnego rogu obszaru wykresu

    // Przeciganie myszy
    wxCoord     mDragStartX;
    wxCoord     mDragStartY;
    wxCoord     mDragWidth; // moe by ujemne
    wxCoord     mDragHeight;// moe by ujemne

    wxCursor    mOldCursor; // Kursor poza oknem (przywracany w OnMouseLeave).

    vector<CountedPtr<PlotLayer> >    mLayers;    ///< Wektor warstw

    int         mWheelRotation; ///< Skumuloway, nieobsuony obrt kka myszki

    double      mMinZoomX;
    double      mMaxZoomX;
    double      mMinZoomY;
    double      mMaxZoomY;

    /// Wsp. zwania widocznego obszaru przy zoomowaniu (tylko kliknicie).
    const static double     ZOOM_RATIO;
    /// Minimalna szeroko widocznego obszaru (maksymalne powikszenie).
    const static double     MIN_ZOOMED_WIDTH;
    /// Maksymalna szeroko widocznego obszaru (minimalne powiszkenie).
    const static double     MAX_ZOOMED_WIDTH;
    /// Szeroko marginesw w okienku (w pikslach).
    const static wxCoord    MARGIN_SIZE;

    const wxCursor          mCursorPan;
    const wxCursor          mCursorZoomIn;
    const wxCursor          mCursorZoomOut;
    
    DECLARE_EVENT_TABLE()
};

// Inlined functions
inline double
PlotControl::X1() const
{
    return mPlotX1;
}// X1

inline double
PlotControl::X2() const
{
    return mPlotX2;
}// X2

inline double
PlotControl::Y1() const
{
    return mPlotY1;
}// Y1

inline double
PlotControl::Y2() const
{
    return mPlotY2;
}// Y2

inline double
PlotControl::ScaleX() const
{
    return mScaleX;
}// ScaleX

inline double
PlotControl::ScaleY() const
{
    return mScaleY;
}// ScaleY

inline double
PlotControl::DcToPlotX(wxCoord x) const
{
    return mPlotX1 + x * mScaleX;
}// DcToPlotX

inline double
PlotControl::DcToPlotY(wxCoord y) const
{
    return mPlotY2 - y * mScaleY;
}// DcToPlotY

inline wxCoord
PlotControl::PlotToDcX(double x) const
{
    return lround((x - mPlotX1) / mScaleX);
}// PlotToDcX


inline wxCoord
PlotControl::PlotToDcY(double y) const
{
    return lround((mPlotY2 - y) / mScaleY);
}// PlotToDcY

#endif
