#include <precomp.h>

#include "../methods/PowellIteration.h"

#include "PowellLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

PowellLayer::PowellLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// PowellLayer

PowellLayer::PowellLayer(const PowellLayer& from)
    : XyUnconstrainedLayer(from)
{
}// PowellLayer

PowellLayer::~PowellLayer()
{
}// ~PowellLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
PowellLayer::Create(const PlotControl&      plotControl,
                    const Result&           result,
                    CountedPtr<PlotLayer>   cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new PowellLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
PowellLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const PowellIteration& iteration =
        dynamic_cast<const PowellIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));

    PowellIteration::TrialPointContainer::const_iterator i;
    for (i = iteration.TrialPoints().begin();
         i != iteration.TrialPoints().end();
         ++i)
    {
         bounding_box.Extend(i->mPoint(mVariableXIndex),
                             i->mPoint(mVariableYIndex));
    }

    if (!iteration.PreliminaryIteration())
    {
        bounding_box.Extend(iteration.EndValuePoint()(mVariableXIndex),
                            iteration.EndValuePoint()(mVariableYIndex));
    }

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////
void
PowellLayer::DrawPreliminaryIteration(wxDC&     dc,
                                      bool      highlight) const
{
    const PowellIteration& iteration =
         dynamic_cast<const PowellIteration&>(mResult.Iteration(0));

    PointList   points_to_draw;

    points_to_draw.push_back(ColourPointType(PlotToDcPoint(
                                    iteration.StartingPoint()),
                                AttributeDatabase::POINT_START, SQUARE));

    const ColumnVector& base_point = iteration.TrialPoints().at(0).mPoint;
    AttributeDatabase::AttributeType attribute = AttributeDatabase::INACTIVE;

    if (highlight)   attribute = AttributeDatabase::STEP_PRELIMINARY;
    DrawLine(dc, iteration.StartingPoint(), base_point, attribute);

    points_to_draw.push_back(ColourPointType(PlotToDcPoint(base_point),
                             attribute));

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawPreliminaryIteration

/* virtual */ void
PowellLayer::DrawIteration(wxDC&       dc,
                           unsigned    iterationNumber,
                           bool        highlight) const
{
    const PowellIteration& iteration =
        dynamic_cast<const PowellIteration&>
            (mResult.Iteration(iterationNumber));

    if (iteration.PreliminaryIteration())
    {
        return DrawPreliminaryIteration(dc, highlight);
    }

    PointList   points_to_draw;

    ColumnVector    current = iteration.StartingPoint();

    if (highlight)
    {
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                            AttributeDatabase::POINT_START));
    }

    PowellIteration::TrialPointContainer::const_iterator i;
    const PowellIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    AttributeDatabase::AttributeType attribute = AttributeDatabase::INACTIVE;
    if (highlight)  attribute = AttributeDatabase::NORMAL;
    DrawLine(dc, current, iteration.EndPoint(), attribute, wxDOT);

    if (highlight)      attribute = AttributeDatabase::STEP_TRIAL_D1;

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        DrawLine(dc, current, i->mPoint, attribute);
        current = i->mPoint;
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(i->mPoint),
                                                     attribute));

        if (highlight)      attribute = AttributeDatabase::STEP_TRIAL_D2;
    }

    if (iteration.StepInNewDirection())
    {
        if (highlight)  attribute = AttributeDatabase::STEP_IN_DIRECTION;

        DrawLine(dc, current, iteration.NewPoint().mPoint, attribute);
        points_to_draw.push_back(ColourPointType(
                    PlotToDcPoint(iteration.NewPoint().mPoint), attribute));
    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* PowellLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            PowellIteration::ClassId(),
            PowellLayer::Create);
