#include <precomp.h>

#include "../Logger.h"
#include "../HistoryManager.h"
#include "../NonlinearParser.h"
#include "../ProblemWithPenalty.h"

#include "ConstraintsValidator.h"
#include "DoubleParameterValidator.h"
#include "ProblemPanelFactory.h"
#include "ProblemWithPenaltyPanel.h"
#include "ProblemWithPenaltyValidator.h"// TODO: ???
#include "PwpAdvancedDialog.h"

LOGGER(DEBUG);

BEGIN_EVENT_TABLE(ProblemWithPenaltyPanel, wxPanel)
    EVT_BUTTON(ID_ADVANCED, ProblemWithPenaltyPanel::OnAdvanced)
END_EVENT_TABLE()

ProblemWithPenaltyPanel::ProblemWithPenaltyPanel(
        wxWindow*           parent,
        int                 id,
        ProblemWithPenalty* pProblem,
        const wxPoint&      pos,
        const wxSize&       size,
        long                style)
    : wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL),
      mpProblem(pProblem),
      mConstraints(),
      mReactionChosen(PenaltyFunction::MAX_VALUE)
{

    const wxString mpTypeChoice_choices[] = {
        _("Outer penalty"),
        _("Inner hyperbolic penalty"),
        _("Inner logarithmic penalty")
    };

    mpTypeChoice        = new wxChoice(this, -1/*ID_PENALTYCHOICE*/,
                                    wxDefaultPosition,
                                    wxDefaultSize, 3, mpTypeChoice_choices);

    mpConstraintsPanel = new ConstraintsPanel(this, -1,
            CountedPtr<const Parser>(new NonlinearParser()),
            pProblem,
            FunctionValidator::ALLOW_UNIVARIATE,
            HistoryManager::GetHistory(HistoryManager::NONLINEAR_FUNCTION),
            HistoryManager::GetHistory(HistoryManager::NONLINEAR_CONSTRAINT));

    mpAdvancedButton    = new wxButton(this, ID_ADVANCED, _("Advanced..."));

    set_properties();
    do_layout();


    mpConstraintsPanel->SetValidator(ConstraintsValidator(
                mcpFunction,
                mcpVariables,
                mConstraints));

}

void ProblemWithPenaltyPanel::set_properties()
{

    if (mpProblem->mpPenaltyFunction != NULL)
    {
        mConstraints    = mpProblem->Penalty().Constraints();
        mReactionChosen = mpProblem->mpPenaltyFunction->OutOfBoundsReaction();
        mpTypeChoice->SetSelection(mpProblem->mpPenaltyFunction->FunctionId());
    }
    else
    {
        mReactionChosen = PenaltyFunction::MAX_VALUE;
        mpTypeChoice->SetSelection(0);
    }

}


void ProblemWithPenaltyPanel::do_layout()
{

    wxStaticBoxSizer*   p_type_sizer    = new wxStaticBoxSizer
                            (wxVERTICAL, this, _("Penalty function type"));
    wxBoxSizer*         p_button_sizer  = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer*         p_main_sizer    = new wxBoxSizer(wxVERTICAL);

    p_type_sizer->Add(mpTypeChoice, 0, wxEXPAND);
    p_button_sizer->AddStretchSpacer();
    p_button_sizer->Add(mpAdvancedButton, 1, wxTOP, 2);

    p_main_sizer->Add(p_type_sizer, 0, wxEXPAND);
    p_main_sizer->Add(mpConstraintsPanel, 1, wxEXPAND);
    p_main_sizer->Add(p_button_sizer, 0, wxEXPAND);

    SetAutoLayout(true);
    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);

}

Constraint::Container&
ProblemWithPenaltyPanel::rConstraints()
{
    return mConstraints;
}// rConstraints

void
ProblemWithPenaltyPanel::OnAdvanced(wxCommandEvent& event)
{
    PwpAdvancedDialog   advanced_dialog(this,
                                        wxID_ANY,
                                        mpProblem,
                                        _("Advanced properties"));

    advanced_dialog.CenterOnParent();
    advanced_dialog.ShowModal();
}

/* static */ wxPanel*
ProblemWithPenaltyPanel::CreatePanel(ProblemBase* pProblem, wxWindow* pParent)
{
    wxPanel* p_panel = new ProblemWithPenaltyPanel(
                            pParent,
                            wxID_ANY,
                            dynamic_cast<ProblemWithPenalty*>(pProblem));

    p_panel->SetValidator(ProblemWithPenaltyValidator(pProblem));

    return p_panel;
}// CreatePanel

// Zarejestruj metod tworzc panel
const void* ProblemWithPenaltyPanel::mpRegistrationDummy =
        ProblemPanelFactory::GetInstance().Register(
                ProblemWithPenalty::ClassId(),
                ProblemWithPenaltyPanel::CreatePanel);

