#include <precomp.h>

#include "../methods/QuadraticApproximationIteration.h"

#include "QuadraticApproximationLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

QuadraticApproximationLayer::QuadraticApproximationLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// QuadraticApproximationLayer

QuadraticApproximationLayer::QuadraticApproximationLayer(
        const QuadraticApproximationLayer& from)
    : UnivariateResultLayer(from)
{
}// QuadraticApproximationLayer


QuadraticApproximationLayer::~QuadraticApproximationLayer()
{
}// ~QuadraticApproximationLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
QuadraticApproximationLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    const QuadraticApproximationIteration& last_iteration =
        dynamic_cast<const QuadraticApproximationIteration&>
            (mResult.Iteration(mResult.LastIterationNumber()));
    if (last_iteration.Succeeded())
    {
        DrawFinalResult(dc);
    }
}// Draw

/* static */ std::auto_ptr<ResultLayer>
QuadraticApproximationLayer::Create(const PlotControl&      plotControl,
                                    const Result&           result,
                                    CountedPtr<PlotLayer>   cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new QuadraticApproximationLayer(plotControl,
                                                        result,
                                                        cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
QuadraticApproximationLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const QuadraticApproximationIteration& iteration =
        dynamic_cast<const QuadraticApproximationIteration&>
            (mResult.Iteration(iterationNumber));

    if (iteration.XOutOfBounds())
    {
        if (iteration.ApproximationStationaryPoint() < iteration.LeftBound())
        {
            return Region(iteration.ApproximationStationaryPoint(), 0,
                          iteration.RightBound(), 0);
        }
        else
        {
            return Region(iteration.LeftBound(), 0,
                          iteration.ApproximationStationaryPoint(), 0);
        }
    }

    return Region(iteration.LeftBound(), 0, iteration.RightBound(), 0);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
QuadraticApproximationLayer::DrawIteration(wxDC&             dc,
                                  unsigned          iterationNumber,
                                  bool              highlight) const
{
    const QuadraticApproximationIteration& iteration =
        dynamic_cast<const QuadraticApproximationIteration&>
            (mResult.Iteration(iterationNumber));

    // Granice przedziau
    DrawVerticalLine(dc, iteration.LeftBound(),
            AttributeDatabase::PLOT_FUNCTION);
    DrawVerticalLine(dc, iteration.RightBound(),
            AttributeDatabase::PLOT_FUNCTION);

    if (highlight)
    {   // Punkty
        DrawBigPoint(dc, iteration.LeftBound(),
                AttributeDatabase::NORMAL);
        DrawBigPoint(dc, iteration.RightBound(),
                AttributeDatabase::NORMAL);

        if (iteration.Succeeded())
        {
            DrawBigPoint(dc, iteration.BetterStep(),
                    AttributeDatabase::POINT_BEST);
            DrawBigPoint(dc, iteration.WorseStep(),
                    AttributeDatabase::POINT_WORST);
        }
        else
        {
            AttributeDatabase::AttributeType attr_x =
                AttributeDatabase::NORMAL;

            if (iteration.XOutOfBounds())
            {
                attr_x = AttributeDatabase::POINT_OUT_OF_BOUNDS;
            }

            DrawBigPoint(dc, iteration.ApproximationStationaryPoint(),
                    attr_x);
            DrawBigPoint(dc, iteration.TrialPoint(),
                    AttributeDatabase::NORMAL);
        }
    }

}// DrawIteration

void
QuadraticApproximationLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const QuadraticApproximationIteration& iteration =
        dynamic_cast<const QuadraticApproximationIteration&>
            (mResult.Iteration(mHighlightedIteration));

    DrawRangeBackground(dc,
                        mTargetPlot.PlotToDcX(iteration.LeftBound()),
                        mTargetPlot.PlotToDcX(iteration.RightBound()),
                        AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* QuadraticApproximationLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            QuadraticApproximationIteration::ClassId(),
            QuadraticApproximationLayer::Create);
