#include <precomp.h>

#include "../methods/RosenbrockDiscreteIteration.h"

#include "RosenbrockDiscreteLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RosenbrockDiscreteLayer::RosenbrockDiscreteLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// RosenbrockDiscreteLayer

RosenbrockDiscreteLayer::RosenbrockDiscreteLayer(
        const RosenbrockDiscreteLayer& from)
    : XyUnconstrainedLayer(from)
{
}// RosenbrockDiscreteLayer

RosenbrockDiscreteLayer::~RosenbrockDiscreteLayer()
{
}// ~RosenbrockDiscreteLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
RosenbrockDiscreteLayer::Create(const PlotControl&      plotControl,
                                const Result&           result,
                                CountedPtr<PlotLayer>   cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new RosenbrockDiscreteLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
RosenbrockDiscreteLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const RosenbrockDiscreteIteration& iteration =
        dynamic_cast<const RosenbrockDiscreteIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));

    RosenbrockDiscreteIteration::TrialPointContainer::const_iterator i;
    for (i = iteration.TrialPoints().begin();
         i != iteration.TrialPoints().end();
         ++i)
    {
         bounding_box.Extend(i->mPoint(mVariableXIndex),
                             i->mPoint(mVariableYIndex));
    }

    bounding_box.Extend(iteration.EndPoint()(mVariableXIndex),
                        iteration.EndPoint()(mVariableYIndex));

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
RosenbrockDiscreteLayer::DrawIteration(wxDC&       dc,
                                       unsigned    iterationNumber,
                                       bool        highlight) const
{
    const RosenbrockDiscreteIteration& iteration =
        dynamic_cast<const RosenbrockDiscreteIteration&>
            (mResult.Iteration(iterationNumber));

    PointList   points_to_draw;

    ColumnVector    current = iteration.StartingPoint();

    if (highlight || iterationNumber == mResult.FirstIterationNumber())
    {
        if (iterationNumber == mResult.FirstIterationNumber())
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                        AttributeDatabase::POINT_START,
                                        SQUARE));
        }
        else
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                        AttributeDatabase::POINT_START,
                                        CIRCLE));
        }
    }

    RosenbrockDiscreteIteration::TrialPointContainer::const_iterator i;
    const RosenbrockDiscreteIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    AttributeDatabase::AttributeType attribute = AttributeDatabase::INACTIVE;

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        if (highlight)
        {
            if ((i - trial_steps.begin()) % 2 == 0)
                attribute = AttributeDatabase::STEP_TRIAL_D1;
            else
                attribute = AttributeDatabase::STEP_TRIAL_D2;
        }

        if (i->mBetter)
        {
            DrawLine(dc, current, i->mPoint, attribute);
            current = i->mPoint;
        }
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(i->mPoint),
                                                     attribute));
    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* RosenbrockDiscreteLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            RosenbrockDiscreteIteration::ClassId(),
            RosenbrockDiscreteLayer::Create);
