#include <precomp.h>

#include "../methods/RosenbrockOptimalIteration.h"

#include "RosenbrockOptimalLayer.h"  // class implemented
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

RosenbrockOptimalLayer::RosenbrockOptimalLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : XyUnconstrainedLayer(plotControl, result, cpFunctionLayer)
{
}// RosenbrockOptimalLayer

RosenbrockOptimalLayer::RosenbrockOptimalLayer(
        const RosenbrockOptimalLayer& from)
    : XyUnconstrainedLayer(from)
{
}// RosenbrockOptimalLayer

RosenbrockOptimalLayer::~RosenbrockOptimalLayer()
{
}// ~RosenbrockOptimalLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ std::auto_ptr<ResultLayer>
RosenbrockOptimalLayer::Create(const PlotControl&       plotControl,
                               const Result&            result,
                               CountedPtr<PlotLayer>    cpFunctionLayer)
{
    return auto_ptr<ResultLayer>
            (new RosenbrockOptimalLayer(plotControl, result, cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================


/* virtual */ Region
RosenbrockOptimalLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{
    Region bounding_box;

    const RosenbrockOptimalIteration& iteration =
        dynamic_cast<const RosenbrockOptimalIteration&>
            (mResult.Iteration(iterationNumber));

    bounding_box.Extend(iteration.StartingPoint()(mVariableXIndex),
                        iteration.StartingPoint()(mVariableYIndex));

    RosenbrockOptimalIteration::TrialPointContainer::const_iterator i;
    for (i = iteration.TrialPoints().begin();
         i != iteration.TrialPoints().end();
         ++i)
    {
         bounding_box.Extend(i->mPoint(mVariableXIndex),
                             i->mPoint(mVariableYIndex));
    }

    bounding_box.Extend(iteration.EndPoint()(mVariableXIndex),
                        iteration.EndPoint()(mVariableYIndex));

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* virtual */ void
RosenbrockOptimalLayer::DrawIteration(wxDC&       dc,
                                      unsigned    iterationNumber,
                                      bool        highlight) const
{
    const RosenbrockOptimalIteration& iteration =
        dynamic_cast<const RosenbrockOptimalIteration&>
            (mResult.Iteration(iterationNumber));

    PointList   points_to_draw;

    ColumnVector    current = iteration.StartingPoint();

    if (highlight || iterationNumber == mResult.FirstIterationNumber())
    {
        if (iterationNumber == mResult.FirstIterationNumber())
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                        AttributeDatabase::POINT_START,
                                        SQUARE));
        }
        else
        {
            points_to_draw.push_back(ColourPointType(PlotToDcPoint(current),
                                        AttributeDatabase::POINT_START,
                                        CIRCLE));
        }
    }

    RosenbrockOptimalIteration::TrialPointContainer::const_iterator i;
    const RosenbrockOptimalIteration::TrialPointContainer& trial_steps =
        iteration.TrialPoints();

    AttributeDatabase::AttributeType attribute = AttributeDatabase::INACTIVE;
    if (highlight)      attribute = AttributeDatabase::STEP_TRIAL_D1;

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        DrawLine(dc, current, i->mPoint, attribute);
        current = i->mPoint;
        points_to_draw.push_back(ColourPointType(PlotToDcPoint(i->mPoint),
                                                     attribute));

        if (highlight)      attribute = AttributeDatabase::STEP_TRIAL_D2;
    }

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));
}// DrawIteration

const /* static */ void* RosenbrockOptimalLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            RosenbrockOptimalIteration::ClassId(),
            RosenbrockOptimalLayer::Create);
