#include <precomp.h>

#include "../methods/SignOfDerivativeIteration.h"

#include "SignOfDerivativeLayer.h"  // class implemented
#include "PlotControl.h"
#include "VisualizerFactory.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SignOfDerivativeLayer::SignOfDerivativeLayer(
        const PlotControl&      plotControl,
        const Result&           result,
        CountedPtr<PlotLayer>   cpFunctionLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer)
{
}// SignOfDerivativeLayer

SignOfDerivativeLayer::SignOfDerivativeLayer(const SignOfDerivativeLayer& from)
    : UnivariateResultLayer(from)
{
}// SignOfDerivativeLayer

SignOfDerivativeLayer::~SignOfDerivativeLayer()
{
}// ~SignOfDerivativeLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
SignOfDerivativeLayer::Draw(wxDC& dc) const
{
    GreyOutBackground(dc);

    DrawActiveIterationBackground(dc);

    mcpFunctionLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i < mHighlightedIteration;
         ++i)
    {
        DrawIteration(dc, i, false);
    }
    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);
}// Draw

/* static */ std::auto_ptr<ResultLayer>
SignOfDerivativeLayer::Create(const PlotControl&       plotControl,
                           const Result&            result,
                           CountedPtr<PlotLayer>    cpFunctionLayer)
{
    return auto_ptr<ResultLayer>(new SignOfDerivativeLayer(plotControl,
                                                     result,
                                                     cpFunctionLayer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
SignOfDerivativeLayer::IterationBoundingBox(unsigned iterationNumber) const
{
    const SignOfDerivativeIteration& iteration =
        dynamic_cast<const SignOfDerivativeIteration&>
            (mResult.Iteration(iterationNumber));

    double  left    = iteration.StartingPoint();
    double  right   = iteration.TrialPoints().front();

    if (left > right)
        swap(left, right);

    return Region(left, 0, right, 0);

}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
SignOfDerivativeLayer::DrawIteration(wxDC&             dc,
                                     unsigned          iterationNumber,
                                     bool              highlight) const
{
    const SignOfDerivativeIteration& iteration =
        dynamic_cast<const SignOfDerivativeIteration&>
            (mResult.Iteration(iterationNumber));
    AttributeDatabase::AttributeType   attribute;

    const std::vector<double>& trial_points = iteration.TrialPoints();

    if (highlight)
        attribute = AttributeDatabase::NORMAL;
    else
        attribute = AttributeDatabase::INACTIVE;

    DrawVerticalLine(dc, iteration.StartingPoint(), attribute);
    DrawVerticalLine(dc, trial_points.back(), attribute);

    if (highlight)
    {
        DrawBigPoint(dc, iteration.StartingPoint(),
                                AttributeDatabase::POINT_START);
        DrawBigPoint(dc, trial_points.back(), AttributeDatabase::POINT_BEST);

        for (size_t i = 0; i < (trial_points.size() - 1); ++i)
        {
            DrawVerticalLine(dc,
                             trial_points[i],
                             AttributeDatabase::INACTIVE);
            DrawBigPoint(dc, trial_points[i], AttributeDatabase::POINT_WORST);
        }
    }

}// DrawIteration

void
SignOfDerivativeLayer::DrawActiveIterationBackground(wxDC& dc) const
{
    const SignOfDerivativeIteration& iteration =
        dynamic_cast<const SignOfDerivativeIteration&>
            (mResult.Iteration(mHighlightedIteration));


    if (iteration.StartingPoint() < iteration.TrialPoints().front())
        DrawRangeBackground(dc,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            dc.GetSize().GetWidth(),
                            AttributeDatabase::PLOT_BACKGROUND);
    else
        DrawRangeBackground(dc,
                            0,
                            mTargetPlot.PlotToDcX(iteration.StartingPoint()),
                            AttributeDatabase::PLOT_BACKGROUND);

}// DrawActiveIterationBackground

/////////////////////////////// PRIVATE    ///////////////////////////////////
const /* static */ void* SignOfDerivativeLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            SignOfDerivativeIteration::ClassId(),
            SignOfDerivativeLayer::Create);
