#include <precomp.h>

#include "../LinearProblem.h"
#include "../methods/SimplexIteration.h"

#include "SimplexLayer.h"  // class implemented
#include "VisualizerFactory.h"

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

SimplexLayer::SimplexLayer(
        const PlotControl&                          plotControl,
        const Result&                               result,
        CountedPtr<PlotLayer>                 cpFunctionLayer,
        std::auto_ptr<XyConstraintsLayer>           apConstraintsLayer)
    : XyResultLayer(plotControl, result, cpFunctionLayer),
      mcpConstraintsLayer(apConstraintsLayer)
{
}// SimplexLayer

SimplexLayer::SimplexLayer(
        const SimplexLayer& from)
    : XyResultLayer(from),
      mcpConstraintsLayer(from.mcpConstraintsLayer)
{
}// SimplexLayer

SimplexLayer::~SimplexLayer()
{
}// ~SimplexLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
SimplexLayer::Draw(wxDC& dc) const
{

    // Wszystko rysujemy na wykresie funkcji.
    mcpFunctionLayer->Draw(dc);
    mcpConstraintsLayer->Draw(dc);


    for (unsigned i = 1; i <= mResult.LastIterationNumber(); i++)
    {
        if (i != mHighlightedIteration)
            DrawIteration(dc, i, false);
    }

    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);

}// Draw

/* virtual */ vector<ToolbarTool>
SimplexLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(XyResultLayer::CreateTools(pPlotControl));
    vector<ToolbarTool> constraints_tools
        (mcpConstraintsLayer->CreateTools(pPlotControl));

    tools.insert(tools.end(),
            constraints_tools.begin(), constraints_tools.end());

    return tools;
}// CreateTools

/* static */ std::auto_ptr<ResultLayer>
SimplexLayer::Create(
            const PlotControl&          plotControl,
            const Result&               result,
            CountedPtr<PlotLayer>       cpFunctionLayer)
{
    const LinearProblem& problem =
        dynamic_cast<const LinearProblem&>(result.Problem());

    auto_ptr<XyConstraintsLayer> ap_constraints_layer(
            new XyConstraintsLayer(plotControl,
                problem.ConstraintsWithAxes(), problem.Variables().GetCount()));

    return auto_ptr<ResultLayer>
            (new SimplexLayer(plotControl,
                              result,
                              cpFunctionLayer,
                              ap_constraints_layer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
SimplexLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{

    const SimplexIteration& iteration =
        dynamic_cast<const SimplexIteration&>
            (mResult.Iteration(iterationNumber));
    const double MARGIN = 1;

    return Region(iteration.ActualPoint()(mVariableXIndex) - MARGIN,
                  iteration.ActualPoint()(mVariableYIndex) - MARGIN,
                  iteration.ActualPoint()(mVariableXIndex) + MARGIN,
                  iteration.ActualPoint()(mVariableYIndex) + MARGIN);
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
SimplexLayer::DrawIteration(wxDC&       dc,
                                            unsigned    iterationNumber,
                                            bool        highlight) const
{

    const SimplexIteration& iteration =
        dynamic_cast<const SimplexIteration&>
            (mResult.Iteration(iterationNumber));

    AttributeDatabase::AttributeType attribute = AttributeDatabase::NORMAL;

    if (highlight)
    {
        if (iteration.MainPhase())
           attribute = AttributeDatabase::SIMPLEX_POINT_MAIN;
        else
           attribute = AttributeDatabase::SIMPLEX_POINT_EXTENDED;
    }


    ColourPointType point(PlotToDcPoint(iteration.ActualPoint()),
                                                 attribute);
    PointPainter(*this, dc)(point);
}// DrawIteration

const /* static */ void* SimplexLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            SimplexIteration::ClassId(),
            SimplexLayer::Create);
