#include <precomp.h>

#include "../Logger.h"
#include "../ProblemBase.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TextVisualizer.h"  // class implemented
#include "ui_util.h"

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TextVisualizer::TextVisualizer
        (wxWindow*                                  pParent,
         const Result&                              result,
         auto_ptr<const TextIterationVisualizer>    apIterationVisualizer)
    : Visualizer(pParent, result),
      mapIterationVisualizer(apIterationVisualizer),
      mNormalAttribute(AttributeDatabase::GetInstance().
              TextAttribute(AttributeDatabase::DEFAULT)),
      mHeaderAttribute(AttributeDatabase::GetInstance().
              TextAttribute(AttributeDatabase::HEADER))
{
    wxSizer* p_main_sizer = new wxBoxSizer(wxVERTICAL);

    mpTextCtrl = new wxTextCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition,
                        wxSize(500, 400), wxTE_MULTILINE | wxTE_READONLY
                        | wxTE_RICH2 | wxTE_DONTWRAP);

    p_main_sizer->Add(mpTextCtrl, 1, wxEXPAND | wxALL, 2);

    SetSizer(p_main_sizer);
    p_main_sizer->Fit(this);
    p_main_sizer->SetSizeHints(this);

    Layout();

    PrintResult(mpTextCtrl);
}// TextVisualizer

TextVisualizer::~TextVisualizer()
{
}// ~TextVisualizer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

/* virtual */ wxString
TextVisualizer::Name() const
{
    return wxGetTranslation(mName);
}// Name

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void
TextVisualizer::PrintResult(wxTextCtrl* pTargetCtrl)
{
    pTargetCtrl->Freeze();
    pTargetCtrl->SetDefaultStyle(mNormalAttribute);

    PrintHeader(pTargetCtrl, _("Optimization problem specification:"));

    wxString text;

    text << mResult.Problem().ToString() << wxT('\n');
    text << _("Variables: ") << wxT('\n');
    text << mResult.Problem().Variables().ToString() << wxT('\n');
    text << wxT("\n\n");

    pTargetCtrl->AppendText(text);
    text.Clear();

    PrintHeader(pTargetCtrl, _("Chosen optimization method:"));
    pTargetCtrl->AppendText(mResult.Problem().UsedMethod().ToString());
    pTargetCtrl->AppendText(wxT('\n'));

    PrintHeader(pTargetCtrl, _("Result summary:"));

    if (mResult.IsAborted())
    {   // Wystpi bd, wywietl komunikat
        pTargetCtrl->SetDefaultStyle(AttributeDatabase::GetInstance().
                TextAttribute(AttributeDatabase::ALERT));
        text <<
            _("An error has been encountered during optimization process:\n");
        text << mResult.ErrorMessage() << wxT("\n\n");
        text << _("No final result has been obtained.\n");

        if (mResult.Iterations().size() > 0)
        {
            text << _("Below is a list of iterations up to the one preceding the error.\n");
        }
        pTargetCtrl->AppendText(text);
        text.Clear();
        pTargetCtrl->SetDefaultStyle(mNormalAttribute);
    }
    else
    {
        wxArrayString strings;

        text << _("Final result: ")
             << PrintPointWithValue(mResult.FinalResult(),
                                    mResult.Problem().BaseFunction())
             << wxT('\n');

        strings.Add(text);
        text.Clear();
        strings.Add(wxString::Format(_("Iteration count: %u\n"),
                    mResult.IterationCount()));
        strings.Add(wxString::Format(_("Function evaluation count: %u\n"),
                    mResult.EvaluationCount()));

        pTargetCtrl->AppendText(AlignedParameterDescriptions(strings));
    }
    pTargetCtrl->AppendText(wxT('\n'));

    Result::IterationContainer::const_iterator iteration;
    const Result::IterationContainer& iterations = mResult.Iterations();

    unsigned iteration_number = mResult.FirstIterationNumber();
    for (iteration = iterations.begin();
         iteration != iterations.end();
         ++iteration, ++iteration_number)
    {
        assert (mapIterationVisualizer.get() != NULL);

        PrintHeader(pTargetCtrl,
                wxString::Format(_("Iteration %2d:"), iteration_number));

        mapIterationVisualizer->Print(pTargetCtrl, **iteration,
                mResult.Problem().BaseFunction(), false);
        if (iteration_number != mResult.LastIterationNumber())  pTargetCtrl->AppendText(wxT("\n\n"));
        else                                                    pTargetCtrl->AppendText(wxT('\n'));
    }

    pTargetCtrl->Thaw();
}// PrintResult

void
TextVisualizer::PrintHeader(wxTextCtrl* pTargetCtrl, const wxString& text)
{
    wxTextAttr old_attribute = pTargetCtrl->GetDefaultStyle();

    pTargetCtrl->SetDefaultStyle(mHeaderAttribute);
    pTargetCtrl->AppendText(text);
    pTargetCtrl->AppendText(wxT("\n\n"));
    pTargetCtrl->SetDefaultStyle(old_attribute);
}// PrintHeader

/////////////////////////////// PRIVATE    ///////////////////////////////////

const /* static */ wxString TextVisualizer::mName(_("Text"));
