#include <precomp.h>

#include "../methods/DichotomousDivisionIteration.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivDichotomousDivision.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivDichotomousDivision::TivDichotomousDivision()
{
}// TivDichotomousDivision

TivDichotomousDivision::~TivDichotomousDivision()
{
}// TivDichotomousDivision

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivDichotomousDivision::Print(wxTextCtrl*             pTargetCtrl,
                              const IterationData&    iterationData,
                              const FunctionBase&     function,
                              bool                    highlight) const
{

    assert (pTargetCtrl != NULL);
    const DichotomousDivisionIteration& iteration =
        dynamic_cast<const DichotomousDivisionIteration&>(iterationData);
    wxString text;
    ColumnVector point(1);

    pTargetCtrl->Freeze();

    text << _("Current section:\n");
    text << PrintSection(iteration.LeftBound(),
                         iteration.RightBound(),
                         function,
                         true);
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintTrialPoints(pTargetCtrl, iteration, function, highlight);

    text << PrintNewSection(pTargetCtrl, iteration, function, highlight);


    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivDichotomousDivision::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivDichotomousDivision);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivDichotomousDivision::PrintTrialPoints
                     (wxTextCtrl*                           pTargetCtrl,
                      const DichotomousDivisionIteration&   iteration,
                      const FunctionBase&                   function,
                      bool                                  highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxTextAttr  attr_left, attr_middle, attr_right;
    wxString    text;

    text << _("Trial points:") << wxT("\n");

    if (iteration.NextMiddle() == DichotomousDivisionIteration::XL)
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
        attr_middle = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    }
    else if (iteration.NextMiddle() == DichotomousDivisionIteration::XR)
    {
        attr_left   = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
        attr_middle = attributes.TextAttribute(AttributeDatabase::POINT_MIDDLE);
        attr_right  = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    }
    else
    {
        attr_middle = attributes.TextAttribute(AttributeDatabase::POINT_BEST);

        if (iteration.BetterPoint() == DichotomousDivisionIteration::LEFT)
        {
            attr_left   = attributes.TextAttribute
                                        (AttributeDatabase::POINT_MIDDLE);
            attr_right  = attributes.TextAttribute
                                        (AttributeDatabase::POINT_WORST);
        }
        else
        {
            attr_left   = attributes.TextAttribute
                                        (AttributeDatabase::POINT_WORST);
            attr_right  = attributes.TextAttribute
                                        (AttributeDatabase::POINT_MIDDLE);
        }
    }

    SetAttributeAndFlush(pTargetCtrl, text, attr_left, highlight);
    text << PrintDoubleWithValue(wxString(_("x_l")),
                                 iteration.LeftStep(),
                                 function);
    Flush(pTargetCtrl, text, highlight);
    text << wxT("\n");

    SetAttributeAndFlush(pTargetCtrl, text, attr_middle, highlight);
    text << PrintDoubleWithValue(wxString(_("x_m")),
                                 iteration.MiddleStep(),
                                 function);
    Flush(pTargetCtrl, text, highlight);
    text << wxT("\n");

    if (iteration.NextMiddle() != DichotomousDivisionIteration::XL)
    {
        SetAttributeAndFlush(pTargetCtrl, text, attr_right, highlight);
        text << PrintDoubleWithValue(wxString(_("x_r")),
                                     iteration.RightStep(),
                                     function);
        Flush(pTargetCtrl, text, highlight);
        text << wxT("\n");
    }

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute
                                        (AttributeDatabase::NORMAL),
                         highlight);

    text << wxT("\n");

    Flush(pTargetCtrl, text, highlight);

    return text;
}

wxString
TivDichotomousDivision::PrintNewSection
                     (wxTextCtrl*                           pTargetCtrl,
                      const DichotomousDivisionIteration&   iteration,
                      const FunctionBase&                   function,
                      bool                                  highlight) const
{
    wxString    text;

    text << _("Chosen section") << wxT(":\n");

    switch (iteration.NextMiddle())
    {
        case DichotomousDivisionIteration::XL :
            text << PrintCustomSection(wxString(wxT("a")),
                                       wxString(wxT("x_m")),
                                       iteration.LeftBound(),
                                       iteration.MiddleStep(),
                                       function,
                                       true);
            break;
        case DichotomousDivisionIteration::XM :
            text << PrintCustomSection(wxString(wxT("x_l")),
                                       wxString(wxT("x_r")),
                                       iteration.LeftStep(),
                                       iteration.RightStep(),
                                       function,
                                       true);
            break;
        case DichotomousDivisionIteration::XR :
            text << PrintCustomSection(wxString(wxT("x_m")),
                                       wxString(wxT("b")),
                                       iteration.MiddleStep(),
                                       iteration.RightBound(),
                                       function,
                                       true);
            break;
       default: assert(false);
    }

    text << wxT("\n");

    Flush(pTargetCtrl, text, highlight);

    return  text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivDichotomousDivision::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            DichotomousDivisionIteration::ClassId(),
            TivDichotomousDivision::Create);

