#include <precomp.h>

#include "../methods/EvolutionaryMethodIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivEvolutionaryMethod.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivEvolutionaryMethod::TivEvolutionaryMethod()
{
}// TivEvolutionaryMethod

TivEvolutionaryMethod::~TivEvolutionaryMethod()
{
}// ~TivEvolutionaryMethod

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ void
TivEvolutionaryMethod::Print(      wxTextCtrl*      pTargetCtrl,
                             const IterationData&   iterationData,
                             const FunctionBase&    function,
                                   bool             highlight) const
{
    assert (pTargetCtrl != NULL);
    const EvolutionaryMethodIteration& iteration =
        dynamic_cast<const EvolutionaryMethodIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    text << PrintStartingStatistics(pTargetCtrl, iteration, function,highlight);
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);
    text << PrintChromosomeCounts(pTargetCtrl, iteration, function, highlight);
    text << wxT('\n');
    Flush(pTargetCtrl, text, highlight);
    text << PrintEndStatistics(pTargetCtrl, iteration, function, highlight);
    Flush(pTargetCtrl, text, highlight);

    pTargetCtrl->AppendText(text);
    pTargetCtrl->Thaw();

}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivEvolutionaryMethod::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivEvolutionaryMethod);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivEvolutionaryMethod::PrintStartingStatistics
                (      wxTextCtrl*                      pTargetCtrl,
                 const EvolutionaryMethodIteration&     iteration,
                 const FunctionBase&                    function,
                       bool                             highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << _("Statistics of initial generation") << wxT(":\n");

    if (highlight)
        SetAttributeAndFlush
               (pTargetCtrl,
                text,
                attributes.TextAttribute(AttributeDatabase::CH_STARTING_BEST),
                highlight);

    text << _("Best point") << wxT(": ");
    text << PrintPointWithValue(wxString(wxT("x")),
                                iteration.StartingBest(),
                                function);
    text << wxT("\n");

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute(AttributeDatabase::NORMAL),
                         highlight);

    text << PrintPoint(wxString(_("Average chromosome")) +
                       wxString(wxT("\t")),
                       iteration.StartingAverageChromosome());
    text << wxT("\n");
    text << (wxString(_("Average value")) +
             wxString(wxT("\t= ")));
    text << PrintDouble(iteration.StartingAverageValue());
    text << wxT("\n");
    text << (wxString(_("Standard deviation of chromosomes")) +
             wxString(wxT("\t= ")));
    text << PrintDouble(iteration.StartingChromosomeStandardDeviation());
    text << wxT("\n");
    text << (wxString(_("Standard deviation of values")) +
             wxString(wxT("\t= ")));
    text << PrintDouble(iteration.StartingValueStandardDeviation());

    return text;
}

wxString
TivEvolutionaryMethod::PrintChromosomeCounts
                (      wxTextCtrl*                      pTargetCtrl,
                 const EvolutionaryMethodIteration&     iteration,
                 const FunctionBase&                    function,
                       bool                             highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    SetAttributeAndFlush
                    (pTargetCtrl,
                     text,
                     attributes.TextAttribute(AttributeDatabase::CH_DECEASED),
                     highlight);

    text << _("Deceased chromosomes")
         << wxString::Format(wxT(" :      %d\n"),
                             iteration.Deceased().size());

    SetAttributeAndFlush
                    (pTargetCtrl,
                     text,
                     attributes.TextAttribute(AttributeDatabase::CH_SURVIVOR),
                     highlight);

    text << _("Chromosomes that survived")
         << wxString::Format(wxT(" : %d\n"),
                             iteration.Survivors().size());

    SetAttributeAndFlush
                    (pTargetCtrl,
                     text,
                     attributes.TextAttribute(AttributeDatabase::CH_MUTANT),
                     highlight);

    text << _("Chromosomes that mutated")
         << wxString::Format(wxT(" :  %d\n"),
                             iteration.Mutants().size());

    SetAttributeAndFlush
                    (pTargetCtrl,
                     text,
                     attributes.TextAttribute(AttributeDatabase::CH_CHILD),
                     highlight);

    text << _("Born chromosomes")
         << wxString::Format(wxT(" :          %d\n"),
                             iteration.Children().size());

    SetAttributeAndFlush
                    (pTargetCtrl,
                     text,
                     attributes.TextAttribute(AttributeDatabase::NORMAL),
                     highlight);

    return text;
}

wxString
TivEvolutionaryMethod::PrintEndStatistics
                (      wxTextCtrl*                      pTargetCtrl,
                 const EvolutionaryMethodIteration&     iteration,
                 const FunctionBase&                    function,
                       bool                             highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << _("Statistics of end generation") << wxT(":\n");

    if (highlight)
        SetAttributeAndFlush
                   (pTargetCtrl,
                    text,
                    attributes.TextAttribute(AttributeDatabase::CH_END_BEST),
                    highlight);

    text << _("Best point") << wxT(": ");
    text << PrintPointWithValue(wxString(wxT("x")),
                                iteration.EndBest(),
                                function);
    text << wxT("\n");

    SetAttributeAndFlush(pTargetCtrl,
                         text,
                         attributes.TextAttribute(AttributeDatabase::NORMAL),
                         highlight);

    text << PrintPoint(wxString(_("Average chromosome")) +
                       wxString(wxT("               ")),
                       iteration.EndAverageChromosome());
    text << wxT("\n");
    text << (wxString(_("Average value")) +
             wxString(wxT("                     = ")));
    text << PrintDouble(iteration.EndAverageValue());
    text << wxT("\n");
    text << (wxString(_("Standard deviation of chromosomes")) +
             wxString(wxT(" = ")));
    text << PrintDouble(iteration.EndChromosomeStandardDeviation());
    text << wxT("\n");
    text << (wxString(_("Standard deviation of values")) +
             wxString(wxT("      = ")));
    text << PrintDouble(iteration.EndValueStandardDeviation());

    return text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivEvolutionaryMethod::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            EvolutionaryMethodIteration::ClassId(),
            TivEvolutionaryMethod::Create);

