#include <precomp.h>

#include "../Symbols.h"
#include "../UnivariateFunction.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivMarquardt.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivMarquardt::TivMarquardt()
{
}// TivMarquardt

TivMarquardt::~TivMarquardt()
{
}// TivMarquardt

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivMarquardt::Print(      wxTextCtrl*       pTargetCtrl,
                    const IterationData&    iterationData,
                    const FunctionBase&     function,
                          bool              highlight) const
{

    assert (pTargetCtrl != NULL);
    const MarquardtIteration& iteration =
        dynamic_cast<const MarquardtIteration&>(iterationData);
    wxString text;
    ColumnVector point(1);

    pTargetCtrl->Freeze();

    text << _("Base point") << wxT(": ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.StartingPoint(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintGradient(wxString::Format(wxT("x")),
                          iteration.Gradient());
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);

    text << PrintMatrix(iteration.Hessian(), wxString(_("H[f(x)]   = ")));
    text << wxT("\n\n");
    Flush(pTargetCtrl, text, highlight);


    text << PrintTrialPoints(pTargetCtrl, iteration, function, highlight);
    text << wxT("\n");


    text << _("New point") << wxT(":  ");
    text << PrintPointWithValue(wxString::Format(wxT("x")),
                                iteration.TrialPoints().back(),
                                function);
    text << wxT("\n");
    Flush(pTargetCtrl, text, highlight);



    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivMarquardt::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivMarquardt);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivMarquardt::PrintTrialPoints
                     (      wxTextCtrl*            pTargetCtrl,
                      const MarquardtIteration&    iteration,
                      const FunctionBase&          function,
                            bool                   highlight) const
{

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    Symbols symbols = Symbols::GetInstance();
    wxTextAttr  attr_rejected, attr_accepted, attr_normal;
    wxString    text;
    double      lambda = iteration.FirstLambda();
    size_t      i;


    attr_rejected = attributes.TextAttribute(AttributeDatabase::POINT_WORST);
    attr_accepted = attributes.TextAttribute(AttributeDatabase::POINT_BEST);
    attr_normal   = attributes.TextAttribute(AttributeDatabase::NORMAL);

    text << _("Trial points:") << wxT("\n");

    const std::vector<ColumnVector>&  trial_points = iteration.TrialPoints();
    const std::vector<ColumnVector>&  marquardt_steps
                                                = iteration.MarquardtSteps();

    for (i = 0; i < (trial_points.size() - 1); i++)
    {
        //Uy MultilineString jeli bdzie jak.
        wxString    lambda_description;
        lambda_description << symbols[Symbols::LAMBDA]
                           << wxT(" = ")
                           << wxString::Format(wxT("%10.8g"), lambda)
                           << wxT(": ");

        wxString    tab(wxT(' '), lambda_description.Length() + 3);

        SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);
        text << lambda_description;
        SetAttributeAndFlush(pTargetCtrl, text, attr_rejected, highlight);
        text << wxString::Format(wxT("%d: "), i);
        text << PrintPoint(_("step"), marquardt_steps[i]);
        text << wxT("\n");
        text << tab;
        text << PrintPointWithValue(wxString::Format(_("x"), i),
                                    trial_points[i],
                                    function);
        text << wxT('\t') << _("- rejected") << wxT("\n");

        lambda /= 2;
    }

    wxString    lambda_description;
    lambda_description << symbols[Symbols::LAMBDA]
                       << wxT(" = ")
                       << wxString::Format(wxT("%10.8g"), lambda)
                       << wxT(": ");

    wxString    tab(wxT(' '), lambda_description.Length() + 3);

    SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);
    text << lambda_description;
    SetAttributeAndFlush(pTargetCtrl, text, attr_accepted, highlight);
    text << wxString::Format(wxT("%d: "), i);
    text << PrintPoint(_("step"), marquardt_steps[i]);
    text << wxT("\n");
    text << tab;
    text << PrintPointWithValue(wxString::Format(_("x"), i),
                                trial_points[i],
                                function);
    text << wxT('\t') << _("- accepted") << wxT("\n");

    SetAttributeAndFlush(pTargetCtrl, text, attr_normal, highlight);

    //Flush(pTargetCtrl, text, highlight);

    return text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivMarquardt::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            MarquardtIteration::ClassId(),
            TivMarquardt::Create);

