#include <precomp.h>

#include "../methods/NelderMeadIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivNelderMead.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivNelderMead::TivNelderMead()
{
}// TivNelderMead

TivNelderMead::~TivNelderMead()
{
}// ~TivNelderMead

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

/* virtual */ void
TivNelderMead::Print(wxTextCtrl*            pTargetCtrl,
                     const IterationData&   iterationData,
                     const FunctionBase&    function,
                     bool                   highlight) const
{
    assert (pTargetCtrl != NULL);
    const NelderMeadIteration& iteration =
        dynamic_cast<const NelderMeadIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();


    text << PrintInitialSimplex(pTargetCtrl, iteration, highlight);
    text << wxT("\n");
    text << PrintTrialPoints(pTargetCtrl, iteration, highlight);
    text << wxT("\n");
    text << PrintNewSimplex(pTargetCtrl, iteration, false);


    pTargetCtrl->AppendText(text);
    pTargetCtrl->Thaw();

}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivNelderMead::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivNelderMead);
}// Create

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

wxString
TivNelderMead::PrintSimplex(wxTextCtrl*                     pTargetCtrl,
                            const vector<PointWithValue>&   simplex,
                            bool                            highlight) const
{

    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString    text;
    std::vector<PointWithValue>::const_iterator iter;

    iter    = simplex.begin();

    SetAttributeAndFlush(pTargetCtrl, text,
        attributes.TextAttribute(AttributeDatabase::POINT_BEST), highlight);
    text << PrintPoint(_("Best point          "), iter->mPoint);
    text << wxT("\t");
    text << PrintFunctionValue(iter->mValue);
    text << wxT("\n");
    SetAttributeAndFlush(pTargetCtrl, text,
        attributes.TextAttribute(AttributeDatabase::DEFAULT), highlight);

    for(++iter;
        (iter != (simplex.end() - 2) && iter != (simplex.end() - 1));
        ++iter)
    {
        text << PrintPoint(_T("            "), iter->mPoint);
        text << wxT("\t");
        text << PrintFunctionValue(iter->mValue);
        text << wxT("\n");
    }

    if (simplex.size() > 2)
    {
        SetAttributeAndFlush(pTargetCtrl, text,
            attributes.TextAttribute(AttributeDatabase::POINT_MIDDLE),
                                     highlight);
        text << PrintPoint(_("Point next to worst "), iter->mPoint);
        text << wxT("\t");
        text << PrintFunctionValue(iter->mValue);
        text << wxT("\n");
        SetAttributeAndFlush(pTargetCtrl, text,
            attributes.TextAttribute(AttributeDatabase::DEFAULT), highlight);

        ++iter;
    }

    SetAttributeAndFlush(pTargetCtrl, text,
        attributes.TextAttribute(AttributeDatabase::POINT_WORST), highlight);
    text << PrintPoint(_("Worst point         "), iter->mPoint);
    text << wxT("\t");
    text << PrintFunctionValue(iter->mValue);
    text << wxT("\n");
    SetAttributeAndFlush(pTargetCtrl, text,
        attributes.TextAttribute(AttributeDatabase::DEFAULT), highlight);


    return      text;
}

wxString
TivNelderMead::PrintInitialSimplex(wxTextCtrl*                  pTargetCtrl,
                                   const NelderMeadIteration&   iteration,
                                   bool                         highlight) const
{
    wxString    text;

    text << _("Initial simplex:\n");
    Flush(pTargetCtrl, text, highlight);
    text << PrintSimplex(pTargetCtrl, iteration.InitialSimplex(), highlight);

    return      text;
}

wxString
TivNelderMead::PrintTrialPoints(wxTextCtrl*                     pTargetCtrl,
                                   const NelderMeadIteration&   iteration,
                                   bool                         highlight) const
{
    NelderMeadIteration::SimplexModificationType modification;
    wxString            text;

    modification    = iteration.SimplexModification();

    text << _("Trial points") << wxT(":\n");

    text << PrintPoint(_("Reflecting point    "),
                                                iteration.Reflected().mPoint);
    text << wxT("\t");
    text << PrintFunctionValue(iteration.Reflected().mValue);
    text << wxT("\t");
    text << ((modification == NelderMeadIteration::REFLECTION) ?
                _("- accepted") : _("- rejected"));
    text << wxT("\n");

    if (iteration.Expanded().mPoint != NullPoint)
    {
        text << PrintPoint(_("Expanding point     "),
                                            iteration.Expanded().mPoint);
        text << wxT("\t");
        text << PrintFunctionValue(iteration.Expanded().mValue);
        text << wxT("\t");
        text << ((modification == NelderMeadIteration::EXPANSION) ?
                _("- accepted") : _("- rejected"));
        text << wxT("\n");
    }

    if (iteration.Contracted().mPoint != NullPoint)
    {
        text << PrintPoint(_("Contracting point   "),
                           iteration.Contracted().mPoint);
        text << wxT("\t");
        text << PrintFunctionValue(iteration.Contracted().mValue);
        text << wxT("\t");
        text << ((modification == NelderMeadIteration::CONTRACTION) ?
                _("- accepted") : _("- rejected"));
        text << wxT("\n");
    }

    return      text;
}

//wxString
//TivNelderMead::PrintIterationSolution
//                                (wxTextCtrl*                pTargetCtrl,
//                                 const NelderMeadIteration& iteration,
//                                 bool                       highlight) const
//{
//    wxString    text;
//
//    switch (iteration.SimplexModification())
//    {
//        case NelderMeadIteration::REFLECTION:
//                    text << _("New simplex built with reflected point");
//                    break;
//        case NelderMeadIteration::EXPANSION:
//                    text << _("New simplex built with further reflected point");
//                    break;
//        case NelderMeadIteration::CONTRACTION:
//                    text << _("New simplex built with contracted point");
//                    break;
//        case NelderMeadIteration::SHRINKAGE:
//                    text << _("New simplex build by shrinking the old one");
//                    break;
//        default:    assert(false);
//                    break;
//    }
//
//    return      text;
//}

wxString
TivNelderMead::PrintNewSimplex(wxTextCtrl*                  pTargetCtrl,
                               const NelderMeadIteration&   iteration,
                               bool                         highlight) const
{
    wxString    text;

    text << _("End simplex:\n");
    text << PrintSimplex(pTargetCtrl, iteration.NewSimplex(), highlight);

    return  text;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* static */ void* TivNelderMead::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            NelderMeadIteration::ClassId(),
            TivNelderMead::Create);

