#include <precomp.h>

#include "../methods/PowellIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "TivPowell.h"  // class implemented
#include "ui_util.h"
#include "VisualizerFactory.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivPowell::TivPowell()
{
}// TivPowell

TivPowell::~TivPowell()
{
}// ~TivPowell

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivPowell::Print(wxTextCtrl*             pTargetCtrl,
                 const IterationData&    iterationData,
                 const FunctionBase&     function,
                 bool                    highlight) const
{
    assert (pTargetCtrl != NULL);
    const PowellIteration& iteration =
        dynamic_cast<const PowellIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    if (iteration.PreliminaryIteration())
    {
        PrintPreliminaryIteration(pTargetCtrl, iteration, function, highlight);
        pTargetCtrl->Thaw();
        return;
    }

    text << PrintStartingParameters(pTargetCtrl, iteration, function,
            highlight);
    Flush(pTargetCtrl, text, highlight);
    text << PrintTrialSteps(pTargetCtrl, iteration, function, highlight);
    text << wxT('\n');
    text << PrintTrialSummary(pTargetCtrl, iteration);
    Flush(pTargetCtrl, text, highlight);

    if (iteration.StepInNewDirection())
    {
        text << PrintNewDirection(pTargetCtrl, iteration, function, highlight);
        text << PrintNewBase(pTargetCtrl, iteration, highlight);
    }

    pTargetCtrl->AppendText(text);

    pTargetCtrl->Thaw();
}// Print

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
void
TivPowell::PrintPreliminaryIteration(
                        wxTextCtrl*                         pTargetCtrl,
                        const PowellIteration&              iterationData,
                        const FunctionBase&                 function,
                        bool                                highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << _("Starting point: ")
         << PrintPointWithValue(wxT("x0"), iterationData.StartingPoint(),
            function)
         << wxT("\n\n");

    Flush(pTargetCtrl, text, highlight);
    text << PrintTwoColumnMatrix(pTargetCtrl, iterationData.D(),
                                 attributes.TextAttribute(
                                    AttributeDatabase::DEFAULT),
                                 attributes.TextAttribute(
                                    AttributeDatabase::STEP_PRELIMINARY),
                                 _("Base (D): "),
                                 highlight);
    text << wxT('\n');

    SetAttributeAndFlush(pTargetCtrl,
                 text,
                 attributes.TextAttribute(AttributeDatabase::STEP_PRELIMINARY),
                 highlight);
    const PowellIteration::TrialPoint& point =
        iterationData.TrialPoints().at(0);
    text << wxT('\n')
         << _("First base point: ")
         << PrintOptimalStep(point.mTau, point.mPoint, function);

    pTargetCtrl->AppendText(text);
}// PrintPreliminaryIteration

wxString
TivPowell::PrintStartingParameters(
                        wxTextCtrl*                         pTargetCtrl,
                        const PowellIteration&              iterationData,
                        const FunctionBase&                 function,
                        bool                                highlight) const
{
    wxString text;

    text << _("Base point: ")
         << PrintPointWithValue(wxT("x_s"), iterationData.StartingPoint(),
                function)
         << wxT("\n\n");

    Flush(pTargetCtrl, text, highlight);
    text << PrintDirectionMatrix(pTargetCtrl, iterationData.D(),
                _("Base (D): "), highlight);
    text << wxT('\n');

    return text;
}// PrintStartingParameters

wxString
TivPowell::PrintTrialSteps(
        wxTextCtrl*                         pTargetCtrl,
        const PowellIteration&              iterationData,
        const FunctionBase&                 function,
        bool                                highlight) const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const wxTextAttr& attr_d1 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D1);
    const wxTextAttr& attr_d2 = attributes.TextAttribute(
        AttributeDatabase::STEP_TRIAL_D2);
    const wxTextAttr& attr_default = attributes.TextAttribute(
        AttributeDatabase::DEFAULT);
    wxString text;

    PowellIteration::TrialPointContainer::const_iterator i;
    const PowellIteration::TrialPointContainer& trial_steps =
        iterationData.TrialPoints();

    text << wxT('\n')
         << _("Trial steps:")
         << wxT('\n');

    for (i = trial_steps.begin(); i != trial_steps.end(); ++i)
    {
        text << wxT("  ");

        if (highlight)
        {
            if (i == trial_steps.begin())
            {
                SetAttributeAndFlush(pTargetCtrl, text, attr_d1, highlight);
            }
            else
            {
                SetAttributeAndFlush(pTargetCtrl, text, attr_d2, highlight);
            }
        }
        text << PrintOptimalStep(i->mTau, i->mPoint, function)
             << wxT('\n');
    }// for

    SetAttributeAndFlush(pTargetCtrl, text, attr_default, highlight);

    return text;
}// PrintTrialSteps

wxString
TivPowell::PrintTrialSummary(wxTextCtrl*                     pTargetCtrl,
                             const PowellIteration&          iterationData)
                                                                    const
{
    wxString text;

    text << _("Final point: ")
         << PrintPoint(wxT("x"), iterationData.EndPoint())
         << wxT('\n')
         << PrintPoint(wxT("x - x_s"), iterationData.TrialDistance())
         << wxString::Format(wxT("   || x - x_s || = %g"),
                 iterationData.TrialDistance().norm_Frobenius())
         << wxT('\n');

    return text;
}// PrintTrialSummary

wxString
TivPowell::PrintNewDirection(wxTextCtrl*                         pTargetCtrl,
                             const PowellIteration&              iterationData,
                             const FunctionBase&                 function,
                             bool                                highlight)
                                                                        const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();

    wxString text;

    SetAttributeAndFlush(pTargetCtrl,
                 text,
                 attributes.TextAttribute(AttributeDatabase::STEP_IN_DIRECTION),
                 highlight);
     text << wxT('\n')
          << PrintPoint(_("New direction"), iterationData.NewDirection())
          << wxT('\n')
          << _("Step in new direction: ")
          << PrintOptimalStep(iterationData.NewPoint().mTau,
                              iterationData.NewPoint().mPoint,
                              function)
          << wxT("\n\n");

    SetAttributeAndFlush(pTargetCtrl,
                 text,
                 attributes.TextAttribute(AttributeDatabase::DEFAULT),
                 highlight);
    return text;
}// PrintNewDirection
/////////////////////////////// PRIVATE    ///////////////////////////////////
