#include <precomp.h>

#include "../methods/SimplexIteration.h"
#include "../Symbols.h"
#include "../text-util.h"

#include "AttributeDatabase.h"
#include "MatrixStyle.h"
#include "MultilineString.h"
#include "TivSimplex.h"  // class implemented
#include "VisualizerFactory.h"
#include "ui_util.h"

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

TivSimplex::TivSimplex()
{
}// TivSimplex

TivSimplex::~TivSimplex()
{
}// ~TivSimplex

//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
/* virtual */ void
TivSimplex::Print(wxTextCtrl*           pTargetCtrl,
                  const IterationData&  iterationData,
                  const FunctionBase&   function,
                  bool                  highlight) const
{
    assert (pTargetCtrl != NULL);
    const SimplexIteration& iteration =
        dynamic_cast<const SimplexIteration&>(iterationData);
    wxString text;

    pTargetCtrl->Freeze();

    if (iteration.Type() == SimplexIteration::FIRST_TRIVIAL)
    {
        PrintMainProblemMatrices(pTargetCtrl, iteration);
        text << _("The A matrix is in canonical form.\nThe first base \
will be constructed from its last m columns.")
             << wxT("\n\n");
        Flush(pTargetCtrl, text);
    }
    else if (iteration.Type() == SimplexIteration::FIRST_EXTENDED)
    {
        PrintMainProblemMatrices(pTargetCtrl, iteration);
        text << _("The A matrix is not in canonical form.\n\
Extended problem will be solved to find the first base indices.")
             << wxT("\n\n");
        Flush(pTargetCtrl, text);
        PrintExtendedProblemMatrices(pTargetCtrl, iteration);
    }
    else if (iteration.Type() == SimplexIteration::FIRST_MAINPHASE)
    {
        text << _("Preliminary phase completed successfully, \
back to the main problem.")
             << wxT('\n');
        Flush(pTargetCtrl, text);
        PrintMainProblemMatrices(pTargetCtrl, iteration);
    }

    PrintStepData(pTargetCtrl, function, iteration);

    if (iteration.Type() == SimplexIteration::LAST_EXTENDED)
    {
        PrintExtendedSolutionSummary(pTargetCtrl, iteration);
    }

    pTargetCtrl->Thaw();
}// Print

/* static */ std::auto_ptr<const TextIterationVisualizer>
TivSimplex::Create()
{
    return auto_ptr<const TextIterationVisualizer>(new TivSimplex);
}// Create
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////
void
TivSimplex::PrintMainProblemMatrices(
                        wxTextCtrl*                         pTargetCtrl,
                        const SimplexIteration&             iterationData)
                                                                const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << _("Augmented form of the main problem: ")
         << wxT("\n\n");
    Flush(pTargetCtrl, text);

    MultilineString multiline;

    // Drukujemy A.
    Matrix A = iterationData.MainA();
    MatrixStyle style(A);
    style.SetColumnStyle(iterationData.ActualVariables() + 1,
                         A.ncols(),
                         attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_SLACK));
    multiline.Append(wxT("A: "));
    multiline.Append(A, style);
    multiline.Flush(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

    // Drukujemy b.
    RowVector b = iterationData.MainB().t();
    style.Reset(b);
    style.SetStyle(attributes.TextAttribute(
                             AttributeDatabase::SIMPLEX_B));
    multiline.Append(wxT("b: "));
    multiline.Append(b, style, wxT("T"));
    multiline.Flush(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

    // Drukujemy c.
    RowVector c = iterationData.MainC().t();
    style.Reset(c);
    style.SetStyle(attributes.TextAttribute(
                   AttributeDatabase::SIMPLEX_C));
    style.AddColumnStyle(iterationData.ActualVariables() + 1,
                         c.ncols(),
                         attributes.TextAttribute(
                         AttributeDatabase::SIMPLEX_VAR_SLACK));
    multiline.Append(wxT("c: "));
    multiline.Append(c, style, wxT("T"));
    multiline.Print(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

}// PrintMainProblemMatrices

void
TivSimplex::PrintExtendedProblemMatrices(
                        wxTextCtrl*                         pTargetCtrl,
                        const SimplexIteration&             iterationData)
                                                                   const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << _("Augmented form of the extended problem: ")
         << wxT("\n\n");
    Flush(pTargetCtrl, text);

    // Drukujemy A.
    MultilineString multiline;
    Matrix A = iterationData.ExtendedA();
    MatrixStyle style(A);
    style.SetColumnStyle(iterationData.ActualVariables() + 1,
                         A.ncols() - A.nrows(),
                         attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_SLACK));
    style.SetColumnStyle(A.ncols() - A.nrows() + 1,
                         A.ncols(),
                         attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_EXTENDING));
    multiline.Append(wxT("A^: "));
    multiline.Append(A, style);
    multiline.Flush(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

    // Drukujemy b.
    RowVector b = iterationData.ExtendedB().t();
    style.Reset(b);
    style.SetStyle(attributes.TextAttribute(
                           AttributeDatabase::SIMPLEX_B));
    multiline.Append(wxT("b^: "));
    multiline.Append(b, style, wxT("T"));
    multiline.Flush(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

    // Drukujemy c.
    RowVector c = iterationData.ExtendedC().t();
    style.Reset(c);
    style.SetStyle(attributes.TextAttribute(
                           AttributeDatabase::SIMPLEX_C));
    style.AddColumnStyle(iterationData.ActualVariables() + 1,
                         c.ncols() - A.nrows(),
                         attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_SLACK));
    style.AddColumnStyle(c.ncols() - A.nrows() + 1,
                         c.ncols(),
                         attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_EXTENDING));
    multiline.Append(wxT("c^: "));
    multiline.Append(c, style, wxT("T"));
    multiline.Print(pTargetCtrl);
    pTargetCtrl->AppendText(wxT("\n\n"));

}// PrintExtendedProblemMatrices

void
TivSimplex::PrintStepData(wxTextCtrl*                         pTargetCtrl,
                          const FunctionBase&                 function,
                          const SimplexIteration&             iterationData)
                                                                const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    const wxTextAttr& attr_default = attributes.TextAttribute(
        AttributeDatabase::DEFAULT);
    wxString text;

    // Drukujemy indeksy bazowe i niebazowe, kolorujac po
    // jednym zmienionym indeksie w kazdej linii.
    text << _("Basic indices (iB)") << wxT(":\t");
    if (iterationData.PreviousQ() == 0)
    {
        text << iterationData.BasicIndices();
    }
    else
    {
        Flush(pTargetCtrl, text);
        text << PrintColouredIndices(pTargetCtrl,
                                     iterationData.BasicIndices(),
                                     iterationData.PreviousQ(),
                                     attributes.TextAttribute(
                                     AttributeDatabase::SIMPLEX_VAR_ENTERING));
    }

    text << wxT('\n')
         << _("Non-basic indices (iN)") << wxT(":\t");
    if (iterationData.NonbasicIndices().size() == 0)
    {
        text << wxT("-");
    }
    else if (iterationData.PreviousS() == 0)
    {
        text << iterationData.NonbasicIndices();
    }
    else
    {
        Flush(pTargetCtrl, text);
        text << PrintColouredIndices(pTargetCtrl,
                                     iterationData.NonbasicIndices(),
                                     iterationData.PreviousS(),
                                     attributes.TextAttribute(
                                     AttributeDatabase::SIMPLEX_VAR_LEAVING));
    }

    // Drukujemy macierze B i N.
    text << wxT("\n\n");
    Flush(pTargetCtrl, text);
    PrintBasicMatrices(pTargetCtrl, iterationData);

    // Drukujemy punkty bazowy i peny.
    SetAttribute(pTargetCtrl, attr_default, true);
    if (iterationData.X() != NullPoint)
    {
        PrintBasicPoint(pTargetCtrl, iterationData);
        PrintFullPoint(pTargetCtrl, function, iterationData);
    }
    else
    {
        return;
    }

    // Drukujemy wektor kosztw ograniczonych i zmienn wchodzc.
    if (iterationData.D() != NullPoint)
    {
        PrintReducedCosts(pTargetCtrl, iterationData);
    }
    else
    {
        return;
    }

    // Drukujemy wektor przesunicia i zmienn opuszczajc.
    if (iterationData.H() != NullPoint)
    {
        PrintLeavingVariable(pTargetCtrl, iterationData);
        if (iterationData.Q() != 0)
        {
            PrintNewPoint(pTargetCtrl, iterationData);
        }
    }
    else
    {
        return;
    }

}// PrintStepData

void
TivSimplex::PrintExtendedSolutionSummary(
                   wxTextCtrl*                      pTargetCtrl,
                   const SimplexIteration&          iterationData) const
{
    wxString text;

    text << wxT('\n')
         << _("Extended solution calculations finished.")
         << wxT('\n')
         << _("Result basic indices (iB)")
         << wxT(":\t")
         << iterationData.BasicIndices();

    pTargetCtrl->AppendText(text);
}// PrintExtendedSolutionSummary

wxString
TivSimplex::PrintColouredIndices(wxTextCtrl*            pTargetCtrl,
                                 const vector<int>&     indices,
                                 int                    highlighted,
                                 const wxTextAttr&      attribute) const
{
    std::vector<int>::const_iterator iter;
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;
    int pos = 1;

    for (iter = indices.begin(); iter != indices.end() && pos < highlighted;
                                                      iter++, pos++)
        text << wxString::Format(wxT("%d "), *iter);

    SetAttributeAndFlush(pTargetCtrl, text, attribute, true);
    text << wxString::Format(wxT("%d"), *iter);;
    Flush(pTargetCtrl, text);

    if (iter != indices.end())  iter++;
    SetAttribute(pTargetCtrl, attributes.TextAttribute(
                AttributeDatabase::DEFAULT), true);
    for (                      ; iter != indices.end(); iter++)
        text << wxString::Format(wxT(" %d"), *iter);

    return text;
}// PrintColouredIndices

void
TivSimplex::PrintBasicMatrices(wxTextCtrl*                      pTargetCtrl,
                               const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    Matrix A = (iterationData.MainPhase() ?
                iterationData.MainA() : iterationData.ExtendedA());
    wxString text;

    // Drukujemy obie macierze.
    MultilineString multiline;
    MatrixStyle style(iterationData.Base());
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_BASE));
    multiline.Append(_("Base (B): "));
    multiline.Append(iterationData.Base(), style);

    if (iterationData.N().ncols() != 0)
    {
        style.Reset(iterationData.N());
        style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_N));
        multiline.Append(_("  Remaining columns (N): "));
        multiline.Append(iterationData.N(), style);
    }
    else
    {
        assert(iterationData.S() == -1);
    }

    multiline.Print(pTargetCtrl);

}// PrintBasicMatrices

void
TivSimplex::PrintBasicPoint(wxTextCtrl*                      pTargetCtrl,
                            const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    text << wxT("\n\n")
         << _("Basic point:")
         << wxT('\n');
    Flush(pTargetCtrl, text);

    MultilineString multiline;
    MatrixStyle style_x(iterationData.X());

    if (iterationData.Type() == SimplexIteration::FIRST_TRIVIAL ||
        iterationData.Type() == SimplexIteration::FIRST_EXTENDED ||
        iterationData.Type() == SimplexIteration::FIRST_MAINPHASE)
    {// Pierwszy punkt bazowy jest obliczany jako rozwizanie
     // rwnania wektorowego, rozpisujemy je.

        MatrixStyle style(iterationData.Base());
        style.SetStyle(attributes.TextAttribute(
                              AttributeDatabase::SIMPLEX_BASE));
        multiline.Append(wxT("x_B = "));
        multiline.Append(iterationData.Base(), style, wxT("-1"));

        style.Reset(iterationData.MainB());
        style.SetStyle(attributes.TextAttribute(
                                AttributeDatabase::SIMPLEX_B));
        multiline.Append(wxT(" "));
        multiline.Append(iterationData.MainB(), style);
        multiline.Append(wxT(" = "));
    }
    else
    {// Kolejne punkty bazowe otrzymujemy w wyniku wykonania kroku
     // rozpisywanego na kocu poprzedniej iteracji.
        multiline.Append(wxT("x_B = "));
    }
    multiline.Append(iterationData.X(), style_x);

    multiline.Print(pTargetCtrl);
}// PrintBasicPoint

void
TivSimplex::PrintFullPoint(wxTextCtrl*                      pTargetCtrl,
                           const FunctionBase&              function,
                           const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;
    const Matrix& A = (iterationData.MainPhase() ?
                       iterationData.MainA() : iterationData.ExtendedA());

    text << wxT("\n\n")
         << _("Full point:")
         << wxT('\n');
    Flush(pTargetCtrl, text);

    RowVector x = iterationData.FullPoint().t();
    MultilineString multiline;
    MatrixStyle style(x);

    if (iterationData.MainPhase())
    {
        style.SetColumnStyle(iterationData.ActualVariables() + 1,
                             x.ncols(),
                             attributes.TextAttribute(
                                    AttributeDatabase::SIMPLEX_VAR_SLACK));
    }
    else
    {
        style.SetColumnStyle(iterationData.ActualVariables() + 1,
                             A.ncols() - A.nrows(),
                             attributes.TextAttribute(
                                    AttributeDatabase::SIMPLEX_VAR_SLACK));

        style.SetColumnStyle(A.ncols() - A.nrows() + 1,
                             A.ncols(),
                             attributes.TextAttribute(
                                    AttributeDatabase::SIMPLEX_VAR_EXTENDING));
    }

    multiline.Append(wxT("x = "));
    multiline.Append(x, style, wxT("T"));

    multiline.Print(pTargetCtrl);

    text << PrintFunctionValue(wxT("x"), function.Evaluate(
                     iterationData.ActualPoint()))
         << wxT('\n');
    pTargetCtrl->AppendText(text);

}// PrintFullPoint

void
TivSimplex::PrintReducedCosts(wxTextCtrl*                      pTargetCtrl,
                              const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    int i;
    wxString text;

    text << wxT('\n')
         << _("Reduced cost vector:")
         << wxT('\n');
    Flush(pTargetCtrl, text);

    const ColumnVector& c = (iterationData.MainPhase() ?
                             iterationData.MainC() : iterationData.ExtendedC());

    // Rozbijamy wektor kosztw na cz bazow
    // i niebazow.
    ColumnVector c_b(iterationData.Base().ncols());
    assert(c.nrows() - c_b.nrows() == iterationData.N().ncols());
    ColumnVector c_n(c.nrows() - c_b.nrows());
    for (i = 1; i <= c_b.nrows(); i++)
    {
       c_b(i) = c(iterationData.BasicIndices()[i - 1]);
    }
    for (i = 1; i <= (c.nrows() - c_b.nrows()); i++)
    {
       c_n(i) = c(iterationData.NonbasicIndices()[i - 1]);
    }

    MultilineString multiline;
    // Dopisujemy koszty niebazowe.
    MatrixStyle style(c_n);
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_C));
    multiline.Append(wxT("d = "));
    multiline.Append(c_n, style);

    // Dopisujemy macierz kolumn niebazowych.
    const Matrix& N = iterationData.N().t();
    style.Reset(N);
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_N));
    multiline.Append(wxT(" - "));
    multiline.Append(N, style);

    // Dopisujemy macierz kolumn bazowych.
    const SquareMatrix& B_t = iterationData.Base().t();
    style.Reset(B_t);
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_BASE));
    multiline.Append(wxT(" "));
    multiline.Append(B_t, style, wxT("-1"));

    // Dopisujemy koszty bazowe.
    style.Reset(c_b);
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_C));
    multiline.Append(wxT(" "));
    multiline.Append(c_b, style);

    // Dopisujemy wynik.
    style.Reset(iterationData.D());
    style.SetRowStyle(iterationData.S(), attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_ENTERING));
    multiline.Append(wxT(" = "));
    multiline.Append(iterationData.D(), style);
    multiline.Print(pTargetCtrl);

    if (iterationData.D()(iterationData.S()) < 0)
    {// Da si ograniczy koszty, drukujemy zmienn wchodzc.
       text << wxT('\n')
            << wxString::Format(wxT("\ns = %d"), iterationData.S())
            << wxT(", ")
            << _("entering column: ")
            << wxString::Format(wxT("iN[%d] = ") , iterationData.S());
       SetAttributeAndFlush(pTargetCtrl,
                            text,
                            attributes.TextAttribute(
                                AttributeDatabase::SIMPLEX_VAR_ENTERING),
                            true);
       pTargetCtrl->AppendText(wxString::Format(wxT("%d"),
                               iterationData.NonbasicIndices()[
                                    iterationData.S() - 1]));
       SetAttribute(pTargetCtrl, attributes.TextAttribute(
                        AttributeDatabase::DEFAULT), true);
    }
    else if (iterationData.MainPhase())
    {// Kosztw nie da si zmniejszy, koniec.
        text << wxT("\n\n")
             << _("Function value (cost) cannot be reduced any more, \
optimization complete.");
        pTargetCtrl->AppendText(text);
    }

}// PrintBasicPoint

void
TivSimplex::PrintLeavingVariable(wxTextCtrl*                      pTargetCtrl,
                                 const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    assert(iterationData.H().nrows() == iterationData.Base().ncols());
    assert(iterationData.S() != 0);

    text << wxT("\n\n")
         << _("Shift vector:")
         << wxT('\n');
    Flush(pTargetCtrl, text);

    MultilineString multiline;
    // Dopisujemy macierz bazow.
    MatrixStyle style(iterationData.Base());
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_BASE));
    multiline.Append(wxT("h = "));
    multiline.Append(iterationData.Base(), style, wxT("-1"));

    // Dopisujemy s-t kolumn macierzy N.
    const ColumnVector& N_s = iterationData.N().column(iterationData.S());
    style.Reset(N_s);
    style.SetStyle(attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_N));
    multiline.Append(wxT(" "));
    multiline.Append(N_s, style);

    // Dopisujemy wynik.
    style.Reset(iterationData.H());
    multiline.Append(wxT(" = "));
    multiline.Append(iterationData.H(), style);
    multiline.Flush(pTargetCtrl);

    if (iterationData.Sigma() != NullPoint)
    {// Dopisujemy wektor sigma i zmienn wchodzc.
        assert(iterationData.Q() != 0);

        text  = wxT("\n\n");
        text << _("Maximal allowed step lengths:")
             << wxT('\n');
        pTargetCtrl->AppendText(text);

        style.Reset(iterationData.Sigma());
        style.SetRowStyle(iterationData.Q(),
                          attributes.TextAttribute(
                            AttributeDatabase::SIMPLEX_VAR_LEAVING));
        text  = Symbols::GetInstance()[Symbols::SIGMA];
        text << wxT(" = ");
        multiline.Append(text);
        multiline.Append(iterationData.Sigma(), style);
        multiline.Print(pTargetCtrl);

        text  = wxT('\n');
        text << wxString::Format(wxT("\nq = %d"), iterationData.Q())
             << wxT(", ")
             << _("leaving column: ")
             << wxString::Format(wxT("iB[%d] = ") , iterationData.Q());
        SetAttributeAndFlush(pTargetCtrl,
                             text,
                             attributes.TextAttribute(
                                AttributeDatabase::SIMPLEX_VAR_LEAVING),
                             true);
        pTargetCtrl->AppendText(wxString::Format(wxT("%d"),
                                iterationData.BasicIndices()[
                                    iterationData.Q() - 1]));
        SetAttribute(pTargetCtrl, attributes.TextAttribute(
                            AttributeDatabase::DEFAULT), true);
    }

}// PrintLeavingVariable

void
TivSimplex::PrintNewPoint(wxTextCtrl*                      pTargetCtrl,
                          const SimplexIteration&          iterationData)
                                                                    const
{
    const AttributeDatabase& attributes = AttributeDatabase::GetInstance();
    wxString text;

    assert(iterationData.Sigma() != NullPoint);
    assert(iterationData.Q() != 0);
    assert(iterationData.NewX() != NullPoint);

    // Drukujemy dugo kroku.
    text << wxT("\n\n")
         << _("Step length:\t")
         << Symbols::GetInstance()[Symbols::ALPHA]
         << wxString::Format(wxT(" = "));
    SetAttributeAndFlush(pTargetCtrl, text, attributes.TextAttribute(
            AttributeDatabase::SIMPLEX_VAR_LEAVING), true);
    text << PrintDouble(iterationData.Alpha());
    SetAttributeAndFlush(pTargetCtrl, text, attributes.TextAttribute(
            AttributeDatabase::DEFAULT), true);

    // Drukujemy nastpny punkt bazowy.
    text << wxT('\n')
         << _("Next point:")
         << wxT('\n');
    Flush(pTargetCtrl, text);

    MultilineString multiline;
    MatrixStyle style(iterationData.NewX());
    style.SetRowStyle(iterationData.Q(), attributes.TextAttribute(
            AttributeDatabase::SIMPLEX_VAR_LEAVING));
    multiline.Append(wxT("x_B = "));
    multiline.Append(iterationData.NewX(), style);
    multiline.Print(pTargetCtrl);

}// PrintNewPoint
/////////////////////////////// PRIVATE    ///////////////////////////////////
/* static */ void* TivSimplex::mpRegistrationDummy =
    VisualizerFactory::RegisterTextVisualizer(
            SimplexIteration::ClassId(),
            TivSimplex::Create);
