#include <precomp.h>

#include "../UnivariateConstrainedIteration.h"
#include "../Logger.h"
#include "../ProblemWithPenalty.h"

#include "UnivariateConstrainedLayer.h"  // class implemented
#include "VisualizerFactory.h"

LOGGER(DEBUG)

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

UnivariateConstrainedLayer::UnivariateConstrainedLayer(
        const PlotControl&                          plotControl,
        const Result&                               result,
        CountedPtr<PlotLayer>                       cpFunctionLayer,
        std::auto_ptr<UnivariateConstraintsLayer>   apConstraintsLayer,
        std::auto_ptr<UnivariateFunctionLayer>      apPenaltyLayer)
    : UnivariateResultLayer(plotControl, result, cpFunctionLayer),
      mcpConstraintsLayer(apConstraintsLayer),
      mcpPenaltyLayer(apPenaltyLayer)
{
}// UnivariateConstrainedLayer

UnivariateConstrainedLayer::UnivariateConstrainedLayer(
        const UnivariateConstrainedLayer& from)
    : UnivariateResultLayer(from),
      mcpConstraintsLayer(from.mcpConstraintsLayer),
      mcpPenaltyLayer(from.mcpPenaltyLayer)
{
}// UnivariateConstrainedLayer

UnivariateConstrainedLayer::~UnivariateConstrainedLayer()
{
}// ~UnivariateConstrainedLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
UnivariateConstrainedLayer::Draw(wxDC& dc) const
{
    const ProblemWithPenalty& problem =
        dynamic_cast<const ProblemWithPenalty&>(mResult.Problem());
    problem.Penalty().rPenaltyParam()
            = dynamic_cast<const UnivariateConstrainedIteration&>
                (mResult.Iteration(mHighlightedIteration)).PenaltyParam();

    // Wszystko rysujemy na wykresie funkcji.
    mcpFunctionLayer->Draw(dc);
    mcpPenaltyLayer->Draw(dc);
    mcpConstraintsLayer->Draw(dc);

    for (unsigned i = mResult.FirstIterationNumber();
         i <= mResult.LastIterationNumber();
         i++)
    {
        if (i != mHighlightedIteration)
            DrawIteration(dc, i, false);
    }

    DrawIteration(dc, mHighlightedIteration, true);

    DrawFinalResult(dc);

}// Draw

/* virtual */ vector<ToolbarTool>
UnivariateConstrainedLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(UnivariateResultLayer::CreateTools(pPlotControl));
    vector<ToolbarTool> constraints_tools
        (mcpConstraintsLayer->CreateTools(pPlotControl));

    tools.insert(tools.end(),
            constraints_tools.begin(), constraints_tools.end());

    return tools;
}// CreateTools

/* static */ std::auto_ptr<ResultLayer>
UnivariateConstrainedLayer::Create(const PlotControl&       plotControl,
                                   const Result&            result,
                                   CountedPtr<PlotLayer>    cpFunctionLayer)
{
    const ProblemWithPenalty& problem =
        dynamic_cast<const ProblemWithPenalty&>(result.Problem());

    auto_ptr<UnivariateConstraintsLayer> ap_constraints_layer(
            new UnivariateConstraintsLayer(plotControl,
                problem.Penalty().Constraints()));

    // TODO: Tu w zasadzie powinienem chyba wykonywa kopi funkcji skoro mam
    // jej parametr. UnivariateFunctionLayerTrzyma consta, wic moe zrobi
    // osobn klas, ktra przy okazji bdzie rysowaa funkcj kary domylnie na
    // czerwono.
    auto_ptr<UnivariateFunctionLayer> ap_penalty_layer(
            new UnivariateFunctionLayer(plotControl,
                dynamic_cast<const UnivariateFunction&>(problem.Function())));

    return auto_ptr<ResultLayer>
            (new UnivariateConstrainedLayer(plotControl,
                                              result,
                                              cpFunctionLayer,
                                              ap_constraints_layer,
                                              ap_penalty_layer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
UnivariateConstrainedLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{

    Region bounding_box;

    const UnivariateConstrainedIteration& iteration =
        dynamic_cast<const UnivariateConstrainedIteration&>
            (mResult.Iteration(iterationNumber));

    if (iterationNumber != mResult.FirstIterationNumber())
        bounding_box.Extend(iteration.PreviousApproximation(), 0);

    bounding_box.Extend(iteration.NewApproximation(), 0);

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
UnivariateConstrainedLayer::DrawIteration(wxDC&       dc,
                                            unsigned    iterationNumber,
                                            bool        highlight) const
{

    const UnivariateConstrainedIteration& iteration =
        dynamic_cast<const UnivariateConstrainedIteration&>
            (mResult.Iteration(iterationNumber));

    PointList       points_to_draw;

    AttributeDatabase::AttributeType attribute = AttributeDatabase::NORMAL;

    if (highlight)
    {
        if (iteration.IsNewApproximationValid())
            attribute = AttributeDatabase::POINT_BEST;
        else
            attribute = AttributeDatabase::POINT_OUT_OF_BOUNDS;

        DrawVerticalLine(dc, iteration.NewApproximation(), attribute);
    }

    points_to_draw.push_back(ColourPointType(PlotToDcPoint
                                                (iteration.NewApproximation()),
                                                 attribute));

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));

}// DrawIteration

const /* static */ void* UnivariateConstrainedLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            UnivariateConstrainedIteration::ClassId(),
            UnivariateConstrainedLayer::Create);
