#include <precomp.h>

#include "../Logger.h"
#include "AttributeDatabase.h"
#include "PlotControl.h"
#include "UnivariateFunctionLayer.h"
#include "UnivariatePlot.h" // implemented class

LOGGER(DEBUG)

UnivariatePlot::UnivariatePlot(
        wxWindow*                               pParent,
        wxWindowID                              id,
        const UnivariateFunction&               function,
        PlotStatus*                             pPlotStatus,
        const wxString&                         variableName,
        const wxPoint&                          position,
        const wxSize&                           size,
        bool                                    integersOnX)
    : PlotControl(pParent, id, function, pPlotStatus, position, size)
{
    mXAxis.SetLabel(variableName);
    mYAxis.SetLabel(_("Value"));

    // Brzydki hak, ale ju nie ma czasu na przerabianie...
    if (integersOnX)
    {
        mMinZoomX = 2.0;
        mXAxis.IntegersOnly() = true;
    }

//    AppendLayer(CountedPtr<const PlotLayer>(
//                new UnivariateFunctionLayer(*this, function)));
}// UnivariatePlot

UnivariatePlot::~UnivariatePlot()
{
}// ~UnivariatePlot

/* virtual */ void
UnivariatePlot::OnAspectChange(bool redraw)
{
    UpdateScale();
    LOG(DEBUG, mPlotY1 << wxT(", ") << mPlotY2);
//    mYAxis.Zoom(mPlotY1, mPlotY2);

    if (redraw)
    {
        Redraw();
        Refresh();
    }
}// AspectChange

/* virtual */ void
UnivariatePlot::Pan(wxCoord x, wxCoord y)
{
    assert(mScaleX > 0);

    double plot_shift = (x - mDragStartX) * mScaleX;

    mPlotX1 -= plot_shift;
    mPlotX2 -= plot_shift;

    mXAxis.Zoom(mPlotX1, mPlotX2);

    UpdateYBounds();
    mYAxis.Zoom(mPlotY1, mPlotY2);
}// Pan

/* virtual */ void
UnivariatePlot::ZoomRegionIn(wxCoord x, wxCoord y, wxCoord width, wxCoord height)
{
    int pixel_width  = mPlotBitmap.GetWidth();

    double shift_on_left;
    double shift_on_right;

    wxCoord drag_end_x = x + width;

    if (x < drag_end_x)
    {
        shift_on_left = x * mScaleX;
        shift_on_right = (pixel_width - drag_end_x) * mScaleX;
    }
    else
    {
        shift_on_left = drag_end_x * mScaleX;
        shift_on_right = (pixel_width - x) * mScaleX;
    }

    mPlotX1 += shift_on_left;
    mPlotX2 -= shift_on_right;

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);

    UpdateYBounds();
}// ZoomRegionIn

/* virtual */ void
UnivariatePlot::ZoomIn(wxCoord x, wxCoord y)
{
    double plot_half_width = (mPlotX2 - mPlotX1)/2;

    double middle = mPlotX1 + x * mScaleX;

    mPlotX1 = middle - (plot_half_width) * ZOOM_RATIO;
    mPlotX2 = middle + (plot_half_width) * ZOOM_RATIO;

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);

    UpdateYBounds();
}// ZoomIn


/* virtual */ void
UnivariatePlot::UpdateStatusOnMove(long x, long y)
{
    const UnivariateFunction& function =
        dynamic_cast<const UnivariateFunction&>(mFunction);

    if (mpPlotStatus != NULL)
    {
        if ((x >= mPlotAreaX) && (x <= mPlotAreaX + mPlotBitmap.GetWidth())
            && (y >= mPlotAreaY) && (y <= mPlotAreaY + mPlotBitmap.GetHeight()))
        {
            mpPlotStatus->UpdatePosition(
                    function.EvaluateNoThrow(DcToPlotX(WindowToPlotAreaX(x))),
                    DcToPlotX(WindowToPlotAreaX(x)));
        }
        else
        {
            mpPlotStatus->ClearPosition();
        }
    }
}// UpdateStatusOnMove

/* virtual */ void
UnivariatePlot::UpdateScale()
{
    int pixel_width  = mPlotBitmap.GetWidth();
//    int pixel_height = mPlotBitmap.GetHeight();

    mScaleX     = (mPlotX2 - mPlotX1) / pixel_width;
//    mScaleY     = (mPlotY2 - mPlotY1) / pixel_height;
    UpdateYBounds();
    mScaleValues= mScaleY;

    if (mpPlotStatus != NULL)
    {
        mpPlotStatus->UpdateFormatStrings(mScaleValues, mScaleX);
    }
}// UpdateScale

/* virtual */ void
UnivariatePlot::ZoomOut(wxCoord x, wxCoord y)
{
    double plot_half_width = (mPlotX2 - mPlotX1)/2;
    double middle = mPlotX1 + x * mScaleX;

    mPlotX1 = middle - plot_half_width * (1/ZOOM_RATIO);
    mPlotX2 = middle + plot_half_width * (1/ZOOM_RATIO);

    LimitZoom(mPlotX1, mPlotX2, mMinZoomX, mMaxZoomX);
    UpdateYBounds();

    mXAxis.Zoom(mPlotX1, mPlotX2);
    mYAxis.Zoom(mPlotY1, mPlotY2);

    LOG(DEBUG, wxT("x1: ") << mPlotX1 << wxT("; x2: ") << mPlotX2);
}// ZoomOut

/* virtual */ void
UnivariatePlot::DrawDisplaced(long x, long y)
{
    wxClientDC dc(this);

    dc.SetBrush(AttributeDatabase::GetInstance().
            Brush(AttributeDatabase::PLOT_BACKGROUND));
    dc.SetPen(AttributeDatabase::GetInstance().
            Pen(AttributeDatabase::PLOT_BACKGROUND));

    if (x > 0)
    {
        dc.Blit(mPlotAreaX + x, mPlotAreaY,
                mPlotBitmap.GetWidth() - x, mPlotBitmap.GetHeight(), &mPlotDc,
                0, 0);
        dc.DrawRectangle(mPlotAreaX, mPlotAreaY, x, mPlotBitmap.GetHeight());
    }
    else if (x < 0)
    {
        dc.Blit(mPlotAreaX, mPlotAreaY,
                mPlotBitmap.GetWidth() + x, mPlotBitmap.GetHeight(), &mPlotDc,
                -x, 0);
        dc.DrawRectangle(mPlotAreaX + mPlotBitmap.GetWidth() + x, mPlotAreaY,
                -x, mPlotBitmap.GetHeight());
    }

    mXAxis.Displace(x);

    RedrawAxes(dc);
}// DrawDisplaced

void
UnivariatePlot::UpdateYBounds()
{
    const UnivariateFunction& function =
        dynamic_cast<const UnivariateFunction&>(mFunction);

    mPlotY1 =  DBL_MAX;
    mPlotY2 = -DBL_MAX;

    wxCoord pixel_width;
    wxCoord pixel_height;
    mPlotDc.GetSize(&pixel_width, &pixel_height);

    for (long i = 0; i < pixel_width; ++i)
    {
        double value = function.EvaluateNoThrow(mPlotX1 + i*mScaleX);

        if (isfinite(value))
        {
            if (value > mPlotY2)    mPlotY2 = value;
            if (value < mPlotY1)    mPlotY1 = value;
        }
    }

    if (mPlotY1 > mPlotY2)
    {
        mPlotY1 = 0.0;
        mPlotY2 = 0.0;
    }

    LimitZoom(mPlotY1, mPlotY2, mMinZoomY, mMaxZoomY);
    mScaleY = (mPlotY2 - mPlotY1) / pixel_height;
}// UpdateYBounds

// TODO: zamieni na PlotLayers w PlotControl
/* virtual */ void
UnivariatePlot::PlotFunction(wxDC& dc)
{
    // Nie jest to super wydajne, ale co tam, bdzie elastyczne

    mLayers[0]->Draw(dc);
}// PlotFunction
