#include <precomp.h>

#include "../Logger.h"
#include "../ProblemBase.h"

#include "IterationValueVisualizer.h"
#include "VisualizerFactory.h"  // class implemented

LOGGER(DEBUG)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* static */ VisualizerFactory::VisualizerContainer
VisualizerFactory::CreateVisualizers(wxWindow*      pParent,
                                     const Result&  result)
{
    VisualizerContainer visualizers;

    if (result.Iterations().size() == 0)
    {   // Brak iteracji, zwracamy jedynie wizualizator tekstowy, ktry wypisze
        // podsumowanie.
        TextVisualizer* p_text_visualizer =
            new TextVisualizer(pParent, result,
                    auto_ptr<const TextIterationVisualizer>(NULL));

        visualizers.push_back(p_text_visualizer);
    }
    else
    {
        vector<IterationData::IterationDataIdType>  iteration_ids
            (result.Iterations().front()->ClassIds());
        vector<IterationData::IterationDataIdType>::const_iterator id_iterator;

        VisualizerCreators visualizer_creators;

        CreatorMapType::const_iterator position;

        for (id_iterator  = iteration_ids.begin();
             id_iterator != iteration_ids.end();
             ++id_iterator)
        {
            position = VisualizerFactory::CreatorMap().find(*id_iterator);

            if (position != VisualizerFactory::CreatorMap().end())
            {
                visualizer_creators.Merge(position->second);
            }

            if (visualizer_creators.IsComplete()) break;   // EXIT LOOP
        }

        if (visualizer_creators.IsEmpty())
        {
            LOG(ERROR, wxT("No visualization creators found!"));
            // TODO: zrobi co?
        }
        else
        {
            if (visualizer_creators.mTextCreator != NULL)
            {
                TextVisualizer* p_text_visualizer =
                    new TextVisualizer(pParent, result,
                                visualizer_creators.mTextCreator());
                visualizers.push_back(p_text_visualizer);
            }

            // Nie ma wizualizatorw dla funkcji wicej ni 2 zmiennych
            if ((visualizer_creators.mFullPlotCreator != NULL)
                    && (result.Problem().Variables().GetCount() <= 2))
            {
                auto_ptr<const TextIterationVisualizer>
                        ap_text_iteration_visualizer;

                if (visualizer_creators.mTextCreator != NULL)
                {
                    ap_text_iteration_visualizer =
                            visualizer_creators.mTextCreator();
                }

                FullPlotVisualizer* p_fullplot_visualizer =
                    new FullPlotVisualizer(pParent, result,
                            visualizer_creators.mFullPlotCreator,
                            ap_text_iteration_visualizer);
                visualizers.push_back(p_fullplot_visualizer);
            }

            // Musz by przynajmniej dwie iteracje
            if ((visualizer_creators.mIterationValueFlag)
                    && (result.IterationCount() > 1))
            {
                visualizers.push_back(
                        new IterationValueVisualizer(pParent, result));
            }
        }
    }// iterations != 0

    return visualizers;
}// CreateVisualizers

/* static */ void*
VisualizerFactory::RegisterTextVisualizer
        (IterationData::IterationDataIdType     id,
         const TextIterationVisualizer::Creator function)
{
    CreatorMapType::iterator position;

    position = VisualizerFactory::CreatorMap().find(id);

    if (position != VisualizerFactory::CreatorMap().end())
    {   // Dodaj
        assert (position->second.mTextCreator == NULL);

        position->second.mTextCreator = function;
    }
    else
    {
        VisualizerCreators creators;

        creators.mTextCreator = function;

        VisualizerFactory::CreatorMap()[id] = creators;
    }

    // Atrapa pozwalajca uy tej funkcji w inicjowaniu statycznym.
    return NULL;
}// RegisterTextVisualizer

/* static */ void*
VisualizerFactory::RegisterFullPlotVisualizer
        (IterationData::IterationDataIdType     id,
         const ResultLayer::Creator             creator)
{
    CreatorMapType::iterator position;

    position = VisualizerFactory::CreatorMap().find(id);

    if (position != VisualizerFactory::CreatorMap().end())
    {   // Dodaj
        assert (position->second.mFullPlotCreator == NULL);

        position->second.mFullPlotCreator = creator;
    }
    else
    {
        VisualizerCreators creators;

        creators.mFullPlotCreator = creator;

        VisualizerFactory::CreatorMap()[id] = creators;
    }

    // Atrapa pozwalajca uy tej funkcji w inicjowaniu statycznym.
    return NULL;
}// RegisterFullPlotVisualizer

/* static */ void*
VisualizerFactory::RegisterIterationValueVisualizer(
        IterationData::IterationDataIdType  id)
{
    CreatorMapType::iterator position;

    position = VisualizerFactory::CreatorMap().find(id);

    if (position != VisualizerFactory::CreatorMap().end())
    {   // Dodaj
        assert (position->second.mIterationValueFlag == false);

        position->second.mIterationValueFlag = true;
    }
    else
    {
        VisualizerCreators creators;

        creators.mIterationValueFlag = true;

        VisualizerFactory::CreatorMap()[id] = creators;
    }

    // Atrapa pozwalajca uy tej funkcji w inicjowaniu statycznym.
    return NULL;
}

//============================= ACCESS     ===================================

/* static */ VisualizerFactory::CreatorMapType&
VisualizerFactory::CreatorMap()
{
    static CreatorMapType creator_map;

    return creator_map;
}// CreatorMap

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////


VisualizerFactory::VisualizerCreators::VisualizerCreators()
    : mTextCreator(NULL), mFullPlotCreator(NULL), mIterationValueFlag(false)
{
}// VisualizerCreators::VisualizerCreators

void
VisualizerFactory::VisualizerCreators::Merge(
        const VisualizerFactory::VisualizerCreators& creators)
{
    if (mTextCreator == NULL)     mTextCreator = creators.mTextCreator;
    if (mFullPlotCreator == NULL) mFullPlotCreator = creators.mFullPlotCreator;
    if (mIterationValueFlag == false)
    {
        mIterationValueFlag = creators.mIterationValueFlag;
    }
}// VisualizerCreators::Merge

bool
VisualizerFactory::VisualizerCreators::IsEmpty() const
{
    if ((mTextCreator != NULL)
            || (mFullPlotCreator != NULL)
            || (mIterationValueFlag == false))
    {
        return false;
    }
    else    return true;
}// VisualizerCreators::IsEmpty

bool
VisualizerFactory::VisualizerCreators::IsComplete() const
{
    if ((mTextCreator == NULL)
            || (mFullPlotCreator == NULL)
            || (mIterationValueFlag == false))
    {
        return false;
    }
    else    return true;
}// VisualizerCreators::IsComplete

/////////////////////////////// PRIVATE    ///////////////////////////////////
