#include <precomp.h>

#include "../MultivariateConstrainedIteration.h"
#include "../Logger.h"
#include "../ProblemWithPenalty.h"

#include "XyConstrainedLayer.h"  // class implemented
#include "VisualizerFactory.h"


LOGGER(DEBUG)

using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

XyConstrainedLayer::XyConstrainedLayer(
        const PlotControl&                          plotControl,
        const Result&                               result,
        CountedPtr<PlotLayer>                       cpFunctionLayer,
        std::auto_ptr<XyConstraintsLayer>           apConstraintsLayer,
        std::auto_ptr<MultivariateFunctionLayer>    apPenaltyLayer)
    : XyResultLayer(plotControl, result, cpFunctionLayer),
      mcpConstraintsLayer(apConstraintsLayer),
      mcpPenaltyLayer(apPenaltyLayer)
{
}// XyConstrainedLayer

XyConstrainedLayer::XyConstrainedLayer(
        const XyConstrainedLayer& from)
    : XyResultLayer(from),
      mcpConstraintsLayer(from.mcpConstraintsLayer),
      mcpPenaltyLayer(from.mcpPenaltyLayer)
{
}// XyConstrainedLayer

XyConstrainedLayer::~XyConstrainedLayer()
{
}// ~XyConstrainedLayer


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================

/* virtual */ void
XyConstrainedLayer::Draw(wxDC& dc) const
{
//    const ProblemWithPenalty& problem =
//        dynamic_cast<const ProblemWithPenalty&>(mResult.Problem());
//    problem.Penalty().rPenaltyParam()
//        = dynamic_cast<const MultivariateConstrainedIteration&>
//            (mResult.Iteration(mHighlightedIteration)).PenaltyParam();


    // Wszystko rysujemy na wykresie funkcji.
    mcpFunctionLayer->Draw(dc);
//    mcpPenaltyLayer->Draw(dc);
    mcpConstraintsLayer->Draw(dc);

    unsigned i = mResult.FirstIterationNumber();

    DrawFirstIteration(dc,
                       i,
                       false);

    for (++i; i <= mResult.LastIterationNumber(); i++)
    {
        if (i != mHighlightedIteration)
            DrawIteration(dc, i, false);
    }

    if (mHighlightedIteration != mResult.FirstIterationNumber())
    {
        DrawIteration(dc, mHighlightedIteration, true);
    }
    else
    {
        DrawFirstIteration(dc, mHighlightedIteration, true);
    }

    DrawFinalResult(dc);

}// Draw

/* virtual */ vector<ToolbarTool>
XyConstrainedLayer::CreateTools(PlotControl* pPlotControl)
{
    vector<ToolbarTool> tools(XyResultLayer::CreateTools(pPlotControl));
    vector<ToolbarTool> constraints_tools
        (mcpConstraintsLayer->CreateTools(pPlotControl));

    tools.insert(tools.end(),
            constraints_tools.begin(), constraints_tools.end());

    return tools;
}// CreateTools

/* static */ std::auto_ptr<ResultLayer>
XyConstrainedLayer::Create(const PlotControl&      plotControl,
                           const Result&           result,
                           CountedPtr<PlotLayer>   cpFunctionLayer)
{
    const ProblemWithPenalty& problem =
        dynamic_cast<const ProblemWithPenalty&>(result.Problem());

    auto_ptr<XyConstraintsLayer> ap_constraints_layer(
            new XyConstraintsLayer(plotControl,
                problem.Penalty().Constraints(), problem.Variables().GetCount()));

    auto_ptr<MultivariateFunctionLayer> ap_penalty_layer(
            new MultivariateFunctionLayer(plotControl,
                    dynamic_cast<const FunctionBase&>(problem.Penalty())));

    return auto_ptr<ResultLayer>
            (new XyConstrainedLayer(plotControl,
                                    result,
                                    cpFunctionLayer,
                                    ap_constraints_layer,
                                    ap_penalty_layer));
}// Create

//============================= ACCESS     ===================================

/* virtual */ Region
XyConstrainedLayer::IterationBoundingBox(unsigned iterationNumber)
        const
{

    Region bounding_box;

    const MultivariateConstrainedIteration& iteration =
        dynamic_cast<const MultivariateConstrainedIteration&>
            (mResult.Iteration(iterationNumber));

    if (iterationNumber != mResult.FirstIterationNumber())
        bounding_box.Extend(iteration.PreviousApproximation()(mVariableXIndex),
                            iteration.PreviousApproximation()(mVariableYIndex));

    bounding_box.Extend(iteration.NewApproximation()(mVariableXIndex),
                        iteration.NewApproximation()(mVariableYIndex));

    return bounding_box;
}// IterationBoundingBox

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void
XyConstrainedLayer::DrawIteration(wxDC&       dc,
                                            unsigned    iterationNumber,
                                            bool        highlight) const
{

    const MultivariateConstrainedIteration& iteration =
        dynamic_cast<const MultivariateConstrainedIteration&>
            (mResult.Iteration(iterationNumber));

    PointList       points_to_draw;

    AttributeDatabase::AttributeType attribute = AttributeDatabase::NORMAL;
/*
    points_to_draw.push_back(ColourPointType(PlotToDcPoint
                                            (iteration.PreviousApproximation()),
                                             attribute));
*/
    if (highlight)
    {
        if (iteration.IsNewApproximationValid())
            attribute = AttributeDatabase::POINT_BEST;
        else
            attribute = AttributeDatabase::POINT_OUT_OF_BOUNDS;
    }

    DrawLine(dc,
             iteration.PreviousApproximation(),
             iteration.NewApproximation(),
             attribute);
    if (highlight)
    {
        if (iteration.IsNewApproximationValid())
            attribute = AttributeDatabase::POINT_BEST;
        else
            attribute = AttributeDatabase::POINT_OUT_OF_BOUNDS;
    }

    points_to_draw.push_back(ColourPointType(PlotToDcPoint
                                                (iteration.NewApproximation()),
                                                 attribute));

    for_each(points_to_draw.begin(), points_to_draw.end(),
            PointPainter(*this, dc));

}// DrawIteration

void
XyConstrainedLayer::DrawFirstIteration(wxDC&       dc,
                                       unsigned    iterationNumber,
                                       bool        highlight) const
{

    const MultivariateConstrainedIteration& iteration =
        dynamic_cast<const MultivariateConstrainedIteration&>
            (mResult.Iteration(iterationNumber));

    AttributeDatabase::AttributeType attribute = AttributeDatabase::NORMAL;

    if (highlight)
    {
        if (iteration.IsNewApproximationValid())
            attribute = AttributeDatabase::POINT_BEST;
        else
            attribute = AttributeDatabase::POINT_OUT_OF_BOUNDS;
    }

    DrawBigPoint(dc, iteration.NewApproximation(), attribute);
}// DrawIteration


const /* static */ void* XyConstrainedLayer::mRegistrationDummy =
    VisualizerFactory::RegisterFullPlotVisualizer(
            MultivariateConstrainedIteration::ClassId(),
            XyConstrainedLayer::Create);
