#include <src/precomp.h>

#include <time.h>
#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/CustomValueCondition.h>
#include <src/InBetweenCrossover.h>
#include <src/Logger.h>
#include <src/NonlinearParser.h>
#include <src/EMIterationsCondition.h>
#include <src/ParseError.h>
#include <src/TournamentSelection.h>
#include <src/UnconstrainedProblem.h>

#include <src/Population.h>
#include <src/methods/EvolutionaryMethod.h>

#include "tests_util.h"
#include <src/util.h>

LOGGER(DEBUG);

using namespace std;

namespace tut
{

    struct EvolutionaryMethodData
    {
    };

    typedef test_group<EvolutionaryMethodData> EvolutionaryMethodTestType;
    typedef EvolutionaryMethodTestType::object EvolutionaryMethodTest;
    EvolutionaryMethodTestType evolutionary_method_group("evolutionary method");


    template<>
    template<>
    void EvolutionaryMethodTest::test<1>()
    {

        try
        {
            LOG(INFO,_T("Test no 1: Population generation test"));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(_T("4*x^2 + 3*y^2 - 4*x*y + x"),
                                 *ap_problem);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector    min_bounds(dimensions);
            ColumnVector    max_bounds(dimensions);

            unsigned    population_size = 50;
            double      left_bound = -10.0;
            double      right_bound = 10.0;

            min_bounds = left_bound;
            max_bounds = right_bound;


            Population population(ap_problem->Function(),
                                  population_size, min_bounds, max_bounds);

            bool passed = true;
            unsigned ch = 0;

            while (passed && ch < population.Size())
            {
                int i = 1;

                while (passed && i <= population[i].mPoint.nrows())
                {
                    double value = population[ch].mPoint(i);

                    if (value > right_bound || value < left_bound)
                        passed = false;

                    ++i;
                }

                ++ch;
            }

            if (population_size != population.Size())
                passed = false;

            ensure("Points out of bounds", passed);

            LOG(INFO,_T("test<1> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, _T("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<1>


    template<>
    template<>
    void EvolutionaryMethodTest::test<2>()
    {
        srand(time(NULL));

        const int length = 41;
        const int focus_tail = length / 2;
        int i, histogram[length];
        double r, sample;

        for (i = 0; i < length; ++i)
            histogram[i] = 0;

        for (i = 0; i < 1000000; ++i)
        {
            r = RandomDouble(0.0, 1.0);

            sample = tan(M_PI * (r - 0.5));
            sample -= 0.5;
            sample = ceil(sample);

            if ((-1 * focus_tail) <= (int)sample && (int)sample <= focus_tail)
            {
                //LOG(DEBUG, wxString::Format(wxT("%d"), (int)sample));
                histogram[(int)sample + length / 2]++;
            }
        }

        for (i = 0; i < length; ++i)
        {
            LOG(DEBUG, wxString::Format(wxT("%2d: %d"),
                                        i - focus_tail,
                                        histogram[i]));
        }

    }// test<2>


    template<>
    template<>
    void EvolutionaryMethodTest::test<3>()
    {

        try
        {
            LOG(INFO, wxT("Test no 3: "));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(_T("x^2 + y^2"),
                                 *ap_problem);

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector correct_result(dimensions);
            correct_result << -0.1875 << -0.125;

            ColumnVector    min_bounds(dimensions);
            ColumnVector    max_bounds(dimensions);

            unsigned    population_size = 10;
            double      left_bound = -10.0;
            double      right_bound = 10.0;
            double      survival_rate = 0.8;

            min_bounds = left_bound;
            max_bounds = right_bound;

            EvolutionaryMethodIteration* p_iteration
                                        = new EvolutionaryMethodIteration();

            Population population(ap_problem->Function(),
                                  population_size, min_bounds, max_bounds);

            SelectionMethod* p_selection
                = SelectionMethod::SelectionMethods()[0]->Clone().release();
//            MutationMethod* p_selection
//                = MutationMethod::MutationMethods()[0]->Clone().release();
            CrossoverMethod*  p_crossover
                = CrossoverMethod::CrossoverMethods()[0]->Clone().release();

            p_selection->Select(population, survival_rate, *p_iteration);
            p_crossover->Crossover(population, 0.1, *p_iteration);

            ensure ("Population size maintained",
                    population.DesiredSize() == population.Size());

            LOG(INFO,_T("test<3> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, _T("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR,wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<3>


    template<>
    template<>
    void EvolutionaryMethodTest::test<4>()
    {

        try
        {
            LOG(INFO,_T("test 4: Test run with 4*x^2 + 3*y^2 - 4*x*y + x."));

            auto_ptr<ProblemBase> ap_problem(
              ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(_T("x^2 + y^2"),
                                 *ap_problem);


            EvolutionaryMethodTester method;

            unsigned int dimensions = ap_problem->VariableCount();

            ColumnVector correct_result(dimensions);
            correct_result = 0.0;

            ColumnVector    min_bounds(dimensions);
            ColumnVector    max_bounds(dimensions);

            min_bounds = -10;
            max_bounds = 10;

            method.SetRangeMinBounds(min_bounds);
            method.SetRangeMaxBounds(max_bounds);
            method.SetRadioactivity(0.2);

            method.SetSelectionMethod(SelectionMethod::SelectionMethods()[0]->
                                                            Clone().release());
            method.SetMutationMethod(MutationMethod::MutationMethods()[0]->
                                                            Clone().release());
            method.SetCrossoverMethod(CrossoverMethod::CrossoverMethods()[0]->
                                                            Clone().release());

            method.SetStopCondition(new EMIterationsCondition);

            auto_ptr<const Result> ap_result
                (method.StartOptimize(*ap_problem).release());

            LOG(INFO,  _T("Result:\n") << *ap_result);
            LOG(DEBUG, _T("CorrectResult:\n") << correct_result);
            LOG(DEBUG, _T("FinalResult:\n") << ap_result->FinalResult());

            ensure("Correct result",
                EqualVectors(ap_result->FinalResult(),
                             correct_result, 0.2));

            LOG(INFO,_T("test<4> finished."));

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, wxT("Parse error: ") << error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") <<
                    wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }

    }// test<4>

}

