#include <src/precomp.h>

#include <tut.h>

#include <src/DistanceCondition.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/UnconstrainedProblem.h>

#include <src/methods/DichotomousDivision.h>
#include <src/methods/HookeJeevesDiscrete.h>
#include <src/methods/HookeJeevesDiscreteIteration.h>
#include <src/methods/HookeJeevesOptimal.h>

#include <src/Logger.h>

#include "tests_util.h"

LOGGER(WARNING);

using namespace std;

namespace tut
{
    struct HookeJeevesData
    {
    };

    typedef test_group<HookeJeevesData> HookeJeevesTestType;
    typedef HookeJeevesTestType::object HookeJeevesTest;
    HookeJeevesTestType hooke_jeeves_discrete_group("hooke jeeves");

    // Test na CoordinateSearch
    template<>
    template<>
    void HookeJeevesTest::test<1>()
    {
        HookeJeevesDiscreteTester tester;

        ColumnVector proper(2);

        proper(1) = -1.0;
        proper(2) =  1.0;

        ensure(proper == tester.Test1());
    }

    // Prbny przebieg
    template<>
    template<>
    void HookeJeevesTest::test<2>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("x^2 + y^2"), *ap_problem);

            HookeJeevesDiscreteTester method;

            ColumnVector startingPoint(ap_problem->Variables().GetCount());

            startingPoint(1) = 1.0;
            startingPoint(2) = 2.0;

            method.SetStartingPoint(startingPoint);

            method.SetEpsilon(0.5);

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            const HookeJeevesDiscreteIteration& iteration =
                dynamic_cast<const HookeJeevesDiscreteIteration&>
                    (ap_result->Iteration(ap_result->FirstIterationNumber()));
            const HookeJeevesIteration::TrialPointContainer& trial_steps =
                iteration.TrialPoints();
            ColumnVector x(2);


            x <<  1.0 <<  2.0;
            ensure("Starting point: ",  iteration.StartingPoint() == x);
            ensure("Old base point: ",  iteration.OldBasePoint() == x);
            LOG(DEBUG, wxT("Trial steps:"));
            LOG(DEBUG, trial_steps[0].mPoint);
            LOG(DEBUG, trial_steps[1].mPoint);
            LOG(DEBUG, trial_steps[2].mPoint);
            LOG(DEBUG, trial_steps[3].mPoint);
            ensure("3 trial steps",   trial_steps.size() == 4);
            x <<  2.0 <<  2.0;
            ensure("1st trial step",  trial_steps[0].mPoint == x);
            ensure("1st trial step worse", ! trial_steps[0].mBetter);
            x <<  0.0 <<  2.0;
            ensure("2nd trial step",  trial_steps[1].mPoint == x);
            ensure("2nd trial step better", trial_steps[1].mBetter);
            x <<  0.0 <<  3.0;
            ensure("3rd trial step",  trial_steps[2].mPoint == x);
            ensure("3rd trial step worse", ! trial_steps[2].mBetter);
            x <<  0.0 <<  1.0;
            ensure("3rd trial step",  trial_steps[3].mPoint == x);
            ensure("3rd trial step better", trial_steps[3].mBetter);
            ensure("Base point",      iteration.BasePoint() == x);
            x << -1.0 << 0.0;
            ensure("Iteration end point: ", iteration.EndPoint() == x);

            ensure("Step not cancelled: ", iteration.IsStepCancelled()
                    == false);

//            x <<  1.0 << -1.0;
//            proper_result.AddStep(x, 3, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            x << -1.0 << -1.0;
//            proper_result.AddStep(x, 3, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            x <<  0.0 <<  0.0;
//            proper_result.AddStep(x, 3);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            proper_result.AddFlag(PointResult::REVERT);
//            //TODO: set option
//
//            x <<  0.5 <<  0.0;
//            proper_result.AddStep(x, 4, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            x << -0.5 <<  0.0;
//            proper_result.AddStep(x, 4, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//
//            x <<  0.0 <<  0.5;
//            proper_result.AddStep(x, 4, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            x <<  0.0 << -0.5;
//            proper_result.AddStep(x, 4, PointResult::NOT_CONNECTED);
//            proper_result.AddFlag(PointResult::CANCELLED);
//            proper_result.AddFlag(PointResult::REVERT);

            x << 0.0 << 0.0;
            ensure("Final result", ap_result->FinalResult() == x);
            LOG(DEBUG, wxT("Iteration count: ") <<
                    ap_result->IterationCount());
            ensure("Number of iterations", ap_result->IterationCount() == 4);

        }
        catch (const ParseError& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Test wersji z krokiem optymalnym
    template<>
    template<>
    void HookeJeevesTest::test<3>()
    {
        try
        {
            auto_ptr<ProblemBase>   ap_problem(
                    ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

            SetNonlinearFunction(wxT("x^2 + 2*x*y + y^4"), *ap_problem);

            HookeJeevesOptimalTester method;

            ColumnVector startingPoint(ap_problem->Variables().GetCount());

            startingPoint(1) = 1.0;
            startingPoint(2) = 2.0;

            method.SetStartingPoint(startingPoint);

            DistanceCondition* p_condition = new DistanceCondition();
            p_condition->rEpsilon() = 0.5;
            method.SetStopCondition(p_condition);

            method.SetLineSearchMethod(auto_ptr<const Method>(
                    Method::GetById(DichotomousDivision::ClassId())->Clone()));

            auto_ptr<const Result> ap_result(method.StartOptimize(*ap_problem));

            const HookeJeevesIteration& iteration =
                dynamic_cast<const HookeJeevesIteration&>
                    (ap_result->Iteration(ap_result->FirstIterationNumber()));

            const HookeJeevesIteration::TrialPointContainer& trial_steps =
                iteration.TrialPoints();
            ColumnVector x(2);

            ensure("Trial point count", trial_steps.size() == 2);
            LOG(DEBUG, wxT("Trial steps:"));
            LOG(DEBUG, trial_steps[0].mPoint);
            LOG(DEBUG, trial_steps[1].mPoint);
            x <<  -2.0 <<   2.0;
            ensure("1st trial step",
                    EqualVectors(trial_steps[0].mPoint, x, 0.01));
            ensure("1st trial better", trial_steps[0].mBetter);
            x <<  -2.0 <<   1.0;
            ensure("2st trial step",
                    EqualVectors(trial_steps[1].mPoint, x, 0.01));
            ensure("2nd trial better", trial_steps[1].mBetter);

            ColumnVector finalPoint(2);
            finalPoint << -0.75 << 0.75;

            ensure("Final point",
                    EqualVectors(ap_result->FinalResult(), finalPoint, 0.05));
        }
        catch (const ParseError& error)
        {
            LOG(ERROR, error.What());
            ensure("Unexpected exception", false);
        }
        catch (const Exception error)
        {
            LOG(ERROR, wxT("Newmat error: ") << wxString::FromAscii(error.what()));
            ensure("Unexpected exception", false);
        }
    }

    // Test na HookeJeevesDiscreteIteration::ClassIDs
    template<>
    template<>
    void HookeJeevesTest::test<4>()
    {
        HookeJeevesDiscreteIteration iteration;
        vector<IterationData::IterationDataIdType>  iteration_ids;
        vector<IterationData::IterationDataIdType>  base_ids;

        iteration_ids = iteration.ClassIds();
        // Tak tego oczywicie nie sprawdz (ale warto byo sprbowa :)
//        base_ids = static_cast<IterationData>(iteration).ClassIDs();

        ensure("Number of ids", iteration_ids.size() == 3);
//        ensure("Number of base ids", base_ids.size() == 1);
//        ensure("Correct base id", iteration.ids[1] == base_ids[0]);
    }

}

