#include <src/precomp.h>

#include <tut.h>

#include <src/LinearFunction.h>
#include <src/LinearParser.h>
#include <src/Logger.h>
#include <src/NonlinearFunction.h>
#include <src/NonlinearParser.h>
#include <src/ParseError.h>
#include <src/UnconstrainedProblem.h>
#include <src/UnivariateNonlinearFunction.h>
#include <src/util.h>

#include "tests_util.h"

LOGGER(INFO);

using namespace std;

namespace tut
{
    struct ParserData
    {
    };

    typedef test_group<ParserData> ParserTestType;
    typedef ParserTestType::object ParserTest;
    ParserTestType parser_group("parsers");

    // Test LinearParsera
    template<>
    template<>
    void ParserTest::test<1>()
    {
        LinearParser parser;
        VariableNames properVariables;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        auto_ptr<ProblemBase> ap_problem(
                ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        try
        {
            auto_ptr<FunctionBase> ap_function
                (parser.CreateFunction(wxT("-x + y"), *ap_variables, true));

            ap_problem->SetFunction(ap_function, ap_variables);

            RowVector properCoefficients(2);
            RowVector coefficients(2);

            properCoefficients(1) = -1.0;
            properCoefficients(2) = 1.0;

            properVariables.Add(wxT("x"));
            properVariables.Add(wxT("y"));

            coefficients = dynamic_cast<const LinearFunction&>
                (ap_problem->Function()).Coefficients();

            LOG(DEBUG, coefficients);

            ensure("Invalid coefficients", coefficients == properCoefficients);

            ensure("Invalid variables",
                    ap_problem->Variables() == properVariables);

        }
        catch (const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("No exceptions", false);
        }
        catch (const Exception& error)
        {
            LOG(ERROR, wxString::FromAscii(error.what()));
            ensure("No exceptions", false);
        }

    }

//    template<>
//    template<>
//    void ParserTest::test<2>()
//    {
//        LinearParser parser;
//        VariableNames variables;
//
//        auto_ptr<const FunctionBase>
//            ap_function(parser.CreateFunction(wxT("2*x + y - 4*y"), variables, true));
//
//        RowVector properCoefficients(2);
//
//        properCoefficients(1) = 2.0;
//        properCoefficients(2) = -3.0;
//
//        ensure("Invalid coefficients", dynamic_cast<const LinearFunction*>
//                (ap_function.get())->Coefficients() == properCoefficients);
//    }

    template<>
    template<>
    void ParserTest::test<3>()
    {
        NonlinearParser parser;
        VariableNames variables;

        try
        {
            auto_ptr<FunctionBase>
                ap_function(parser.CreateFunction(wxT("sin(1) + sqrt(4)"),
                            variables, true));
        }
        catch (const ParseError& error)
        {
            LOG(INFO, error.What());
            ensure("Successful parse", false);
        }
    }


    // Bd parsowania
    template<>
    template<>
    void ParserTest::test<4>()
    {
        LinearParser parser;
        VariableNames variables;
        bool gotException = false;

        parser.CreateFunction(wxT("3*x + y"), variables, true);

        VariableNames variables_copy(variables);
        ensure(variables == variables_copy);

        try
        {
            parser.CreateFunction(wxT("2*x +"), variables, true);
        }
        catch (const ParseError& error)
        {
            gotException = true;
            LOG(DEBUG, error.What());
            ensure("Subtest 1. Expected position of error",
                    error.Position() == 5);
        }

        ensure("Subtest 1. Got exception.", gotException);
        ensure("Subtest 1. Variables unchanged.", variables == variables_copy);

        gotException = false;
        try
        {
            parser.CreateFunction(wxT("2*x*4*y + 5"), variables, true);
        }
        catch (const ParseError& error)
        {
            gotException = true;
            LOG(DEBUG, error.What());
            ensure("Subtest 2. Expected position of error",
                    error.Position() == 6);
        }

        ensure("Subtest 2. Got exception.", gotException);
        ensure("Subtest 2.Variables unchanged.", variables == variables_copy);
    }

    // Parsowanie z ustalonymi zmiennymi
    template<>
    template<>
    void ParserTest::test<5>()
    {
        LinearParser parser;
        VariableNames variables;
        bool gotException = false;

        parser.CreateFunction(_T("x + y"), variables, true);

        VariableNames variables_copy(variables);
        try
        {
            parser.CreateFunction(_T("y"), variables, false);
        }
        catch(const Error& error)
        {
            gotException = true;
            LOG(ERROR, error.What());
        }

        ensure("No exceptions", !gotException);
        ensure("Variables unchanged.", variables == variables_copy);

    }

    // Usuwanie starych zmiennych
    template<>
    template<>
    void ParserTest::test<6>()
    {
        LinearParser parser;
        VariableNames variables;
        VariableNames proper_variables;

        parser.CreateFunction(wxT("x + y + z"), variables, true);

        parser.CreateFunction(wxT("a + b"), variables, true);

        proper_variables.Add(wxT("a"));
        proper_variables.Add(wxT("b"));

        ensure("Only new variables", proper_variables == variables);
    }

    // Parser nieliniowy
    template<>
    template<>
    void ParserTest::test<7>()
    {
        // Bananoida
        auto_ptr<const ProblemBase> ap_problem(
                ProblemWithFunction(wxT("100*(y - x^2)^2 + (1 - x)^2")));

        const FunctionBase& function = ap_problem->Function();

        ensure("Function is differentiable", function.IsDifferentiable());

        ColumnVector x(2);


        x << 0.0 << 0.0;
        ensure("Proper value 1", function.Evaluate(x) == 1.0);

        x << -10 << 0.0;
        ensure("Proper value 2", function.Evaluate(x) == 1000121);

        x << 1 << 10;
        ensure("Proper value 3", function.Evaluate(x) == 8100);
    }

    // Funkcja jednej zmiennej
    template<>
    template<>
    void ParserTest::test<8>()
    {
        NonlinearParser parser;
        VariableNames variables;

        try
        {
            auto_ptr<FunctionBase> ap_function(
                    parser.CreateFunction(wxT("x^2 + sin(x)"), variables, true));

            ensure("Function is univariate",
                    dynamic_cast<UnivariateFunction*>(ap_function.get()) != NULL);
            ensure("Function is differentiable", ap_function->IsDifferentiable());
        }
        catch(const Exception& error)
        {
            LOG(ERROR, wxT("Exception: ") << wxString::FromAscii(error.what()));
            ensure("No exceptions", false);
        }
        catch(const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("No exceptions", false);
        }
    }

    // Kolejno dziaa - i ^ :)
    template<>
    template<>
    void ParserTest::test<9>()
    {
        NonlinearParser parser;
        VariableNames variables;

        try
        {
            auto_ptr<const ProblemBase> ap_problem(
                    ProblemWithFunction(wxT("-x^2")));
            const FunctionBase& function = ap_problem->Function();

            ensure("Function is univariate",
                    dynamic_cast<const UnivariateFunction*>(&function) != NULL);
            ensure("Function is differentiable", function.IsDifferentiable());

            ColumnVector x(1);
            x(1) = 1;

            ensure("Power stronger than negation", function.Evaluate(x) == -1);
        }
        catch(const Exception& error)
        {
            LOG(ERROR, wxT("Exception: ") << wxString::FromAscii(error.what()));
            ensure("No exceptions", false);
        }
        catch(const Error& error)
        {
            LOG(ERROR, error.What());
            ensure("No exceptions", false);
        }
    }
}
