#include <src/precomp.h>

#include <tut.h>

#include <src/Logger.h>

#include <src/DistanceCondition.h>
#include <src/NMBestWorstCondition.h>
#include <src/NMCircumferenceCondition.h>
#include <src/NMPreviousWorstCondition.h>
#include <src/NMWorstReplacementCondition.h>
#include <src/NonlinearParser.h>
#include <src/SlopeCondition.h>
#include <src/SquaredCustomValueCondition.h>
#include <src/UnconstrainedProblem.h>
#include <src/ValueCondition.h>
#include <src/VariableNames.h>

LOGGER(WARNING);

namespace tut
{
    struct StopConditionData
    {
        VariableNames variables;
        std::auto_ptr<const FunctionBase> ap_function;
    };

    typedef test_group<StopConditionData> StopConditionTestType;
    typedef StopConditionTestType::object StopConditionTest;
    StopConditionTestType stopcondition_group("stop conditions");

    // Test DistanceCondition
    template<>
    template<>
    void StopConditionTest::test<1>()
    {
        DistanceCondition distanceCondition;

        distanceCondition.rEpsilon() = 1.50001;

        ColumnVector x1(3);
        ColumnVector x2(3);
        ColumnVector x3(3);

        x1 << -1.0 << -1.0 << 1.0;
        x2 << -0.5 << -0.5 << 0.0;
        x3 <<  0.5 <<  0.5 << 0.5;

        LOG(INFO, wxT("Stop condition: ") << distanceCondition.Name());
        ensure("DistanceCondition, check 1",
                distanceCondition.IsMet(x1, x2, 0.0, 0.0) == true);
        ensure("DistanceCondition, check 2",
                distanceCondition.IsMet(x3, x2, 0.0, 0.0) == true);

        ensure("DistanceCondition, check 3",
                distanceCondition.IsMet(x1, x3, 0.0, 0.0) == false);
    }

    // Test ValueCondition
    template<>
    template<>
    void StopConditionTest::test<2>()
    {
        NonlinearParser parser;
        auto_ptr<VariableNames> ap_variables(new VariableNames);
        std::auto_ptr<FunctionBase> ap_function(
            parser.CreateFunction(wxT("x^2 + y^2"), *ap_variables, true));
        auto_ptr<ProblemBase>   ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

        ap_problem->SetFunction(ap_function, ap_variables);

        ValueCondition valueCondition;

        valueCondition.rEpsilon() = 0.5;

        ColumnVector x1(2);
        ColumnVector x2(2);
        ColumnVector x3(2);
        ColumnVector x4(2);

        x1 << -1.0  <<  0.5;
        x2 <<  1.0  <<  0.0;
        x3 << 10.0  <<  0.0;
        x4 << 10.2  <<  0.0;

        LOG(INFO, wxT("Stop condition: ") << valueCondition.Name());

        ensure("ValueCondition, check 1",
                valueCondition.IsMet(x1, x2, ap_problem->Function()) == true);
        ensure("ValueCondition, check 2",
                valueCondition.IsMet(x4, x3, ap_problem->Function()) == false);
    }

    // Test SlopeCondition
    template<>
    template<>
    void StopConditionTest::test<3>()
    {
        NonlinearParser parser;
        VariableNames variables;
        std::auto_ptr<const FunctionBase> ap_function(
            parser.CreateFunction(wxT("x^2 + y^2"), variables, true));

        SlopeCondition slopeCondition;

        slopeCondition.rEpsilon() = 0.5;

        ColumnVector x1(2);
        ColumnVector x2(2);
        ColumnVector x3(2);
        ColumnVector x4(2);

        x1 << 0.5 << 0.5;
        x2 << 0.0 << 0.0;


        LOG(INFO, wxT("Stop condition: ") << slopeCondition.Name());

        // TODO: dokoczy test
    }


    // Test NMBestWorstCondition
    template<>
    template<>
    void StopConditionTest::test<4>()
    {
        PointWithValue x1(2);
        PointWithValue x2(2);
        PointWithValue x3(2);
        PointWithValue x;

        x1.mPoint << 1 << 1;
        x2.mPoint << 5 << 5;
        x3.mPoint << 6 << 6;

        x1.mValue = 21;
        x2.mValue = 22;
        x3.mValue = 23;

        NMBestWorstCondition condition(CountedPtr<StandardStopCondition>(
                    new DistanceCondition));
        condition.rEpsilon() = 2;

        ensure("NMBestWorstCondition, check 1",
                condition.IsMet(x1, x2, x, x, 0.0) == false);
        ensure("NMBestWorstCondition, check 2",
                condition.IsMet(x2, x3, x, x, 0.0) == true);

        condition = NMBestWorstCondition(CountedPtr<StandardStopCondition>(
                    new ValueCondition));
        condition.rEpsilon() = 2;
        ensure("NMBestWorstCondition, check 3",
                condition.IsMet(x1, x2, x, x, 0.0) == true);
        ensure("NMBestWorstCondition, check 4",
                condition.IsMet(x1, x3, x, x, 0.0) == false);

        LOG(INFO, wxT("Stop condition: ") << condition.Name());
    }

    // Test NMCircumferenceCondition
    template<>
    template<>
    void StopConditionTest::test<5>()
    {
        NMCircumferenceCondition condition;
        condition.rEpsilon() = 11;

        PointWithValue x;

        ensure("NMCircumferenceCondition, check 1",
                condition.IsMet(x, x, x, x, 10) == true);
        ensure("NMCircumferenceCondition, check 2",
                condition.IsMet(x, x, x, x, 12) == false);
    }

    // Test NMWorstReplacementCondition
    template<>
    template<>
    void StopConditionTest::test<6>()
    {
        PointWithValue x1(2);
        PointWithValue x2(2);
        PointWithValue x3(2);
        PointWithValue x;

        x1.mPoint << 1 << 1;
        x2.mPoint << 5 << 5;
        x3.mPoint << 6 << 6;

        x1.mValue = 21;
        x2.mValue = 22;
        x3.mValue = 23;

        NMWorstReplacementCondition condition(CountedPtr<StandardStopCondition>(
                    new DistanceCondition));
        condition.rEpsilon() = 2;

        ensure("NMWorstReplacementCondition, check 1",
                condition.IsMet(x, x2, x, x1, 0.0) == false);
        ensure("NMWorstReplacementCondition, check 2",
                condition.IsMet(x, x2, x, x3, 0.0) == true);

        condition = NMWorstReplacementCondition(
                        CountedPtr<StandardStopCondition>(new ValueCondition));
        condition.rEpsilon() = 2;

        ensure("NMWorstReplacementCondition, check 3",
                condition.IsMet(x, x1, x, x2, 0.0) == true);
        ensure("NMWorstReplacementCondition, check 4",
                condition.IsMet(x, x1, x, x3, 0.0) == false);

        LOG(INFO, wxT("Stop condition: ") << condition.Name());
    }

    // Test NMPreviousWorstCondition
    template<>
    template<>
    void StopConditionTest::test<7>()
    {
        PointWithValue x1(2);
        PointWithValue x2(2);
        PointWithValue x3(2);
        PointWithValue x;

        x1.mPoint << 1 << 1;
        x2.mPoint << 5 << 5;
        x3.mPoint << 6 << 6;

        x1.mValue = 21;
        x2.mValue = 22;
        x3.mValue = 23;

        NMPreviousWorstCondition condition(CountedPtr<StandardStopCondition>(
                    new DistanceCondition));
        condition.rEpsilon() = 2;

        ensure("NMPreviousWorstCondition, check 1",
                condition.IsMet(x, x1, x2, x, 0.0) == false);
        ensure("NMPreviousWorstCondition, check 2",
                condition.IsMet(x, x3, x2, x, 0.0) == true);

        condition = NMPreviousWorstCondition(
                        CountedPtr<StandardStopCondition>(new ValueCondition));
        condition.rEpsilon() = 2;

        ensure("NMPreviousWorstCondition, check 3",
                condition.IsMet(x, x2, x1, x, 0.0) == true);
        ensure("NMPreviousWorstCondition, check 4",
                condition.IsMet(x, x3, x1, x, 0.0) == false);

        LOG(INFO, wxT("Stop condition: ") << condition.Name());
    }

    // Test wektorw warunkw zwracanych przez metody.
    template<>
    template<>
    void StopConditionTest::test<8>()
    {
        vector< CountedPtr<StopConditionBase> > conditions;

        conditions = StopConditionBase::StandardStopConditions();

        vector< CountedPtr<StopConditionBase> >::iterator iter;
        for (iter = conditions.begin(); iter != conditions.end(); iter++)
        {
            LOG(INFO, (*iter)->Name());
        }
        LOG(INFO, wxT("----"));
        conditions = StopConditionBase::GradientStopConditions();
        conditions.at(0)->rName() = wxT("||d||");
        for (iter = conditions.begin(); iter != conditions.end(); iter++)
        {
            LOG(INFO, (*iter)->Name());
        }

    }

}// namespace tut
