#include <src/precomp.h>

#include <src/UnconstrainedProblem.h>
#include <src/DirectionalUnivariateAdapter.h>

#include "tests_util.h"

void
SetNonlinearFunction(const wxString&  input, ProblemBase& rProblem)
{
    NonlinearParser parser;
    std::auto_ptr<VariableNames>    ap_variables(new VariableNames);
    std::auto_ptr<FunctionBase>     ap_function;

    ap_function = parser.CreateFunction(input, *ap_variables, true);

    rProblem.SetFunction(ap_function, ap_variables);
}

void
SetLinearFunction(const wxString& input, ProblemBase& rProblem)
{
    LinearParser parser;
    std::auto_ptr<VariableNames>    ap_variables(new VariableNames);
    std::auto_ptr<FunctionBase>     ap_function;

    ap_function = parser.CreateFunction(input, *ap_variables, true);

    rProblem.SetFunction(ap_function, ap_variables);
}

std::auto_ptr<const ProblemBase>
ProblemWithFunction(const wxString& input)
{
    NonlinearParser parser;
    std::auto_ptr<VariableNames> ap_variables(new VariableNames);
    std::auto_ptr<FunctionBase>  ap_function;
    auto_ptr<ProblemBase> ap_problem(
        ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

    ap_function = parser.CreateFunction(input, *ap_variables, true);

    ap_problem->SetFunction(ap_function, ap_variables);

    return auto_ptr<const ProblemBase>(ap_problem);
}

std::auto_ptr<const ProblemBase>
GetUnivariateFunction(const wxString& input)
{
    NonlinearParser parser;
    std::auto_ptr<VariableNames> ap_variables(new VariableNames);
    std::auto_ptr<FunctionBase>  ap_function;
    auto_ptr<ProblemBase>   ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());
    ColumnVector x(1);
    ColumnVector d(1);

    x = 0;
    d = 1;
    ap_function = parser.CreateFunction(input, *ap_variables, true);

    ap_function->SetProblem(ap_problem.get());

    std::auto_ptr<FunctionBase> ap_univariate(
            new DirectionalUnivariateAdapter(*ap_function.release(), x, d));

    ap_problem->SetFunction(ap_univariate, ap_variables);

    // Uwalniamy wskaznik. Bedzie wisial do konca testow, trudno.

    return auto_ptr<const ProblemBase>(ap_problem);
}

bool
EqualVectors(const  ColumnVector& v1,
             const  ColumnVector& v2,
             double delta)
{
    //assert(v1.size() == v2.size());
    if (v1.size() != v2.size())
        return false;

    for (int i = 1; i <= v1.size(); ++i)
    {
        if (fabs(v1(i) - v2(i)) > delta)
        {
            return false;
        }
    }

    return true;
}

bool
EqualMatrices(const  Matrix& m1,
              const  Matrix& m2,
              double delta)
{
    assert((m1.nrows() == m2.nrows()) && (m1.ncols() == m2.ncols()));

    for (int i = 1; i <= m1.nrows(); ++i)
        for (int j = 1; j <= m1.ncols(); ++j)
            if (fabs(m1(i,j) - m2(i,j)) > delta)
                return false;

    return true;
}


///////////////////////// LINEAR METHODS //////////////////////////////////////

///////////////////////// AlphaDivisionTester /////////////////////////////////

AlphaDivisionTester::AlphaDivisionTester()
    : AlphaDivision()
{
}

void
AlphaDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

void
AlphaDivisionTester::SetAlpha(double alpha)
{
    mAlpha = alpha;
    mGoldenSection = false;
}

///////////////////////// BolzanoDivisionTester ///////////////////////////////

BolzanoDivisionTester::BolzanoDivisionTester()
    : BolzanoDivision()
{
}

void
BolzanoDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

void
BolzanoDivisionTester::SetA(double a)
{
    mA  = a;
    mInitialA  = a;
}

void
BolzanoDivisionTester::SetB(double b)
{
    mB  = b;
    mInitialB  = b;
}

///////////////////////// DerivativeTester //////////////////////////////////

DerivativeTester::DerivativeTester()
    : Derivative()
{
}

void
DerivativeTester::SetEpsilon(double epsilon)
{
    mEpsilon    = epsilon;
}

void
DerivativeTester::SetAlpha(double alpha)
{
    mAlpha      = alpha;
}

void
DerivativeTester::SetGamma(double gamma)
{
    mGamma      = gamma;
}

void
DerivativeTester::SetStartingPoint(double startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

///////////////////////// DichotomousDivisionTester ///////////////////////////

DichotomousDivisionTester::DichotomousDivisionTester()
    : DichotomousDivision()
{
}

void
DichotomousDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

void
DichotomousDivisionTester::SetA(double a)
{
    mA  = a;
    mInitialA  = a;
}

void
DichotomousDivisionTester::SetB(double b)
{
    mB  = b;
    mInitialB  = b;
}

///////////////////////// FibonacciDivisionTester /////////////////////////////

FibonacciDivisionTester::FibonacciDivisionTester()
    : FibonacciDivision()
{
}

vector<unsigned int>
FibonacciDivisionTester::GenerateFibonacciSequence(double a,
                                                   double b,
                                                   double epsilon) const
{
    return FibonacciDivision::GenerateFibonacciSequence(a, b, epsilon);
}

void
FibonacciDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon    = epsilon;
}

void
FibonacciDivisionTester::SetA(double a)
{
    mA  = a;
    mInitialA  = a;
}

void
FibonacciDivisionTester::SetB(double b)
{
    mB  = b;
    mInitialB  = b;
}

///////////////////////// NewtonRaphsonTester //////////////////////////////////


NewtonRaphsonTester::NewtonRaphsonTester()
    : NewtonRaphson()
{
}

void
NewtonRaphsonTester::SetEpsilon(double epsilon)
{
    mEpsilon    = epsilon;
}

void
NewtonRaphsonTester::SetStartingPoint(double startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

///////////////////////// QuadraticApproximationTester ////////////////////////

QuadraticApproximationTester::QuadraticApproximationTester()
    : QuadraticApproximation()
{
}

void
QuadraticApproximationTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

///////////////////////// SecantDivisionTester ///////////////////////////////

SecantDivisionTester::SecantDivisionTester()
    : SecantDivision()
{
}

void
SecantDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

void
SecantDivisionTester::SetA(double a)
{
    mA  = a;
    mInitialA  = a;
}

void
SecantDivisionTester::SetB(double b)
{
    mB  = b;
    mInitialB  = b;
}

///////////////////////// SignOfDerivativeTester //////////////////////////////////

SignOfDerivativeTester::SignOfDerivativeTester()
    : SignOfDerivative()
{
}

void
SignOfDerivativeTester::SetEpsilon(double epsilon)
{
    mEpsilon    = epsilon;
}

void
SignOfDerivativeTester::SetAlpha(double alpha)
{
    mAlpha      = alpha;
}

void
SignOfDerivativeTester::SetGamma(double gamma)
{
    mGamma      = gamma;
}

void
SignOfDerivativeTester::SetStartingPoint(double startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

///////////////////////// UniformDivisionTester ///////////////////////////////

UniformDivisionTester::UniformDivisionTester()
    : UniformDivision()
{
}

void
UniformDivisionTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

///////////////////////// MULTIDIMENSIONAL METHODS ////////////////////////////

///////////////////////// DFPTester ///////////////////////////////////////////

DFPTester::DFPTester()
{
}// DFPTester

void
DFPTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
DFPTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
DFPTester::SetStopCondition(const StandardStopCondition *pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// EvolutionaryMethodTester ////////////////////////////

EvolutionaryMethodTester::EvolutionaryMethodTester()
{
}

void
EvolutionaryMethodTester::SetRangeMinBounds(const ColumnVector& rangeMinBounds)
{
    mInitialRangeMinBounds  = rangeMinBounds;
    mRangeMinBounds         = rangeMinBounds;
}

void
EvolutionaryMethodTester::SetRangeMaxBounds(const ColumnVector& rangeMaxBounds)
{
    mInitialRangeMaxBounds  = rangeMaxBounds;
    mRangeMaxBounds         = rangeMaxBounds;
}

void
EvolutionaryMethodTester::SetCrossoverRate(double crossoverRate)
{
    mCrossoverRate = crossoverRate;
}

void
EvolutionaryMethodTester::SetSurvivalRate(double survivalRate)
{
    mSurvivalRate = survivalRate;
}

void
EvolutionaryMethodTester::SetRadioactivity(double radioactivity)
{
    mRadioactivity = radioactivity;
}

void
EvolutionaryMethodTester::SetNumberOfChromosomes(unsigned numberOfChromosomes)
{
    mNumberOfChromosomes = numberOfChromosomes;
}

void
EvolutionaryMethodTester::SetSelectionMethod
                                    (const SelectionMethod* pSelectionMethod)
{
    mcpSelectionMethod.Reset(pSelectionMethod);
}

void
EvolutionaryMethodTester::SetMutationMethod
                                    (const MutationMethod* pMutationMethod)
{
    mcpMutationMethod.Reset(pMutationMethod);
}

void
EvolutionaryMethodTester::SetCrossoverMethod
                                    (const CrossoverMethod* pCrossoverMethod)
{
    mcpCrossoverMethod.Reset(pCrossoverMethod);
}

void
EvolutionaryMethodTester::SetStopCondition
                            (const EMStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// FletcherReevesTester ////////////////////////////////

FletcherReevesTester::FletcherReevesTester()
    : FletcherReeves()
{
}

void
FletcherReevesTester::SetModification(bool modification)
{
    mModification = modification;
}

void
FletcherReevesTester::SetStartingPoint(const ColumnVector& startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

void
FletcherReevesTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
FletcherReevesTester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// HookeJeevesDiscreteTester ///////////////////////////

HookeJeevesDiscreteTester::HookeJeevesDiscreteTester()
{
}// HookeJeevesDiscreteTester

const ColumnVector
HookeJeevesDiscreteTester::Test1()
{
    NonlinearParser parser;
    VariableNames variables;
    HookeJeevesDiscreteIteration iteration_data;

    auto_ptr<VariableNames> ap_variables(new VariableNames);
    auto_ptr<FunctionBase>
        ap_function(parser.CreateFunction(wxT("x^2 + y^2"),
                    *ap_variables, true));
    auto_ptr<ProblemBase>   ap_problem(
            ProblemBase::GetById(UnconstrainedProblem::ClassId())->Clone());

    ap_problem->SetFunction(ap_function, ap_variables);
    const FunctionBase& function = ap_problem->Function();

    mInitialStartingPoint.resize(2);

    mInitialStartingPoint(1) = -2.0;
    mInitialStartingPoint(2) =  2.0;

    ColumnVector x = mInitialStartingPoint;

    CoordinateSearch(function, x, 1.0, iteration_data);

    return x;

}

void
HookeJeevesDiscreteTester::SetEpsilon(double epsilon)
{
    mEpsilon = epsilon;
}

void
HookeJeevesDiscreteTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

///////////////////////// HookeJeevesOptimalTester ////////////////////////////

HookeJeevesOptimalTester::HookeJeevesOptimalTester()
{
}// HookeJeevesOptimalTester

void
HookeJeevesOptimalTester::SetStopCondition
                                    (const StandardStopCondition *pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

void
HookeJeevesOptimalTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
HookeJeevesOptimalTester::SetLineSearchMethod(
        auto_ptr<const Method> apLineSearch)
{
    mcpLineSearch.Reset(dynamic_cast<const LineSearch*>(apLineSearch.release()));
}

///////////////////////// MarquardtTester ////////////////////////////////////

MarquardtTester::MarquardtTester()
    : Marquardt()
{
}

void
MarquardtTester::SetStartingPoint(const ColumnVector& startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

void
MarquardtTester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// NelderMeadTester ////////////////////////////////////

NelderMeadTester::NelderMeadTester()
    : NelderMead()
{
}

const vector<PointWithValue>
NelderMeadTester::TestInitialSimplexGeneration(const ColumnVector& centralPoint,
                                                     double        sideLength,
                                               const FunctionBase* function)
{

    NelderMead::Simplex simplex(centralPoint,
                                DoubleParameter(sideLength, 0, true),
                                function);

    return simplex.rPoints();
}

ColumnVector
NelderMeadTester::TestCentroidFinding(vector<ColumnVector>& initialSimplex,
                                      const ProblemBase& problem)
{

    const FunctionBase& function    = problem.Function();
    ColumnVector        result_point;

    NelderMead::Simplex simplex(initialSimplex,&function);

    return simplex.Centroid(0, simplex.Size() - 2).mPoint;

}

vector<ColumnVector>
NelderMeadTester::TestShrinking(      vector<ColumnVector> initialSimplex,
                                const ProblemBase& problem,
                                      double shrinkageCoefficient)
{
    const FunctionBase& function    = problem.Function();
    vector<ColumnVector> result_vector;
    size_t  i;


    NelderMead::Simplex simplex(initialSimplex,&function);

    simplex.ShrinkTowardBest(shrinkageCoefficient);


    for (i = 0; i < simplex.rPoints().size(); ++i)
    {
        //LOG(DEBUG, wxT("P") << static_cast<int>(i) << wxT(" = ")
        //        << simplex.rPoints()[i].mPoint.as_row());
        //LOG(DEBUG,wxT("Value = ") << simplex.rPoints()[i].mValue);
        result_vector.push_back(simplex.rPoints()[i].mPoint);
    }

    return result_vector;
}

void
NelderMeadTester::SetReflectionCoefficient(double d)
{
    mReflectionCoefficient = d;
}

void
NelderMeadTester::SetExpansionCoefficient(double d)
{
    mExpansionCoefficient = d;
}

void
NelderMeadTester::SetContractionCoefficient(double d)
{
    mContractionCoefficient = d;
}

void
NelderMeadTester::SetShrinkageCoefficient(double d)
{
    mShrinkageCoefficient = d;
}

void
NelderMeadTester::SetStartingPoint(ColumnVector& point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
NelderMeadTester::SetSideLength(double length)
{
    mSideLength  = length;
}

void
NelderMeadTester::SetStopCondition(const NMStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// NewtonTester ////////////////////////////////////////

NewtonTester::NewtonTester()
    : Newton()
{
}

void
NewtonTester::SetStartingPoint(const ColumnVector& startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

void
NewtonTester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// NewtonModifiedTester ////////////////////////////////

NewtonModifiedTester::NewtonModifiedTester()
    : NewtonModified()
{
}

void
NewtonModifiedTester::SetStartingPoint(const ColumnVector& startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

void
NewtonModifiedTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
NewtonModifiedTester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// PolakRibiereTester //////////////////////////////////

PolakRibiereTester::PolakRibiereTester()
    : PolakRibiere()
{
}

void
PolakRibiereTester::SetStartingPoint(const ColumnVector& startingPoint)
{
    mInitialStartingPoint = startingPoint;
    mStartingPoint = startingPoint;
}

void
PolakRibiereTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
PolakRibiereTester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// PowellVariant1Tester ////////////////////////////////

PowellVariant1Tester::PowellVariant1Tester()
{
}// PowellVariant1Tester

void
PowellVariant1Tester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
PowellVariant1Tester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
PowellVariant1Tester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// PowellVariant2Tester ////////////////////////////////

PowellVariant2Tester::PowellVariant2Tester()
{
}// PowellVariant2Tester

void
PowellVariant2Tester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
PowellVariant2Tester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
PowellVariant2Tester::SetStopCondition(const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// SteepestDescentTester ///////////////////////////////

SteepestDescentTester::SteepestDescentTester()
{
}// SteepestDescentTester

void
SteepestDescentTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
SteepestDescentTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
SteepestDescentTester::SetStopCondition(const StandardStopCondition *pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// RosenbrockDiscreteTester ////////////////////////////

RosenbrockDiscreteTester::RosenbrockDiscreteTester()
{
}// RosenbrockDiscreteTester

void
RosenbrockDiscreteTester::SetTau(ColumnVector tau)
{
    mTau = tau;
}

void
RosenbrockDiscreteTester::SetDelta(double delta)
{
    mDelta = delta;
}

void
RosenbrockDiscreteTester::SetAlpha(double alpha)
{
    mAlpha = alpha;
}

void
RosenbrockDiscreteTester::SetBeta(double beta)
{
    mBeta = beta;
}

void
RosenbrockDiscreteTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
RosenbrockDiscreteTester::SetStopCondition
                                    (const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

///////////////////////// RosenbrockOptimalTester /////////////////////////////

RosenbrockOptimalTester::RosenbrockOptimalTester()
{
}// RosenbrockOptimalTester

void
RosenbrockOptimalTester::SetStartingPoint(ColumnVector point)
{
    mInitialStartingPoint = point;
    mStartingPoint = point;
}

void
RosenbrockOptimalTester::SetLineSearchMethod(const LineSearch* pLineSearch)
{
    mcpLineSearch.Reset(pLineSearch);
}

void
RosenbrockOptimalTester::SetStopCondition
                                    (const StandardStopCondition* pCondition)
{
    mcpStopCondition.Reset(pCondition);
}

